c  cosmology--cosmology geometry relations needed for SNAPsim 
c                          -- Eric Linder  29 Feb 2004 
c
c     By defining each relation in terms of preceding ones, down 
c     to a few "atoms", we minimize changes between different 
c     cosmological models.  There will be pathological cosmologies 
c     for which the given relations are invalid, but the relations 
c     below are valid for all Friedmann-Lemaitre cosmologies. 
c
      double precision om,blahblah
      external fnr,yadayada
      common om,q,u,v
      open(1,file='input.dat')
      open(2,file='results.dat')
c
c     read in parameters, e.g. 
c     om = matter energy density
c     omk = curvature energy density 
c     omw = dark energy density 
c     w0 = dark energy equation of state parameter1, e.g. present value
c     wp = dark energy equation of state parameter2, e.g. time variation
c     wfn = read in EOS function as expression, binned values, or solution 
c       to Klein-Gordon ODE solver
      read(1,*) om,omk,omw,w0,wp
c     define independent variable, i.e. time marker 
      z=0.d0 ! redshift - probably use as loop; allow in [-1,+infty]
      y=1.d0+z ! redshift in convenient alternate notation 
      a=1.d0/y ! scale factor - useful for long redshift intervals, e.g. age
c     1st breakpoint - either keep general EOS or give analytic integral
c     define function(x)=(1.d0+wfn(x))/x
c     use integrator to evaluate integral(function, lower bound, upper bound)
      ew=dexp(3.d0*integral(function,1.d0,y))
c     OR do analytically for wfn(a)=w0+wa*(1.d0-a)
      wa=wp
      ew=y**(3.d0*(1.d0+w0+wa))*dexp(-3.d0*wa*(1.d0-a))
c     Note density variables are not independent: omk=1.d0-om-omw
c     We may wish to have even more components, e.g. radiation or 
c     a second dark energy component; omk=1.d0-sum of all
c
c     Define the first atom; this holds for flat (omk=0.d0) or nonflat models 
      hh=dsqrt(om*y**3.+omk*y*y+omw*ew) ! reduced Hubble parameter H/H0
      didz=1.d0/hh ! coordinate interval
      rcoord=integral(didz,1.d0,y) ! coordinate distance 
c     2nd breakpoint - easier to do separately for flat and nonflat models
c     if (omk.gt.1.d-6) ! nonflat, positive omk, negative curvature
      eta=dsqrt(omk)*dsinh(dsqrt(omk)*rcoord) ! conformal time
      detadz=dcosh(dsqrt(omk)*rcoord)*didz
c     if (omk.lt.-1.d-6) ! nonflat, negative omk, positive curvature
      eta=dsqrt(-omk)*dsin(dsqrt(-omk)*rcoord) ! conformal time
      detadz=dcos(dsqrt(omk)*rcoord)*didz
c     if (dabs(omk).lt.1.d-6) ! flat 
      eta=rcoord
      detadz=1.d0/hh 
c     All further definitions now independent of flat or nonflat 
      deta=detadz ! conformal time interval ; convenient to define in dz
      dt=deta/y ! time interval
      rlum=y*eta ! luminosity distance 
      rang=eta/y ! angular diameter distance 
      rpm=eta ! proper motion distance 
      t=integral(dt,1.d0,y) ! lookback time 
c for age of universe better to transform integral in terms of a to 
c keep finite interval; then dt=da/(a*hh), integral from 0 to 1
      rprop=t
      drprop=dt
      distmod=5.d0*dlog10(rlum) ! distance modulus
      solidangle=4.d0*pi ! choose sky area in steradians
      dvprop=drprop*solidangle*rang**2. ! proper volume element
      dvcomov=dvprop*y**3. ! comoving volume element 
c make sure to use proper element with proper densities and comoving 
c element with comoving densities 
c 
c     Derivatives: wherever they cannot be done analytically in 
c     general, use two sided, stable, finite differences.  Be sure to 
c     test step size; generally 1e-3 of typical value should be ok. 
c
c     Integrator function should be of adaptive Romberg type for 
c     best combination of accuracy and speed, since the functions are 
c     smooth and well behaved.  If the EOS wfn is not well behaved then 
c     ew should be evaluated through gaussian quadrature. 
c     Below is an example subroutine for evaluating eta in a flat model, 
c     given either EOS (w0,w1), (w0,wa), or function wfn.  Replace 
c     everywhere I've written above integral(function,a,b) with the 
c     expression answer and add a prior line: 
c       call dqromb(function,a,b,answer) 
c     Note the common block below. 
c
c       call dqromb(fnr,1.d0,y,answer) 
c
c        function fnr(x)
c        double precision fnr,x,om,q,u,v,ew,res,ewfull,powarg,exparg
c        external ew
c        common om,q,u,v
cc  for {w0,w'} comment out call qgausy and uncomment next 
cc  first pair is for v=w1, second for v=wa; comment out one! 
cc        powarg=3.d0*(1.d0+u-v)
cc        exparg=3.d0*v*(x-1.d0)
c       powarg=3.d0*(1.d0+u+v)
c       exparg=-3.d0*v*(1.d0-1.d0/x)
c        ewfull=x**powarg*dexp(exparg)
c        call qgausy(ew,1.d0,x,res)
c        ewfull=x**3.*dexp(3.d0*res)
c        fnr=1.d0/dsqrt(om*x**3.+q*ewfull)
c        return 
c        end
