package gov.fnal.eag.shapelets.functions;

/**
 * 
 * calculates B3 tensor eq  8 - 11 of paper II
 * 
 * @author kuropat
 */
public class B3Tensor extends Factorial {
	/** value of the square root of two */
	private static double sqrt2 = Math.sqrt(2.0);

	private double[][][] bt; // b3tensor

	private double[][][] p;

	private double[][][] lt;

	private static double sqrtPI = Math.sqrt(Math.PI);

	/**
	 * fully-specified constructor 
	 * @param alpha scale length 
	 * @param beta scale length
	 * @param gamma scale length
	 * @param nMax maximum order to consider
	 */	
	public B3Tensor(double alpha, double beta, double gamma, int nMax) {
	double nu = 1.0 / Math.sqrt(1. / (alpha * alpha) + 1. / (beta * beta)
					+ 1. / (gamma * gamma));
		double a = sqrt2 * nu / alpha;
		double b = sqrt2 * nu / beta;
		double c = sqrt2 * nu / gamma;
		double c1 = 0;
		double c2 = 0;
		double c3 = 0;
		lt = new double[nMax + 1][nMax + 1][nMax + 1];
		bt = new double[nMax + 1][nMax + 1][nMax + 1];
		p = new double[nMax + 1][nMax + 1][nMax + 1];
		lt[0][0][0] = 1.0;
		for (int i = 0; i < nMax+1; i++) {
			for (int l = 0; l <= i; l++) {
				for (int m = 0; m <= i; m++) {
					for (int n = 0; n <= i; n++) {

						if ((l >= m) && (l >= n)) {
							c1 = 0;
							c2 = 0;
							c3 = 0;
							if (((l - 1) >= 0) && ((l - 2) >= 0)) {
								c1 = 2. * (l - 1) * (a * a - 1)
										* lt[l - 2][m][n];
							}
							if (((l - 1) >= 0) && ((m - 1) >= 0)) {
								c2 = 2. * m * a * b * lt[l - 1][m - 1][n];
							}
							if (((l - 1) >= 0) && ((n - 1) >= 0)) {
								c3 = 2. * n * a * c * lt[l - 1][m][n - 1];
							}
						}

						if ((m >= l) && (m >= n)) {
							c1 = 0;
							c2 = 0;
							c3 = 0;
							if (((m - 1) >= 0) && ((m - 2) >= 0)) {
								c1 = 2. * (m - 1) * (b * b - 1)
										* lt[l][m - 2][n];
							}
							if (((m - 1) >= 0) && ((l - 1) >= 0)) {
								c2 = 2. * l * a * b * lt[l - 1][m - 1][n];
							}
							if (((m - 1) >= 0) && ((n - 1) >= 0)) {
								c3 = 2. * n * b * c * lt[l][m - 1][n - 1];
							}
						}

						if ((n >= l) && (n >= m)) {
							c1 = 0;
							c2 = 0;
							c3 = 0;
							if (((n - 1) >= 0) && ((n - 2) >= 0)) {
								c1 = 2. * (n - 1) * (c * c - 1)
										* lt[l][m][n - 2];
							}
							if (((n - 1) >= 0) && ((l - 1) >= 0)) {
								c2 = 2. * l * a * c * lt[l - 1][m][n - 1];
							}
							if (((n - 1) >= 0) && ((m - 1) >= 0)) {
								c3 = 2. * m * b * c * lt[l][m - 1][n - 1];
							}
						}

						lt[l][m][n] = c1 + c2 + c3;
						if ((l == 0) && (m == 0) && (n == 0)) {
							lt[l][m][n] = 1.0;
						}
					}
				}
			}
		}

		//double temp = 0.;
		//double f = 0.;
		// Calculate normalization factor matrix
		for (int i = 0; i < nMax+1; i++) {
			for (int j = 0; j < nMax+1; j++) {
				for (int k = 0; k < nMax+1; k++) {
					p[i][j][k] = nu
							/ Math.sqrt(Math.pow(2., (i + j + k - 1.)) * sqrtPI
									* factorial[i] * factorial[j]
									* factorial[k] * alpha * beta * gamma);

				}
			}
		}

		for (int l = 0; l < nMax+1; l++) {
			for (int m = 0; m < nMax+1; m++) {
				for (int n = 0; n < nMax+1; n++) {
					bt[l][m][n] = lt[l][m][n] * p[l][m][n];
				}
			}
		}

	}
	
	/**
	 * @return the b3 tensor
	 */
	public double[][][] getBt() {
		return bt;
	}
	
	/**
	 * @return the lt tensor
	 */
	public double[][][] getLt() {
		return lt;
	}
}