
package gov.fnal.eag.shapelets.functions;

/**
 * implements paper 2 BasisFunction tensor, one-dimensional, equation 8 
 * 
 * @author stoughton
 * 
 */
public class CTensor extends Factorial{
	private double[][][] bt;  // b3tensor
	//private double[][][] p;
	protected double[][][] ct;  // one dimensional convolution tensor
	private static double sqrt2 = Math.sqrt(2.0);
	private static double sqrtPI = Math.sqrt(Math.PI);
	//private BasisFunction b;
	/**
	 * fills in the bt tensor for the given values (using gamma, alpha, beta instead of a1, a2, a3);
	 * @param gamma -- scale of the resultin image
	 * @param alpha -- scale of the original image
	 * @param beta  -- scale of the Psf image
	 * @param nMax  -- resulting matrix will go up to nMax
	 */
	public CTensor(double gamma,    // scale factor of the convolved image
			double alpha,       // scale factor  of the original image
			double beta,      // scale factor of the PSF image
			int nMax) {
		//b = new BasisFunction(nMax);
		B3Tensor b3t = new B3Tensor(1./gamma,1./alpha,1./beta,nMax);
		bt = b3t.getBt();
		ct = new double[nMax+1][nMax+1][nMax+1];
		//double temp = 0.;
		double f = 0.;
		// Calculate normalization factor matrix
		for (int i=0; i<=nMax; i++) {
			for (int j= 0; j<=nMax; j++) {
				for (int k=0; k<=nMax; k++) {
					ct[i][j][k] = 0.;
					if (Math.IEEEremainder((i + j + k), 2.0) == 0) {
                        f = Math.pow(-1, i + (i + j + k) / 2.);
 					    ct[i][j][k] = sqrt2*sqrtPI*f*bt[i][j][k]; 
					}	
				}
			}
		}	


  }
	/**
	 * @return the one dimensional convolution tensor
	 */
	public double[][][] getCt() {
		return ct;
	}


}	
	
	
	
	
	
	
	
	
	
	
