package gov.fnal.eag.shapelets.functions;

//import hep.aida.IAnalysisFactory;
//import hep.aida.IBaseHistogram;
//import hep.aida.IFitData;
//import hep.aida.IFitFactory;
//import hep.aida.IFitResult;
//import hep.aida.IFitter;
//import hep.aida.IFunction;
//import hep.aida.IFunctionFactory;
//import hep.aida.IHistogram;
//import hep.aida.IHistogram1D;
//import hep.aida.IHistogramFactory;
//import hep.aida.IPlotter;
//import hep.aida.ITree;

//import java.io.IOException;
import java.util.Random;

/**
 *  generates random numbers according to the Epanechinikov distribution, paper 4 equation 3
 */
public class Epanechinikov {
	/**
	 * 	random number generator
	 */
	public Random r = new Random();

	
	/**
	 * constructor with seed
	 * @param seed
	 */
	public Epanechinikov(long seed) {
		super();
		setSeed(seed);
	}
	/**
	 * default constructor with seed of 0
	 */
	public Epanechinikov() {
		this(0);
	}
	/**
	 * get the next random number in the distribution with lambda
	 * @param lambda
	 * @return the random number
	 */
	public double next(double lambda) {
		//double x;
		double x1 = lambda*(2.0*r.nextDouble()-1.0);
		double x2 = lambda*(2.0*r.nextDouble()-1.0);
		double x3 = lambda*(2.0*r.nextDouble()-1.0);
		if (Math.abs(x1) < Math.abs(x2) || Math.abs(x1) < Math.abs(x3)) {
			return x1;
		}
		return x2;
	}
	/**
	 * get the next vector of random numbers in the distribution with lambdas
	 * @param lambda double[]
	 * @return the vector of random numbers  
	 */
	public double[] nextVector(double[] lambda) {
		double[] delta = new double[lambda.length];
		for (int i=0; i<lambda.length; i++) {
			delta[i] = next(lambda[i]);
		}
		return delta;
	}
	/**
	 * set the random number seed
	 * @param seed
	 */
	public void setSeed(long seed) {
		r.setSeed(seed);
	}


	/**
	 * @param argv
	 * @throws IOException
	 */
/*
	public  static void main(String argv[]) throws IOException {
		Epanechinikov e = new Epanechinikov();

		IAnalysisFactory af = IAnalysisFactory.create();
		ITree tree = af.createTreeFactory().create("e.aida", null, false, true);
		IHistogramFactory hf = af.createHistogramFactory(tree);
		long seed = 1234567;
		for (double lambda = 0.1; lambda < 20; lambda *= 5) {
			System.out.println("begin lambda="+lambda);
			IHistogram h1d = (IHistogram) hf.createHistogram1D("e", 100,
					-lambda, lambda);

			e.setSeed(seed++);
			int npt = 50000;
			for (int i = 0; i < npt; i++) {
				((IHistogram1D) h1d).fill(e.next(lambda));
			}

			IFunctionFactory functionFactory1 = af.createFunctionFactory(tree);
			IFunction function1 = functionFactory1.createFunctionFromScript(
					"parabola", 1, "a+b*x[0]+c*x[0]*x[0]", "a,b,c", "parabola");
			IFitFactory fitFactory1 = af.createFitFactory();
			IFitter fitter1 = fitFactory1.createFitter("Chi2", "minuit");
			IFitResult f1 = fitter1.fit((IFitData) h1d, function1);

			IPlotter plotter = af.createPlotterFactory().create("Plot");
			plotter.region(0).plot((IBaseHistogram) h1d);
			plotter.region(0).plot(f1.fittedFunction());
			plotter.writeToFile("e"+lambda+".jpg", "jpg");

			double a = f1.fittedParameter("a");
			double c = f1.fittedParameter("c");
	
		}
	}
	*/
}

