package gov.fnal.eag.shapelets.functions;

/**
 * convolve shapelet object with a gaussian, implementing equations 48 through 55 of paper 1.
 * @author stoughto
 *
 */
public class GaussConvolve {
	static double[][] gnm;
	static int  nMax;
	double beta;
	static {
		nMax = BasisFunction.nMaxBasis;
		//System.out.println(" static GaussConvolve:  nMax="+nMax);
		gnm = new double[nMax+1][nMax+1];
		for (int n=0; n<=nMax; n++) {
			for (int m=n; m<=nMax; m+=2) {
				gnm[n][m] = Math.pow(2, (n-m)/2) * 
		Math.sqrt(Factorial.factorial[m]/Factorial.factorial[n])/Factorial.factorial[(m-n)/2];
			}
		}
	}

	/**
	 * convolve the shapelet by a gaussian
	 * @param alpha scale of the input shapelet
	 * @param f eigenvalues of the input shapelet
	 * @param beta scale (sigma) of the gaussian
	 * @return eigenvalues of the convolved shapelet; use getGamma to find its scale
	 */
	public static double[][] convolve(double alpha, double[][] f, double beta) {
		int convolveNMax = f.length-1;
		double omega = alpha*beta/Math.sqrt(alpha*alpha+beta*beta);
		double[] alphaToThe = new double[f.length];
		for (int i=0; i<=convolveNMax; i+=2) {
			alphaToThe[i] = Math.pow(alpha, i);
		}
		double[] betaToThe = new double[f.length];
		for (int i=0; i<=convolveNMax; i++) {
			betaToThe[i] = Math.pow(beta, i+0.5);
		}
		double[] omegaToThe = new double[f.length];
		for (int i=0; i<=convolveNMax; i++) {
			omegaToThe[i] = Math.pow(omega, i+0.5);
		}	
		double[][] retval = new double[f.length][f.length];
		for (int n1=0; n1<=convolveNMax; n1++) {
			for (int n2=0; n2<=convolveNMax-n1; n2++) {
				for (int m1=n1; m1<=convolveNMax; m1+=2) {
					for (int m2=n2; m2<=convolveNMax; m2+=2) {
						double increment = f[m1][m2] *
						(
								gnm[n1][m1]*omegaToThe[m1]/(alphaToThe[m1-n1]*betaToThe[n1])
						)
						*
						(
								gnm[n2][m2]*omegaToThe[m2]/(alphaToThe[m2-n2]*betaToThe[n2])
						);
						retval[n1][n2] += increment;
						//System.out.println(" m1="+m1+" m2="+m2+" increment="+increment+" retval["+n1+"]["+n2+"]="+retval[n1][n2]);
					}
				}
			}
		}
		return retval;	
	}
	/**
	 * find the scale of a shapelet of scale alpha convolved with a gaussian of scale (sigma) beta
	 * @param alpha scale of input shapelet
	 * @param beta scale (sigma) of gaussina
	 * @return the scale of the convolved shapelet
	 */
	public static double getGamma(double alpha, double beta) {
		return Math.sqrt(alpha*alpha + beta*beta);
	}
	
	/**
	 * calculate eivenvalues for a shapelet convolved with a gaussian
	 * @param alpha scale of the input shapelet
	 * @param shapelets eigenvalues of the input shapelet
	 * @param beta sigma of the gaussian
	 * @return resulting eigenvalues
	 */
	
	public static double[] convolve(double alpha, double[] shapelets, double beta) {
		if (alpha == 0) {
			return shapelets;
		}
		double[][] shapeletsMatrix = BasisFunction.vector2Tab(shapelets);
		double[][] convolvedMatrix = GaussConvolve.convolve(alpha, shapeletsMatrix, beta);
		return BasisFunction.tab2Vector(convolvedMatrix);
	}
	
}
