package gov.fnal.eag.shapelets.functions;

/**
 * constructs and evaluates the Hermite polynomials
 * 
 * @author stoughton
 */
public class Hermite {
	protected static int nMaxHermite;
	protected static Polynomial[] H = null;
	
	/**
	 * construct the hermite polynomials up to the maximum number of orders
	 * @param nMax
	 */
	static {
		// nMaxHermite is the master constant.   A value of 20 allows 16 shapelet orders.
		// 25 allows nMax=21 which takes 231 coefficients
		nMaxHermite = 25;            
		H = new Polynomial[nMaxHermite];
		H[0] = new Polynomial(1, 0);   //  1
		H[1] = new Polynomial(2, 1);   //  2x

		// compute Hermite polynomials
		for (int n = 2; n < nMaxHermite; n++) {
			Polynomial temp1 = H[1].times(H[n-1]);
			Polynomial temp2 = new Polynomial(2 * (n-1), 0);   // 2(n-1)
			Polynomial temp3 = temp2.times(H[n-2]);
			H[n] = temp1.minus(temp3);
		}
	}

	/**
	 * evaluate the Hermite polynomila
	 * @param n order to evaluate
	 * @param x where to evaluate
	 * @return the value
	 * @throws Exception
	 */
	public static double evaluate(int n, double x) throws Exception {
		double value;
		try {
			value = H[n].evaluate(x);
		} catch (Exception e){
			throw e;
		}
		return value;
	}

	/**
	 * reports the number of orders defined
	 * @return the number of orders
	 */
	public static int getNMax() {return nMaxHermite;}

	
	/**
	 * @return the Hermite polynomials
	 */
	public static Polynomial[] getH() {
		return H;
	}
}
