package gov.fnal.eag.shapelets.functions;

/**
 *
 * calculates image moments, implements eq 26 - 28 of Paper I.
 * 
 * @author kuropat
 */

public class ImageAnalyser {

	/**
	 * convenience storage for the square root of pi
	 */
	private static double sqrtPI = Math.sqrt(Math.PI);
	/**
	 * the object's flux
	 */
	private double Flux;  // Flux from the object
	/**
	 * the object's centroid y position
	 */
	private double Yc;    // Centroid coordinates
	/**
	 * the object's centroid x position
	 */
	private double Xc;    //


	
	
	/**
	 * fully-specified constructor
	 * 
	 * @param nMax the maximum rank to evaluate
	 * @param beta scale factor
	 * @param fnm matrix of eigenvalues
	 */
	public ImageAnalyser( double beta, double[][] fnm) {
		int nMax = Math.min(fnm.length-1, BasisFunction.nMaxBasis);
		//First calculate Flux

		for (int n1=0; n1<=nMax; n1+=2) {
			for (int n2=0; n2<=nMax; n2+=2) {
				Flux += Math.pow(2.,(2. -n1 -n2)/2.)*Math.sqrt(BasisFunction.binom[n1][n1/2]) * Math.sqrt(BasisFunction.binom[n2][n2/2])*fnm[n1][n2];
			}
		}
		Flux *=sqrtPI*beta;
		// Now calculate centroids
		for (int n1 = 1; n1<nMax; n1+=2) {
			for (int n2 = 0; n2<nMax; n2+=2) {
				Yc += Math.sqrt(n1+1.)*Math.pow(2.,(2.-n1-n2)/2.)*Math.sqrt(BasisFunction.binom[n1+1][(n1+1)/2]) *
				      Math.sqrt(BasisFunction.binom[n2][n2/2])*fnm[n1][n2];
			}
		}
		Yc = Yc*sqrtPI*beta*beta/Flux;
		for (int n2 =1; n2<nMax; n2 +=2) {
			for (int n1=0; n1<=nMax; n1 +=2) {
				Xc += Math.sqrt(n2+1.)*Math.pow(2.,(2.-n1-n2)/2.)*Math.sqrt(BasisFunction.binom[n2+1][(n2+1)/2]) *
			      Math.sqrt(BasisFunction.binom[n1][n1/2])*fnm[n1][n2];
			}
		}
		Xc = Xc*sqrtPI*beta*beta/Flux;


	}

	/**
	 * @return double flux
	 */
	public double getFlux() {return Flux; }
	/**
	 * @return y
	 */
	public double getYc() {return Yc; }
	/**
	 * @return x
	 */
	public double getXc() {return Xc; }


}
