package gov.fnal.eag.shapelets.functions;
/**
	 *
	 * calculates image moments, implements eq 26 - 28 of Paper I.
	 * 
	 * @author kuropat
	 */

public class ImageMoments {


		private double beta3; // beta in power 3
		/**
		 * convenience storage for the square root of pi
		 */
		private static double sqrtPI = Math.sqrt(Math.PI);
		/**
		 * the object's flux
		 */
		private double Flux;  // Flux from the object
		/**
		 * the object's centroid y position
		 */
		private double Yc;    // Centroid coordinates
		/**
		 * the object's centroid x position
		 */
		private double Xc;    //
		/**
		 * effective radius of the image, used to evaluate beta
		 */
		private double R2f;   // RMS of the image
		//private double R2;
		private double[] ellipticity = new double[2];
		
		
		/**
		 * fully-specified constructor
		 * 
		 * @param nMax the maximum rank to evaluate
		 * @param beta scale factor
		 * @param fnm matrix of eigenvalues
		 */
		public ImageMoments( double beta, double[][] fnm) {
			double q11 = 0.;
			double q12 = 0.;
			double q22 = 0.;
			double A = 0.;
			double B = 0.;
			beta3 = beta*beta*beta;
			int nMax = Math.min(fnm.length-1, BasisFunction.nMaxBasis);
			//First calculate Flux

			for (int n1=0; n1<=nMax; n1+=2) {
				for (int n2=0; n2<=nMax; n2+=2) {
					Flux += Math.pow(2.,(2. -n1 -n2)/2.)*Math.sqrt(BasisFunction.binom[n1][n1/2]) * Math.sqrt(BasisFunction.binom[n2][n2/2])*fnm[n1][n2];
				}
			}
			Flux *=sqrtPI*beta;
			// Now calculate centroids
			for (int n1 = 1; n1<nMax; n1+=2) {
				for (int n2 = 0; n2<nMax; n2+=2) {
					Yc += Math.sqrt(n1+1.)*Math.pow(2.,(2.-n1-n2)/2.)*Math.sqrt(BasisFunction.binom[n1+1][(n1+1)/2]) *
					      Math.sqrt(BasisFunction.binom[n2][n2/2])*fnm[n1][n2];
				}
			}
			Yc = Yc*sqrtPI*beta*beta/Flux;
			for (int n2 =1; n2<nMax; n2 +=2) {
				for (int n1=0; n1<=nMax; n1 +=2) {
					Xc += Math.sqrt(n2+1.)*Math.pow(2.,(2.-n1-n2)/2.)*Math.sqrt(BasisFunction.binom[n2+1][(n2+1)/2]) *
				      Math.sqrt(BasisFunction.binom[n1][n1/2])*fnm[n1][n2];
				}
			}
			Xc = Xc*sqrtPI*beta*beta/Flux;
			// Finally calculate RMS
			for (int n1=0; n1<=nMax; n1 +=2) {
				for (int n2=0; n2<=nMax; n2 +=2) {
					R2f +=Math.pow(2.,(4.-n1-n2)/2.)*(1.+n1+n2)*Math.sqrt(BasisFunction.binom[n1][n1/2])*
					      Math.sqrt(BasisFunction.binom[n2][n2/2])*fnm[n1][n2];
				}
			}
			R2f = Math.sqrt(R2f*sqrtPI*beta*beta*beta/Flux);
			// Calculate moments
			for (int n1=0; n1<=nMax; n1 +=2) {
				for (int n2=0; n2<=nMax; n2 +=2) {
					A = Math.sqrt(Factorial.factorial[n1]*Factorial.factorial[n2]);
					B = 1./(Factorial.factorial[n1/2]*Factorial.factorial[n2/2]);
					q11 += Math.pow(2.,(2.-n1-n2)/2.)*(1.+2.*n1)*A*B*fnm[n1][n2];
					q22 += Math.pow(2.,(2.-n1-n2)/2.)*(1.+2.*n2)*A*B*fnm[n1][n2];
				}
			}
			for (int n1=1; n1<=nMax; n1 +=2) {
				for (int n2=1; n2<=nMax; n2 +=2) {
				q12 += 	Math.pow(2.,(2.-n1-n2)/2.)*Math.sqrt((n1+1.)*(n2+1.))*
				Math.sqrt(Factorial.factorial[n1+1]*Factorial.factorial[n2+1])/
				(Factorial.factorial[(n1+1)/2]*Factorial.factorial[(n2+1)/2])*fnm[n1][n2];
					
				}
			}
			q11 *= sqrtPI*beta3;
			q22 *= sqrtPI*beta3;
			q12 *= sqrtPI*beta3;

			ellipticity[0] = (q11-q22)/(q11+q22);
			ellipticity[1] = 2.*q12/(q11+q22); 
		}

		/**
		 * @return double flux
		 */
		public double getFlux() {return Flux; }
		/**
		 * @return y
		 */
		public double getYc() {return Yc; }
		/**
		 * @return x
		 */
		public double getXc() {return Xc; }
		/**
		 * @return r2
		 */
		public double getR2() {return R2f; }
		

		/**
		 * @return double[] ellipticity e1,e2
		 */
		public double[] getEll() {return ellipticity; }


}
