package gov.fnal.eag.shapelets.functions;

/**
 * 
 * calculates L Tensor, equation 11 paper 2
 * 
 * 
 * @author stoughto
 */
public class LTensor {

	/**
	 * generate the L tensor of equation 11 paper 2
	 * @param a
	 * @param b
	 * @param c
	 * @param nMax
	 * @return the tensor
	 */
	public static double[][][] getLTensor(double a, double b, double c, int nMax) {

		double[][][] lt = new double[nMax + 1][nMax + 1][nMax + 1];
		boolean[][][] done = new boolean[nMax + 1][nMax + 1][nMax + 1];
		lt[0][0][0] = 1.0;
		done[0][0][0] = true;
		for (int i = 0; i < nMax + 1; i++) {
			for (int l = 0; l <= i; l++) {
				for (int m = 0; m <= i; m++) {
					for (int n = 0; n <= i; n++) {
						if ((l + m + n) % 2 == 0) {
							if (!done[l][m][n]) {
								double sum = 0.0;
								if ((l >= m) && (l >= n)) {
									if (((l - 2) >= 0)) {
										sum += 2 * (l - 1) * (a * a - 1)
												* lt[l - 2][m][n];
									}
									if (((l - 1) >= 0) && ((m - 1) >= 0)) {
										sum += 2. * m * a * b
												* lt[l - 1][m - 1][n];
									}
									if (((l - 1) >= 0) && ((n - 1) >= 0)) {
										sum += 2. * n * a * c
												* lt[l - 1][m][n - 1];
									}
									lt[l][m][n] = sum;
								} else if ((m >= l) && (m >= n)) {
									if (((m - 1) >= 0) && ((m - 2) >= 0)) {
										sum += 2. * (m - 1) * (b * b - 1)
												* lt[l][m - 2][n];
									}
									if (((m - 1) >= 0) && ((l - 1) >= 0)) {
										sum += 2. * l * a * b
												* lt[l - 1][m - 1][n];
									}
									if (((m - 1) >= 0) && ((n - 1) >= 0)) {
										sum += 2. * n * b * c
												* lt[l][m - 1][n - 1];
									}
									lt[l][m][n] = sum;
								} else {
									if (((n - 1) >= 0) && ((n - 2) >= 0)) {
										sum += 2. * (n - 1) * (c * c - 1)
												* lt[l][m][n - 2];
									}
									if (((n - 1) >= 0) && ((l - 1) >= 0)) {
										sum += 2. * l * a * c
												* lt[l - 1][m][n - 1];
									}
									if (((n - 1) >= 0) && ((m - 1) >= 0)) {
										sum += 2. * m * b * c
												* lt[l][m - 1][n - 1];
									}
									lt[l][m][n] = sum;
									done[l][m][n] = true;
								}
							}
						}
					}
				}
			}

		}
		return lt;
	}
}
