

/*
 * Created on Nov 10, 2004
 *
 * @author stoughto
 */
package gov.fnal.eag.shapelets.functions;

/**
 * computes polynomials 
 */
public class Polynomial {
	private double[] coef; // coefficients

	private int degree; // degree of polynomial (-1 for the zero polynomial)

	
	/**
	 * constructor to evaluate a * x^b
	 * @param a 
	 * @param b 
	 */
	public Polynomial(double a, int b) {
		coef = new double[b + 1];
		coef[b] = a;
		if (a == 0)
			degree = -1;
		else
			degree = b;
	}

	/**
	 * use Horner's method to compute and return the polynomial evaluated at x
	 * @param x location
	 * @return computed value
	 */
	public double evaluate(double x) {
		double p = 0;
		for (int i = degree; i >= 0; i--)
			p = coef[i] + (x * p);
		return p;
	}
	/**
	 * add two Polynomials
	 * @param b the Polynomial to add to this one
	 * @return the sum
	 */
	public Polynomial plus(Polynomial b) {
    		Polynomial a = this;
        int 	N = Math.max(a.degree, b.degree);
        Polynomial c = new Polynomial(0, N);  // zero polynomial of degree N
        for (int i =	 0; i <= a.degree; i++) c.coef[i] += a.coef[i];
        for (int i = 0; 	i <= b.degree; i++) c.coef[i] += b.coef[i];
        for (int i = 0; i <=	 N; i++) if (c.coef[i] != 0) c.degree = i;
        return c;	
    }	


	/**
	 * multiply two Polynomials
	 * @param b the Polynomial to multiply this one by
	 * @return the product
	 */
	public Polynomial times(Polynomial b) {
		Polynomial a = this;
		if (a.degree == -1 || b.degree == -1)
			return new Polynomial(0, 0);
		Polynomial c = new Polynomial(0, a.degree + b.degree);
		for (int i = 0; i <= a.degree; i++)
			for (int j = 0; j <= b.degree; j++)
				c.coef[i + j] += (a.coef[i] * b.coef[j]);
		c.degree = a.degree + b.degree;
		return c;
	}

	/**
	 * difference two Polynomials
	 * @param b the Polynomial to subtract from this one
	 * @return the difference
	 */
	public Polynomial minus(Polynomial b) {
		Polynomial a = this;
		int N = Math.max(a.degree, b.degree);
		Polynomial c = new Polynomial(0, N);
		for (int i = 0; i <= a.degree; i++)
			c.coef[i] += a.coef[i];
		for (int i = 0; i <= b.degree; i++)
			c.coef[i] -= b.coef[i];
		for (int i = 0; i <= N; i++)
			if (c.coef[i] != 0)
				c.degree = i;
		return c;
	}

}