
package gov.fnal.eag.shapelets.functions;

/**
 * recomputes pixeled image from eigenvalue matrix and a set of shapelets
 *
 * @author kuropat
 *
 */
public class ShapeRecomp extends BasisFunction {
		private int rpix; // row pixels that with col pixels define rectangular
		private int cpix; // area to hold the image positioned in the center
		private double rc; // row of the image center
		private double cc; // column of the image center
		private double scale; // the image scale
		private double fluxFactor;
		private double [][] ev; // 2D version of eigenvalues
		//private double[] ev1;   // vector version of eigenvalues
		private int nMax;       // maximal rank of eigenfunctions to use
		
	/**
	 * @param rpix - max number of pixels in row
	 * @param cpix - max number of pixels in col
	 * @param fluxFactor - factor to change the original flux
	 * @param scale - beta factor
	 * @param ev - matrix of eigenvalues
	 * @param nMax - maximal rank to be used for reconstruction
	 */
	public ShapeRecomp(int rpix,int cpix, double fluxFactor, double scale,  double[][]ev, int nMax) {
		//double sum;
		this.rpix = rpix;
		this.cpix = cpix;
		this.fluxFactor = fluxFactor;
		this.ev = ev;
		this.scale = scale;
		this.nMax = nMax;
		//this.ev1 = tab2Vector(ev);
		rc = rpix/2. ;
		cc = cpix/2. ;


	}
	/**
	 * @param rpix
	 * @param cpix
	 * @param fluxFactor
	 * @param scale
	 * @param ev1
	 * @param nMax
	 * @throws Exception
	 */
	
	
	public ShapeRecomp(int rpix,int cpix, double fluxFactor, double scale,  double[]ev1, int nMax) throws Exception {		
		this(rpix, cpix, fluxFactor, scale, vector2Tab(ev1), nMax);
	}
	
	
	
		/**
		 * @return max row pixel
		 */
		public int getRow() {return rpix;}
		/**
		 * @return max col pixel
		 */
		public int getCol() {return cpix;}
		/**
		 * @return flux factor
		 */
		public double getFluxFactor() {return fluxFactor;}
		
		/**
		 * @param r
		 * @param c
		 * @return flux at point r,c
		 * @throws shapeletException
		 */
		public double getFlux(double r, double c) {
			double flux = 0.0;
			double dr = r - rc;
			double dc = c - cc;
			double[][] Fnm = integrateAll(nMax,dr,dc,scale,1.,1.);
			for (int n1=0; n1<=nMax; n1++) {
				for (int n2=nMax-n1; n2>=0; n2--) {
					flux += ev[n1][n2]*Fnm[n1][n2];
				}
			}
			return fluxFactor*flux;
		}
		/**
		 * @return pixel array with reconstructed image
		 */
		public double[][] getImage() {
			double dr = 0.;
			double dc = 0.;
			double sum = 0.;
			double[][]Fnm;
			double[][] image = new double[rpix][cpix];
			for (int i=0; i<rpix; i++) {
				for (int j=0; j<cpix; j++) {
					image[i][j] = 0.;
					dr = i +0.5 - rc;
					dc = j + 0.5 - cc;
					sum = 0.;
					Fnm = integrateAll(nMax,dr,dc,scale,1.,1.);

					for (int k=0; k<nMax; k++) {

						for (int l = nMax-1; l>=0; l--) {
							sum += ev[k][l]*Fnm[k][l]; //
						}
					}
					image[i][j] = sum;
				}
			}
			return image;
		}
		
	}

