/*
 * Created on Dec 21, 2004
 *
 * @author kuropat
 * 
 */
package gov.fnal.eag.shapelets.functions;
import gov.fnal.eag.matrix.Matrix;


import com.imsl.math.Complex;


/**
 *
 * creates a transformation matrix to convert Cartesian
 * shapelets coefficients into polar form and back.
 * 
 * @author kuropat
 * 
 */
public class ShapeletsTransform extends BasisFunction {
	private static int[][] index;

	private static int[] n;

	private static int[] m;

	private static int nCoefs;

	private final static Complex I = new Complex(0., 1.);
    private final static Complex czero = new Complex(0., 0.);
    //private final static Complex cone = new Complex(1., 0.);
	private static double DEGRAD = (Math.PI/180.);
	private static 	double sqrt2 = Math.sqrt(2.);

	private static Complex[][] matrix;
	private static Matrix A;
    private static Matrix Ainv;
    private static Matrix Pvect;
    private static Matrix Cvect;
    
	
	static {
		
		nCoefs = (nMaxBasis+1) * (nMaxBasis + 2) / 2;
		matrix = new Complex[nCoefs][nCoefs];
		n = new int[nCoefs];
		m = new int[nCoefs];

		double[][] pl = BasisFunction.binom;
		double[][] pr = BasisFunction.binom;
		index = BasisFunction.ind; 
		int n_r = 0;
		int n_l = 0;
		int n_1 = 0;
		int n_2 = 0;
		//
		// Calculate polar indexes
		//
		for (int i=0; i<nCoefs; i++) {
			n_r = index[0][i];
			n_l = index[1][i];
			n[i] = n_r + n_l;
			m[i] = n_r - n_l;

		}

//
		for (int i = 0; i < nCoefs; i++) {
			for (int j = 0; j < nCoefs; j++) {
                matrix[j][i] = czero;
            }
		}
		for (int i = 0; i < nCoefs; i++) { //Cartesian
			for (int j = 0; j < nCoefs; j++) { // Polar
				n_r = index[0][j];
				n_l = index[1][j];
				n_1 = index[0][i];
				n_2 = index[1][i];
				//int n = n_r + n_l;
				//int m = n_r - n_l;

				if ((n_1 + n_2) == (n_r + n_l)) {
                    Complex factor = czero;
                    for (int n_rd = 0; n_rd <= n_r; n_rd++) {
						for (int n_ld = 0; n_ld <= n_l; n_ld++) {
							if (n_rd + n_ld == n_1) {
								double Cnl = pr[n_r][n_rd]
										* pl[n_l][n_ld];
								factor = factor.plus(Complex.times(Complex.pow(
										I, (n_ld - n_rd)), Cnl));
							}
						}
					}

					double fact1 = Math.pow(2., -(n_r + n_l) / 2.);
					double root = Math.sqrt(Factorial.factorial[n_1]
							* Factorial.factorial[n_2]
							/ (Factorial.factorial[n_r] * Factorial.factorial[n_l]));

					matrix[j][i] = Complex.times(Complex.times(factor, Complex
							.times(Complex.pow(I, n_r - n_l), root)), fact1);

				}

			}
		}

		A = new Matrix(matrix);
		Ainv = A.inverse();
        Pvect = new Matrix(1,nCoefs); // Column vector 
        Cvect = new Matrix(nCoefs,1); // Row vector
	}
	
	/**
	 * Direct convertion of Cartesian shapelets coefficients to polar
	 * coefficients
	 * @param cart
	 * @return Complex array of polar coefficients
	 */
	
	public static Complex[] c2p(double[] cart) {

		int nInp = cart.length;
		int nCoefsC2p = nInp;
		Matrix D = A.getMatrix(0, nCoefsC2p-1, 0, nCoefsC2p-1);
	
		Complex[] res = new Complex[nInp];
		Matrix C = Cvect.getMatrix(0,nCoefsC2p-1,0,0); // Cartesian vector as a column
		Matrix P = Pvect.getMatrix(0,0,0,nCoefsC2p-1); // Polar vector as a row

		
		Complex[][] Cc = C.getArray();
		for (int i = 0; i < nCoefsC2p; i++) {

			if (i<nInp) {
				Cc[i][0] = new Complex(cart[i],0.);
			} else {
				Cc[i][0] = czero;
			}
			}

		P = D.times(C);
		Complex[] Pc = P.getColumnPackedCopy();
		//
		for (int i=0; i<nInp; i++) {
			res[i] = Pc[i];
		}
		return res;
	}
	
	/**
	 * Converts polar coefficients into Cartesian ones
	 * @param pol  Complex array of polar coefficients
	 * @return double array of Cartesian coefficients
	 */
	public static double[] p2c(Complex[] pol) {

	    int nInp = pol.length;
	    int nCoefsP2c = nInp;
	    double[] res = new double[nInp];
		//Matrix D = A.getMatrix(0, nCoefsP2c-1, 0, nCoefsP2c-1);

        Matrix Dinv = Ainv.getMatrix(0, nCoefsP2c-1, 0, nCoefsP2c-1);
	    
		Complex[] Cc = new Complex[nCoefsP2c];

		Matrix P = Cvect.getMatrix(0, nCoefsP2c-1,0,0);
//		Matrix P = new Matrix(nCoefsP2c,1);
		Complex[][] Pc = P.getArray();

		for (int i=0; i<nCoefsP2c; i++) {
			if (i<nInp) {
			Pc[i][0] = pol[i];
			} else {
                Pc[i][0] = czero;
            }
		}

		Matrix C = Dinv.times(P);
		Cc = C.getColumnPackedCopy();
		for (int i=0; i<nInp; i++) {
			if (i<Cc.length) {
			res[i] = Complex.real(Cc[i]);
			} else {
				res[i] = 0.;
			}
		}
		return res;

	}
	
	/**
	 * Perform rotation in the polar coordinates by angle phi
	 * @param pol  Complex array of polar coefficients
	 * @param phi  double rotation angle (in degrees)
	 * @return Complex array of rotated polar coefficients
	 */
	public static Complex[] rotate(Complex[] pol,double phi) {
		int nCoefsRotate = pol.length;
	    Complex[] res = new Complex[nCoefsRotate];
	    int rn = 0;
	    int ln = 0;
	    int nInp = pol.length;
	    double rho = DEGRAD*phi;

		for (int i=0; i<nCoefsRotate; i++) {
			rn = index[0][i];
			ln = index[1][i];
			if (i<nInp) {
			res[i] = Complex.exp(I.times(rho*(ln - rn))).times(pol[i]);
			} else {

                res[i] = czero;
            }
		}

		return res;

	}
	
	/**
	 * rotate the cartesian, 1-d representation of this shapelet
	 * @param ev1 the input shapelet object
	 * @param phi rotation angle (degrees)
	 * @return cartesian, 1-d representation of the rotated shapelet
	 */
	public static double[] rotate(double[] ev1, double phi) {
		Complex[] ev1Complex = c2p(ev1);
		ev1Complex = rotate(ev1Complex, phi);
		double[] retval = p2c(ev1Complex);
		return retval;
	}

	
	/**
	 * Performs shear operation with shear factors gamma1 and gamma2
	 * @param pol  Complex array of polar coefficients
	 * @param gamma1  double shear factor in x
	 * @param gamma2  double shear factor in 45 degrees to x
	 * @return Complex array of sheared polar coefficients
	 */
	public static Complex[] shear(Complex[] pol,double gamma1, double gamma2) {
		int nCoefsShear = pol.length;
	    Complex[] res = new Complex[nCoefs];
	    Complex f1,f2,f3,f4;
	    int ind1 = 0;
	    int ind2 = 0;
	    int ind3 = 0;
	    int ind4 = 0;

	    Complex fact1 = Complex.plus((new Complex(gamma1,0.)),(new Complex(0.,gamma2)));
	    Complex fact2 = Complex.minus((new Complex(gamma1,0.)),(new Complex(0.,gamma2)));
		for (int i=0; i<nCoefs; i++) {
			if (i<nCoefsShear) {	
				ind1 = Search(n[i]-2,m[i]-2);
				if (ind1>=0 & ind1 < nCoefsShear) {
					f1 =Complex.times(pol[ind1],(Math.sqrt((n[i]+m[i])*(n[i]+m[i]-2))));
				} else { 
                    f1 = czero;
                }
				ind2 = Search(n[i]+2,m[i]-2);
				if (ind2>=0 & ind2 < nCoefsShear) {
					f2 =Complex.times(pol[ind2],(Math.sqrt((n[i]-m[i]+2)*(n[i]-m[i]+4))));
				} else {
                    f2 = czero;}		
				ind3 = Search(n[i]-2,m[i]+2);
				if (ind3>=0 & ind3 < nCoefsShear) {
					f3 =Complex.times(pol[ind3],(Math.sqrt((n[i]-m[i])*(n[i]-m[i]-2))));
				} else {
                    f3 = czero;}
				ind4 = Search(n[i]+2,m[i]+2);
				if (ind4>=0 & ind4 < nCoefsShear) {
					f4 =Complex.times(pol[ind4],(Math.sqrt((n[i]+m[i]+2)*(n[i]+m[i]+4))));
				} else { 
                    f4 = czero;}		

				res[i] = Complex.plus(pol[i],Complex.plus(Complex.over(Complex.times(fact1,
						Complex.minus(f1,f2)),4.),
					Complex.over(Complex.times(fact2,Complex.minus(f3,f4)),4.)));
			} else {
                res[i] = czero;
			}
		}

		return res;

	}
    /**
     * Performs dilation operation with dilation factor k
     * @param pol  Complex array of polar coefficients
     * @param k  double dilation factor
     * @return Complex array of dilated polar coefficients
     */
    public static Complex[] dilate(Complex[] pol,double k) {
        int nCoefsDelate = pol.length;
        //int retvalLength = Math.min(nCoefs, pol.length+4);
//        Complex[] res = new Complex[nCoefsDelate];
        Complex[] res = new Complex[nCoefs];
        Complex down;
        Complex up;
        int indD = 0;
        int indU = 0;

        for (int i=0; i<nCoefs; i++) {
            if (i<nCoefsDelate) {
            indD = Search(n[i]-2,m[i]);
            if (indD>=0 & indD < nCoefsDelate) {
                down =Complex.times(pol[indD],(k*Math.sqrt((n[i]-m[i])*(n[i]+m[i]))/2.));
            } else { down = czero;}
            indU = Search(n[i]+2,m[i]);
            if (indU>=0 & indU<nCoefsDelate) {
                up = Complex.times(pol[indU],(k*Math.sqrt((n[i]-m[i]+2)*(n[i]+m[i]+2))/2.));
            } else {
                up = czero; }
            res[i] = Complex.plus(Complex.times(pol[i], (1.-k)), Complex.minus(down, up));
                
            } else {
                res[i] = czero;
            }
        }

        return res;

    }
	/**
	 * performs dilation to first order in Cartesian space
	 * @param in input eigenvalues array
	 * @param kappa
	 * @return converted double[][]
     * 
	 */
	public static double[][] dilate (double[][] in, double kappa) {
//		int retvalLength = in.length;
		int nMax = Math.min(nMaxBasis, in.length+2);
		int retvalLength = in.length;
        double stepR = 0.0;
        double stepL = 0.0;
        double stepU = 0.0;
        double stepD = 0.0;
	    double[][] retval = new double[nMax][nMax];
		double[][] inPadded = new double[nMaxBasis][nMaxBasis];
		for (int i=0; i< nMaxBasis; i++) {
			for (int j=0; j<nMaxBasis; j++) {
			if (i < retvalLength && j< retvalLength) {
			inPadded[i][j] = in[i][j];
			retval[i][j] = in[i][j];
			} else { inPadded[i][j] = 0.; }
			

			}
		}
		
		for (int n1=0; n1<nMax; n1++) {
			for (int n2 = 0; n2< nMax; n2++) {
				if (n1+2<nMaxBasis) {	
					stepL = kappa * Math.sqrt((n1+1)*(n1+2))*inPadded[n1+2][n2]/2.0;
					
                } else { stepL = 0.0; }

				if (n2+2 < nMaxBasis) {
					stepD = kappa * Math.sqrt((n2+1)*(n2+2))*inPadded[n1][n2+2]/2.0;
				} else { stepD = 0.0; }
				if (n1 -2 >= 0 ) {
					stepR = kappa * Math.sqrt((n1)*(n1-1))*inPadded[n1-2][n2]/2.0;

				} else { stepR = 0.0; }
				if (n2 -2 >= 0 ) {
					stepU = kappa * Math.sqrt((n2)*(n2-1))*inPadded[n1][n2-2]/2.0;

				} else { stepU = 0.0; }
                retval[n1][n2] = retval[n1][n2]*(1. -kappa) + stepU + stepR - stepD - stepL;
			}
		}
		return retval;
	}
	/**
	 * performs dilation to first order in Cartesian space
	 * @param in input eigenvalues array
	 * @param kappa
	 * @return converted double[]
     * This does not work should test it more thorougly
	 */
	/*
	public static double[] dilate (double[] in, double kappa) {
		int retvalLength = in.length;
//        int retvalLength = Math.min(nCoefs, in.length+4);
	    int indD = 0;
	    int indU = 0;
	    int indR = 0;
	    int indL = 0;
        double stepR = 0.0;
        double stepL = 0.0;
        double stepU = 0.0;
        double stepD = 0.0;
	    double[] retval = new double[retvalLength];
		double[] inPadded = new double[nCoefs];
		for (int i=0; i< nCoefs; i++) {
			if (i < retvalLength) {
			inPadded[i] = in[i];
			} else { inPadded[i] = 0.; }
			if (i< retvalLength) retval[i] = in[i];
		}
		
		for (int i=0; i<nCoefs; i++) {
			if (i<retvalLength) {	
				indL = Search(n[i]+2,m[i]);
				if (indL>=0 & indL < nCoefs) {
					stepL = kappa * Math.sqrt((n[i]+1)*(n[i]+2))*inPadded[indL]/2.0;
//					System.out.println("ni="+n[i]+" mi="+m[i]+" ind="+indL+" step="+stepL);
                } else { stepL = 0.0; }
				indD = Search(n[i],m[i]+2);
				if (indD>=0 & indD < nCoefs) {
					stepD = kappa * Math.sqrt((m[i]+1)*(m[i]+2))*inPadded[indD]/2.0;
//                    System.out.println("ni="+n[i]+" mi="+m[i]+" ind="+indD+" step="+stepD);
				} else { stepD = 0.0; }
				indR = Search(n[i]-2,m[i]);
				if (indR>=0 & indR < nCoefs) {
					stepR = kappa * Math.sqrt((n[i])*(n[i]-1))*inPadded[indR]/2.;
//                    System.out.println("ni="+n[i]+" mi="+m[i]+" ind="+indR+" step="+stepR);
				} else { stepR = 0.0; }
				indU = Search(n[i],m[i]-2 );
				if (indU>=0 & indU < nCoefs) {
					stepU = kappa * Math.sqrt((m[i])*(m[i]-1))*inPadded[indU];
//                    System.out.println("ni="+n[i]+" mi="+m[i]+" ind="+indU+" step="+stepU);
				} else { stepU = 0.0; }
                retval[i] = retval[i]*(1. -kappa) + stepU + stepR - stepD - stepL;
			}
		}
		return retval;
	}
	*/
	
	
	/*
	public static double[][] shear(double[][] in, double gamma1, double gamma2) {
		int retvalLength = Math.min(nMaxBasis, in.length+2);
		double[][] inPadded = new double[retvalLength][retvalLength];
		for (int i=0; i<retvalLength; i++) {
			for (int j=0; j<retvalLength; j++) {
				if ((i<in.length) && (j< in.length)) {
				inPadded[i][j] = in[i][j];
				} else { inPadded[i][j] = 0.0; }
			}
		}
		double[][] retval = new double[retvalLength][retvalLength];
		for (int n1=0; n1<retvalLength; n1++) {
			for (int n2=0; n2<retvalLength-n1; n2++) {
				for (int m1=Math.max(0,n1-2); m1<=Math.min(retvalLength-1,n1+2); m1++) {
					for (int m2=Math.max(0,n2-2); m2<=Math.min(retvalLength-1,n2+2); m2++) {
						if (m1==n1 && m2==n2) {
							retval[n1][n2] += inPadded[n1][n2];
						} else if (m1==n1+2 && m2==n2) {
							retval[n1][n2] -= 0.5*gamma1*Math.sqrt((n1+1)*(n1+2))*inPadded[m1][m2];
						} else if (m1==n1 && m2==n2+2) {
							retval[n1][n2] += 0.5*gamma1*Math.sqrt((n2+1)*(n2+2))*inPadded[m1][m2];
						} else if (m1==n1-2 && m2==n2) {
							retval[n1][n2] += 0.5*gamma1*Math.sqrt(n1*(n1-1))*inPadded[m1][m2];
						} else if (m1==n1 && m2==n2-2) {
							retval[n1][n2] -= 0.5*gamma1*Math.sqrt(n2*(n2-1))*inPadded[m1][m2];
						} else if (m1==n1+1 && m2==n2+1) {
							retval[n1][n2] -= gamma2*Math.sqrt((n1+1)*(n2+1))*inPadded[m1][m2];
						} else if (m1==n1-1 && m2==n2-1) {
							retval[n1][n2] += gamma2*Math.sqrt(n1*n2)*inPadded[m1][m2];
						}
					}
				}
			}
		}
		return retval;
	}
	*/
	
	/**
	 * @param in
	 * @param gamma1
	 * @param gamma2
	 * @return the result of the shear
	 */
	public static double[][] shear(double[][] in, double gamma1, double gamma2) {
		int retvalLength = Math.min(nMaxBasis, in.length+2);
		int insize = in.length;
		double[][] inPadded = new double[retvalLength][retvalLength];
		double[][] retval = new double[retvalLength][retvalLength];
		for (int i=0; i<retvalLength; i++) {
			for (int j=0; j<retvalLength; j++) {
				retval[i][j] = 0.;
				if (i<insize && j < insize) {
				inPadded[i][j] = in[i][j];
				} else { inPadded[i][j] = 0.0; }
			}
		}

		for (int n1=0; n1<retvalLength; n1++) {
			for (int n2=0; n2<retvalLength-n1; n2++) {
				retval[n1][n2] += inPadded[n1][n2];
				if (n1+2 < retvalLength) {
					retval[n1][n2] -= 0.5*gamma1*Math.sqrt((n1+1)*(n1+2))*inPadded[n1+2][n2];	
				}
				if (n2+2 < retvalLength) {
					retval[n1][n2] += 0.5*gamma1*Math.sqrt((n2+1)*(n2+2))*inPadded[n1][n2+2];
				}
				if (n1 > 1) {
					retval[n1][n2] += 0.5*gamma1*Math.sqrt(n1*(n1-1))*inPadded[n1-2][n2];
				}
				if (n2 > 1) {
					retval[n1][n2] -= 0.5*gamma1*Math.sqrt(n2*(n2-1))*inPadded[n1][n2-2];
				}
				if (n1+1 < retvalLength && n2+1 < retvalLength) {
					retval[n1][n2] -= gamma2*Math.sqrt((n1+1)*(n2+1))*inPadded[n1+1][n2+1];
				}	
				if (n1*n2 > 0) {
					retval[n1][n2] += gamma2*Math.sqrt(n1*n2)*inPadded[n1-1][n2-1];
				}	
			}
		}
		return retval;
	}

	/**
	 * Performs translation operation with shift factors eps1 and eps2
	 * @param pol  Complex array of polar coefficients
	 * @param eps1  double translation factor in x
	 * @param eps2  double translation factor in y
	 * @return Complex array of translated polar coefficients
	 */
	
	public static Complex[] trans(Complex[] pol,double eps1, double eps2) {
		int nCoefsTrans = pol.length;
	    Complex[] res = new Complex[nCoefsTrans];
	    Complex f1,f2,f3,f4;
	    int ind1 = 0;
	    int ind2 = 0;
	    int ind3 = 0;
	    int ind4 = 0;

	    Complex fact1 = (Complex.plus((new Complex(eps1,0.)),(new Complex(0.,eps2)))).over(2.*sqrt2);
	    Complex fact2 = (Complex.minus((new Complex(eps1,0.)),(new Complex(0.,eps2)))).over(2.*sqrt2);
		for (int i=0; i<nCoefsTrans; i++) {
			if (i<nCoefsTrans) {
			ind1 = Search(n[i]-1,m[i]-1);
			if (ind1>=0 & ind1 < nCoefsTrans) {
				f1 =pol[ind1].times(Math.sqrt(n[i]+m[i]));
			} else {
                f1 = czero;
                }
			ind2 = Search(n[i]+1,m[i]-1);
			if (ind2>=0 & ind2 < nCoefsTrans) {
				f2 =pol[ind2].times(Math.sqrt(n[i]-m[i]+2));
			} else { 
                f2 = czero;
                }		
			ind3 = Search(n[i]-1,m[i]+1);
			if (ind3>=0 & ind3 < nCoefsTrans) {
				f3 =pol[ind3].times(Math.sqrt(n[i]-m[i]));
			} else { 
                f3 = czero;
                }
			ind4 = Search(n[i]+1,m[i]+1);
			if (ind4>=0 & ind4 < nCoefsTrans) {
				f4 =pol[ind4].times(Math.sqrt(n[i]+m[i]+2));
			} else {
                f4 = czero; 
                }           
			res[i] = pol[i].plus(Complex.plus((fact1.times(f1.minus(f2))),
					(fact2.times(f3.minus(f4)))));
			} else {
                res[i] = czero;
			}
		}
		return res;
	}
	
	/**
	 * Performs circularization of the polar shapelets
	 * @param pol  Complex array of polar coefficients
	 * @return Complex array of rotated polar coefficients
	 */
	/*
	public static Complex[] circularise(Complex[] pol) {
		int nCoefsCircularise = pol.length;
	    Complex[] res = new Complex[nCoefsCircularise];
	    int rn = 0;
	    int ln = 0;


		for (int i=0; i<nCoefsCircularise; i++) {
			rn = index[0][i];
			ln = index[1][i];
			if (i<nCoefsCircularise & ((rn-ln) == 0)) {
			  res[i] = pol[i];
			} else {
                res[i] = czero;
			}
		}

		return res;

	}
	*/
	private static int Search(int k, int l){
		int res = -1;
		for (int i=0; i<nCoefs; i++) {
			if ((n[i] == k) && (m[i] == l)) return i;
		}
		return res;
	}
}