package gov.fnal.eag.sim.catalog;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.NoSuchElementException;
/**
 * combines a set of AstroObjectGenerator objects to generate a complete catalog
 * @author stoughto
 *
 */
public class AstroCatalogGenerator implements Iterable<AstroObject> {
	private LinkedList<AstroObjectGenerator> astroObjectGenerators = new LinkedList<AstroObjectGenerator>();
	private Iterator<AstroObjectGenerator> aogIterator = new LinkedList<AstroObjectGenerator>().iterator();
	private Iterator<AstroObject> aoIterator = new LinkedList<AstroObject>().iterator();
	/**
	 * @return the generators
	 */
	public LinkedList<AstroObjectGenerator> getAstroObjectGenerators() {
		return astroObjectGenerators;
	}
	/**
	 * construct an empty catalog generator
	 */
	public AstroCatalogGenerator() {
	}
	/**
	 * construct with one AstroObjectGenerator
	 * @param aog
	 */
	public AstroCatalogGenerator(AstroObjectGenerator aog) {
		add(aog);
	}
	/**
	 * add this generator
	 * @param aog
	 */
	public void add(AstroObjectGenerator aog) {
		astroObjectGenerators.add(aog);
		aogIterator = astroObjectGenerators.iterator();
		aoIterator = aogIterator.next().iterator();
	}
	/**
	 * @see java.lang.Iterable#iterator()
	 */
	public Iterator<AstroObject> iterator() {
		return new AstroCatalogIterator();
	}
	private class AstroCatalogIterator implements Iterator<AstroObject> {
		public boolean hasNext() {
			boolean aoIteratorHasNext = aoIterator.hasNext();
			if (aoIteratorHasNext) return true;
			boolean aogIteratorHasNext = aogIterator.hasNext();			
			if (aogIteratorHasNext) return true;
			return false;
		}

		public AstroObject next() {
			if (aoIterator.hasNext()) {
				return aoIterator.next();
			}
			if (aogIterator.hasNext()) {
				aoIterator = aogIterator.next().iterator();
				return next();
			}
			throw new NoSuchElementException();
		}

		public void remove() {
			throw new UnsupportedOperationException();
		}
	}
	/**
	 * add the AstroObjectGenerators from acg to this AstroCatalogGenerator
	 * @param acg
	 */
	public void add(AstroCatalogGenerator acg) {
		for (AstroObjectGenerator aog : acg.astroObjectGenerators) {
			add(aog);
		}		
	}

}