package gov.fnal.eag.sim.catalog;

import gov.fnal.eag.sim.random.SimRandomLuminosityConstant;
import gov.fnal.eag.util.astronomy.Cosmo;
import gov.fnal.eag.util.misc.StringUtil;
import gov.fnal.eag.wcs.SphericalCoordinate;

/**
 * generates a set of objects that are all the same, and on a grid in ra,dec
 * @author stoughto
 *
 */
public class AstroObjectGeneratorGrid extends AstroObjectGenerator {

	SphericalCoordinate detectorMinRadec, detectorMaxRadec, gridCenter;
	int gridSize;
	double gridAbsMag, gridDeltaArcsec, gridObjectSizeArcsec, redshiftMin, redshiftMax;
	String lfBandpass;
	SED sed;
	Cosmo cosmo;
	/**
	 * @param detectorMinRadec
	 * @param detectorMaxRadec
	 * @param gridCenter
	 * @param gridSize
	 * @param gridDeltaArcsec
	 * @param gridAbsMag
	 * @param lfBandpass
	 * @param gridObjectShape 
	 * @param gridObjectSizeArcsec
	 * @param gridObjectReferenceRedshift 
	 * @param cosmo
	 * @param redshiftMin
	 * @param redshiftMax
	 * @param sed
	 * @param distanceIsRedshift 
	 */
	public AstroObjectGeneratorGrid(SphericalCoordinate detectorMinRadec, SphericalCoordinate detectorMaxRadec, 
			SphericalCoordinate gridCenter, int gridSize, 
			double gridDeltaArcsec, double gridAbsMag, String lfBandpass, 
			String gridObjectShape, double gridObjectSizeArcsec, double gridObjectReferenceRedshift,
			Cosmo cosmo, double redshiftMin, double redshiftMax, SED sed, boolean distanceIsRedshift) {
		this.detectorMinRadec = detectorMinRadec;
		this.detectorMaxRadec = detectorMaxRadec;
		this.gridCenter = gridCenter;
		this.gridSize = gridSize;
		this.gridDeltaArcsec = gridDeltaArcsec;
		this.gridAbsMag = gridAbsMag;
		this.lfBandpass = lfBandpass;
		this.gridObjectSizeArcsec = gridObjectSizeArcsec;
		this.cosmo= cosmo;
		this.redshiftMin = redshiftMin;
		this.redshiftMax = redshiftMax;
		this.sed = sed;		
		coordinateGenerator = new GridCoordinateGenerator(detectorMinRadec, detectorMaxRadec, gridCenter, gridSize, gridDeltaArcsec);
		redshiftGenerator = new RedshiftSequence(redshiftMin, redshiftMax, 1000);
		redshiftGenerator.setDefinedAsDistancePc(!distanceIsRedshift);
		sedGenerator = new SEDGeneratorConstant(sed);
		Morphology morphology;
		if (gridObjectSizeArcsec == 0) {
			morphology = new MorphologyShapelet();
		} else {
			String[] tokens = StringUtil.tokenize(gridObjectShape, " ");
			if (tokens[0].equalsIgnoreCase("profile")) {
				String shape = tokens[1];
				double ellipticity = Double.parseDouble(tokens[2]);
				double positionAngle = Double.parseDouble(tokens[3]);
				int nMax = Integer.parseInt(tokens[4]);
				morphology = new MorphologyShapelet(shape, gridObjectSizeArcsec, ellipticity, positionAngle, gridObjectReferenceRedshift, nMax, distanceIsRedshift);
			} else {
				double[] ev = StringUtil.toDoubleArray(gridObjectShape);
				morphology = new MorphologyShapelet(ev, gridObjectSizeArcsec, gridObjectReferenceRedshift, distanceIsRedshift);
			}
		}		
		morphologyGenerator = new MorphologyGeneratorConstant(morphology);
		luminosityGenerator = new SimRandomLuminosityConstant(gridAbsMag, lfBandpass);
		setFilter();
	}
	/**
	 * @return the cosmology
	 */
	public Cosmo getCosmology() {
		return cosmo;

	}
}
