package gov.fnal.eag.sim.catalog;

import gov.fnal.eag.sim.repository.image.ImageRepository;
import gov.fnal.eag.util.misc.StringUtil;

import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.StringTokenizer;
import java.util.logging.Level;
import java.util.logging.Logger;

import uk.ac.starlink.table.ColumnInfo;
import uk.ac.starlink.table.DefaultValueInfo;
import uk.ac.starlink.table.DescribedValue;
import uk.ac.starlink.table.RowListStarTable;

/**
 * Implements a stand-alone utility for converting PEGASE2 format data files to fits.
 * 
 * @author stoughto
 * @version $Name:  $ ($Id: FiltersCreateFromPegase.java,v 1.6 2008/01/02 17:25:59 descvs Exp $)
 */
public class FiltersCreateFromPegase {
	private static Logger logger = Logger.getLogger("gov.fnal.eag.sim");
	private static int transmissionType;
	private static int calibrationType;
	private static String name;
	private static String description;
	private static RowListStarTable rlst = null;
	private static ColumnInfo[] colInfo = {
		new ColumnInfo("wavelength", Double.class, "wavelength in Angstroms"),
		new ColumnInfo("transmission", Double.class, "transmission")		
		};
	
	/**
	 * read in PEGASE2-filters.dat and write filter-pegase-*.fit files, one for each filter.
	 * <p>
	 * note:  this code was run to create the files gov/fnal/eag/sim/catalog/data/filters/filter-pegase-*.fit
	 * 
	 * @param args not used
	 * @throws Exception
	 * 
	 */
	public static void main(String[] args) throws Exception {
		String fileName = "data"+File.separator+"filters"+File.separator+"PEGASE2-filters.dat";
		InputStream inputStream;
		if ((inputStream = FiltersCreateFromPegase.class.getResourceAsStream(fileName)) != null) {
			BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(inputStream));
			String line;
			try {
				while ( (line = bufferedReader.readLine()) != null) {
					String clean = (String) StringUtil.removeDuplicateWhitespace(line);
					StringTokenizer st = new StringTokenizer(clean);
					int nt = st.countTokens();
					if (nt > 2) {
						writeFile();
						Integer.parseInt(st.nextToken());
						transmissionType = Integer.parseInt(st.nextToken());
						calibrationType = Integer.parseInt(st.nextToken());
						name = st.nextToken();
						description = "";
						while (st.hasMoreTokens()) {
							description += st.nextToken()+" ";
						}	
						description = description.trim().replace("'","_prime");	
						rlst = new RowListStarTable(colInfo);
						rlst.setParameter(new DescribedValue(new DefaultValueInfo("TRNSTYPE", Integer.class, "transmission type: 0 (energy), 1 (photons), 2 (D4000Bruzual 1983)"), new Integer(transmissionType)));
						rlst.setParameter(new DescribedValue(new DefaultValueInfo("CLBRTYPE", Integer.class, "calibration type: 0 (D4000), 1 (standard), 2 (AB), 3 (Thuan), 4 (WFPC), 5 (FOCA)"), new Integer(calibrationType)));
						rlst.setParameter(new DescribedValue(new DefaultValueInfo("FDESCRIP", String.class, "filter description"), description));
					} else if (nt == 2) {					
						Object[] row = {
								new Double(st.nextToken()),
								new Double(st.nextToken())
						};	
						rlst.addRow(row);
					}
				}
			} catch (IOException e) {
				throw new Error(e);
			}
			writeFile();
		} 
	}
	private static void writeFile() throws Exception {
		if (rlst != null) {
			ImageRepository ir = new ImageRepository();
			String fileName = "filter-pegase-"+name.replaceAll("'","")+".fit";
			logger.log(Level.INFO,"now write fileName="+fileName);
			ir.put(fileName, rlst);
		}
	}
 }

