package gov.fnal.eag.sim.catalog;

import gov.fnal.eag.wcs.SphericalCoordinate;


/**
 * trivial generator that returns the same SED every time
 * 
 * @author stoughto
 * @version $Name:  $ ($Id: SEDGeneratorConstant.java,v 1.11 2008/08/04 17:06:42 descvs Exp $)
 */
public class SEDGeneratorConstant implements SEDGenerator {

	private final SED sed;
	/**
	 * fully-specified constructor
	 * @param ws defines the wavelength scale
	 * @param name determines which spectrum to load
	 */
	public SEDGeneratorConstant(WavelengthScale ws, String name) {
		this(new SED(ws,name));
	}
	
	/**
	 * construct a generator that returns copies of the SED
	 * @param sed the original to clone
	 */
	public SEDGeneratorConstant(SED sed) {
		this.sed = sed;
	}
	
	/** 
	 * return a copy the SED
	 * @return the SED
	 */
	public SED next() {
		return sed.clone();
	}
	
	/**
	 * @see gov.fnal.eag.sim.catalog.SEDGenerator#next(boolean)
	 */
	public SED next(boolean returnValue) {
		SED retval = null;
		if (returnValue) {
			retval = next();
		}
		return retval;
	}

	/**
	 * @see gov.fnal.eag.sim.catalog.SEDGenerator#copy()
	 */
	public SEDGenerator copy() {
		SEDGeneratorConstant retval = new SEDGeneratorConstant(next());
		return retval;
	}


	/**
	 * @see gov.fnal.eag.sim.catalog.SEDGenerator#getWavelengthScale()
	 */
	public WavelengthScale getWavelengthScale() {
		return sed.getLambda();
	}

	/**
	 * @see gov.fnal.eag.sim.catalog.SEDGenerator#getISeed()
	 */
	public int getISeed() {
		return 0;
	}

	/**
	 * @see gov.fnal.eag.sim.catalog.SEDGenerator#next(gov.fnal.eag.wcs.SphericalCoordinate)
	 */
	public SED next(SphericalCoordinate sc) {
		return next(true);
	}
}

