/**
 * 
 */
package gov.fnal.eag.sim.catalog;

import java.util.Iterator;

/**
 * An iterator over wavelength bins
 * 
 * @author neilsen
 * @version $Name:  $ ($Id: ValuedBinIterator.java,v 1.5 2009/03/23 14:47:08 stoughto Exp $)
 * 
 */
public class ValuedBinIterator implements Iterator<ValuedBin> {
	
	private WavelengthBinIterator wavelengths = null;
	private Double[] values = null;
	private int currentValueIndex = 0;
	
	/**
	 * Create an iterator over valued bins.
	 * 
	 * @param values the values over which to iterate
	 * @param scale the bins over which to iterate
	 */
	public ValuedBinIterator(Double[] values, WavelengthScale scale) {
		if (scale == null || values == null) {
			throw new IllegalArgumentException("can not be null but scale="+null+" values="+null);
		}
		if (scale.size() != values.length) {
			throw new IllegalArgumentException("need to be same size:  scale.size()="+scale.size()+" values.length="+values.length);
		}
		wavelengths = scale.iterator();
		currentValueIndex = 0;
		this.values = values;
	}
	
	/**
	 * Return true if there is another bin.
	 * @see java.util.Iterator#hasNext()
	 */
	public boolean hasNext() {
		//assert wavelengths.hasNext() == (currentValueIndex < values.length);
		return wavelengths.hasNext();
	}

	/**
	 * Get the next bin.
	 * 
	 * @see java.util.Iterator#next()
	 */
	public ValuedBin next() {
		ValuedBin bin = new ValuedBin(wavelengths.next(),values[currentValueIndex++]);
		return bin;
	}

	/**
	 * Remove a bin (not implemented).
	 * 
	 * @see java.util.Iterator#remove()
	 */
	public void remove() {
		throw new UnsupportedOperationException("Bins cannot be removed");

	}

}
