package gov.fnal.eag.sim.crt;

import flanagan.complex.Complex;
import gov.fnal.eag.util.astronomy.Integrate;
import gov.fnal.eag.util.astronomy.Simple2dFunction;


/**
 * describes the response of an antenna
 * @author stoughto
 *
 */
public abstract class AntennaResponse {
	/**
	 * @param k
	 * @return the response
	 */
	abstract public Complex response(KVector k);
	
	/**
	 * @param k
	 * @return the E value
	 */
	abstract public Complex getE(KVector k);

	/**
	 * @param nu frequency in Hz
	 * @param dSinTheta bin size in sinTheta
	 * @param dPhi bin size in Phi
	 * @return the result of eq 34, with E_0 set to 1
	 */
	public double getE2rms(final double nu, double dSinTheta, double dPhi) {
		double sinThetaMin = -1.0;
		double sinThetaMax = 1.0;
		double phiMin = -Math.PI/2;
		double phiMax =  Math.PI/2;
		Simple2dFunction antennaResponse = 	new Simple2dFunction() {
			@Override
		public double f(double theta, double phi) {
				KVector kVector = new KVector(theta, phi, nu);
				Complex response = getE(kVector);
				double retval = response.squareAbs();
				return retval;
			}
		};
		double retval = Integrate.integrateSphere(antennaResponse, sinThetaMin, sinThetaMax, phiMin, phiMax, dSinTheta, dPhi);
		retval /= 4*Math.PI;
		return retval;
	}
}
