package gov.fnal.eag.sim.crt;

import java.util.Hashtable;

import flanagan.complex.Complex;

/**
 * simple AntennaResponse
 * @author stoughto
 *
 */
public class AntennaResponseDefault extends AntennaResponse {

	double dx, dy;
	final double sqrt4Pi = Math.sqrt(4.0*Math.PI);
	Hashtable<Double,Double> ermsTable = new Hashtable<Double,Double>();
	/**
	 * @param dx separation between detectors along a cylinder, in meters
	 * @param dy separation between cylinders, in meters
	 */
	public AntennaResponseDefault(double dx, double dy) {
		this.dx = dx;
		this.dy = dy;
	}

	/**
	 * @see gov.fnal.eag.sim.crt.AntennaResponse#response(gov.fnal.eag.sim.crt.KVector)
	 * 
	 * the real part of the response is lambda * E / (Erms*sqrt(4*pi))
	 * and the imaginary part of the response is zero.
	 */
	public Complex response(KVector k) {
		Complex e = getE(k);	

		double nu = k.geFrequency();
		Double erms = ermsTable.get(nu);
		if (erms == null) {
			double e2rms = getE2rms(nu, 0.01, 0.01);
			erms = Math.sqrt(e2rms);
			ermsTable.put(nu, erms);
		}
		double norm = k.getLambda()/erms/sqrt4Pi;
		Complex retval = e.times(norm);
		return retval;
	}

	/**
	 * @param k
	 * @return electric field, assuming E_0=1, as in equation 35
	 */
	public Complex getE(KVector k) {
		double kxdx = k.getX()*dx;
		double kydy = k.getY()*dy;
		
		double realx = sinXByX(kxdx);
		double realy = sinXByX(kydy);
		double realxy = realx*realy;
		Complex retval = new Complex(realxy, 0);
		return retval;		
	}
	
	private double sinXByX(double x) {
		if (Math.abs(x) < 1e-6) {
			return 1.0;
		}
		return Math.sin(x)/x;
	}
}
