package gov.fnal.eag.sim.crt;

/**
 * defines bins in one dimension
 * @author stoughto
 *
 */
public class Bins {
	double min,max;
	int nBin;
	double delta;
	/**
	 * @param min minimum value
	 * @param max maximum value
	 * @param nBin number of bins
	 */
	public Bins(double min, double max, int nBin) {
		if (min >= max || nBin <=0) {
			throw new IllegalArgumentException("min="+min+" max="+max+" nBin="+nBin);
		}
		this.min = min;
		this.max = max;
		this.nBin = nBin;
		delta = (max-min)/nBin;
	}
	/**
	 * @return the minimum value
	 */
	public double getMin() {
		return min;
	}
	/**
	 * @return the maximum value
	 */
	public double getMax() {
		return max;
	}
	/**
	 * @return the number of bins
	 */
	public int getNBin() {
		return nBin;
	}
	/**
	 * @return the bin size
	 */
	public double getDelta() {
		return delta;
	}
	/**
	 * @param iBin
	 * @return minimum value of the ith bin
	 */
	public double minValue(int iBin) {
		return min + iBin*delta;
	}
	/**
	 * @param iBin
	 * @return maximum value of ith bin
	 */
	public double maxValue(int iBin) {
		return min + (iBin+1)*delta;
	}
	/**
	 * @param iBin
	 * @return middle value of ith bin
	 */
	public double value(int iBin) {
		return min + (iBin+0.5)*delta;
	}
	
	/**
	 * @param value
	 * @return the bin that contains this value, with no bounds checking
	 */
	public int getIBin(double value) {
		int iBin = (int)(Math.floor((value-min)/delta));
		return iBin;
	}
}
