package gov.fnal.eag.sim.crt;

import gov.fnal.eag.util.misc.Constants;

/**
 * describes a vector locating a position and frequency
 * @author stoughto
 */

public class KVector {
	/**
	 * frequency in Hz
	 */
	private double frequency;
	/**
	 * the x component of this vector, in meters^-1
	 */
	private double x;
	/**
	 * the y component of this vector, in meters^-1
	 */
	private double y;
	/**
	 * the z component of this vector, in meters^-1
	 */
	private double z;
	
	/**
	 * @param theta radians from zenith along cylinder axis
	 * @param phi radians from zenith around cylinder axis
	 * @param frequency frequency in Hz;
	 */
	public KVector(double theta, double phi, double frequency) {
		this.frequency = frequency;
		double norm = 2*Math.PI*frequency/(1e-10*Constants.c); // in meters^-1
		x = norm * Math.sin(theta);
		y = norm * Math.cos(theta) * Math.sin(phi);
		z = norm * Math.cos(theta) * Math.cos(phi);		
	}

	/**
	 * @return frequency in Hz
	 */
	public double geFrequency() {
		return frequency;
	}
	
	/**
	 * @return wavelength in meters
	 */
	public double getLambda() {
		double retval = 1e-10*Constants.c/(frequency);
		return retval;
	}
	
	/**
	 * @return the x component in 1/meters
	 */
	public double getX() {
		return x;
		
	}
	/**
	 * @return the y component in 1/meters
	 */
	public double getY() {
		return y;		
	}
	/**
	 * @return the z component in 1/meters
	 */
	public double getZ() {
		return z;
		
	}
}
