package gov.fnal.eag.sim.crt;

import gov.fnal.eag.wcs.SphericalCoordinate;

/**
 * describes the location and orientation of a telescope
 * @author stoughto
 *
 */
public class RadioTelescope {
	SphericalCoordinate location;
	/* Rate of rotation of earth in Degrees per millisecond */
	static double earthRotation=0.001/239.344695918981;
	/**
	 * default constructor located at longitude,latitude=0,0
	 */
	public RadioTelescope() {
		location = new SphericalCoordinate(0.0, 0.0);
	}
	/**
	 * @param location longitude,latitude of the telescope
	 */
	public RadioTelescope(SphericalCoordinate location) {
		setLocation(location);
	}
	/**
	 * @param location longitude,latitude of the telescope
	 */
	public void setLocation(SphericalCoordinate location) {
		this.location=location;
		}
	/**
	 * @return the location of the telescope
	 */
	public SphericalCoordinate getLocation() {
		return location;
	}
	/**
	 * @param raDec
	 * @param ut
	 * @return the location in thetaPhi coordinates
	 */
	public SphericalCoordinate getThetaPhi(SphericalCoordinate raDec, UniversalTime ut) {
		double[] kxyz = new double[3];
		double thetap = Math.PI*location.getLatitude()/180.0;
		double phip = Math.PI*(location.getLongitude()+earthRotation*ut.getUtc())/180.0;
		double thetae = Math.PI*raDec.getLatitude()/180.0;
		double phie = Math.PI*raDec.getLongitude()/180.0;
		kxyz[2]=Math.cos(thetap)*Math.sin(thetae)-Math.sin(thetap)*Math.cos(thetae)*Math.cos(phie-phip);
	  	kxyz[1]=Math.cos(thetae)*Math.sin(phie-phip);
	    kxyz[0]=Math.cos(thetap)*Math.cos(thetae)*Math.cos(phie-phip) + Math.sin(thetap)*Math.sin(thetae);
/*
 * 	    System.out.println("thetap="+thetap+" phip="+phip);
 * 	    System.out.println("thetae="+thetae+" phie="+phie);
 *	    System.out.println(kxyz[2]);
 *	    System.out.println(kxyz[1]);
 *	    System.out.println(kxyz[0]);
 */

	    SphericalCoordinate ThetaPhi = new SphericalCoordinate(kxyz);
			
		return ThetaPhi;

	}
	
	/**
	 * @param thetaPhi
	 * @param ut
	 * @return the location in raDec coordinates
	 */
	public SphericalCoordinate getRaDec(SphericalCoordinate thetaPhi, UniversalTime ut) {
		double[] kxyz = new double[3];
		double thetap = Math.PI*location.getLatitude()/180.0;
		double phip = Math.PI*(location.getLongitude()+earthRotation*ut.getUtc())/180.0;
		double thetat = Math.PI*thetaPhi.getLatitude()/180.0;
		double phit = Math.PI*thetaPhi.getLongitude()/180.0;
		kxyz[2]=Math.cos(thetap)*Math.sin(thetat)+Math.sin(thetap)*Math.cos(thetat)*Math.cos(phit);
	  	kxyz[1]=Math.cos(thetap)*Math.sin(phip)*Math.cos(thetat)*Math.cos(phit)
	  	+Math.cos(phip)*Math.cos(thetat)*Math.sin(phit)
	  	-Math.sin(thetap)*Math.sin(phip)*Math.sin(thetat);
	    kxyz[0]=Math.cos(thetap)*Math.cos(phip)*Math.cos(thetat)*Math.cos(phit)
	    -Math.sin(thetap)*Math.cos(phip)*Math.sin(thetat)
	    -Math.sin(phip)*Math.cos(thetat)*Math.sin(phit);
/*
 * 	    System.out.println("thetap="+thetap+" phip="+phip);
 *	    System.out.println("thetat="+thetat+" phie="+phit);
 *	    System.out.println(kxyz[2]);
 *	    System.out.println(kxyz[1]);
 *	    System.out.println(kxyz[0]);
 */
	    SphericalCoordinate RaDec = new SphericalCoordinate(kxyz);	 
		return RaDec;
    

	}
}
