package gov.fnal.eag.sim.crt;

import gov.fnal.eag.wcs.SphericalCoordinate;

/**
 * @author stoughto
 *
 */
public class SkyTemperaturePointSource implements SkyTemperature {

	Bins frequencyBins, sinThetaBins, phiBins;
	int nu, iota, mu;
	double tPointSource;
	/**
	 * a point source in the center of one bin at the specified temperature
	 * @param frequencyBins bins in the frequency dimension
	 * @param sinThetaBins  bins in the sinTheta dimension
	 * @param phiBins bins in the phi dimension
	 * @param nu frequency bin
	 * @param mu sinTheta bin
	 * @param iota phi bin
	 * @param tPointSource temperature in the bin 
	 */
	public SkyTemperaturePointSource(Bins frequencyBins, Bins sinThetaBins, Bins phiBins, int nu, int mu, final int iota, double tPointSource) {
		this.frequencyBins = frequencyBins;
		this.sinThetaBins = sinThetaBins;
		this.phiBins = phiBins;
		this.nu = nu;
		this.mu = mu;
		this.iota = iota;
		this.tPointSource = tPointSource;
		System.out.println(" SkyTemperaturePointSource at nu="+nu+" iota="+iota+" mu="+mu);
		System.out.println(" SkyTemperaturePointSource:  frequency="+frequencyBins.value(nu));
		System.out.println(" SkyTemperaturePointSource:      theta="+Math.toDegrees(Math.asin(sinThetaBins.value(mu))));
		System.out.println(" SkyTemperaturePointSource:        phi="+phiBins.value(iota)+" radians");
	}
	/**
	 * @see gov.fnal.eag.sim.crt.SkyTemperature#getT(gov.fnal.eag.wcs.SphericalCoordinate, double)
	 */
	public double getT(SphericalCoordinate sc, double frequency) {
		double retval = 0;
		int iotaIn = phiBins.getIBin(Math.toRadians(sc.getLongitude()));
		if (iotaIn == iota) {
			int muIn = sinThetaBins.getIBin(Math.sin(Math.toRadians(sc.getLatitude())));
			if (muIn == mu) {
				int nuIn = frequencyBins.getIBin(frequency);
				if (nuIn == nu) {
					retval = tPointSource;
				}
			}
		}
		return retval;
	}
}
