package gov.fnal.eag.sim.crt;

import gov.fnal.eag.wcs.SphericalCoordinate;

/**
 * describes foreground as a power law
 * @author stoughto
 *
 */
public class SkyTemperaturePowerLaw implements SkyTemperature {

	private double t0, nu0, alpha;
	
	/**
	 * @param t0 temperature normalization, in degrees K
	 * @param nu0 reference frequency, in Hz
	 * @param alpha power law coefficient
	 */
	public SkyTemperaturePowerLaw(double t0, double nu0, double alpha) {
		this.t0 = t0;
		this.nu0 = nu0;
		this.alpha = alpha;
	}
	
	/**
	 * @see gov.fnal.eag.sim.crt.SkyTemperature#getT(gov.fnal.eag.wcs.SphericalCoordinate, double)
	 * return the temperature = t0*(nu/nu0)^alpha
	 */
	public double getT(SphericalCoordinate sc, double nu) {
		double retval = t0 * Math.pow(nu/nu0, alpha); // degrees K
		return retval;
	}

}
