package gov.fnal.eag.sim.demo;

import gov.fnal.eag.sim.pixel.Pixel;
import gov.fnal.eag.sim.pixel.PixelCtiConstant;
import gov.fnal.eag.sim.pixel.PixelRegister;
import gov.fnal.eag.util.densityEstimate.Histogram;
import gov.fnal.eag.util.misc.StringUtil;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;

import org.jfree.chart.ChartUtilities;
import org.jfree.chart.JFreeChart;

/**
 * demonstrate various methods for simulating CTE in a column of pixels
 * @author stoughto
 *
 */
public class CTEDemo1 {

	/**
	 * @param args ignored
	 * @throws IOException 
	 */
	public static void main(String[] args) throws IOException {
		int nPixels = 1020;
		int iPixel = 1000;
		demoCtiConstant(nPixels, iPixel);
		//demoCtiSqrtQ(nPixels, iPixel);
		//demoAllPixelsHaveElectronTrapRandom(nPixels, iPixel);
		//demoSomePixelsHaveElectronTrapRandom(nPixels, iPixel);
		//demoSomePixelsHaveElectronTrapCB1(nPixels, iPixel);
	}
	
	private static void demoCtiConstant(int nPixels, int iPixel) throws IOException {	
		ArrayList<Pixel> pixels = new ArrayList<Pixel>();
		double cti = 1e-4;
		double cte = 1-cti;
		double background = 100.0;
		double charge = 1000;
		Histogram hgInput = new Histogram("input", "pixel number", "charge", nPixels-iPixel, iPixel, nPixels);
		Histogram hgReadout = new Histogram("readout", "pixel number", "charge", nPixels-iPixel, iPixel, nPixels);
		
		for (int i=0; i<nPixels; i++) {
			double total = background;
			if (i == iPixel) {
				total += charge;
			}
			pixels.add(new PixelCtiConstant(1.0-cte, total));
			hgInput.fill(iPixel, total);
		}
		
		PixelRegister pr = new PixelRegister(pixels);
		double chargeOut = -1;
		double time = 0.0; // ignored for PixelCti
		for (int i=0; i<nPixels; i++) {
			double temp = pr.clock(nPixels, time, true);
			hgReadout.fill(i, Math.floor(temp));
			if (i == iPixel) {
				chargeOut = temp;
			}
		}
		//double truth = charge*Math.pow(cte, iPixel+1);
		//System.out.println(" truth="+truth);
		double cteCalculated = Math.pow(chargeOut/charge, 1.0/(iPixel+1));
		//System.out.println(" cte="+cte+" cteCalculated="+cteCalculated);
		
		Histogram[] list = {hgInput, hgReadout};
		boolean logarithmicRange = true;
		JFreeChart chart = Histogram.getChart(list, "constant CTI="+cti, logarithmicRange);
		ChartUtilities.saveChartAsPNG(new File(StringUtil.getFileName("png")), chart, 500, 300);
		String line = String.format("Constant CTE="+cte+"  measured CTE="+cteCalculated);
		System.out.println(line);
	}

}
