package gov.fnal.eag.sim.demo;

import gov.fnal.eag.shapelets.functions.BasisFunction;
import gov.fnal.eag.sim.catalog.MorphologyGeneratorShapeletCatalog;
import gov.fnal.eag.sim.catalog.MorphologyShapelet;
import gov.fnal.eag.sim.extractor.Extractor;
import gov.fnal.eag.sim.extractor.SourceObject;
import gov.fnal.eag.sim.pixel.DetectorObjectSurfaceBrightness;
import gov.fnal.eag.sim.pixel.PixeledImage;
import gov.fnal.eag.sim.pixel.PixeledImage.Direction;
import gov.fnal.eag.sim.random.SimRandomNormal;
import gov.fnal.eag.sim.random.SimRandomPoisson;
import gov.fnal.eag.util.densityEstimate.Histogram;
import gov.fnal.eag.util.misc.StringUtil;

import java.io.File;
import java.io.IOException;

import org.jfree.chart.ChartUtilities;
import org.jfree.chart.JFreeChart;

/**
 * Demonstration of using Extractor on catalog of shapelets in shapecatS.fits
 * @author dcharms
 *
 */
public class ExtractorDemo {

	/**
	 * @param args
	 * @throws IOException 
	 */
	public static void main(String[] args) throws IOException {
		MorphologyGeneratorShapeletCatalog ssg = new MorphologyGeneratorShapeletCatalog("shapecatS.fits", false);
		double row = 100.5;
		double col = 100.5;
		double nPhoton = 100000;
		double scale = 5;
		double background = 10;
		double readNoise = 5;
		ssg.setRotationRange(0, 0);
		ssg.calcVariationLimits(0, 0);
		ssg.setCatRange(0, ssg.getMaxObjNum());
		Histogram xHist = new Histogram("Col", "x", "dN", 100, 90, 110);
		Histogram diffHist = new Histogram("Delta Col", "dC", "dN", 100, -10, 10);
		Histogram sumHist = new Histogram("True-Measured/Error", "sum", "dN", 100, -100, 100);
		Histogram meanHist = new Histogram("Means", "mean", "dN", 100, -100, 100);
		int nToDo = ssg.getMaxObjNum();
		SimRandomPoisson srp = new SimRandomPoisson();
		SimRandomNormal srn = new SimRandomNormal(0, 0, readNoise, null, false);
		for (int i=0; i<nToDo; i++) {
			MorphologyShapelet sh = ssg.next();
			double[] vals = sh.getEigenvalues();
			double[][] eVals = BasisFunction.vector2Tab(vals);
			DetectorObjectSurfaceBrightness dosb = new DetectorObjectSurfaceBrightness(row, col, nPhoton, eVals, scale);
			PixeledImage pi = dosb.makePixeledImage(100, false);
			pi.addScalar(background);	
			pi.poisson(srp);		
			pi.addRandomNumber(srn);	
			pi.addScalar(-background);	
			Extractor ex = new Extractor(pi);	
			SourceObject so = ex.getLargestSource();
			double x = so.getX();
			double diff = col-x;
			double errX = Math.sqrt(so.getErrX2());
			xHist.fill(x);
			diffHist.fill(diff);
			sumHist.fill(diff/errX);
			Histogram histogram = pi.getHistogram(Direction.YDIRECTION);
			double m = histogram.getMean();
			double e = histogram.getMeanError();
			//System.out.println("m="+m+" e="+e);
			double sum = (row-m)/e;
			meanHist.fill(sum);
		}

		Histogram[] colList = {xHist};
		JFreeChart chartCol = Histogram.getChart(colList, "Col", "col", "dN", Double.NaN, Double.NaN, false);	
		ChartUtilities.saveChartAsPNG(new File(StringUtil.getFileName("x","png")), chartCol, 600, 600);
		Histogram[] colList1 = {diffHist};
		JFreeChart chartCol1 = Histogram.getChart(colList1, "Delta Col", "dCol", "dN", Double.NaN, Double.NaN, false);	
		ChartUtilities.saveChartAsPNG(new File(StringUtil.getFileName("diff","png")), chartCol1, 600, 600);

		Histogram[] colList2 = {sumHist};
		JFreeChart chartCol2 = Histogram.getChart(colList2, "True-Measured/Error", "sum", "dN", Double.NaN, Double.NaN, false);	
		ChartUtilities.saveChartAsPNG(new File(StringUtil.getFileName("sum","png")), chartCol2, 600, 600);
		Histogram[] colList3 = {meanHist};
		JFreeChart chartCol3 = Histogram.getChart(colList3, "Mean", "mean", "dN", Double.NaN, Double.NaN, false);	
		ChartUtilities.saveChartAsPNG(new File(StringUtil.getFileName("mean","png")), chartCol3, 600, 600);
		System.out.println("MeanHist="+meanHist.getMean()+" SigHist="+meanHist.getSigma(meanHist.getMean()));

	}

}
