package gov.fnal.eag.sim.demo;

import gov.fnal.eag.sim.random.SimRandomNormal;
import gov.fnal.eag.util.densityEstimate.Histogram;
import gov.fnal.eag.util.misc.StringUtil;

import java.io.File;
import java.io.IOException;

import org.jfree.chart.ChartUtilities;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.annotations.XYTextAnnotation;

/**
 * demo of using histograms. Makes histograms displaying results of
 * True-Measured/Error results.
 * 
 * @author dcharms
 *
 */
public class HistogramDemo {

	/**
	 * @param args
	 * @throws IOException 
	 */
	public static void main(String[] args) throws IOException {
		int nToDo = 100;

		int nBins = 100;
		double lowerBound = -10;
		double upperBound = 10;
		double meanXUp = 1;
		double meanXLow = -1;
		double sigXUp = 1.75;
		double sigXLow = .25;
		double errorXUp = .15;
		double errorXLow = .05;
		Histogram meanHist = new Histogram("Mean", "mean", "dN", 100, meanXLow, meanXUp);
		Histogram sigmaHist = new Histogram("Sigma", "sigma", "dN", 100, sigXLow, sigXUp);
		Histogram error = new Histogram("Error", "error in mean", "dN", 100, errorXLow, errorXUp);
		Histogram finalHist = new Histogram("Final", "true-mean/error", "dN", 100, -20, 20);
		Histogram mSig = new Histogram("measured sigma^2", "sigma^2", "dN", 100, -2, 5);
		Histogram sig2Err = new Histogram("sigma^2 error", " sigma^2 error", "dN", 100, -.1, .4);
		Histogram total = new Histogram("Total", "true-measured/error", "dN", 100, -6, 6);
		for (int num=0; num<nToDo; num++) {
			Histogram hist = new Histogram(null, null, null, nBins, lowerBound, upperBound);
			SimRandomNormal srn = new SimRandomNormal(num);
			for (int i=0; i<nToDo; i++) {
				hist.fill(srn.next());
			}
			meanHist.fill(hist.getMean());
			double s = hist.getSigma(hist.getMean());
			sigmaHist.fill(s);
			double e = hist.getMeanError();
			error.fill(e);
			finalHist.fill(-hist.getMean()/e);
			double se2 = hist.getSigma2Error();
			sig2Err.fill(se2);
			mSig.fill(s*s);
			double sum = (1-s*s)/se2;
			total.fill(sum);
		}
		Histogram[] colList = {meanHist};
		JFreeChart chartCol = Histogram.getChart(colList, "Mean", "mean", "dN", Double.NaN, Double.NaN, false);	
		double meanYUp = chartCol.getXYPlot().getRangeAxis().getUpperBound();
		double meanYLow = chartCol.getXYPlot().getRangeAxis().getLowerBound();
		double meanXRange = meanXUp-meanXLow;
		double meanYRange = meanYUp-meanYLow;
		chartCol.getXYPlot().addAnnotation(new XYTextAnnotation("mean="+meanHist.getMean(), meanXUp-meanXRange/10, meanYUp-meanYRange/10));
		ChartUtilities.saveChartAsPNG(new File(StringUtil.getFileName("mean","png")), chartCol, 600, 600);

		Histogram[] colList1 = {sigmaHist};
		JFreeChart chartCol1 = Histogram.getChart(colList1, "Sigma", "sigma", "dN", Double.NaN, Double.NaN, false);	
		double sigYUp = chartCol1.getXYPlot().getRangeAxis().getUpperBound();
		double sigYLow = chartCol1.getXYPlot().getRangeAxis().getLowerBound();
		double sigXRange = sigXUp-sigXLow;
		double sigYRange = sigYUp-sigYLow;
		chartCol1.getXYPlot().addAnnotation(new XYTextAnnotation("mean="+sigmaHist.getMean(), sigXUp-sigXRange/10, sigYUp-sigYRange/10));
		ChartUtilities.saveChartAsPNG(new File(StringUtil.getFileName("sigma","png")), chartCol1, 600, 600);

		Histogram[] colList2 = {error};
		JFreeChart chartCol2 = Histogram.getChart(colList2, "Error", "error", "dN", Double.NaN, Double.NaN, false);	
		double errorYUp = chartCol2.getXYPlot().getRangeAxis().getUpperBound();
		double errorYLow = chartCol2.getXYPlot().getRangeAxis().getLowerBound();
		double errorXRange = errorXUp-errorXLow;
		double errorYRange = errorYUp-errorYLow;
		chartCol2.getXYPlot().addAnnotation(new XYTextAnnotation("mean="+error.getMean(), errorXUp-errorXRange/10, errorYUp-errorYRange/10));
		ChartUtilities.saveChartAsPNG(new File(StringUtil.getFileName("error","png")), chartCol2, 600, 600);

		System.out.println("final sigma="+finalHist.getSigma(finalHist.getMean()));

		Histogram[] colList3 = {error};
		JFreeChart chartCol3 = Histogram.getChart(colList3, "Final", "true-mean/error", "dN", Double.NaN, Double.NaN, false);	
		ChartUtilities.saveChartAsPNG(new File(StringUtil.getFileName("final","png")), chartCol3, 600, 600);

		Histogram[] colList4 = {mSig};
		JFreeChart chartCol4 = Histogram.getChart(colList4, "Measured Sigma", "measured", "dN", Double.NaN, Double.NaN, false);	
		ChartUtilities.saveChartAsPNG(new File(StringUtil.getFileName("measured","png")), chartCol4, 600, 600);

		Histogram[] colList5 = {sig2Err};
		JFreeChart chartCol5 = Histogram.getChart(colList5, "Sigma2 Error", "sigma2 error", "dN", Double.NaN, Double.NaN, false);	
		ChartUtilities.saveChartAsPNG(new File(StringUtil.getFileName("sig2error","png")), chartCol5, 600, 600);

		System.out.println("Total mean="+total.getMean());
		System.out.println("Total sigma="+total.getSigma(total.getMean()));

		Histogram[] colList6 = {total};
		JFreeChart chartCol6 = Histogram.getChart(colList6, "True-Measured/Error", "sum", "dN", Double.NaN, Double.NaN, false);	
		ChartUtilities.saveChartAsPNG(new File(StringUtil.getFileName("total","png")), chartCol6, 600, 600);


	}

}
