package gov.fnal.eag.sim.demo;

import gov.fnal.eag.sim.pixel.DetectorObjectProfile;
import gov.fnal.eag.sim.pixel.PixeledImage;
import gov.fnal.eag.sim.pixel.Profiles;
import gov.fnal.eag.sim.repository.image.ImageRepository;

/**
 * demonstrate how to make an image with one object
 * @author stoughto
 *
 */
public class MakeOneDetectorObject {

	private Profiles profiles = new Profiles();
	
	/**
	 * @param args are ignored
	 */
	public static void main(String[] args) {
		new MakeOneDetectorObject();
	}
	
	/**
	 * write one detector object on a PixeledImage and save it as a FITS file
	 */
	public MakeOneDetectorObject() {
		// size of the image
		int nrow = 500;
		int ncol = 500;
		PixeledImage original = new PixeledImage(nrow, ncol);
		
		// location, size, and shape of the object
		int row = 200;
		int col = 300;
		int nPhoton = 1000;
		String shape = "exp";
		double scale = 10;
		double ellipticity = 0.1;
		double positionAngle = 30.0;		
		DetectorObjectProfile dosb = new DetectorObjectProfile(row, col, nPhoton, shape, scale, ellipticity, positionAngle, profiles);

		// put the object in the image
		int nGrid = 3;
		for (int r=0; r<nrow; r++) {
			for (int c=0; c<ncol; c++) {
				double value = dosb.getFluxByPix(r,c,nGrid);
				original.setByRowCol(r, c, value);
			}
		}		
		// write the image 
		ImageRepository.write("original.fit",original);

		// This line simply copies the original, but we want to convolve it
		PixeledImage convolved = new PixeledImage(original);  // Hemant, you can do better than this!
		ImageRepository.write("convolved.fit", convolved);
		
	}

}
