package gov.fnal.eag.sim.demo;

import java.util.ArrayList;

import gov.fnal.eag.sim.pixel.BackgroundLightConstant;
import gov.fnal.eag.sim.pixel.DetectorObject;
import gov.fnal.eag.sim.pixel.DetectorObjectProfile;
import gov.fnal.eag.sim.pixel.PSFFactoryNull;
import gov.fnal.eag.sim.pixel.PixelDetector;
import gov.fnal.eag.sim.pixel.PixeledImage;
import gov.fnal.eag.sim.pixel.PixeledImageFactory;
import gov.fnal.eag.sim.pixel.Profiles;
import gov.fnal.eag.sim.random.SimRandomNormal;
import gov.fnal.eag.sim.random.SimRandomPoisson;
import gov.fnal.eag.sim.random.SimRandomUniform;
import gov.fnal.eag.sim.repository.image.ImageRepository;

/**
 * Demonstration of how to make a single star and write it to
 * the ImageRepository.
 * @author dcharms
 *
 */
public class MakeOneStar {

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		drawStar(.3, 1, 20, 20);
		
	}
	
	/**
	 * a simple method to draw one star and save the image
	 * @param pixelSize size of a pixel in arseconds
	 * @param sigmaInArcSec value of sigma in arcseconds
	 * @param row row position
	 * @param col column position
	 */
	public static void drawStar(double pixelSize, double sigmaInArcSec, double row, double col) {
		int nRow = (int)(6*2*(sigmaInArcSec/pixelSize));
		int nCol = nRow;
		PixelDetector pixelDetector = new PixelDetector(nRow, nCol);
		double sky = 0;
		BackgroundLightConstant backgroundLight = new BackgroundLightConstant(sky);
		PSFFactoryNull psf = new PSFFactoryNull();
		double cosmicRayDensity = 0;
		int seed = 12;
		double nPhoton = 1e6;
		Profiles profile = new Profiles();
		SimRandomPoisson srp = new SimRandomPoisson(seed+1);
		SimRandomUniform sru = new SimRandomUniform(seed+2);
		SimRandomNormal srn = new SimRandomNormal(seed+3);
		PixeledImage pi = null;
		
		ArrayList<DetectorObject> list = new ArrayList<DetectorObject>();
		String shape = "gauss";
		double scale = sigmaInArcSec/pixelSize;
		DetectorObjectProfile detectorObject = new DetectorObjectProfile(row, col, nPhoton, shape, scale, 0, 0, profile);
		list.add(detectorObject);
		PixeledImageFactory pif = new PixeledImageFactory(pixelDetector, backgroundLight, psf, list, cosmicRayDensity, srp, sru, srn);
		pif.setNGridBright(5);
		pi = pif.planAndProduce();
		
		ImageRepository.write("OneStar.FITS", pi, true);
	}

}
