package gov.fnal.eag.sim.demo;

import gov.fnal.eag.sim.pixel.BackgroundLightConstant;
import gov.fnal.eag.sim.pixel.DetectorObject;
import gov.fnal.eag.sim.pixel.DetectorObjectProfile;
import gov.fnal.eag.sim.pixel.PSFFactoryNull;
import gov.fnal.eag.sim.pixel.PixelDetector;
import gov.fnal.eag.sim.pixel.PixeledImage;
import gov.fnal.eag.sim.pixel.PixeledImageFactory;
import gov.fnal.eag.sim.pixel.Profiles;
import gov.fnal.eag.sim.random.SimRandomNormal;
import gov.fnal.eag.sim.random.SimRandomPoisson;
import gov.fnal.eag.sim.random.SimRandomUniform;
import gov.fnal.eag.sim.repository.image.ImageRepository;
import gov.fnal.eag.util.yanny.Table;

import java.awt.geom.Point2D;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.LinkedList;

import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartUtilities;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.renderer.xy.XYLineAndShapeRenderer;
import org.jfree.data.xy.XYSeries;
import org.jfree.data.xy.XYSeriesCollection;

import uk.ac.starlink.table.ColumnInfo;
import uk.ac.starlink.table.RowListStarTable;
import cern.colt.list.DoubleArrayList;
import cern.jet.stat.Descriptive;

/**
 * Demonstration of how noise effects measurements of shape and location.
 * @author dcharms
 *
 */
public class NoiseStudy {

	private static RowListStarTable starTable = null;

	/**
	 * @param args
	 * @throws IOException 
	 */
	public static void main(String[] args) throws IOException {
		NoiseStudy ns = new NoiseStudy();
		ns.run();
	}
	private void run() throws IOException {
		int nToDo = 100;

		ColumnInfo ci0 = new ColumnInfo("ps", Double.class, "pixel size (arcseconds)");
		ColumnInfo ci1 = new ColumnInfo("sig", Double.class, "sigma (arcseconds)");
		ColumnInfo ci2 = new ColumnInfo("rn", Double.class, "read noise (e)");
		ColumnInfo ci3 = new ColumnInfo("sky", Double.class, "sky value (e/pixel)");
		ColumnInfo ci4 = new ColumnInfo("np", Double.class, "number of photons");
		ColumnInfo ci5 = new ColumnInfo("pois", Double.class, "poisson (1=true, 0=false)");
		ColumnInfo ci6 = new ColumnInfo("dr", Double.class, "delta row (arcseconds)");
		ColumnInfo ci7 = new ColumnInfo("dre", Double.class, "delta row error (arcseconds)");
		ColumnInfo ci8 = new ColumnInfo("dc", Double.class, "delta column (arcseconds)");
		ColumnInfo ci9 = new ColumnInfo("dce", Double.class, "delta column error (arcseconds)");
		ColumnInfo ci10 = new ColumnInfo("dsr", Double.class, "delta sigma row (arcseconds)");
		ColumnInfo ci11 = new ColumnInfo("dsre", Double.class, "delta sigma row error (arcseconds)");
		ColumnInfo ci12 = new ColumnInfo("dsc", Double.class, "delta sigma column (arcseconds)");
		ColumnInfo ci13 = new ColumnInfo("dsce", Double.class, "delta sigma column error (arcseconds)");
		ColumnInfo[] ci = {ci0, ci1, ci2, ci3, ci4, ci5, ci6, ci7, ci8, ci9, ci10, ci11, ci12, ci13};
		starTable = new RowListStarTable(ci);
		double sigmaInArcSec[] = {.6, 1.2};
		double readNoise[] = {0};
		double sky[] = {10, 100};
		double nPhoton[] = {500.0, 2000.0};
		double poisson[] = {1};
		int n = 0;
		int nTests = sigmaInArcSec.length * readNoise.length * sky.length * nPhoton.length * poisson.length;
		String fileName = "NoiseStudyTable.fit";
		ImageRepository ir = new ImageRepository();
		for (int i=0; i<sigmaInArcSec.length; i++) {
			for (int j=0; j<readNoise.length; j++) {
				for (int k=0; k<sky.length; k++) {
					for (int l=0; l<nPhoton.length; l++) {
						for (int m=0; m<poisson.length; m++) {
							if (n%1 == 0) System.out.println("n="+n+" nTests="+nTests);
							simple(.27, sigmaInArcSec[i], readNoise[j], sky[k], nPhoton[l], poisson[m], nToDo, true, false, false);
							n++;
						}
					}
				}
			}
		}
		ir.put(fileName, starTable, true);		
		plot("NoiseStudyTable.fit", "sig", "dr", "np", nPhoton, "pois", poisson[0], "rn", readNoise[0], "sky", sky[0]);
	}

	/**
	 * @param pixelSize size of one pixel in arcseconds
	 * @param sigmaInArcSec value of sigma in arcseconds
	 * @param readNoise 
	 * @param sky 
	 * @param nPhoton number of photons
	 * @param poisson 
	 * @param nToDo 
	 * @param makeTable 
	 * @param fits 
	 * @param chatter 
	 */
	public static void simple(double pixelSize, double sigmaInArcSec, double readNoise, double sky, double nPhoton, double poisson, int nToDo, boolean makeTable, boolean fits, boolean chatter) {
		int nRow = (int)(6*2*(sigmaInArcSec/pixelSize));
		int nCol = nRow;
		int nDRow = 0, nDCol = 0, nDRowSig = 0, nDColSig = 0;
		double quantumEfficiency = 1;
		double gain = 1;
		SimRandomNormal srn = new SimRandomNormal();
		PixelDetector pixelDetector = new PixelDetector(new Point2D.Float(nCol, nRow), quantumEfficiency, gain, readNoise, srn);
		BackgroundLightConstant backgroundLight = new BackgroundLightConstant(sky);
		PSFFactoryNull psf = new PSFFactoryNull();
		double cosmicRayDensity = 0;
		int seed = 12;
		SimRandomPoisson srp = new SimRandomPoisson(seed+1);
		SimRandomUniform sru = new SimRandomUniform(seed+2);
		Profiles profile = new Profiles();
		DoubleArrayList dRowArray = new DoubleArrayList();
		DoubleArrayList dColArray = new DoubleArrayList();
		DoubleArrayList dRowSigArray = new DoubleArrayList();
		DoubleArrayList dColSigArray = new DoubleArrayList();
		for (int i=0; i<nToDo; i++) {
			ArrayList<DetectorObject> list = new ArrayList<DetectorObject>();
			double row = sru.next(nRow/2, nRow/2+1);
			double col = sru.next(nCol/2, nCol/2+1);
			String shape = "gauss";
			double scale = sigmaInArcSec/pixelSize;
			DetectorObjectProfile detectorObject = new DetectorObjectProfile(row, col, nPhoton, shape, scale, 0, 0, profile);
			list.add(detectorObject);
			PixeledImageFactory pif = new PixeledImageFactory(pixelDetector, backgroundLight, psf, list, cosmicRayDensity, srp, sru, srn);
			pif.setNGridBright(5);
			pif.setCountsPerPixelThreshold(0.01);
				pif.setDoPoisson(true);
			
			PixeledImage pi = pif.planAndProduce();
			Hashtable<String,Number> ht = pi.centroid(sky);
			double measuredRow = ((Double)(ht.get("rowCentroid"))).doubleValue();
			double deltaRow = (measuredRow-row)*pixelSize;
			double measuredCol = ((Double)(ht.get("colCentroid"))).doubleValue();
			double deltaCol = (measuredCol-col)*pixelSize;
			double sigRow = ((Double)(ht.get("rowSigma"))).doubleValue();
			double sigCol = ((Double)(ht.get("colSigma"))).doubleValue();
			double deltaSigRow = (sigRow-scale)*pixelSize;
			double deltaSigCol = (sigCol-scale)*pixelSize;
			if (chatter) {
				
			}
			if (deltaRow == deltaRow) {
				dRowArray.add(deltaRow);
				nDRow++;
			}
			if (deltaCol == deltaCol) {
				dColArray.add(deltaCol);
				nDCol++;
			}
			if (deltaSigRow == deltaSigRow) {
				dRowSigArray.add(deltaSigRow);
				nDRowSig++;
			}
			if (deltaSigCol == deltaSigCol)	{
				dColSigArray.add(deltaSigCol);
				nDColSig++;
			}

			if (fits) {

			}

		}

		double meanDRow = Descriptive.mean(dRowArray);
		double meanDRowError = Descriptive.standardDeviation(Descriptive.sampleVariance(dRowArray, meanDRow))/Math.sqrt(nDRow);
		double meanDCol = Descriptive.mean(dColArray);
		double meanDColError = Descriptive.standardDeviation(Descriptive.sampleVariance(dColArray, meanDCol))/Math.sqrt(nDCol);
		double meanDRowSig = Descriptive.mean(dRowSigArray);
		double meanDRowSigError = Descriptive.standardDeviation(Descriptive.sampleVariance(dRowSigArray, meanDRowSig))/Math.sqrt(nDRowSig);
		double meanDColSig = Descriptive.mean(dColSigArray);
		double meanDColSigError = Descriptive.standardDeviation(Descriptive.sampleVariance(dColSigArray, meanDColSig))/Math.sqrt(nDColSig);

		
		Object[] row0 = { new Double(pixelSize), new Double(sigmaInArcSec), new Double(readNoise), new Double(sky), new Double(nPhoton), new Double(poisson), 
				new Double(meanDRow), new Double(meanDRowError),
				new Double(meanDCol), new Double(meanDColError), 
				new Double(meanDRowSig), new Double(meanDRowSigError), 
				new Double(meanDColSig), new Double(meanDColSigError)};
		if (makeTable)
			starTable.addRow(row0);




	}


	/**
	 * make a plot of one variable versus another and see how it changes based on the variation variable.
	 * @param fileName filename to save plot to
	 * @param xVariable name of variable to go on x-axis
	 * @param yVariable name of variable to go on y-axis
	 * @param variation name of variable that will be changing
	 * @param varValues array of values of variation variable
	 * @param constant1 first constant name
	 * @param value1 value of first constant
	 * @param constant2 second constant name
	 * @param value2 value of second constant
	 * @param constant3 third constant name
	 * @param value3 value of third constant
	 * @throws IOException 
	 */
	public void plot(String fileName, String xVariable, String yVariable, String variation, double[]varValues, String constant1, double value1, String constant2, double value2, String constant3, double value3) throws IOException {			
		ImageRepository ir = new ImageRepository();	
		LinkedList<Object> list = ir.get(fileName);
		RowListStarTable rlst = (RowListStarTable) list.get(0);
		int xIndex = Table.getIndexByName(rlst,xVariable);
		int yIndex = Table.getIndexByName(rlst, yVariable);
		int yErrorIndex = Table.getIndexByName(rlst, yVariable+"e");
		int varIndex = Table.getIndexByName(rlst, variation);
		int cons1Index = Table.getIndexByName(rlst, constant1);
		int cons2Index = Table.getIndexByName(rlst, constant2);
		int cons3Index = Table.getIndexByName(rlst, constant3);
		double sigma[][] = new double [10][100];
		XYSeriesCollection xy = new XYSeriesCollection();
		for (int i=0; i<varValues.length; i++) {
			xy.addSeries(new XYSeries(variation+"="+varValues[i]));
			System.out.println("add series with variation="+variation+" value="+varValues[i]);
		}
		for (int i=0; i< varValues.length; i++) {
			int count = 0;
			for (long j=0; j<rlst.getRowCount(); j++) {
				Object[] row = rlst.getRow(j);
				double x = (Double) row[xIndex];
				double y = (Double) row[yIndex];
				double varCheck = (Double) row[varIndex];
				double cons1Check = (Double) row[cons1Index];
				double cons2Check = (Double) row[cons2Index];
				double cons3Check = (Double) row[cons3Index];
				if(varCheck==varValues[i]&&cons1Check==value1&&cons2Check==value2&&cons3Check==value3) {
					if (!Double.isInfinite(y)) {
						xy.getSeries(i).add(x, y);
						sigma[i][count] = (Double) row[yErrorIndex];
						count++;			  
					}
				}
			}
		}
		String c1 = rlst.getColumnInfo(cons1Index).getDescription();
		String c2 = rlst.getColumnInfo(cons2Index).getDescription();
		String c3 = rlst.getColumnInfo(cons3Index).getDescription();
		String title = String.format("%s=%.2f %s=%.2f %s=%.2f", c1, value1, c2, value2, c3, value3);
		String xVar = rlst.getColumnInfo(xIndex).getDescription();
		String yVar = rlst.getColumnInfo(yIndex).getDescription();
		JFreeChart chart = ChartFactory.createScatterPlot(title, xVar, yVar, xy, PlotOrientation.VERTICAL, true, false, false);
		XYLineAndShapeRenderer xyItemRenderer = (XYLineAndShapeRenderer) chart.getXYPlot().getRendererForDataset(xy);
		for (int i=0; i<xy.getSeriesCount(); i++) {
			xyItemRenderer.setSeriesLinesVisible(i, true);
			xyItemRenderer.setSeriesShapesVisible(i, true);
		}
		String file = String.format("%Svs%Svar%S.%S%.2f.%S%.2f.%S%.2f", xVariable, yVariable, variation, constant1, value1, constant2, value2, constant3, value3);
		ChartUtilities.saveChartAsPNG(new File(file+".png"), chart, 700, 600);
	}
}
