package gov.fnal.eag.sim.demo;

import gov.fnal.eag.sim.pixel.BackgroundLightConstant;
import gov.fnal.eag.sim.pixel.DetectorObject;
import gov.fnal.eag.sim.pixel.DetectorObjectProfile;
import gov.fnal.eag.sim.pixel.PSFFactoryNull;
import gov.fnal.eag.sim.pixel.PixelDetector;
import gov.fnal.eag.sim.pixel.PixeledImage;
import gov.fnal.eag.sim.pixel.PixeledImageFactory;
import gov.fnal.eag.sim.pixel.Profiles;
import gov.fnal.eag.sim.random.SimRandomNormal;
import gov.fnal.eag.sim.random.SimRandomPoisson;
import gov.fnal.eag.sim.random.SimRandomUniform;
import gov.fnal.eag.util.astronomy.SimpleFunction;
import gov.fnal.eag.util.densityEstimate.KernelDensityEstimate;
import gov.fnal.eag.util.densityEstimate.UniformKernel;
import gov.fnal.eag.util.misc.StringUtil;
import gov.fnal.eag.util.plot.JFreeChartUtil;
import gov.fnal.eag.util.plot.XYErrorBarRenderer;
import gov.fnal.eag.util.plot.XYErrorBarSeries;
import gov.fnal.eag.util.plot.XYErrorBarSeriesCollection;

import java.awt.Color;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Hashtable;

import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartUtilities;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.plot.XYPlot;
import org.jfree.chart.renderer.xy.XYLineAndShapeRenderer;
import org.jfree.data.xy.XYSeries;
import org.jfree.data.xy.XYSeriesCollection;

import cern.colt.list.DoubleArrayList;
import cern.jet.stat.Descriptive;

/**
 * demonstrates the effect of just pixelization
 * @author dcharms
 *
 */
public class Pixelization {

	/**
	 * @param args ignored
	 * @throws IOException 
	 */
	public static void main(String[] args) throws IOException {
		XYErrorBarSeriesCollection mean = new XYErrorBarSeriesCollection();
		XYSeriesCollection sigma = new XYSeriesCollection();
		XYSeriesCollection sigmaDelta = new XYSeriesCollection();
		mean.addSeries(new XYErrorBarSeries("DeltaRow"));
		sigmaDelta.addSeries(new XYSeries("SigmaDeltaRow"));
		mean.addSeries(new XYErrorBarSeries("DeltaCol"));
		sigmaDelta.addSeries(new XYSeries("SigmaDeltaCol"));
		sigma.addSeries(new XYSeries("DeltaRowSigma"));
		sigma.addSeries(new XYSeries("DeltaColSigma"));
		double sig = 1.00;
		simple(.1, sig, mean, sigma, sigmaDelta);
		simple(.2, sig, mean, sigma, sigmaDelta);
		simple(.3, sig, mean, sigma, sigmaDelta);
		String title = String.format("Distribution Due to Pixelization Sigma = %.2f", sig);
		JFreeChart chart = ChartFactory.createScatterPlot(title, "pixel size", "mean (Arc Seconds)", mean, PlotOrientation.VERTICAL, false, false, false);
		title = String.format("Sigma Distribution Due to Pixelization Sigma = %.2f", sig);
		JFreeChart chart1 = ChartFactory.createScatterPlot(title, "pixel size", "sigma (Arc Seconds)", sigma, PlotOrientation.VERTICAL, true, false, false);
		XYLineAndShapeRenderer xyItemRenderer = (XYLineAndShapeRenderer) chart.getXYPlot().getRendererForDataset(mean);
		JFreeChart chart2 = ChartFactory.createScatterPlot("SigmaDelta Distribution", "Pixel Size", "Arcseconds", sigmaDelta, PlotOrientation.VERTICAL, true, false, false);
		for (int i=0; i<mean.getSeriesCount(); i++) {
			xyItemRenderer.setSeriesLinesVisible(i, true);
			xyItemRenderer.setSeriesShapesVisible(i, true);
		}
		XYLineAndShapeRenderer xyItemRenderer1 = (XYLineAndShapeRenderer) chart1.getXYPlot().getRendererForDataset(sigma);
		for (int i=0; i<sigma.getSeriesCount(); i++) {
			xyItemRenderer1.setSeriesLinesVisible(i, true);
			xyItemRenderer1.setSeriesShapesVisible(i, true);
		}
		XYLineAndShapeRenderer xyItemRenderer2 = (XYLineAndShapeRenderer) chart2.getXYPlot().getRendererForDataset(sigmaDelta);
		for (int i=0; i<sigma.getSeriesCount(); i++) {
			xyItemRenderer2.setSeriesLinesVisible(i, true);
			xyItemRenderer2.setSeriesShapesVisible(i, true);
		}
		final XYPlot plot = chart.getXYPlot();
		plot.setBackgroundPaint(Color.white);
	    final XYErrorBarRenderer renderer = new XYErrorBarRenderer();
	    plot.setRenderer(renderer);
	    renderer.addChangeListener(plot);
	    JFreeChartUtil.toPng(chart, StringUtil.getFileName("-mean", "png"));
	    JFreeChartUtil.toPng(chart1, StringUtil.getFileName("-sigma", "png"));
	    JFreeChartUtil.toPng(chart2, StringUtil.getFileName("-sigmaDelta", "png"));
	}
	
	/**
	 * @param pixelSize size of a pixel in arcseconds
	 * @param sigmaInArcSec value of sigma in arcseconds
	 * @param series errorbar collection of means
	 * @param sigma collection of sigmas
	 * @param sigmaDelta collection of delta sigmas
	 * @throws IOException 
	 */
	public static void simple(double pixelSize, double sigmaInArcSec, XYErrorBarSeriesCollection series, XYSeriesCollection sigma, XYSeriesCollection sigmaDelta) throws IOException {
		int nToDo = 100;
		int nRow = (int)(6*2*(sigmaInArcSec/pixelSize));
		int nCol = nRow;
		PixelDetector pixelDetector = new PixelDetector(nRow, nCol);
		double sky = 0;
		BackgroundLightConstant backgroundLight = new BackgroundLightConstant(sky);
		PSFFactoryNull psf = new PSFFactoryNull();
		double cosmicRayDensity = 0;
		int seed = 12;
		SimRandomPoisson srp = new SimRandomPoisson(seed+1);
		SimRandomUniform sru = new SimRandomUniform(seed+2);
		SimRandomNormal srn = new SimRandomNormal(seed+3);
		double nPhoton = 1e6;
		Profiles profile = new Profiles();
		XYSeries dRow = new XYSeries("deltaRow");
		XYSeries dCol = new XYSeries("deltaColumn");
		XYSeries dSigRow = new XYSeries("deltaSigmaRow");
		XYSeries dSigCol = new XYSeries("deltaSigmaColumn");
		XYSeries dRowvRow = new XYSeries("Row,DeltaRow");
		XYSeries dColvRow = new XYSeries("Row,DeltaCol");
		XYSeries sRowvRow = new XYSeries("Row,DeltaSigRow");
		XYSeries sColvRow = new XYSeries("Row,DeltaSigCol");
		XYSeries dRowvCol = new XYSeries("Col,DeltaRow");
		XYSeries dColvCol = new XYSeries("Col,DeltaCol");
		XYSeries sRowvCol = new XYSeries("Col,DeltaSigRow");
		XYSeries sColvCol = new XYSeries("Col,DeltaSigCol");
		SimpleFunction kernel = new UniformKernel(.0025);			
		KernelDensityEstimate density1 = new KernelDensityEstimate(-0.025, 0.025, 100, kernel);
		KernelDensityEstimate density2 = new KernelDensityEstimate(-0.025, 0.025, 100, kernel);
		KernelDensityEstimate density3 = new KernelDensityEstimate(-0.025, 0.025, 100, kernel);
		KernelDensityEstimate density4 = new KernelDensityEstimate(-0.025, 0.025, 100, kernel);
		DoubleArrayList dRowArray = new DoubleArrayList();
		DoubleArrayList dColArray = new DoubleArrayList();
		DoubleArrayList dRowSigArray = new DoubleArrayList();
		DoubleArrayList dColSigArray = new DoubleArrayList();
		for (int i=0; i<nToDo; i++) {
			ArrayList<DetectorObject> list = new ArrayList<DetectorObject>();
			double row = sru.next(nRow/2, nRow/2+1);
			double col = sru.next(nCol/2, nCol/2+1);
			String shape = "gauss";
			double scale = sigmaInArcSec/pixelSize;
			DetectorObjectProfile detectorObject = new DetectorObjectProfile(row, col, nPhoton, shape, scale, 0, 0, profile);
			list.add(detectorObject);
			PixeledImageFactory pif = new PixeledImageFactory(pixelDetector, backgroundLight, psf, list, cosmicRayDensity, srp, sru, srn);
			pif.setNGridBright(5);
			PixeledImage pi = pif.planAndProduce();
			Hashtable<String,Number> ht = pi.centroid(sky);
			double measuredRow = ((Double)(ht.get("rowCentroid"))).doubleValue();
			double deltaRow = row-measuredRow;
			double measuredCol = ((Double)(ht.get("colCentroid"))).doubleValue();
			double deltaCol = col-measuredCol;
			double sigRow = ((Double)(ht.get("rowSigma"))).doubleValue();
			double sigCol = ((Double)(ht.get("colSigma"))).doubleValue();
			double deltaSigRow = scale-sigRow;
			double deltaSigCol = scale-sigCol;
			density1.add(pixelSize*deltaRow);
			density2.add(pixelSize*deltaCol);
			density3.add(pixelSize*deltaSigRow);
			density4.add(pixelSize*deltaSigCol);
			dRowvRow.add(pixelSize*deltaRow, row);
			dColvRow.add(pixelSize*deltaCol, row);
			sRowvRow.add(pixelSize*deltaSigRow, row);
			sColvRow.add(pixelSize*deltaSigCol, row);
			dRowvCol.add(pixelSize*deltaRow, col);
			dColvCol.add(pixelSize*deltaCol, col);
			sRowvCol.add(pixelSize*deltaSigRow, col);
			sColvCol.add(pixelSize*deltaSigCol, col);
			dRowArray.add(pixelSize*deltaRow);
			dColArray.add(pixelSize*deltaCol);
			dRowSigArray.add(pixelSize*deltaSigRow);
			dColSigArray.add(pixelSize*deltaSigCol);
		}
		double y[] = density1.getEstimates();
		double x[] = density1.getX();
		double max1 = 0;
		for(int i=0; i<x.length; i++) {
			dRow.add(x[i], y[i]);
			if (max1 < y[i])
				max1 = y[i];
		}
		XYSeriesCollection first = new XYSeriesCollection();
		first.addSeries(dRow);
		y = density2.getEstimates();
		x = density2.getX();
		double max2 = 0;
		for(int i=0; i<x.length; i++) {
			dCol.add(x[i], y[i]);
			if (max2 < y[i])
				max2 = y[i];
		}
		XYSeriesCollection second = new XYSeriesCollection();
		second.addSeries(dCol);
		y = density3.getEstimates();
		x = density3.getX();
		double max3 = 0;
		for(int i=0; i<x.length; i++) {
			dSigRow.add(x[i], y[i]);
			if (max3 < y[i])
				max3 = y[i];
		}
		XYSeriesCollection third = new XYSeriesCollection();
		third.addSeries(dSigRow);
		y = density4.getEstimates();
		x = density4.getX();
		double max4 = 0;
		for(int i=0; i<x.length; i++) {
			dSigCol.add(x[i], y[i]);
			if (max4 < y[i])
				max4 = y[i];
		}
		XYSeriesCollection fourth = new XYSeriesCollection();
		fourth.addSeries(dSigCol);
		XYSeriesCollection cloud = new XYSeriesCollection();
		cloud.addSeries(dRowvRow);
		cloud.addSeries(dColvRow);
		cloud.addSeries(sRowvRow);
		cloud.addSeries(sColvRow);
		cloud.addSeries(dRowvCol);
		cloud.addSeries(dColvCol);
		cloud.addSeries(sRowvCol);
		cloud.addSeries(sColvCol);
		
		double meanDRow = Descriptive.mean(dRowArray);
		double sigmaDRow = Descriptive.standardDeviation(Descriptive.sampleVariance(dRowArray, meanDRow));
		double meanDCol = Descriptive.mean(dColArray);
		double sigmaDCol = Descriptive.standardDeviation(Descriptive.sampleVariance(dColArray, meanDCol));
		double meanDRowSig = Descriptive.mean(dRowSigArray);
		double sigmaDRowSig = Descriptive.standardDeviation(Descriptive.sampleVariance(dRowSigArray, meanDRowSig));
		double meanDColSig = Descriptive.mean(dColSigArray);
		double sigmaDColSig = Descriptive.standardDeviation(Descriptive.sampleVariance(dColSigArray, meanDColSig));
		
		XYSeries mean1 = new XYSeries("mean");
		mean1.add(meanDRow, 0);
		mean1.add(meanDRow, max1);
		first.addSeries(mean1);
		XYSeries mean2 = new XYSeries("mean");
		mean2.add(meanDCol, 0);
		mean2.add(meanDCol, max2);
		second.addSeries(mean2);
		XYSeries mean3 = new XYSeries("mean");
		mean3.add(meanDRowSig, 0);
		mean3.add(meanDRowSig, max3);
		third.addSeries(mean3);
		XYSeries mean4 = new XYSeries("mean");
		mean4.add(meanDColSig, 0);
		mean4.add(meanDColSig, max4);
		fourth.addSeries(mean4);
		
		XYSeries sigma1 = new XYSeries("sigma");
		sigma1.add(meanDRow-sigmaDRow*2.355/2, max1/2);
		sigma1.add(meanDRow+sigmaDRow*2.355/2, max1/2);
		first.addSeries(sigma1);
		XYSeries sigma2 = new XYSeries("sigma");
		sigma2.add(meanDCol-sigmaDCol*2.355/2, max2/2);
		sigma2.add(meanDCol+sigmaDCol*2.355/2, max2/2);
		second.addSeries(sigma2);
		XYSeries sigma3 = new XYSeries("sigma");
		sigma3.add(meanDRowSig-sigmaDRowSig*2.355/2, max3/2);
		sigma3.add(meanDRowSig+sigmaDRowSig*2.355/2, max3/2);
		third.addSeries(sigma3);
		XYSeries sigma4 = new XYSeries("sigma");
		sigma4.add(meanDColSig-sigmaDColSig*2.355/2, max4/2);
		sigma4.add(meanDColSig+sigmaDColSig*2.355/2, max4/2);
		fourth.addSeries(sigma4);
		
		String DeltaRow = String.format("DeltaRow Pixel=%.2f Sigma=%.2f", pixelSize, sigmaInArcSec);
		String DeltaColumn = String.format("DeltaColumn Pixel=%.2f Sigma=%.2f", pixelSize, sigmaInArcSec);
		String DeltaSigmaRow = String.format("DeltaSigmaRow Pixel=%.2f Sigma=%.2f", pixelSize, sigmaInArcSec);
		String DeltaSigmaColumn = String.format("DeltaSigmaColumn Pixel=%.2f Sigma=%.2f", pixelSize, sigmaInArcSec);
		String Cloud = String.format("Cloud Pixel=%.2f Sigma=%.2f", pixelSize, sigmaInArcSec);
		JFreeChart chart1 = ChartFactory.createXYLineChart(DeltaRow, "x", "y", first, PlotOrientation.VERTICAL, true, false, false);
		JFreeChart chart2 = ChartFactory.createXYLineChart(DeltaColumn, "x", "y", second, PlotOrientation.VERTICAL, true, false, false);
		JFreeChart chart3 = ChartFactory.createXYLineChart(DeltaSigmaRow, "x", "y", third, PlotOrientation.VERTICAL, true, false, false);
		JFreeChart chart4 = ChartFactory.createXYLineChart(DeltaSigmaColumn, "x", "y", fourth, PlotOrientation.VERTICAL, true, false, false);
		JFreeChart chart5 = ChartFactory.createScatterPlot(Cloud, "x", "y", cloud, PlotOrientation.VERTICAL, true, false, false);
		String deltRow = String.format("DeltaRowPixel%.2f.png", pixelSize);
		String deltCol = String.format("DeltaColumnPixel%.2f.png", pixelSize);
		String deltSigRow = String.format("DeltaSigmaRowPixel%.2f.png", pixelSize);
		String deltSigCol = String.format("DeltaSigmaColumn%.2f.png", pixelSize);
		String cloudTitle = String.format("Cloud%.2f.png", pixelSize);
		{
			ChartUtilities.saveChartAsPNG(new File(deltRow), chart1, 500, 300);
		} 
		 {
			ChartUtilities.saveChartAsPNG(new File(deltCol), chart2, 500, 300);
		} 
		 {
			ChartUtilities.saveChartAsPNG(new File(deltSigRow), chart3, 500, 300);
		} 
		 {
			ChartUtilities.saveChartAsPNG(new File(deltSigCol), chart4, 500, 300);
		} 
		 {
			ChartUtilities.saveChartAsPNG(new File(cloudTitle), chart5, 500, 500);
		} 
		series.getSeries(0).add(pixelSize, meanDRow, meanDRow-sigmaDRow/Math.sqrt(nToDo), meanDRow+sigmaDRow/Math.sqrt(nToDo));
		sigmaDelta.getSeries(0).add(pixelSize, sigmaDRow);
		series.getSeries(1).add(pixelSize, meanDCol, meanDCol-sigmaDCol/Math.sqrt(nToDo), meanDCol+sigmaDCol/Math.sqrt(nToDo));
		sigmaDelta.getSeries(1).add(pixelSize, sigmaDCol);
		sigma.getSeries(0).add(pixelSize, meanDRowSig);
		sigma.getSeries(1).add(pixelSize, meanDColSig);
	}
	

}
