package gov.fnal.eag.sim.demo;

import gov.fnal.eag.sim.pixel.BackgroundLightConstant;
import gov.fnal.eag.sim.pixel.DetectorObject;
import gov.fnal.eag.sim.pixel.DetectorObjectProfile;
import gov.fnal.eag.sim.pixel.PSFFactoryNull;
import gov.fnal.eag.sim.pixel.PixelDetector;
import gov.fnal.eag.sim.pixel.PixeledImage;
import gov.fnal.eag.sim.pixel.PixeledImageFactory;
import gov.fnal.eag.sim.pixel.Profiles;
import gov.fnal.eag.sim.random.SimRandomNormal;
import gov.fnal.eag.sim.random.SimRandomPoisson;
import gov.fnal.eag.sim.random.SimRandomUniform;
import gov.fnal.eag.sim.repository.image.ImageRepository;
import gov.fnal.eag.sim.utils.Sexec;
import gov.fnal.eag.util.yanny.Table;

import java.awt.geom.Point2D;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.LinkedList;

import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartUtilities;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.renderer.xy.XYLineAndShapeRenderer;
import org.jfree.data.xy.XYSeries;
import org.jfree.data.xy.XYSeriesCollection;

import uk.ac.starlink.table.ColumnInfo;
import uk.ac.starlink.table.RowListStarTable;
import cern.colt.list.DoubleArrayList;
import cern.jet.stat.Descriptive;

/**
 * Demonstration on how to use SExtractor. Includes making a star, finding
 * it and taking simple measurements.
 * @author dcharms
 *
 */
public class SExtractorStudy {

private static RowListStarTable starTable = null;
	
	/**
	 * @param args
	 */
	public static void main(String[] args) {
		
		int nToDo = 5;
		
		ColumnInfo ci0 = new ColumnInfo("ps", Double.class, "pixel size (arcseconds)");
        ColumnInfo ci1 = new ColumnInfo("sig", Double.class, "sigma (arcseconds)");
        ColumnInfo ci2 = new ColumnInfo("rn", Double.class, "read noise (e)");
        ColumnInfo ci3 = new ColumnInfo("sky", Double.class, "sky value (e/pixel)");
        ColumnInfo ci4 = new ColumnInfo("#p", Double.class, "number of photons");
        ColumnInfo ci5 = new ColumnInfo("pois", Double.class, "poisson (1=true, 0=false)");
        ColumnInfo ci6 = new ColumnInfo("dr", Double.class, "delta row (arcseconds)");
        ColumnInfo ci7 = new ColumnInfo("dre", Double.class, "delta row error (arcseconds)");
        ColumnInfo ci8 = new ColumnInfo("dc", Double.class, "delta column (arcseconds)");
        ColumnInfo ci9 = new ColumnInfo("dce", Double.class, "delta column error (arcseconds)");
        ColumnInfo ci10 = new ColumnInfo("dsr", Double.class, "delta sigma row (arcseconds)");
        ColumnInfo ci11 = new ColumnInfo("dsre", Double.class, "delta sigma row error (arcseconds)");
        ColumnInfo ci12 = new ColumnInfo("dsc", Double.class, "delta sigma column (arcseconds)");
        ColumnInfo ci13 = new ColumnInfo("dsce", Double.class, "delta sigma column error (arcseconds)");
        ColumnInfo[] ci = {ci0, ci1, ci2, ci3, ci4, ci5, ci6, ci7, ci8, ci9, ci10, ci11, ci12, ci13};
        starTable = new RowListStarTable(ci);
		double sigmaInArcSec[] = {.6, 1.2};
		double readNoise[] = {0, 20};
		double sky[] = {20};
		double nPhoton[] = {1e3, 1e5};
		double poisson[] = {1};
		int total = 0, n = 0;
		for (int i=0; i<sigmaInArcSec.length; i++) {
			for (int j=0; j<readNoise.length; j++) {
				for (int k=0; k<sky.length; k++) {
					for (int l=0; l<nPhoton.length; l++) {
						for (int m=0; m<poisson.length; m++) {
							simple(.27, sigmaInArcSec[i], readNoise[j], sky[k], nPhoton[l], poisson[m], nToDo, true, false, false);
							total++;
							System.out.print(".");
							if (total>n) {
								System.out.println();
								n++;
								total=0;
							}
							
						}
					}
				}
			}
		}
		String fileName = "SExtractorStudyTable.fit";
        ImageRepository ir = new ImageRepository();
        ir.put(fileName, starTable, true);
		
	}

	/**
	 * @param pixelSize size of one pixel in arcseconds
	 * @param sigmaInArcSec value of sigma in arcseconds
	 * @param readNoise 
	 * @param sky 
	 * @param nPhoton number of photons
	 * @param poisson 
	 * @param nToDo number of stars to do in one experiment
	 * @param makeTable 
	 * @param fits 
	 * @param chatter 
	 */
	public static void simple(double pixelSize, double sigmaInArcSec, double readNoise, double sky, double nPhoton, double poisson, int nToDo, boolean makeTable, boolean fits, boolean chatter) {
		int nRow = (int)(6*2*(sigmaInArcSec/pixelSize))*2;
		int nCol = nRow;
		int nSDRow = 0, nSDCol = 0, nSDRowSig = 0, nSDColSig = 0;
		double quantumEfficiency = 1;
		double gain = 1;
		SimRandomNormal srn = new SimRandomNormal();
		PixelDetector pixelDetector = new PixelDetector(new Point2D.Float(nCol, nRow), quantumEfficiency, gain, readNoise, srn);
		BackgroundLightConstant backgroundLight = new BackgroundLightConstant(sky);
		PSFFactoryNull psf = new PSFFactoryNull();
		double cosmicRayDensity = 0;
		int seed = 12;
		SimRandomPoisson srp = new SimRandomPoisson(seed+1);
		SimRandomUniform sru = new SimRandomUniform(seed+2);
		Profiles profile = new Profiles();
		DoubleArrayList simpleDRowArray = new DoubleArrayList();
		DoubleArrayList simpleDColArray = new DoubleArrayList();
		DoubleArrayList simpleDRowSigArray = new DoubleArrayList();
		DoubleArrayList simpleDColSigArray = new DoubleArrayList();
		Sexec se = new Sexec();
		se.setParam("XWIN_IMAGE", true);
		se.setParam("YWIN_IMAGE", true);
		se.setParam("X2WIN_IMAGE", true);
		se.setParam("Y2WIN_IMAGE", true);
		se.setParam("XMIN_IMAGE", true);
		se.setParam("YMIN_IMAGE", true);
		se.setParam("XMAX_IMAGE", true);
		se.setParam("YMAX_IMAGE", true);
		se.setParam("BACKGROUND", true);
		se.setParam("THRESHOLD", true);
		if (poisson==0 && readNoise==0) {
			se.setSex("THRESH_TYPE", "ABSOLUTE");
			se.setSex("DETECT_THRESH", ""+(sky+.01));
		}
		else {
			se.setSex("DETECT_THRESH", "1.5");
		}
		se.setSex("PIXEL_SCALE", ""+pixelSize);
		se.setSex("GAIN", ""+gain);
		se.setSex("CHECKIMAGE_TYPE", "SEGMENTATION FILTERED IDENTICAL");
		se.setSex("CHECKIMAGE_NAME", "SEGMENTATION.fits FILTERED.fits IDENTICAL.fits");
		se.setSex("FILTER", "Y");
		//se.setSex("MASK_TYPE", "NONE");
		se.setSex("DETECT_MINAREA", "4");
		se.setSegRegion("segmentation.reg");
		se.setCleanupWhenDone(false);
		for (int i=0; i<nToDo; i++) {
			ArrayList<DetectorObject> list = new ArrayList<DetectorObject>();
			double row = sru.next(nRow/2, nRow/2+1);
			double col = sru.next(nCol/2, nCol/2+1);
			String shape = "gauss";
			double scale = sigmaInArcSec/pixelSize;
			DetectorObjectProfile detectorObject = new DetectorObjectProfile(row, col, nPhoton, shape, scale, 0, 0, profile);
			se.setDefConv("gauss_2.5_5x5.conv");
			list.add(detectorObject);
			PixeledImageFactory pif = new PixeledImageFactory(pixelDetector, backgroundLight, psf, list, cosmicRayDensity, srp, sru, srn);
			pif.setNGridBright(5);
			pif.setCountsPerPixelThreshold(.01);
			if (poisson==1)
				pif.setDoPoisson(true);
			else
				pif.setDoPoisson(false);
			PixeledImage pi = pif.planAndProduce();
			String catalogName = se.exec(pi);
			RowListStarTable rlst = se.getRlst(catalogName);
			int xImageIndex = Table.getIndexByName(rlst, "XWIN_IMAGE");
			int yImageIndex = Table.getIndexByName(rlst, "YWIN_IMAGE");
			int xSigIndex = Table.getIndexByName(rlst, "X2WIN_IMAGE");
			int ySigIndex = Table.getIndexByName(rlst, "Y2WIN_IMAGE");
			int xMinIndex = Table.getIndexByName(rlst, "XMIN_IMAGE");
			int yMinIndex = Table.getIndexByName(rlst, "YMIN_IMAGE");
			int xMaxIndex = Table.getIndexByName(rlst, "XMAX_IMAGE");
			int yMaxIndex = Table.getIndexByName(rlst, "YMAX_IMAGE");
			int backIndex = Table.getIndexByName(rlst, "BACKGROUND");
			int threshIndex = Table.getIndexByName(rlst, "THRESHOLD");
			Object[] firstRow = null;
			if (rlst.getRowCount() != 0) {
				firstRow = rlst.getRow(0);
				double xImage = (Double) firstRow[xImageIndex];
				double yImage = (Double) firstRow[yImageIndex];		
				double sigCol = Math.sqrt((Double) firstRow[xSigIndex]);
				double sigRow = Math.sqrt((Double) firstRow[ySigIndex]);
				double measuredRow = yImage-0.5;
				double measuredCol = xImage-0.5;	
				double deltaRow = (measuredRow-row)*pixelSize;
				double deltaCol = (measuredCol-col)*pixelSize;
				double deltaSigRow = (sigRow-scale)*pixelSize;
				double deltaSigCol = (sigCol-scale)*pixelSize;
				double xMin = (Double) firstRow[xMinIndex];
				double yMin = (Double) firstRow[yMinIndex];
				double xMax = (Double) firstRow[xMaxIndex];
				double yMax = (Double) firstRow[yMaxIndex];
				double background = (Double) firstRow[backIndex];
				double threshold = (Double) firstRow[threshIndex];
				
				
				String line = se.getSex("CHECKIMAGE_TYPE");
				String[] tokens = line.split(" ");
				String file = null;
				for (int j=0; j<tokens.length; j++) {
					if (tokens[j].compareTo("SEGMENTATION")==0) {
							file = se.getSegmentation(j);
					}
				}
				ImageRepository irSeg = new ImageRepository(".");
				PixeledImage piSeg = irSeg.getPixeledImage(file);
				Hashtable<?, ?>[] ht = new Hashtable<?,?>[(int)rlst.getRowCount()];
				double simpleRow[] = new double[(int)rlst.getRowCount()];
				double simpleCol[] = new double[(int)rlst.getRowCount()];
				double simpleSigRow[] = new double[(int)rlst.getRowCount()];
				double simpleSigCol[] = new double[(int)rlst.getRowCount()];
				double simpleDeltaRow[] = new double[(int)rlst.getRowCount()];
				double simpleDeltaCol[] = new double[(int)rlst.getRowCount()];
				double simpleDeltaSigRow[] = new double[(int)rlst.getRowCount()];
				double simpleDeltaSigCol[] = new double[(int)rlst.getRowCount()];
				for (int j=0; j<rlst.getRowCount(); j++) {
					ht[j] = pi.centroid(background, piSeg, j+1);
					simpleRow[j] = ((Double)(ht[j].get("rowCentroid"))).doubleValue();
					simpleCol[j] = ((Double)(ht[j].get("colCentroid"))).doubleValue();
					simpleSigRow[j]= ((Double)(ht[j].get("rowSigma"))).doubleValue();
					simpleSigCol[j] = ((Double)(ht[j].get("colSigma"))).doubleValue();
					simpleDeltaRow[j] = (simpleRow[j]-row)*pixelSize;
					simpleDeltaCol[j] = (simpleCol[j]-col)*pixelSize;
					simpleDeltaSigRow[j] = (simpleSigRow[j]-scale)*pixelSize;
					simpleDeltaSigCol[j] = (simpleSigCol[j]-scale)*pixelSize;
				}
	
				if (simpleDeltaRow[0] == simpleDeltaRow[0]) {
					simpleDRowArray.add(simpleDeltaRow[0]);
					nSDRow++;
				}
				if (simpleDeltaCol[0] == simpleDeltaCol[0]) {
					simpleDColArray.add(simpleDeltaCol[0]);
					nSDCol++;
				}
				if (simpleDeltaSigRow[0] == simpleDeltaSigRow[0]) {
					simpleDRowSigArray.add(simpleDeltaSigRow[0]);
					nSDRowSig++;
				}
				if (simpleDeltaSigCol[0] == simpleDeltaSigCol[0])	{
					simpleDColSigArray.add(simpleDeltaSigCol[0]);
					nSDColSig++;
				}
				
				if (chatter) {
					System.out.println("x="+xImage+" y="+yImage+" xMin="+xMin+" yMin="+yMin+" xMax="+xMax+" yMax="+yMax+" background="+background+" threshold="+threshold);
					System.out.println("row="+row+" measuredRow="+measuredRow+" deltaRow="+deltaRow+" col="+col+" measuredCol="+measuredCol+" deltaCol="+deltaCol+
							" scale="+scale+" sigRow="+sigRow+" deltaSigRow="+deltaSigRow+" sigCol="+sigCol+" deltaSigCol="+deltaSigCol+" simpleRow="+simpleRow[0]+
							" simpleDeltaRow="+simpleDeltaRow[0]+" simpleCol="+simpleCol[0]+" simpleDeltaCol="+simpleDeltaCol[0]+" simpleSigRow="+simpleSigRow[0]+" simpleDeltaSigRow="+
							simpleDeltaSigRow[0]+" simpleSigCol="+simpleSigCol[0]+" simpleDeltaSigCol="+simpleDeltaSigCol[0]);
				}
			}
			else
				if (chatter)
					System.out.println("BadStar at seeing="+sigmaInArcSec+" readNoise="+readNoise+" sky="+sky+" #p="+nPhoton+" pois="+poisson);
			if (fits) {
				String title = String.format("StarCheck%d.FITS", i);
				ImageRepository.write(title, pi, true);
			}

		}
		se.cleanup();
		double meanSDRow = Descriptive.mean(simpleDRowArray);
		double meanSDRowError = Descriptive.standardDeviation(Descriptive.sampleVariance(simpleDRowArray, meanSDRow))/Math.sqrt(nSDRow);
		double meanSDCol = Descriptive.mean(simpleDColArray);
		double meanSDColError = Descriptive.standardDeviation(Descriptive.sampleVariance(simpleDColArray, meanSDCol))/Math.sqrt(nSDCol);
		double meanSDRowSig = Descriptive.mean(simpleDRowSigArray);
		double meanSDRowSigError = Descriptive.standardDeviation(Descriptive.sampleVariance(simpleDRowSigArray, meanSDRowSig))/Math.sqrt(nSDRowSig);
		double meanSDColSig = Descriptive.mean(simpleDColSigArray);
		double meanSDColSigError = Descriptive.standardDeviation(Descriptive.sampleVariance(simpleDColSigArray, meanSDColSig))/Math.sqrt(nSDColSig);
		
		if (chatter) {
			System.out.println("Summary: meanSDRow="+meanSDRow+" meanSDCol="+meanSDCol+" meanSDRowSig="+meanSDRowSig+" meanSDColSig="+meanSDColSig);
		}
		Object[] row0 = { new Double(pixelSize), new Double(sigmaInArcSec), new Double(readNoise), new Double(sky), new Double(nPhoton), new Double(poisson), 		
				new Double(meanSDRow), new Double(meanSDRowError),
				new Double(meanSDCol), new Double(meanSDColError), 
				new Double(meanSDRowSig), new Double(meanSDRowSigError), 
				new Double(meanSDColSig), new Double(meanSDColSigError)};
		if (makeTable)
			starTable.addRow(row0);
	}

	/**
	 * @param fileName
	 * @param xVariable
	 * @param yVariable
	 * @param variation
	 * @param varValues
	 * @param constant1
	 * @param value1
	 * @param constant2
	 * @param value2
	 * @param constant3
	 * @param value3
	 */
	public void plot(String fileName, String xVariable, String yVariable, String variation, double[]varValues, String constant1, double value1, String constant2, double value2, String constant3, double value3) {			
		ImageRepository ir = new ImageRepository();	
	    LinkedList<Object> list = ir.get(fileName);
	    RowListStarTable rlst = (RowListStarTable) list.get(0);
	    int xIndex = Table.getIndexByName(rlst,xVariable);
	    int yIndex = Table.getIndexByName(rlst, yVariable);
	    int yErrorIndex = Table.getIndexByName(rlst, yVariable+"e");
	    int varIndex = Table.getIndexByName(rlst, variation);
	    int cons1Index = Table.getIndexByName(rlst, constant1);
	    int cons2Index = Table.getIndexByName(rlst, constant2);
	    int cons3Index = Table.getIndexByName(rlst, constant3);
	    double sigma[][] = new double [10][100];
	    XYSeriesCollection xy = new XYSeriesCollection();
	    for (int i=0; i<varValues.length; i++) {
	    	xy.addSeries(new XYSeries(variation+"="+varValues[i]));
	    }
	    String yVar = null;
	    for (int i=0; i< varValues.length; i++) {
	    	int count = 0;
	    	for (long j=0; j<rlst.getRowCount(); j++) {
			  Object[] row = rlst.getRow(j);
			  double x = (Double) row[xIndex];
			  double y = (Double) row[yIndex];
			  yVar = rlst.getColumnInfo(yIndex).getDescription();
			  if ((xVariable.compareTo("sig")==0)&&(yVariable.compareTo("dsr")==0||yVariable.compareTo("dsc")==0)) {
			    	y /= x;
			    	y *= 100;
			    	if (yVariable.compareTo("dsr")==0)
			    		yVar = "delta sigma row (%)";
			    	else 
			    		yVar = "delta sigma column (%)";
			  }
			  else if ((constant1.compareTo("sig")==0)&&(yVariable.compareTo("dsr")==0||yVariable.compareTo("dsc")==0)) {
			    	y /= value1;
			    	y *= 100;
			    	if (yVariable.compareTo("dsr")==0)
			    		yVar = "delta sigma row (%)";
			    	else 
			    		yVar = "delta sigma column (%)";
			  }
			  else if (constant2.compareTo("sig")==0&&(yVariable.compareTo("dsr")==0||yVariable.compareTo("dsc")==0)) {
				  	y /= value2;
				  	y *= 100;
				  	if (yVariable.compareTo("dsr")==0)
			    		yVar = "delta sigma row (%)";
			    	else 
			    		yVar = "delta sigma column (%)";
			  }
			  else if (constant3.compareTo("sig")==0&&(yVariable.compareTo("dsr")==0||yVariable.compareTo("dsc")==0)) {
				  	y /= value3;
				  	y *= 100;
				  	if (yVariable.compareTo("dsr")==0)
			    		yVar = "delta sigma row (%)";
			    	else 
			    		yVar = "delta sigma column (%)";
			  }
			  double varCheck = (Double) row[varIndex];
			  double cons1Check = (Double) row[cons1Index];
			  double cons2Check = (Double) row[cons2Index];
			  double cons3Check = (Double) row[cons3Index];
			  if(varCheck==varValues[i]&&cons1Check==value1&&cons2Check==value2&&cons3Check==value3) {
				  xy.getSeries(i).add(x, y);
				  sigma[i][count] = (Double) row[yErrorIndex];
				  count++;			  
			  }
		    }
	    }
	    String c1 = rlst.getColumnInfo(cons1Index).getDescription();
	    String c2 = rlst.getColumnInfo(cons2Index).getDescription();
	    String c3 = rlst.getColumnInfo(cons3Index).getDescription();
	    String title = String.format("%s=%.2f %s=%.2f %s=%.2f", c1, value1, c2, value2, c3, value3);
	    String xVar = rlst.getColumnInfo(xIndex).getDescription();
	    JFreeChart chart = ChartFactory.createScatterPlot(title, xVar, yVar, xy, PlotOrientation.VERTICAL, true, false, false);
	    XYLineAndShapeRenderer xyItemRenderer = (XYLineAndShapeRenderer) chart.getXYPlot().getRendererForDataset(xy);
	    for (int i=0; i<xy.getSeriesCount(); i++) {
			xyItemRenderer.setSeriesLinesVisible(i, true);
			xyItemRenderer.setSeriesShapesVisible(i, true);
		}
	    String file = String.format("SEX%Svs%Svar%S.%S%.2f.%S%.2f.%S%.2f", xVariable, yVariable, variation, constant1, value1, constant2, value2, constant3, value3);
	    try {
			ChartUtilities.saveChartAsPNG(new File(file+".png"), chart, 700, 600);
		} catch (IOException e) {
			throw new Error(e);
		}
	}

}
