package gov.fnal.eag.sim.demo;

import gov.fnal.eag.shapelets.functions.BasisFunction;
import gov.fnal.eag.shapelets.functions.Decompose;
import gov.fnal.eag.shapelets.functions.ImageMoments;
import gov.fnal.eag.shapelets.functions.ShapeletsTransform;
import gov.fnal.eag.sim.extractor.Extractor;
import gov.fnal.eag.sim.extractor.SourceObject;
import gov.fnal.eag.sim.pixel.ColorTable;
import gov.fnal.eag.sim.pixel.DetectorObjectSurfaceBrightness;
import gov.fnal.eag.sim.pixel.PixeledImage;
import gov.fnal.eag.sim.pixel.PixeledImagePlot;
import gov.fnal.eag.sim.pixel.ScaleTable;
import gov.fnal.eag.sim.repository.image.FitsWriter;

import java.awt.Color;
import java.awt.Font;
import java.io.File;
import java.io.IOException;
import java.text.NumberFormat;
import java.util.ArrayList;
import java.util.Hashtable;

import org.jfree.chart.ChartUtilities;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.annotations.XYTextAnnotation;
import org.jfree.chart.plot.XYPlot;

import com.imsl.math.Complex;

/**
 * demonstrated the shear transformations in shapelet space
 * @author stoughto
 *
 */
public class ShearDemo {

	/**
	 * @param args
	 * @throws IOException 
	 */
	public static void main(String[] args) throws IOException {
		//ShapeletsTransform st = new ShapeletsTransform();
		double row = 100.5;
		double col = 100.5;
		double nPhoton = 100000000;
		double sqrt2PI = Math.sqrt(2.*Math.PI);
		// Create an object
		double [][] resPic = new double[200][200];
		for (int i=0; i<200; i++) {
			for (int j=0; j<200; j++) {
				resPic[i][j] = 0.;
			}
		}
		int size = 200;
		double[][] pix = new double[size][size];
		double i0 = size/2.0;
		double j0 = size/2.0;

		double height = 1000000.;
		double scale = 10.;
		double factor = 1./(scale*sqrt2PI);
		for (int i=0; i<size; i++) {
			double di=i+0.5-i0;
			for (int j=0; j<size; j++) {
				double dj=j+0.5-j0;
				double r = Math.sqrt(di*di+dj*dj);
//				double flux = height-r*height/width;
				double flux = height*factor*Math.exp(-(r*r)/(2.*scale*scale));
				if (flux > 0) pix[i][j] = flux;
			}
		}
		System.out.println(" scale="+scale);
		Decompose d = new Decompose(pix, i0, j0, scale);
		int nMax = 15;
		double[][] eVals = d.getFnn2(nMax);
        double[] vals = BasisFunction.tab2Vector(eVals);
        ImageMoments ia = new ImageMoments(scale,eVals);
        double r2 = ia.getR2();
        double [] ell = ia.getEll();
        System.out.println("orig image r2="+r2+" e1="+ell[0]+ " e2="+ell[1]);

		DetectorObjectSurfaceBrightness dosb = new DetectorObjectSurfaceBrightness(
				row, col, nPhoton, eVals, scale);
		
		PixeledImage pi = dosb.makePixeledImage(400, false);

		String outf = "origImage.fit";
		FitsWriter fwr = new FitsWriter(outf);
		fwr.put(pi);
		fwr.close();
		Extractor ex = new Extractor(pi);
		SourceObject so = ex.getLargestSource();
		double e1 = so.getE1();
		double x2 = so.getX2();
		double y2 = so.getY2();
		double e2 = so.getE2();
		System.out.println(" x2="+x2+" y2="+y2+" e1="+e1+" e2="+e2);
		double gamma1 = 0.1;
		double gamma2 = 0.1;
		//double kappa = 0.0;
//
		long start = System.currentTimeMillis();
		Complex[] pol = ShapeletsTransform.c2p(vals);
		pol = ShapeletsTransform.shear(pol, gamma1, gamma2);
		double[] tVals = ShapeletsTransform.p2c(pol);
		double[][] valsT = BasisFunction.vector2Tab(tVals);
//
		long stop = System.currentTimeMillis();
		double elapsed = (double) stop - (double) start;
		System.out.println("Transformation time msec ="+ elapsed);

	      ImageMoments iaT = new ImageMoments(scale,valsT);
	        double r2T = iaT.getR2();
	        double [] ellT = iaT.getEll();
	        System.out.println("sheared image r2T="+r2T+" e1T="+ellT[0]+ " e2T="+ellT[1]);
		DetectorObjectSurfaceBrightness dosb1 = new DetectorObjectSurfaceBrightness(
				row, col, nPhoton, valsT, scale);
		PixeledImage piT =  dosb1.makePixeledImage(400, false);

		String outfT = "Image-gamma1-1-polar.fit";
		FitsWriter fwrT = new FitsWriter(outfT);
		fwrT.put(piT);
		fwrT.close();
		Extractor exT = new Extractor(piT);
		SourceObject soT = exT.getLargestSource();
		double e1T = soT.getE1();
		double x2T = soT.getX2();
		double y2T = soT.getY2();
		double e2T = soT.getE2();
		System.out.println(" x2T="+x2T+" y2T="+y2T+" e1T="+e1T+" e2T="+e2T);
		

		//
		long startC = System.currentTimeMillis();

		double[][] valsTC = ShapeletsTransform.shear(eVals, gamma1, gamma2);
//
		long stopC = System.currentTimeMillis();
		double elapsedC = (double) stopC - (double) startC;
		System.out.println("Cartesian Transformation time msec ="+ elapsedC);

	      ImageMoments iaTC = new ImageMoments(scale,valsTC);
	        double r2TC = iaTC.getR2();
	        double [] ellTC = iaTC.getEll();
	        System.out.println("Cartesian sheared image r2T="+r2TC+" e1T="+ellTC[0]+ " e2T="+ellTC[1]);
		DetectorObjectSurfaceBrightness dosb1C = new DetectorObjectSurfaceBrightness(
				row, col, nPhoton, valsTC, scale);
		PixeledImage piTC =  dosb1C.makePixeledImage(400, false);

		String outfTC = "Image-gamma1-1-cart.fit";
		FitsWriter fwrTC = new FitsWriter(outfTC);
		fwrTC.put(piTC);
		fwrTC.close();
		Extractor exTC = new Extractor(piTC);
		SourceObject soTC = exTC.getLargestSource();
		double e1TC = soTC.getE1();
		double x2TC = soTC.getX2();
		double y2TC = soTC.getY2();
		double e2TC = soTC.getE2();
		System.out.println(" x2T="+x2TC+" y2T="+y2TC+" e1T="+e1TC+" e2T="+e2TC);		
		
		String title = "Shear with gamma1=0.02";
		String outName = "shear1";
		try {
			makePlot(title,pi, piT,e1,e2,x2,y2,e1T,
					e2T,x2T,y2T, outName);
		} catch (IOException e ) {
			e.printStackTrace();
		}
	}
	private static void makePlot(String title, PixeledImage orig,
			PixeledImage trans, double e1, double e2, double x2, double y2,
			double te1, double te2, double tx2, double ty2, String outName)
			throws IOException {
		double ObSize = Math.sqrt(x2 + y2);
		double TObSize = Math.sqrt(tx2 + ty2);
		String text = "";
		int size = 400;
		// make formats for annotation
		NumberFormat defForm = NumberFormat.getInstance();
		// Remember some annotations for the png file
		ArrayList<XYTextAnnotation> annotations = new ArrayList<XYTextAnnotation>();
		Font boldFont = new Font("Helvetica", Font.BOLD, 16);
		Font aFont = new Font("Helvetica", Font.PLAIN, 14);
		double[][] resPic = new double[400][800];
		for (int i = 0; i < size; i++) {
			for (int j = 0; j < 2 * size; j++) {
				resPic[i][j] = 0.;
			}
		}
		for (int i = 0; i < size; i++) {
			for (int j = 0; j < size; j++) {
				resPic[i][j] = orig.getByRowCol(i, j);
			}
		}
		int xshift = size;
		//int yshift = 0;
		for (int i = 0; i < size; i++) {
			for (int j = 0; j < size; j++) {
				resPic[i][j + xshift] = trans.getByRowCol(i, j);
			}
		}
		PixeledImage resIm = new PixeledImage(resPic);
		Hashtable<String, Double> extrema = resIm.extrema();
		double xMin = extrema.get("min");
		double xMax = extrema.get("max");
		// XYTextAnnotation ann = new XYTextAnnotation(text, x, y);
		int annWidth = 20;
		text = formatText(title, annWidth);
		XYTextAnnotation ann = new XYTextAnnotation(text, 400, 380);
		ann.setPaint(Color.YELLOW);
		ann.setFont(boldFont);
		annotations.add(ann);
		text = formatText("Original Image", annWidth);
		ann = new XYTextAnnotation(text, 150, 360);
		ann.setPaint(Color.YELLOW);
		ann.setFont(boldFont);
		annotations.add(ann);
		text = formatText(" e1=" + defForm.format(e1)+" e2="+defForm.format(e2), annWidth);
		ann = new XYTextAnnotation(text, 150, 340);
		ann.setFont(aFont);
		ann.setPaint(Color.YELLOW);
		annotations.add(ann);

		text = formatText(" size=" + defForm.format(ObSize), annWidth);
		ann = new XYTextAnnotation(text, 150, 320);
		ann.setFont(aFont);
		ann.setPaint(Color.YELLOW);
		annotations.add(ann);
		//
		text = formatText(" Transformed image", annWidth);
		ann = new XYTextAnnotation(text, 650, 360);
		ann.setPaint(Color.YELLOW);
		ann.setFont(boldFont);
		annotations.add(ann);
		text = formatText(" e1=" + defForm.format(te1)+" e2="+defForm.format(te2), annWidth);
		ann = new XYTextAnnotation(text, 650, 340);
		ann.setFont(aFont);
		ann.setPaint(Color.YELLOW);
		annotations.add(ann);
		text = formatText(" size=" + defForm.format(TObSize), annWidth);
		ann = new XYTextAnnotation(text, 650, 320);
		ann.setFont(aFont);
		ann.setPaint(Color.YELLOW);
		annotations.add(ann);

		ScaleTable st = new ScaleTable(xMin, xMax);
		ColorTable ct = new ColorTable("gray");
		PixeledImagePlot pip = new PixeledImagePlot(resIm, st, ct);
		JFreeChart chart = pip.getChart();
		XYPlot xyPlot = chart.getXYPlot();
		for (XYTextAnnotation ann1 : annotations) {
			xyPlot.addAnnotation(ann1);
		}
		ChartUtilities.saveChartAsPNG(new File(outName + ".png"), chart, 1200,
				600);

	}
	private static String formatText(String string, int annWidth) {
		String output = string;
		if (output.length() < annWidth) {
			int extention = annWidth - output.length();
			for (int i = 0; i < extention; i++) {
				output = output + " ";
			}
		}
		return output;
	}
}
