package gov.fnal.eag.sim.demo;

import gov.fnal.eag.shapelets.functions.BasisFunction;
import gov.fnal.eag.shapelets.functions.ImageMoments;
import gov.fnal.eag.shapelets.functions.ShapeletsTransform;
import gov.fnal.eag.sim.catalog.MorphologyGeneratorShapeletCatalog;
import gov.fnal.eag.sim.catalog.MorphologyShapelet;
import gov.fnal.eag.sim.pixel.ColorTable;
import gov.fnal.eag.sim.pixel.DetectorObjectSurfaceBrightness;
import gov.fnal.eag.sim.pixel.PixeledImage;
import gov.fnal.eag.sim.pixel.PixeledImagePlot;
import gov.fnal.eag.sim.pixel.ScaleTable;

import java.awt.Color;
import java.awt.Font;
import java.io.File;
import java.io.IOException;
import java.text.NumberFormat;
import java.util.ArrayList;
import java.util.Hashtable;

import org.jfree.chart.ChartUtilities;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.annotations.XYTextAnnotation;
import org.jfree.chart.plot.XYPlot;

/**
 * The demonstration program to demonstrate 
 * image operations (rotation, convergence, shear)
 * @author kuropat
 *
 */
public class TransformDemo {
	/**
	 * This is an example of rotation, compression - delution and shear
	 * transformations of a shapelet object
	 * 
	 * @param args
	 * @throws Exception
	 */
	public static void main(String[] args) throws Exception {

		MorphologyGeneratorShapeletCatalog ssg = new MorphologyGeneratorShapeletCatalog("shapecatS.fits",
				false);
		double row = 200.5;
		double col = 200.5;
		double nPhoton = 10000000;
		double scale = 20;
		//double background = 10.;
		//double readNoise = 5.;
//		double size = 400.;
		ssg.setRotationRange(0, 0);
		ssg.calcVariationLimits(0, 0);
		ssg.setCatRange(0, 0);

		// select first object from the catalog
		MorphologyShapelet sh = ssg.next();
		double[] vals = sh.getEigenvalues();
		double[][] eVals = BasisFunction.vector2Tab(vals); // original matrix
		// of eigenvalues	
		
//		double[] rotated = ShapeletsTransform.rotate(vals, 90);

		ImageMoments im = new ImageMoments(scale, eVals);
		double r2 = im.getR2();
		double e1 = im.getEll()[0];
		double e2 = im.getEll()[1];
		double x2 = im.getXc();
		double y2 = im.getYc();
//		double[][] rVals = BasisFunction.vector2Tab(rotated); // rotated
																// matrix
		DetectorObjectSurfaceBrightness dosb1 = new DetectorObjectSurfaceBrightness(
				row, col, nPhoton, eVals, scale);
/*		
		DetectorObjectSurfaceBrightness dosb2 = new DetectorObjectSurfaceBrightness(
				row, col, nPhoton, rVals, scale);

		PixeledImage pi = dosb1.makePixeledImage(400, false);
		PixeledImage rotatedPi = dosb2.makePixeledImage(400, false);
		pi.addScalar(background);
		rotatedPi.addScalar(background);
		SimRandomPoisson srp = new SimRandomPoisson();
		pi.poisson(srp);
		rotatedPi.poisson(srp);
		SimRandomNormal srn = new SimRandomNormal(0, 0, readNoise, null, false);
		pi.addRandomNumber(srn);
		rotatedPi.addRandomNumber(srn);

		Extractor ex = new Extractor(pi);
		Extractor rotatedEx = new Extractor(rotatedPi);
		SourceObject so = ex.getLargestSource();
		SourceObject rotatedSo = rotatedEx.getLargestSource();
		double e1 = so.getE1();
		double x2 = so.getX2();
		double y2 = so.getX2();
		double e2 = so.getE2();
		double rotatedE1 = rotatedSo.getE1();
		double rotatedX2 = rotatedSo.getX2();
		double rotatedE2 = rotatedSo.getE2();
		double rotatedY2 = rotatedSo.getY2();

		System.out.println("e1=" + e1 + " e2=" + e2 + " rotatedE1=" + rotatedE1
				+ " rotatedE2=" + rotatedE2);
		String title = "Rotation by 90 deg.";
		String outName = "rotated";
		try {
			makePlot(title,pi, rotatedPi,e1,e2,x2,y2,rotatedE1,
					rotatedE2,rotatedX2,rotatedY2, outName);
		} catch (IOException e ) {
			e.printStackTrace();
		}
	*/	
		// delution
		PixeledImage pi = dosb1.makePixeledImage(400, false);
		vals = sh.getEigenvalues();

//		Complex[] pol = ShapeletsTransform.c2p(vals);
		double epsilon = -0.1;
//		Complex[] transf = ShapeletsTransform.delate(pol, epsilon);
//		double[] tVect = ShapeletsTransform.p2c(transf);
		double[][] tVals = ShapeletsTransform.dilate(eVals, epsilon);



		DetectorObjectSurfaceBrightness dosbT = new DetectorObjectSurfaceBrightness(
				row, col, nPhoton, tVals, scale);
		ImageMoments imT = new ImageMoments(scale, tVals);
		double r2T = imT.getR2();
		double transfE1 = imT.getEll()[0];
		double transfE2 = imT.getEll()[1];
		double transfX2 = imT.getXc();
		double transfY2 = imT.getYc();
		System.out.println("Compression orig image r2="+r2+" new image r2="+r2T);
		PixeledImage transfPi = dosbT.makePixeledImage(400, false);
//		transfPi.addScalar(background);
//		transfPi.poisson(srp);
//		transfPi.addRandomNumber(srn);
//		Extractor transfEx = new Extractor(transfPi);
//		SourceObject transfSo = transfEx.getLargestSource();
//		double transfE1 = transfSo.getE1();
//		double transfX2 = transfSo.getX2();
//		double transfE2 = transfSo.getE2();
//		double transfY2 = transfSo.getY2();
		
		String title = "Convergence with k=-0.1";
		String outName = "compressed";
		makePlot(title, pi, transfPi,e1,e2,x2,y2,r2,transfE1,transfE2,transfX2,transfY2,r2T,outName);
		// compression

		epsilon = 0.1;
		vals = sh.getEigenvalues();

		pi = dosb1.makePixeledImage(400, false);

		tVals = ShapeletsTransform.dilate(eVals, epsilon);

		dosbT = new DetectorObjectSurfaceBrightness(
				row, col, nPhoton, tVals, scale);
		ImageMoments imT1 = new ImageMoments(scale, tVals);
		double r2T1 = imT1.getR2();
		transfE1 = imT.getEll()[0];
		transfE2 = imT.getEll()[1];
		transfX2 = imT.getXc();
		transfY2 = imT.getYc();
		System.out.println("dilation orig image r2="+r2+" new image r2="+r2T1);
		transfPi = dosbT.makePixeledImage(400, false);
//		transfPi.addScalar(background);
//		transfPi.poisson(srp);
//		transfPi.addRandomNumber(srn);
//		transfEx = new Extractor(transfPi);
//		transfSo = transfEx.getLargestSource();
//		transfE1 = transfSo.getE1();
//		transfX2 = transfSo.getX2();
//		transfE2 = transfSo.getE2();
//		transfY2 = transfSo.getY2();
		
		title = "Convergence with k=0.1";
		outName = "dilated";
		 {
			makePlot(title,pi, transfPi,e1,e2,x2,y2,r2,transfE1,
					transfE2,transfX2,transfY2, r2T1, outName);
		} 
		
	/*	
		
		// Shear
		double gamma1 = 0.05;
		double gamma2 = 0.;
		vals = sh.getEigenvalues();
		Complex[] pol = ShapeletsTransform.c2p(vals);
		Complex[] transf = ShapeletsTransform.shear(pol, gamma1, gamma2);
		tVect = ShapeletsTransform.p2c(transf);


		dosbT = new DetectorObjectSurfaceBrightness(
				row, col, nPhoton, tVect, scale);
		transfPi = dosbT.makePixeledImage(400, false);
		transfPi.addScalar(background);
		transfPi.poisson(srp);
		transfPi.addRandomNumber(srn);
		transfEx = new Extractor(transfPi);
		transfSo = transfEx.getLargestSource();
		transfE1 = transfSo.getE1();
		transfX2 = transfSo.getX2();
		transfE2 = transfSo.getE2();
		transfY2 = transfSo.getY2();
		
		title = "Shear with gamma1=0.05";
		outName = "shear1";
		try {
			makePlot(title,pi, transfPi,e1,e2,x2,y2,transfE1,
					transfE2,transfX2,transfY2, outName);
		} catch (IOException e ) {
			e.printStackTrace();
		}
		// Shear
		gamma1 = 0.;
		gamma2 = 0.05;
		vals = sh.getEigenvalues();
		pol = ShapeletsTransform.c2p(vals);
		transf = ShapeletsTransform.shear(pol, gamma1, gamma2);
		tVect = ShapeletsTransform.p2c(transf);


		dosbT = new DetectorObjectSurfaceBrightness(
				row, col, nPhoton, tVect, scale);
		transfPi = dosbT.makePixeledImage(400, false);
		transfPi.addScalar(background);
		transfPi.poisson(srp);
		transfPi.addRandomNumber(srn);
		transfEx = new Extractor(transfPi);
		transfSo = transfEx.getLargestSource();
		transfE1 = transfSo.getE1();
		transfX2 = transfSo.getX2();
		transfE2 = transfSo.getE2();
		transfY2 = transfSo.getY2();
		
		title = "Shear with gamma2=0.05";
		outName = "shear2";
		try {
			makePlot(title,pi, transfPi,e1,e2,x2,y2,transfE1,
					transfE2,transfX2,transfY2, outName);
		} catch (IOException e ) {
			e.printStackTrace();
		}
*/
	}

	private static void makePlot(String title, PixeledImage orig,
			PixeledImage trans, double e1, double e2, double x2, double y2, double r2,
			double te1, double te2, double tx2, double ty2, double tr2, String outName)
			throws IOException {
		double ObSize = r2;
		double TObSize = tr2;
		String text = "";
		int size = 400;
		// make formats for annotation
		NumberFormat defForm = NumberFormat.getInstance();
		// Remember some annotations for the png file
		ArrayList<XYTextAnnotation> annotations = new ArrayList<XYTextAnnotation>();
		Font boldFont = new Font("Helvetica", Font.BOLD, 16);
		Font aFont = new Font("Helvetica", Font.PLAIN, 14);
		double[][] resPic = new double[400][800];
		for (int i = 0; i < size; i++) {
			for (int j = 0; j < 2 * size; j++) {
				resPic[i][j] = 0.;
			}
		}
		for (int i = 0; i < size; i++) {
			for (int j = 0; j < size; j++) {
				resPic[i][j] = orig.getByRowCol(i, j);
			}
		}
		int xshift = size;
		//int yshift = 0;
		for (int i = 0; i < size; i++) {
			for (int j = 0; j < size; j++) {
				resPic[i][j + xshift] = trans.getByRowCol(i, j);
			}
		}
		PixeledImage resIm = new PixeledImage(resPic);
		Hashtable<String,Double> extrema = resIm.extrema();
		double xMin = extrema.get("min");
		double xMax = extrema.get("max");
		// XYTextAnnotation ann = new XYTextAnnotation(text, x, y);
		int annWidth = 20;
		text = formatText(title, annWidth);
		XYTextAnnotation ann = new XYTextAnnotation(text, 400, 380);
		ann.setPaint(Color.YELLOW);
		ann.setFont(boldFont);
		annotations.add(ann);
		text = formatText("Original Image", annWidth);
		ann = new XYTextAnnotation(text, 150, 360);
		ann.setPaint(Color.YELLOW);
		ann.setFont(boldFont);
		annotations.add(ann);
		text = formatText(" e1=" + defForm.format(e1)+" e2="+defForm.format(e2), annWidth);
		ann = new XYTextAnnotation(text, 150, 340);
		ann.setFont(aFont);
		ann.setPaint(Color.YELLOW);
		annotations.add(ann);

		text = formatText(" size=" + defForm.format(ObSize), annWidth);
		ann = new XYTextAnnotation(text, 150, 320);
		ann.setFont(aFont);
		ann.setPaint(Color.YELLOW);
		annotations.add(ann);
		//
		text = formatText(" Transformed image", annWidth);
		ann = new XYTextAnnotation(text, 650, 360);
		ann.setPaint(Color.YELLOW);
		ann.setFont(boldFont);
		annotations.add(ann);
		text = formatText(" e1=" + defForm.format(te1)+" e2="+defForm.format(te2), annWidth);
		ann = new XYTextAnnotation(text, 650, 340);
		ann.setFont(aFont);
		ann.setPaint(Color.YELLOW);
		annotations.add(ann);
		text = formatText(" size=" + defForm.format(TObSize), annWidth);
		ann = new XYTextAnnotation(text, 650, 320);
		ann.setFont(aFont);
		ann.setPaint(Color.YELLOW);
		annotations.add(ann);

		ScaleTable st = new ScaleTable(xMin, xMax);
		ColorTable ct = new ColorTable("gray");
		PixeledImagePlot pip = new PixeledImagePlot(resIm, st, ct);
		JFreeChart chart = pip.getChart();
		XYPlot xyPlot = chart.getXYPlot();
		for (XYTextAnnotation ann1 : annotations) {
			xyPlot.addAnnotation(ann1);
		}
		ChartUtilities.saveChartAsPNG(new File(outName + ".png"), chart, 1200,
				600);

	}

	private static String formatText(String string, int annWidth) {
		String output = string;
		if (output.length() < annWidth) {
			int extention = annWidth - output.length();
			for (int i = 0; i < extention; i++) {
				output = output + " ";
			}
		}
		return output;
	}
}
