/**
 * 
 */
package gov.fnal.eag.sim.extractor;

import cern.colt.list.DoubleArrayList;
import cern.jet.stat.Descriptive;
import gov.fnal.eag.sim.pixel.PixeledImage;
import gov.fnal.eag.util.densityEstimate.Histogram;

/**
 * handles background calculation for a mesh (sub-PixeledImage) of pixels
 * @author stoughto
 *
 */

public class Backmesh {
	/**
	 * Set boolean to false to do calculations in floating point precision: otherwise calculations done in double precision
	 */
	public static boolean doDouble = true;
	Histogram hg;
	double mean, sigma, lcut, hcut, med;
	int npix;
	boolean isGood;
	float qscale, qzero;

	// magic copied from SExtractor back.c
	static final int QUANTIF_NSIGMA = 5;
	static final int QUANTIF_AMIN = 4;
	static final int QUANTIF_NMAXLEVELS = 4096;
	static final double step = Math.sqrt(2.0/Math.PI)*QUANTIF_NSIGMA/QUANTIF_AMIN;
	static final double EPS = 1e-4;

	/**
	 * constructor with an image but no weight 
	 * @param image
	 */
	public Backmesh(PixeledImage image) {
		if (image.getColumnCount()>20) {
			PixeledImage subImage = new PixeledImage(image, 20, 20, 0, 0);
			init(subImage, null);
		}
		else {
		init(image, null);
	}
	}

	/**
	 * constructor with image and weight
	 * @param image
	 * @param weight
	 */
	public Backmesh(PixeledImage image, PixeledImage weight) {
		// TODO Make this better
		if (image.getColumnCount()>20) {
			PixeledImage subImage0 = new PixeledImage(image, 20, 20, 0, 0);
			PixeledImage subImage1 = new PixeledImage(image, 20, 20, 20, 0);
			PixeledImage subImage2 = new PixeledImage(image, 20, 20, 0, 20);
			PixeledImage subImage3 = new PixeledImage(image, 20, 20, 20, 20);
			
			init(subImage0, null);
		double mean0 = mean;
			init(subImage1, null);
		double mean1 = mean;
			init(subImage2, null);
		double mean2 = mean;
			init(subImage3, null);
		double mean3 = mean;
		System.out.println("mean0="+mean0+" mean1="+mean1+" mean2="+mean2+" mean3="+mean3);
		
		DoubleArrayList values = new DoubleArrayList();
		values.add(mean0);
		values.add(mean1);
		values.add(mean2);
		values.add(mean3);
		values.sort();
		med = Descriptive.median(values);
		System.out.println("BACKGROUND="+ med);
		
		
		
		}
		else {
		init(image, null);
	}
	}
	

	/**
	 * initialize mean, sigma, and a histogram, based on the logic in subroutine backstat, backhisto, and backGuess
	 */
	private double init(PixeledImage image, PixeledImage weight) {
		isGood = true;
		mean = 0.0;
		sigma = 0.0;
		double weightSum = 0.0;
		for (int row=image.getRowMin(); row<image.getRowMax(); row++) {
			for (int col=image.getColumnMin(); col<image.getColumnMax(); col++) {
				double imageValue = image.getByRowCol(row,col);
				double weightValue = 1.0;
				if (weight != null) {
					weightValue=weight.getByRowCol(row, col);
					imageValue*=weightValue;
					
					
				
				}
				mean += imageValue;
				sigma += imageValue*imageValue;
				weightSum += weightValue; 
			}
		}
		npix = image.getRowCount()*image.getColumnCount();
		mean /= weightSum;
		sigma = Math.sqrt(sigma/weightSum - mean*mean);
		//System.out.println("mean"+mean+"sigma"+sigma);
		lcut = mean - 2.0*sigma;
		hcut = mean + 2.0*sigma;
		mean = 0;
		sigma = 0;
		npix = 0;
		for (int row=image.getRowMin(); row<image.getRowMax(); row++) {
			for (int col=image.getColumnMin(); col<image.getColumnMax(); col++) {
				double value = image.getByRowCol(row,col);
				if (value >= lcut && value <= hcut) {
					mean += value;
					sigma += value*value;
					npix++;
				}
			}
		}
		mean /= npix;
		sigma = Math.sqrt(sigma/npix - mean*mean);		

		int nLevels = (int) (step*npix+1);
		nLevels = Math.min(QUANTIF_NMAXLEVELS, nLevels);
		if (sigma > 0.0) {
			qscale = (float) (2 * QUANTIF_NSIGMA*sigma/nLevels);
		} else {
			qscale = (float) 1.0;
		}
		qzero = (float) (mean - QUANTIF_NSIGMA*sigma);
		
		//System.out.println("mean="+mean+" QUANTIF_NSIGMA="+QUANTIF_NSIGMA+" sigma="+sigma+" qzero="+String.format("%30.10f", qzero));
		double hgMin = qzero - qscale/2.0;
		double hgMax = hgMin + nLevels*qscale;
	//	System.out.println("hgMax="+hgMax+" nLevels="+nLevels);
		hg = image.getHistogram(hgMin, hgMax, nLevels);
		
		
		
		 
			hcut = hg.getNBin(); 
			lcut = 0;
			double sig = 10*(hg.getNBin()-1);
			double sig1 = 1.0;
			double mea = mean;
			double med = mean;
			int iBinLowCut = 0;
			int iBinHighCut = hg.getNBin()-1;
			for (int n=99; (sig>=0.1) && Math.abs(sig/sig1-1.0)>EPS && n >= 0; n--){
				sig1 = sig;
				long sum = 0;
				sig = 0.0;
				mea = 0.0;
				long lowsum = 0;
				long highsum = 0;
				int iBinLowIndex = iBinLowCut;
				int iBinHighIndex = iBinHighCut;
				for (int iBin=iBinLowCut; iBin<=iBinHighCut; iBin++) {
					if (lowsum < highsum) {
						double contents = hg.getContents(iBinLowIndex++);
						lowsum += contents;
					} else {
						double contents = hg.getContents(iBinHighIndex--);
						highsum += contents;

					}
					double contents = hg.getContents(iBin);
					sum += contents;
					mea += contents*iBin;
					sig += contents*iBin*iBin;
				}
				double temp1;
				if (iBinHighIndex >= 0.0) {
					if (hg.getContents(iBinLowIndex) > hg.getContents(iBinHighIndex)) {
						temp1 = hg.getContents(iBinLowIndex);
					} else {

						temp1 = hg.getContents(iBinHighIndex);

					}
					// TODO Why is this?
					med = iBinHighIndex + 0.5 + (highsum-lowsum)/(2.0*temp1);
					
					//System.out.println("med "+med+" iBinHighIndex "+iBinHighIndex+" temp1 "+temp1+" highsum "+highsum+" lowsum "+lowsum);
				} else {
					med = 0.0;
				}

				mea /= sum;
				sig = sig/sum - mea*mea;
				if (sig > 0) {
					sig = Math.sqrt(sig);
				} else {
					sig = 0.0;
				}
				{
					double ftemp = med-3.0*sig;
					if (ftemp > 0) {
						iBinLowCut = (int) (ftemp+0.5);
					} else {
						iBinLowCut = 0;
					}
				}
				{
					double ftemp = med+3.0*sig;
					//System.out.println("med="+med+" sig="+sig);
					//System.out.println("ftemp= "+ftemp+" NBin<< subt 1 ="+hg.getNBin());
					if (ftemp < hg.getNBin()-1) {
						if (ftemp > 0) {
							iBinHighCut = (int) (ftemp+0.5);
						} else {
							iBinHighCut = (int) (ftemp-0.5);
						}
					} else {
						iBinHighCut = hg.getNBin()-1;
					}
				}
			}
			double temp1 = Math.abs(sig);
			// note that these were already defined
			//double qscale = hg.getBinSize();
			//double qzero = hg.getMin() + qscale/2.0;
			double temp3 = qzero+mea*qscale;
			double temp4 = Math.abs((mea-med)/sig);
			double temp5 = qzero+(2.5*med-1.5*mea)*qscale;
			double temp6 = qzero + med*qscale;
			//System.out.println("qzero= "+qzero+" med= "+med+" qscale= "+qscale);
			if (temp1>0.0) {
				if (temp4 < 0.3) {
					mean = temp5; 
				

				} else {
					mean = temp6;
					

				}
			}

			else {
				mean = temp3;
			}	
			
		return mean;
	}

	/**
	 * @return the histogram
	 */
	public Histogram getHg() {
		return hg;
	}

	/**
	 * performs the logic of the backguess subroutine, using the Histogram hg and mean that are initialized in the constructor.  
	 * The mean value is overwritten.
	 * 
	 */
	/*
	public void backGuess() {
		hcut = hg.getNBin(); 
		lcut = 0;
		double sig = 10*(hg.getNBin()-1);
		double sig1 = 1.0;
		double mea = mean;
		double med = mean;
		int iBinLowCut = 0;
		int iBinHighCut = hg.getNBin()-1;
		for (int n=99; (sig>=0.1) && Math.abs(sig/sig1-1.0)>EPS && n >= 0; n--){
			sig1 = sig;
			long sum = 0;
			sig = 0.0;
			mea = 0.0;
			long lowsum = 0;
			long highsum = 0;
			int iBinLowIndex = iBinLowCut;
			int iBinHighIndex = iBinHighCut;
			for (int iBin=iBinLowCut; iBin<=iBinHighCut; iBin++) {
				if (lowsum < highsum) {
					double contents = hg.getContents(iBinLowIndex++);
					lowsum += contents;
				} else {
					double contents = hg.getContents(iBinHighIndex--);
					highsum += contents;

				}
				double contents = hg.getContents(iBin);
				sum += contents;
				mea += contents*iBin;
				sig += contents*iBin*iBin;
			}
			double temp1;
			if (iBinHighIndex >= 0.0) {
				if (hg.getContents(iBinLowIndex) > hg.getContents(iBinHighIndex)) {
					temp1 = hg.getContents(iBinLowIndex);
				} else {

					temp1 = hg.getContents(iBinHighIndex);

				}
				// TODO Why is this?
				med = iBinHighIndex + 0.5 + (highsum-lowsum)/(2.0*temp1);
				
				//System.out.println("med "+med+" iBinHighIndex "+iBinHighIndex+" temp1 "+temp1+" highsum "+highsum+" lowsum "+lowsum);
			} else {
				med = 0.0;
			}

			mea /= sum;
			sig = sig/sum - mea*mea;
			if (sig > 0) {
				sig = Math.sqrt(sig);
			} else {
				sig = 0.0;
			}
			{
				double ftemp = med-3.0*sig;
				if (ftemp > 0) {
					iBinLowCut = (int) (ftemp+0.5);
				} else {
					iBinLowCut = 0;
				}
			}
			{
				double ftemp = med+3.0*sig;
				//System.out.println("med="+med+" sig="+sig);
				//System.out.println("ftemp= "+ftemp+" NBin<< subt 1 ="+hg.getNBin());
				if (ftemp < hg.getNBin()-1) {
					if (ftemp > 0) {
						iBinHighCut = (int) (ftemp+0.5);
					} else {
						iBinHighCut = (int) (ftemp-0.5);
					}
				} else {
					iBinHighCut = hg.getNBin()-1;
				}
			}
		}
		double temp1 = Math.abs(sig);
		// note that these were already defined
		//double qscale = hg.getBinSize();
		//double qzero = hg.getMin() + qscale/2.0;
		double temp3 = qzero+mea*qscale;
		double temp4 = Math.abs((mea-med)/sig);
		double temp5 = qzero+(2.5*med-1.5*mea)*qscale;
		double temp6 = qzero + med*qscale;
		//System.out.println("qzero= "+qzero+" med= "+med+" qscale= "+qscale);
		if (temp1>0.0) {
			if (temp4 < 0.3) {
				mean = temp5; 
				System.out.println("mean"+mean);

			} else {
				mean = temp6;
				System.out.println("mean"+mean);

			}
		}

		else {
			mean = temp3;
		}	
		System.out.println("mean"+mean);
	}

	/**
	 * @return the mean value of the pixel values
	 */
	public double getBackground() {
		return med;
	}

	/**
	 * @return the sigma of the mean value of the pixel values
	 */
	public double getSigma() {
		return sigma;
	}

	/**
	 * If all data is identical, sigma is automatically set to 1.0
	 */
	public void filterBack() {
		if (sigma == 0) {
			sigma = 1.0;
			//System.out.println("sigma"+sigma);
		}

	}

}
