package gov.fnal.eag.sim.extractor;

import flanagan.math.MinimisationFunction;
import gov.fnal.eag.sim.pixel.DetectorObjectProfile;
import gov.fnal.eag.sim.pixel.PixeledImage;
import gov.fnal.eag.sim.pixel.Profiles;
import gov.fnal.eag.sim.repository.image.ImageRepository;

/**
 * @author dcharms
 *
 */
public class EllipseMinFunction implements MinimisationFunction{

	private PixeledImage win;
	
	private double x2M;
	
	private double y2M;
	
	private double xyM;
	
	private double row;
	
	private double col;
	
	/**
	 * set to true to print debugging information
	 */
	public static boolean debug = false;
	
	/**
	 * @param row 
	 * @param col 
	 * @param eW 
	 * @param sW 
	 * @param tW 
	 * @param x2M 
	 * @param y2M 
	 * @param xyM 
	 * 
	 */
	public EllipseMinFunction(double row, double col, double eW, double sW, double tW, double x2M, double y2M, double xyM) {
		DetectorObjectProfile dop = new DetectorObjectProfile(row, col, 1, "guass", sW, eW, tW, new Profiles());
		win = dop.makePixeledImage((int)(sW*30));
		if (debug) {
			//System.out.println(" row="+row+" col="+col+" eW="+eW+" sW="+sW+" tW="+tW);
			ImageRepository.write("EllipseMinFunctionWin.fit", win);
			//System.out.println(" row="+row+" col="+col+" row0="+win.getRow0()+" col0="+win.getCol0());
			//SourceObject temp = new SourceObject(win);
			//System.out.println("temp x="+temp.getX()+" ellip="+temp.getEllipticity()+" a="+Math.sqrt(temp.getA())+" theta="+temp.getTheta());
		}
		this.row = row;
		this.col = col;
		this.x2M = x2M;
		this.y2M = y2M;
		this.xyM = xyM;
	}
	
	/**
	 * @see flanagan.math.MinimisationFunction#function(double[])
	 */
	public double function(double[] param) {
		double x2 = param[0];
		double y2 = param[1];
		double xy = param[2];
		double a = Math.sqrt((x2+y2)/2+Math.sqrt(Math.pow((x2-y2)/2, 2)+Math.pow(xy, 2)));
		double b = Math.sqrt((x2+y2)/2-Math.sqrt(Math.pow((x2-y2)/2, 2)+Math.pow(xy, 2)));
		double eT = 1-b/a;
		double sT = a;
		double tT =0.5*Math.atan2(2*xy, (x2-y2))*180/Math.PI;
		//System.out.println("x2="+param[0]+" y2="+param[1]+" xy="+param[2]);
		//System.out.println("e="+eT+" s="+sT+" t="+tT);
		DetectorObjectProfile dop = new DetectorObjectProfile(row, col, 1, "gauss", sT, eT, tT, new Profiles());
		PixeledImage truePi = dop.makePixeledImage(win.getRowCount());
		truePi.multiply(win);
		SourceObject so = new SourceObject(truePi);
		x2 = so.getX2();
		y2 = so.getY2();
		xy = so.getXY();
		
		double deltaX2 = x2M-x2;
		double deltaY2 = y2M-y2;
		double deltaXY = xyM-xy;
		return Math.pow(deltaX2, 2)+Math.pow(deltaY2, 2)+Math.pow(deltaXY, 2);
	}

}
