package gov.fnal.eag.sim.extractor;

import java.awt.geom.Point2D;

import gov.fnal.eag.sim.pixel.DetectorObject;
import gov.fnal.eag.sim.pixel.DetectorObjectProfile;
import gov.fnal.eag.sim.pixel.DetectorObjectSurfaceBrightness;
import gov.fnal.eag.util.densityEstimate.FitFunction;

/**
 * describes profile through a DetectorObject
 * @author stoughto
 *
 */
public class FitFunctionDetectorObject extends FitFunction {

	private DetectorObject detectorObject;
	private Point2D.Double rowCol0;
	private double cosTheta, sinTheta;
	/**
	 * @param detectorObject object to profile
	 * @param rowCol0 location of 0
	 * @param thetaDegrees angle of profile (0 for x=column; 90 for x=row)
	 */
	public FitFunctionDetectorObject(DetectorObject detectorObject, Point2D.Double rowCol0, double thetaDegrees) {
		this.detectorObject = detectorObject;
		this.rowCol0 = rowCol0;
		cosTheta = Math.cos(Math.toDegrees(thetaDegrees));
		sinTheta = Math.sin(Math.toDegrees(thetaDegrees));
		
		//String name = "";
		if (detectorObject instanceof DetectorObjectProfile) {
			name = "profile "+((DetectorObjectProfile)detectorObject).getShape();
		}
		if (detectorObject instanceof DetectorObjectSurfaceBrightness) {
			name = "shapelet nOrder="+((DetectorObjectSurfaceBrightness)detectorObject).getNOrder();
		}
		setName(name);
	}
	
	/**
	 * @see gov.fnal.eag.util.densityEstimate.FitFunction#valueAt(double)
	 */
	@Override
	public double valueAt(double x) {
		double row = rowCol0.x + x*sinTheta;
		double col = rowCol0.y + x*cosTheta;
		double retval = detectorObject.getFlux(row,col);
		return retval;
	}

}
