package gov.fnal.eag.sim.extractor;

import gov.fnal.eag.shapelets.functions.Decompose;
import gov.fnal.eag.sim.pixel.DetectorObjectSurfaceBrightness;
import gov.fnal.eag.sim.pixel.PixeledImage;

import java.awt.Rectangle;

/**
 * measure the shapelets for an object in a pixeled image
 * @author stoughto
 *
 */
public class MeasureShapelets {
	private PixeledImage pi;
	/**
	 * @param piIn the image to measure
	 */
	public MeasureShapelets(PixeledImage piIn) {
		pi = piIn;
	}
	/**
	 * @param row
	 * @param col
	 * @param scale
	 * @param nMax
	 * @param bounding
	 * @return the object
	 * @throws shapeletException
	 */
	public DetectorObjectSurfaceBrightness getDosb(double row, double col, double scale, int nMax, Rectangle bounding) {
		double nPhoton = 1.0;
		int nRow = bounding.height;
		int nCol = bounding.width;
		{
			double[][] pixo = pi.getPix();
			double vLeft = pixo[nRow/2-1][nCol/2];
			double vMid  = pixo[nRow/2][nCol/2];
			double vRight = pixo[nRow/2+1][nCol/2];
			System.out.println("MeasureShapelets:  vLeft vRight "+vLeft+" "+vMid+" "+vRight);
		}
		double[][] pix = new double[nRow][nCol];
		for (int r=0; r<nRow; r++) {
			for (int c=0; c<nCol; c++) {
				pix[r][c] = pi.getByRowCol(r+row-nRow/2, c+col-nCol/2);
			}
		}
		{
			double vLeft = pix[nRow/2-1][nCol/2];
			double vMid = pix[nRow/2][nCol/2];
			double vRight = pix[nRow/2+1][nCol/2];
			System.out.println("MeasureShapelets:  vLeft vRight "+vLeft+" "+vMid+" "+vRight);
		}
		double x0 = nCol/2.0;
		double y0 = nRow/2.0;
		Decompose d = new Decompose(pix, y0, x0, scale);
		double[][] eigenvalues = d.getFnn2(nMax);
		DetectorObjectSurfaceBrightness dosb = new DetectorObjectSurfaceBrightness(row, col, nPhoton, eigenvalues, scale);
		return dosb;
	}
	
}
