package gov.fnal.eag.sim.extractor;

/**
 * convenience methods for calculating moments
 * @author stoughto
 *
 */
public class Moment {
	
	/**
	 * @param sigma
	 * @param rMax
	 * @return the integral, from zero to rMax, of r*exp(-r*r/(2*sigma*sigma) dr
	 */
	public static double integrateR(double sigma, double rMax) {
		return sigma*sigma*(1.0-Math.exp(-rMax*rMax/2/sigma/sigma));
	}
	/**
	 * @param sigma
	 * @return the integral, from zero to infinity, of r*exp(-r*r/(2*sigma*sigma) dr
	 */
	public static double integrateR(double sigma) {
		return sigma*sigma;
	}
	
	
	/**
	 * @param sigma
	 * @param rMax
	 * @return the integral, from zero to rMax, of r^3*exp(-r*r/(2*sigma*sigma) dr
	 */
	public static double integrateR3(double sigma, double rMax) {
		double s2 = sigma*sigma;
		double r2 = rMax*rMax;
		return s2*(2*s2 - (r2+2*s2)*Math.exp(-r2/2/s2));
	}
	
	/**
	 * @param sigma
	 * @return the integral, from zero to infinity, of r^3*exp(-r*r/(2*sigma*sigma) dr
	 */
	public static double integrateR3(double sigma) {
		double s2 = sigma*sigma;
		return s2*(2*s2);
	}
	
	/**
	 * when sigma is measured inside a finite aperture, the expected value is less than the true value
	 * @param sigma the true sigma
	 * @param rMax radius that includes data used for the measurement
	 * @return the expected sigma measurement
	 */
	public static double sigmaExpected(double sigma, double rMax) {
		double sigmaExpected;
		if (sigma > 0) {
			double numerator = integrateR3(sigma, rMax);
			double denominator = integrateR(sigma, rMax);
			sigmaExpected = Math.sqrt(numerator/2/denominator);
		} else {
			sigmaExpected = 0;
		}
		return sigmaExpected;
	}
	
	/**
	 * @param radius radius of object
	 * @param sigmaWindow value of windowing sigma
	 * @param sigmaTrue the true sigma
	 * @return return the expected sigma windowed
	 */
	public static double sigmaExpectedWindowed(double radius, double sigmaWindow, double sigmaTrue) {
		double sigmaEffective = sigmaTrue*sigmaWindow/Math.sqrt((sigmaTrue*sigmaTrue+sigmaWindow*sigmaWindow));
		return sigmaExpected(sigmaEffective, radius);
	}
}
