package gov.fnal.eag.sim.extractor.test;

import gov.fnal.eag.sim.extractor.Extractor;
import gov.fnal.eag.sim.extractor.SourceObject;
import gov.fnal.eag.sim.pixel.DetectorObjectProfile;
import gov.fnal.eag.sim.pixel.PixeledImage;
import gov.fnal.eag.sim.pixel.Profiles;
import gov.fnal.eag.sim.random.SimRandomPoisson;
import gov.fnal.eag.sim.random.SimRandomUniform;
import gov.fnal.eag.sim.repository.image.ImageRepository;
import gov.fnal.eag.util.densityEstimate.Histogram;
import gov.fnal.eag.util.misc.StringUtil;

import java.io.File;
import java.io.IOException;

import junit.framework.TestCase;

import org.jfree.chart.ChartUtilities;
import org.jfree.chart.JFreeChart;

/**
 * demonstrates how to use the Extractor class
 * @author stoughto
 *
 */
public class ExtractorDemo extends TestCase {
	/**
	 * plot measured-true for positions and sigmas, with and without gaussian weighting
	 * @throws IOException
	 */
	public void testPositionApertureAndWeighted() throws IOException {
		Histogram hDx =    new Histogram("aperture dColumn", "dx", "dn", 40, -0.5, 0.5);
		Histogram hDxWin = new Histogram("weighted dColumn", "dxWin", "dn", 40, -0.5, 0.5);
		Histogram hDy =    new Histogram("aperture dRow", "dy", "dn", 40, -0.5, 0.5);
		Histogram hDyWin = new Histogram("weighted dRow", "dyWin", "dn", 40, -0.5, 0.5);
		double sigma = 5.0;
		Histogram hSigmaX = new Histogram("aperture sigma col", "sigma x", "dn", 40, sigma*0.6, sigma*1.0);
		Histogram hSigmaXWin = new Histogram("weighted sigma col", "sigma x", "dn", 40, sigma*0.6, sigma*1.0);
		Histogram hSigmaY = new Histogram("aperture sigma row", "sigma y", "dn", 40, sigma*0.6, sigma*1.0);
		Histogram hSigmaYWin = new Histogram("weighted sigma row", "sigma y", "dn", 40, sigma*0.6, sigma*1.0);
		Histogram hE1 = new Histogram("aperture e1", "e1", "dn", 40, -0.1, 0.1);
		Histogram hE2 = new Histogram("aperture e2", "e2", "dn", 40, -0.1, 0.1);
		Histogram hE1Win = new Histogram("weighted e1", "e1", "dn", 40, -0.1, 0.1);
		Histogram hE2Win = new Histogram("weighted e2", "e2", "dn", 40, -0.1, 0.1);
		double pos = 100;
		SimRandomUniform sru = new SimRandomUniform(1, pos, pos+1);
		double sky = 10.0;
		int pixelDiameter = (int) (sigma*12);
		double nPhoton = 10000;
		double ellipticity = 0;
		double positionAngle = 0;
		Profiles profiles = new Profiles();
		SimRandomPoisson srp = new SimRandomPoisson(2);
		int nToDo = 100;
		//DetectorObjectProfile dopKernel = new DetectorObjectProfile(0.0, 0.0, 1.0, "gauss", sigma, 0.0, 0.0, profiles);
		//PixeledImage kernel = dopKernel.makePixeledImage(5);
		for (int i=0; i<nToDo; i++) {
			double trueRow = sru.next();
			double trueCol = sru.next();
			DetectorObjectProfile dop = new DetectorObjectProfile(trueRow, trueCol, nPhoton, "gauss", sigma, ellipticity, positionAngle, profiles);
			PixeledImage pi = dop.makePixeledImage(pixelDiameter);
			pi.addScalar(sky);
			pi.poisson(srp);
			Extractor ex = new Extractor(pi);
			//ex.setKernel(kernel);
			ex.setAbsoluteThresh(sky+3*Math.sqrt(sky));
			SourceObject so = ex.getLargestSource();
			hDx.fill(so.getX()-trueCol);
			hDxWin.fill(so.getXWin()-trueCol);
			hDy.fill(so.getY()-trueRow);
			hDyWin.fill(so.getYWin()-trueRow);
			
			double sigmaX = Math.sqrt(so.getX2());
			hSigmaX.fill(sigmaX);		
			double sigmaXWin = Math.sqrt(so.getX2Win());
			hSigmaXWin.fill(sigmaXWin);
			double sigmaY = Math.sqrt(so.getY2());
			hSigmaY.fill(sigmaY);		
			double sigmaYWin = Math.sqrt(so.getY2Win());
			hSigmaYWin.fill(sigmaYWin);
			
			hE1.fill(so.getE1());
			hE2.fill(so.getE2());
			hE1Win.fill(so.getE1Win());
			hE2Win.fill(so.getE2Win());
			
		}
		System.out.println("AAAA");
		{
			Histogram[] colList = {hDxWin, hDx};
			JFreeChart chartCol = Histogram.getChart(colList, "column offsets", "delta column", "dN", Double.NaN, Double.NaN, false);
			ChartUtilities.saveChartAsPNG(new File(StringUtil.getFileName("Column","png")), chartCol, 600, 600);		
		}
		{
			Histogram[] rowList = {hDyWin, hDy};
			JFreeChart chartRow = Histogram.getChart(rowList, "row offsets", "delta row", "dN", Double.NaN, Double.NaN, false);
			ChartUtilities.saveChartAsPNG(new File(StringUtil.getFileName("Row","png")), chartRow, 500, 300);		
		}
		{
			Histogram[] sigmaColList = {hSigmaXWin, hSigmaX};
			JFreeChart chartSigmaX = Histogram.getChart(sigmaColList, "sigma column", "sigma column", "dN", Double.NaN, Double.NaN, false);
			ChartUtilities.saveChartAsPNG(new File(StringUtil.getFileName("SigmaColumn","png")), chartSigmaX, 500, 300);		
		}
		{
			Histogram[] sigmaRowList = {hSigmaYWin, hSigmaY};
			JFreeChart chartSigmaY = Histogram.getChart(sigmaRowList, "sigma row", "sigma row", "dN", Double.NaN, Double.NaN, false);
			ChartUtilities.saveChartAsPNG(new File(StringUtil.getFileName("SigmaRow","png")), chartSigmaY, 500, 300);		
		}
		{
			Histogram[] e1List = {hE1Win, hE1};
			JFreeChart chartE1 = Histogram.getChart(e1List, "e1", "e1", "dN", Double.NaN, Double.NaN, false);
			ChartUtilities.saveChartAsPNG(new File(StringUtil.getFileName("E1","png")), chartE1, 500, 300);		
		}
		{
			Histogram[] e2List = {hE2Win, hE2};
			JFreeChart chartE2 = Histogram.getChart(e2List, "e2", "e2", "dN", Double.NaN, Double.NaN, false);
			ChartUtilities.saveChartAsPNG(new File(StringUtil.getFileName("E2","png")), chartE2, 500, 300);		
		}
		/*
		System.out.println(hDx);
		System.out.println(hDxWin);
		System.out.println(hDy);
		System.out.println(hDyWin);
		System.out.println(hSigmaX);
		System.out.println(hSigmaXWin);
		System.out.println(hSigmaY);
		System.out.println(hSigmaYWin);
		System.out.println(hE1);
		System.out.println(hE2);
		System.out.println(hE1Win);
		System.out.println(hE2Win);		
		*/
	}	
	
	/**
	 * test the ctiColumn method
	 */
	public void testCtiColumn() {
		double trueRow = 100;
		double trueCol = 100;
		double nPhoton = 10000;
		double sky = 10.0;
		double ellipticity = 0;
		double positionAngle = 0;
		Profiles profiles = new Profiles();
		double sigma = 5.0;
		int pixelDiameter = (int) (sigma*12);		
		DetectorObjectProfile dop = new DetectorObjectProfile(trueRow, trueCol, nPhoton, "gauss", sigma, ellipticity, positionAngle, profiles);
		PixeledImage pi = dop.makePixeledImage(pixelDiameter);
		pi.addScalar(sky);
		ImageRepository.write(StringUtil.getFileName("Original","fit"), pi);
		Extractor exOriginal = new Extractor(pi);
		exOriginal.setAbsoluteThresh(sky+3*Math.sqrt(sky));
		
		//SourceObject soOriginal = exOriginal.getLargestSource();
		//System.out.println("original: x="+soOriginal.getXWin()+" y="+soOriginal.getYWin()+" e1="+soOriginal.getE1Win()+" e2="+soOriginal.getE2Win());
		double cti = 0.001;
		pi.doCtiCol(cti);
		ImageRepository.write(StringUtil.getFileName("Cti","fit"), pi);
		Extractor exCti = new Extractor(pi);
		exCti.setAbsoluteThresh(sky+3*Math.sqrt(sky));
		//SourceObject soCti = exCti.getLargestSource();
		//System.out.println("cti: x="+soCti.getXWin()+" y="+soCti.getYWin()+" e1="+soCti.getE1Win()+" e2="+soCti.getE2Win());		
	}
	
	/**
	 * see how e1,e2 measurements change with cti
	 * @throws IOException
	 */
	public void testE1E2Cti() throws IOException {
		double sigma = 5.0;
		Histogram hE1Original = new Histogram("original e1", "e1", "dn", 40, -0.1, 0.1);
		Histogram hE2Original = new Histogram("original e2", "e2", "dn", 40, -0.1, 0.1);
		Histogram hE1Cti = new Histogram("cti e1", "e1", "dn", 40, -0.1, 0.1);
		Histogram hE2Cti = new Histogram("cti e2", "e2", "dn", 40, -0.1, 0.1);
		double pos = 100;
		SimRandomUniform sru = new SimRandomUniform(1, pos, pos+1);
		double sky = 10.0;
		int pixelDiameter = (int) (sigma*12);
		double nPhoton = 10000;
		double ellipticity = 0;
		double positionAngle = 0;
		Profiles profiles = new Profiles();
		SimRandomPoisson srp = new SimRandomPoisson(2);
		int nToDo = 1000;
		for (int i=0; i<nToDo; i++) {
			double trueRow = sru.next();
			double trueCol = sru.next();
			DetectorObjectProfile dop = new DetectorObjectProfile(trueRow, trueCol, nPhoton, "gauss", sigma, ellipticity, positionAngle, profiles);
			PixeledImage pi = dop.makePixeledImage(pixelDiameter);
			pi.addScalar(sky);
			pi.poisson(srp);
			
			{
				Extractor ex = new Extractor(pi);
				ex.setAbsoluteThresh(sky+3*Math.sqrt(sky));
				SourceObject so = ex.getLargestSource();
				hE1Original.fill(so.getE1Win());
				hE2Original.fill(so.getE2Win());
			}
			pi.doCtiCol(0.001);
			{
				Extractor ex = new Extractor(pi);
				ex.setAbsoluteThresh(sky+3*Math.sqrt(sky));
				SourceObject so = ex.getLargestSource();			
				hE1Cti.fill(so.getE1Win());
				hE2Cti.fill(so.getE2Win());
			}
			
			
			
		}
		{
			Histogram[] e1List = {hE1Original, hE1Cti};
			JFreeChart chartCol = Histogram.getChart(e1List, "e1", "e1", "dN", Double.NaN, Double.NaN, false);
			ChartUtilities.saveChartAsPNG(new File(StringUtil.getFileName("E1","png")), chartCol, 500, 300);		
		}
		{
			Histogram[] e2List = {hE2Original, hE2Cti};
			JFreeChart chartCol = Histogram.getChart(e2List, "e2", "e2", "dN", Double.NaN, Double.NaN, false);
			ChartUtilities.saveChartAsPNG(new File(StringUtil.getFileName("E2","png")), chartCol, 500, 300);		
		}
	}	

	
	
	
}

