package gov.fnal.eag.sim.extractor.test;
import gov.fnal.eag.sim.pixel.DetectorObjectProfile;
import gov.fnal.eag.sim.pixel.PixeledImage;
import gov.fnal.eag.sim.pixel.Profiles;
import gov.fnal.eag.sim.random.SimRandomPoisson;
import gov.fnal.eag.sim.random.SimRandomUniform;
import gov.fnal.eag.sim.utils.Sexec;
import gov.fnal.eag.util.densityEstimate.Histogram;
import gov.fnal.eag.util.misc.StringUtil;
import gov.fnal.eag.util.yanny.Table;

import java.io.File;
import java.io.IOException;
import java.util.Hashtable;

import junit.framework.TestCase;

import org.jfree.chart.ChartUtilities;
import org.jfree.chart.JFreeChart;

import uk.ac.starlink.table.RowListStarTable;

/**
 * Demonstrate how to use the SExtractor class
 * @author dcharms
 *
 */
public class SExtractorDemo extends TestCase {
	/**
	 * plot measured-true for positions and sigmas, with and without gaussian weighting
	 * @throws IOException
	 */
	public void testSPositionApertureAndWeighted() throws IOException {
		Histogram hDx =    new Histogram("aperture dColumn", "dx", "dn", 80, -10.0, 10.0);
		Histogram hDxWin = new Histogram("weighted dColumn", "dxWin", "dn", 80, -10.0, 10.0);
		Histogram hDy =    new Histogram("aperture dRow", "dy", "dn", 80, -10.0, 10.0);
		Histogram hDyWin = new Histogram("weighted dRow", "dyWin", "dn", 80, -10.0, 10.0);
		double sigma = 10.0;
		Histogram hSigmaX = new Histogram("aperture sigma col", "sigma x", "dn", 40, 0., sigma*2.0);
		Histogram hSigmaXWin = new Histogram("weighted sigma col", "sigma x", "dn", 40, 0., sigma*2.0);
		Histogram hSigmaY = new Histogram("aperture sigma row", "sigma y", "dn", 40, 0., sigma*2.0);
		Histogram hSigmaYWin = new Histogram("weighted sigma row", "sigma y", "dn", 40, 0., sigma*2.0);

		double pos = 100.5;
		SimRandomUniform sru = new SimRandomUniform(1, pos-0.5, pos+0.5);
		double sky = 200.0;
//		int pixelDiameter = (int) (sigma*12); // pixel diameter = 60 of what?
		int pixelDiameter = (200*1); 
		double nPhoton = 30000;
		double ellipticity = 0;
		double positionAngle = 0;
		Profiles profiles = new Profiles();
		SimRandomPoisson srp = new SimRandomPoisson(2);
		int nToDo = 10;
		
		
		for (int i=0; i<nToDo; i++) {
			Hashtable<String,Number> hashtable = new Hashtable<String,Number>();
			double trueRow = sru.next(); // generate raw,col as random withing a pixel 
			double trueCol = sru.next();
//			System.out.println("trueX="+trueCol+" trueY="+trueRow);
	//		generate detector object profile as gaussian at true pos. and sigma=5.0
			DetectorObjectProfile dop = new DetectorObjectProfile(trueRow, trueCol, nPhoton, "gauss", sigma, ellipticity, positionAngle, profiles);
	// create pixeled image and add sky level and poisson noise
			PixeledImage pi = dop.makePixeledImage(pixelDiameter);
			
			pi.addScalar(sky);
			pi.poisson(srp);
			System.out.println("create Sexec");
			Sexec se = new Sexec();
			se.setSex("FILTER","N");
//			se.setSex("CHECKIMAGE_TYPE","SEGMENTATION");
//			se.setSex("CHECKIMAGE_NAME","(footprint.fits)");
//			System.out.println("create region");
			se.setSegRegion("region.reg");
//			System.out.println("set parameters");
			String[] params = {"X_IMAGE", "Y_IMAGE", "X2_IMAGE", "Y2_IMAGE", 
					"XWIN_IMAGE","YWIN_IMAGE","X2WIN_IMAGE","Y2WIN_IMAGE","BACKGROUND"};
			for (String param : params) {
				se.setParam(param, true);
			}
//			System.out.println("set clean false");
			se.setCleanupWhenDone(false);
			System.out.println("process image");
			String catalogName = se.exec(pi);
			System.out.println("catalog name="+catalogName);
			RowListStarTable rlst = se.getRlst(catalogName);
//			System.out.println("N raws in the table="+rlst.getRowCount());
			Object[] sexRow = rlst.getRow(0);

			for (String param : params) {
				Number number = (Number) sexRow[Table.getIndexByName(rlst,param)];
//				System.out.println(param+": "+number.toString());
				hashtable.put(param, number);
			}
			double xI = (Double) hashtable.get("X_IMAGE");
			double xIwin = (Double) hashtable.get("XWIN_IMAGE");
			double yI = (Double) hashtable.get("Y_IMAGE");
			double yIwin = (Double) hashtable.get("YWIN_IMAGE");
			double x2 = (Double) hashtable.get("X2_IMAGE");
			double x2win = (Double) hashtable.get("X2WIN_IMAGE");
			double y2 = (Double) hashtable.get("Y2_IMAGE");
			double y2win = (Double) hashtable.get("Y2WIN_IMAGE");

			// fill pool histograms
			hDx.fill(xI-trueCol -0.5);
			hDxWin.fill(xIwin-trueCol -0.5);
			hDy.fill(yI-trueRow -0.5);
			hDyWin.fill(yIwin-trueRow -0.5);
			
			double sigmaX = Math.sqrt(x2);
			hSigmaX.fill(sigmaX);		
			double sigmaXWin = Math.sqrt(x2win);
			hSigmaXWin.fill(sigmaXWin);
			double sigmaY = Math.sqrt(y2);
			hSigmaY.fill(sigmaY);		
			double sigmaYWin = Math.sqrt(y2win);
			hSigmaYWin.fill(sigmaYWin);


			
		}
		{
			Histogram[] colList = {hDxWin, hDx};
			JFreeChart chartCol = Histogram.getChart(colList, "column offsets", "delta column", "dN", Double.NaN, Double.NaN, false);
			ChartUtilities.saveChartAsPNG(new File(StringUtil.getFileName("Column","png")), chartCol, 500, 300);		
		}
		{
			Histogram[] rowList = {hDyWin, hDy};
			JFreeChart chartRow = Histogram.getChart(rowList, "row offsets", "delta row", "dN", Double.NaN, Double.NaN, false);
			ChartUtilities.saveChartAsPNG(new File(StringUtil.getFileName("Row","png")), chartRow, 500, 300);		
		}
		{
			Histogram[] sigmaColList = {hSigmaXWin, hSigmaX};
			JFreeChart chartSigmaX = Histogram.getChart(sigmaColList, "sigma column", "sigma column", "dN", Double.NaN, Double.NaN, false);
			ChartUtilities.saveChartAsPNG(new File(StringUtil.getFileName("SigmaColumn","png")), chartSigmaX, 500, 300);		
		}
		{
			Histogram[] sigmaRowList = {hSigmaYWin, hSigmaY};
			JFreeChart chartSigmaY = Histogram.getChart(sigmaRowList, "sigma row", "sigma row", "dN", Double.NaN, Double.NaN, false);
			ChartUtilities.saveChartAsPNG(new File(StringUtil.getFileName("SigmaRow","png")), chartSigmaY, 500, 300);		
		}

		System.out.println(hDx);
		System.out.println(hDxWin);
		System.out.println(hDy);
		System.out.println(hDyWin);
		System.out.println(hSigmaX);
		System.out.println(hSigmaXWin);
		System.out.println(hSigmaY);
		System.out.println(hSigmaYWin);

	}	
	

	
	
}
