package gov.fnal.eag.sim.extractor.test;

import uk.ac.starlink.table.RowListStarTable;
import gov.fnal.eag.sim.extractor.Extractor;
import gov.fnal.eag.sim.extractor.SourceObject;
import gov.fnal.eag.sim.pixel.PixeledImage;
import gov.fnal.eag.sim.random.SimRandomPoisson;
import gov.fnal.eag.sim.repository.image.ImageRepository;
import gov.fnal.eag.sim.utils.Sexec;
import gov.fnal.eag.util.misc.StringUtil;
import gov.fnal.eag.util.yanny.Table;
import junit.framework.TestCase;

/**
 * compares SExtractor to Extractor/SourceObject
 * @author stoughto
 *
 */
public class SourceObjectSExtractorComparison extends TestCase {
	/**
	 * compares the results of SExtractor with Extractor for four pixels well above the threshold.
	 * This test uses no filtering and sets an absolute threshold.
	 * Note that the positions reported by SExtractor are 0.5 larger than the positions reported by Extractor.
	 */
	public void testBackGuessTestBacksize() {

		int nRow = 40;
		int nCol = 40;
		PixeledImage pi = new PixeledImage(nRow, nCol);
		double trueMean = 100.0;
		pi.setScalar(trueMean); 
		pi.setByRowCol(nRow-3, nCol-3, trueMean*100);
		pi.setByRowCol(nRow-3, nCol-2, trueMean*100);
		pi.setByRowCol(nRow-2, nCol-3, trueMean*100);
		pi.setByRowCol(nRow-2, nCol-2, trueMean*100);
		SimRandomPoisson srp = new SimRandomPoisson(1);
		pi.poisson(srp);




		ImageRepository.write(StringUtil.getFileName("fit"), pi);
		Sexec se = new Sexec();
		se.setSex("BACK_SIZE", "20");
		se.setSex("FILTER","N");
		se.setSex("FILTER_NAME", "ThisFilterDoesNotExist");
		se.setSex("THRESH_TYPE", "ABSOLUTE");
		se.setSex("DETECT_THRESH", "500.0");
		se.setSex("DETECT_MINAREA", "3");
		se.setSegRegion("region.reg");
		String[] params = {"X_IMAGE", "Y_IMAGE", "X2_IMAGE", "Y2_IMAGE", 
				"XWIN_IMAGE","YWIN_IMAGE","X2WIN_IMAGE","Y2WIN_IMAGE","BACKGROUND"};

		for (String param : params) {
			se.setParam(param, true);
		}
		se.setCleanupWhenDone(false);
		String catalogName = se.exec(pi);
		RowListStarTable rlst = se.getRlst(catalogName);
		assertEquals(1, rlst.getRowCount());
		Object[] row = rlst.getRow(0);
		double xImage = ((Number)row[Table.getIndexByName(rlst, "X_IMAGE")]).doubleValue();
		double yImage = ((Number)row[Table.getIndexByName(rlst, "Y_IMAGE")]).doubleValue();
		double background = ((Number)row[Table.getIndexByName(rlst, "BACKGROUND")]).doubleValue();
		System.out.println("X_IMAGE="+xImage+"  Y_IMAGE="+yImage+"  BACKGROUND"+background);
		Extractor extractor = new Extractor(pi);
		SourceObject so = extractor.getSource(0);
		assertEquals(xImage-0.5, so.getX(), 5e-4);
		assertEquals(yImage-0.5, so.getY(), 5e-4);
		assertEquals( 97.9118042,background, 1e-4);
	}
}




