
package gov.fnal.eag.sim.pixel;

/**
 * a simple background light model, a constant number of photons in each pixel
 */
public class BackgroundLightConstant implements BackgroundLight {

	/** 
	 * the constant background level
	 */
	private double constantPhotonsPerPixel;
	
	/**
	 * constant background light per pixel
	 * @param constantPhotonsPerPixel the mean rate
	 */
	public BackgroundLightConstant(double constantPhotonsPerPixel) {
		this.constantPhotonsPerPixel = constantPhotonsPerPixel;
	}
	/**
	 * default constructor with no background light
	 *
	 */
	public BackgroundLightConstant() {
		this(0.0);
	}
	/**
	 * @param row position
	 * @param col column position
	 * @return the background light in the pixel at this position
	 */
	public double getLightInPixel(double row, double col) {
		return constantPhotonsPerPixel;
	}
	
	/**
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		String retval = "BackgroundLightConstant with constantPhotonsPerPixel="+constantPhotonsPerPixel;
		return retval;
	}

	/**
	 * @return the constant number of photons per pixel
	 */
	public double getConstantPhotonsPerPixel() {
		return constantPhotonsPerPixel;
	}
}
