package gov.fnal.eag.sim.pixel;

import gov.fnal.eag.wcs.Coordinate;
import gov.fnal.eag.wcs.TransformLinear;

/**
 * provides a pupil image background
 * @author stoughto
 *
 */
public class BackgroundLightPupilImageCoreRadius implements BackgroundLight {

	private TransformLinear colRowToMm;
	private double constant;
	private Coordinate center;
	private double coreRadiusMm;
	
	/**
	 * @param colRowToMm transforms col,row to position in the focal plane
	 * @param constant amount of sky per pixel at the center
	 * @param center coordinate (in mm) of the center of the pupil image
	 * @param coreRadiusMm size of the pupil image, in mm
	 */
	public BackgroundLightPupilImageCoreRadius(TransformLinear colRowToMm, double constant, Coordinate center, double coreRadiusMm) {
		this.colRowToMm = colRowToMm;
		this.constant = constant;
		this.center = center;
		this.coreRadiusMm = coreRadiusMm;
	}
	/**
	 * @see gov.fnal.eag.sim.pixel.BackgroundLight#getLightInPixel(double, double)
	 */
	public double getLightInPixel(double row, double col) {
		Coordinate position = new Coordinate(col, row);
		colRowToMm.forwardInPlace(position);
		double r = center.linearSeparation(position);
		double extra = 0.06*Math.pow(coreRadiusMm,2)/Math.pow(r+coreRadiusMm,2);
		double value = constant*(1.0+extra);
		return value;
	}

	/**
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		return "BackgroundLightPupilImage:  colRowToMm="+colRowToMm+" center="+center+" coreRadiusMm="+coreRadiusMm+" constant="+constant;
	}
}
