package gov.fnal.eag.sim.pixel;

import gov.fnal.eag.sim.random.SimRandomExponential;
import gov.fnal.eag.sim.random.SimRandomPoisson;
import gov.fnal.eag.sim.random.SimRandomUniform;
import gov.fnal.eag.sim.repository.image.FitsDataFormat;
import gov.fnal.eag.sim.repository.image.ImageRepository;
import gov.fnal.eag.util.densityEstimate.FitFunction;
import gov.fnal.eag.util.densityEstimate.FitFunctionDoubleExponential;
import gov.fnal.eag.util.densityEstimate.Histogram;
import gov.fnal.eag.util.misc.StringUtil;
import gov.fnal.eag.util.plot.JFreeChartUtil;
import gov.fnal.eag.util.stopwatch.StopwatchCollection;

import java.awt.Rectangle;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.Iterator;

import org.jfree.chart.JFreeChart;

import uk.ac.starlink.table.DefaultValueInfo;
import uk.ac.starlink.table.DescribedValue;
import uk.ac.starlink.table.RowListStarTable;

import com.martiansoftware.jsap.FlaggedOption;
import com.martiansoftware.jsap.JSAP;
import com.martiansoftware.jsap.JSAPException;
import com.martiansoftware.jsap.JSAPResult;
import com.martiansoftware.jsap.UnflaggedOption;

/**
 * main program to run clock charging algorithm from the command line
 * @author stoughto
 *
 */
public class ClockCharge {

	private JSAPResult config;
	private static Hashtable<String,double[]> truth = new Hashtable<String,double[]>();
	private FitsDataFormat format = FitsDataFormat.DOUBLE;

	/*
	 * data with dates taken from Kyle's log files from his trails program.
	 * Data from "table-iv" from Kyle's paper 14-Nov-2007, assuming times
	 * per transfer are 27.71e-3 sec (parallel) and 14.29e-6 sec (serial)
	 */
	static {
		//double tParallel = 27.71E-3; taken by eye from the plot
		//double tSerial   = 14.29E-6;
		
		/*
		 * email from Kyle March 3, 2009
		serial clocking time:
			1.42857e-05 seconds between pixels

			parallel clocking time:
			0.0264286 seconds between rows


			TABLE IV:  Background level was 30 e-/pixel
	*/

		
		double tParallel = 0.0264286;
		double tSerial = 1.42857e-05;
		truth.put("Post-Anneal-11sep-serial", new double[]{ 3.37364E-5, 0.531099788456812, 2.15388E-6, 16.13007620726331});
		truth.put("Post-Anneal-11sep-parallel", new double[]{ 5.39552E-4, 0.42638927872871873, 1.31666E-6, 12.042858925921335});
		truth.put("Post-Anneal-14sep-serial", new double[]{ 1.75618E-5, 0.7460389530111164, 2.23852E-6, 15.443234941896991});
		truth.put("Post-Anneal-14sep-parallel", new double[]{ 4.83213E-4, 0.44368992486515835, 1.35011E-6, 13.914039294301574});
		truth.put("Post-Anneal-15sep-serial", new double[]{ 1.7941E-5, 0.7187265638017006, 2.28687E-6, 15.654530155803858});
		truth.put("Post-Anneal-15sep-parallel", new double[]{ 4.85135E-4, 0.44769547422081296, 1.21603E-6, 16.441380271981803});
		truth.put("Pre-Anneal-14aug06-serial", new double[]{ 3.31774E-5, 0.5999675530601316, 1.08511E-6, 7.7088876097480785});
		truth.put("Pre-Anneal-14aug06-parallel", new double[]{ 1.4602E-4, 0.5873430937435892, 2.04759E-6, 13.115638617412122});
		truth.put("Pre-Anneal-15aug06-serial", new double[]{ 5.83095E-5, 0.43946946201531073, 5.65595E-7, 15.763993766058945});
		truth.put("Pre-Anneal-15aug06-parallel", new double[]{ 1.73348E-4, 0.5440719201633862, 1.61455E-6, 17.117490541165797});
		truth.put("Pre-Anneal-16aug06-serial", new double[]{ 3.06295E-5, 0.5409500186873831, 6.20471E-7, 17.52747790115625});
		truth.put("Pre-Anneal-16aug06-parallel", new double[]{ 1.90829E-4, 0.508583531813011, 2.05073E-6, 14.103651486128737});
		truth.put("Pre-Anneal-17aug06-serial", new double[]{ 3.04156E-5, 0.5743166807635517, 7.04632E-7, 15.452951141792882});
		truth.put("Pre-Anneal-17aug06-parallel", new double[]{ 1.72798E-4, 0.5301929817536213, 1.75313E-6, 16.483351985628794});
		truth.put("Pre-Anneal-TableIV-parallel",  new double[]{1.75E-4, 1.40E-2/tParallel, 1.84E-6, 4.06E-1/tParallel});
		truth.put("Post-Anneal-TableIV-parallel", new double[]{5.02E-4, 1.16E-2/tParallel, 1.28E-6, 3.70E-1/tParallel});
		truth.put("Pre-Anneal-TableIV-serial",    new double[]{3.42E-5, 7.72E-6/tSerial,   0.64E-6, 2.23E-4/tSerial});
		truth.put("Post-Anneal-TableIV-serial",   new double[]{2.21E-5, 9.30E-6/tSerial,   2.22E-6, 2.25E-4/tSerial});
	}

	//

	boolean userSpecifiedParallelCti = false;
	int trapLocationRandomSeed;
	int trapDecayRandomSeed;
	double[] parallelTrapsPerPixels;
	double[] parallelTrapLifetimes;
	double[] serialTrapsPerPixels;
	double[] serialTrapLifetimes;
	double parallelFullWell;
	double parallelPower;
	double parallelNotchDepth;
	int nLevels = 100;
	double serialFullWell;
	double serialPower;
	double serialNotchDepth;
	String testTag = "default-";
	double sky=0.0;
	double gain=1.0;
	int testRow;
	int testCol;
	int testTrailLength = 45;
	boolean quantizeTraps = true; 
	boolean quantizeCharge = true;
	int nToDo = 200;
	int verbosity=0;
	double defaultPower = 0.5;
	double defaultNotchDepth = 0.0;
	double serialCtiChargeNought = 1620.0;
	double parallelCtiChargeNought = 1620.0;
	String testTruth = "Post-Anneal-11sep";
	//

	/**
	 * @param name for example, Post-Anneal-11sep
	 * @return array to define FitFunctionDoubleExponential
	 */
	static public double[] getTruth(String name) {
		double[] temp = truth.get(name);
		if (temp==null) {
			System.out.println("can not find name="+name);
			for (String key : truth.keySet()){
				System.out.println("  key="+key);
			}
			throw new IllegalArgumentException("can not find "+name);
		}
		double[] retval = new double[temp.length];
		for (int i=0; i<temp.length; i++) {
			retval[i] =  temp[i];
		}
		return retval;
	}

	/**
	 * @return the names we know
	 */
	static public Iterable<String> getTruthNames() {
		Iterable<String> retval = truth.keySet();
		return retval;
	}
	/**
	 * @param args parsed with <a href=http://www.martiansoftware.com/jsap/doc/javadoc/com/martiansoftware/jsap/JSAP.html>JSAP</a>
	 * 
	 * <br>--doTest false if true, make plots to compare to measured trails
	 * <br>--testRow 100 if doTest, row position of pixel that is filled with parallelCtiChargeNought electrons
	 * <br>--testCol 100 if doTest, col position of pixel that is filled with parallelCtiChargeNought electrons
	 * <br>--testNToDo 200 if doTest, number of time to simulate the charge transfer to coadd into histograms
	 * <br>--testTag default if doTest, preface to file names for histogram plots
	 * <br>--testTrailLength if doTest, number of pixels to histogram
	 * <br>--testTruth Post-Anneal-11sep if doTest, true data set to superimpose (one of Post-Anneal-11sep, Post-Anneal-14sep,Post-Anneal-15sep, Pre-Anneal-14aug06, Pre-Anneal-15aug06, Pre-Anneal-16aug06, Pre-Anneal-17aug06 
	 * <br>--parallelCti 1e-6 if set by user, use cti scaled by 1/sqrt(charge) with this value at parallelCtiChargeNought
	 * <br>--serialCti 1e-6, scale cti by 1/sqrt(charge) with this value at serialCtiChargeNought
	 * <br>--serialCtiChargeNought 1620
	 * <br>--parallelCtiChargeNought 1620
	 * <br>--parallelTrapsPerPixel 0.13,7.0 mean number of traps per pixel for parallel transfers (comma-separated list)
	 * <br>--parallelTrapLifetime  10.0, 0.5 trap lifetime, in units of parallel transfer time (comma-separated list)
	 * <br>--parallelFullWell 150000 full well for parallel pixels
	 * <br>--parallelPower the height that charge fills up a well is calculated as (charge/parallelFullWell)^parallelPower
	 * <br>--serialTrapsPerPixel 0.7,0.3 mean number of traps per serial pixel (comma-separated list)
	 * <br>--serialTrapLifetime 15.0,1.0 trap lifetime, in units of serial transfer time (comma-separated list)
	 * <br>--serialFullWell 150000 full well for serial pixels
	 * <br>--serialPower the height that charge fills up a well is calculated as (charge/serialFullWell)^serialPower
	 * <br>--fileNames if specified, the name of the input file to read and apply charge transfer effect, and the name of the output file to write
	 * <br>--trapLocationRandomSeed 12 index to random seeds that select where to place charge traps
	 * <br>--trapDecayRandomSeed 13 index to random seeds for decaying traps
	 * <br>--outputFormat float precision of output file written, one of float, double, ushort 
	 * <br>--showStopwatch false if true, show stopwatch information for timing 
	 * <br>--skyPerPixel 0.0 counts to add (in units of the input image) to each pixel
	 * <br>--gain 1.0 after adding sky, multiply by this to convert to electrons per pixel
	 * <br>--range "default" or "rowMin colMin rowMax colMax" four integers to define range to consider
	 * <br>--verbosity 0 is quiet; 1 prints two messages; 2 prints at start of each column and row
	 * <br>--quantizeCharge true to quantize charge (default); false to release the mean number of electrons
	 * <br>--quantizeTraps true to quantize traps (default); false to use means to release charge
	 * @throws JSAPException if there is trouble creating options
	 */
	public ClockCharge(String[] args) throws JSAPException {
		parseArgs(args);
	}
	/**
	 * @return the configuration built by JSAP from command line arguments
	 */
	public JSAPResult getConfig() {
		return config;
	}
	/**
	 * get ready to clock this the following parameters.
	 * @param trapLocationRandomSeed
	 * @param trapDecayRandomSeed
	 * @param parallelTrapsPerPixels
	 * @param parallelTrapLifetimes
	 * @param serialTrapsPerPixels
	 * @param serialTrapLifetimes
	 * @param parallelFullWell
	 * @param parallelPower
	 * @param parallelNotchDepth
	 * @param serialFullWell
	 * @param serialPower
	 * @param serialNotchDepth
	 */
	public ClockCharge(int trapLocationRandomSeed, int trapDecayRandomSeed, 
			double[] parallelTrapsPerPixels, double[] parallelTrapLifetimes,
			double[] serialTrapsPerPixels, double[] serialTrapLifetimes,
			double parallelFullWell, double parallelPower, double parallelNotchDepth,
			double serialFullWell, double serialPower, double serialNotchDepth) {
		/*
		if (parallelTrapsPerPixels.length != parallelTrapLifetimes.length || serialTrapsPerPixels.length != serialTrapLifetimes.length) {
			throw new Error("the TrapsPerPixels and TrapLifetimes arrays must be the same length in each direction");
		}
		 */
		this.trapLocationRandomSeed = trapLocationRandomSeed;
		this.trapDecayRandomSeed = trapDecayRandomSeed;
		this.parallelTrapsPerPixels = parallelTrapsPerPixels;
		this.parallelTrapLifetimes = parallelTrapLifetimes;
		this.serialTrapsPerPixels = serialTrapsPerPixels;
		this.serialTrapLifetimes = serialTrapLifetimes;
		this.parallelFullWell = parallelFullWell;
		this.parallelPower = parallelPower;
		this.parallelNotchDepth = parallelNotchDepth;
		this.serialFullWell = serialFullWell;
		this.serialPower = serialPower;
		this.serialNotchDepth = serialNotchDepth;
		quantizeTraps = true;
	}
	/**
	 * @param args as defined in the ClockCharge(String[]) constructor
	 * @throws JSAPException
	 */
	public static void main(String[] args) throws JSAPException {
		/*
		System.out.println("begin ClockCharge with these args:");
		for (String arg : args) {
			System.out.println("    "+arg);
		}
		 */
		StopwatchCollection.start("all");
		ClockCharge cc = new ClockCharge(args);
		if (cc.getConfig().getBoolean("test")) {
			cc.doTest();
		}
		String[] fileNames = cc.getConfig().getStringArray("fileNames");
		if (fileNames.length == 2) {
			Rectangle range = null;
			String rangeOpt = cc.getConfig().getString("range");
			if (!rangeOpt.equals("default")) {
				int[] ri = StringUtil.toIntArray(rangeOpt,",");
				// ri[0] is rowMin
				// ri[1] is colMin
				// ri[2] is rowMax
				// ri[3] is colMax
				// range is defines with x0, y0, width, height
				range = new Rectangle(ri[1], ri[0], ri[3]-ri[1], ri[2]-ri[0]);
			}
			boolean quantizeCharge = cc.getConfig().getBoolean("quantizeCharge");
			cc.doClock2(fileNames[0], fileNames[1], range, quantizeCharge);
		}
		StopwatchCollection.stop("all");
		if (cc.getConfig().getBoolean("showStopwatch")) {
			StopwatchCollection.report(System.out);
		}
	}
	/**
	 * make the test plots
	 */
	public void doTest() {
		doTestSerialParallel("parallel", true, 0.0);
		doTestSerialParallel("serial", true, 0.0);
	}


	/**
	 * do the test in one direction
	 * @param sp "parallel" or "serial"
	 * @param writeFiles
	 * @param skyValue 
	 * @return chi2 for the fit of the results of this clocking 
	 */
	public double doTestSerialParallel(String sp, boolean writeFiles, double skyValue) {
		if (testTag.equals("default-")) {
			testTag = "";
		}
		double[] tpps;
		if (sp.equals("parallel")) {
			tpps = parallelTrapsPerPixels;
		} else {
			tpps = serialTrapsPerPixels;
		}
		boolean tppsAllZero = true;
		for (double tpp : tpps) {
			if (tpp > 0.0) tppsAllZero = false;
		}
		if (tppsAllZero) {
			return 0.0;
		}
		int pix0;
		int nType;
		double charge0;

		double power, fullWell, notchDepth;
		if (sp.equals("parallel")) {
			pix0 = testRow;
			nType = parallelTrapLifetimes.length;
			charge0 = parallelCtiChargeNought;
			power = parallelPower;
			fullWell = parallelFullWell;
			notchDepth = parallelNotchDepth;
		} else {
			pix0 = testCol;
			nType = serialTrapLifetimes.length;
			charge0 = serialCtiChargeNought;
			power = serialPower;
			fullWell = serialFullWell;
			notchDepth = serialNotchDepth;
		}
		ArrayList<Pixel> pixels = new ArrayList<Pixel>();
		SimRandomPoisson srp       = new SimRandomPoisson();
		SimRandomUniform sruHeight = new SimRandomUniform();
		double xMin = 0.5;
		double xMax = testTrailLength+0.5;
		int nBin = testTrailLength;
		String yTitle = String.format("charge from "+charge0+" electrons in pixel "+pix0);
		//String hgTitle = "doTest "+sp+" charge0="+charge0+" quantizeTraps="+quantizeTraps+" tau="+config.getString(sp+"TrapLifetime")+" rho="+config.getString(sp+"TrapsPerPixel");
		String hgTitle = "doTest "+sp+" charge0="+charge0+" quantizeTraps="+quantizeTraps;
		if (sp.equals("parallel")) {
			hgTitle += " tau="+StringUtil.toString(parallelTrapLifetimes)+" rho="+StringUtil.toString(parallelTrapsPerPixels);
		} else {
			hgTitle += " tau="+StringUtil.toString(serialTrapLifetimes)+" rho="+StringUtil.toString(serialTrapsPerPixels);			
		}
		hgTitle += " nToDo="+nToDo+" sky="+sky+" pix0="+pix0;
		Histogram hg = new Histogram(hgTitle, "pixel after "+pix0, yTitle, nBin, xMin, xMax);

		int nPixels = pix0 + testTrailLength;

		for (int iPixel=0; iPixel<nPixels; iPixel++) {
			ArrayList<ElectronTrap> traps = new ArrayList<ElectronTrap>();
			Pixel pixel = new PixelWithTraps(traps, 0.0, fullWell, power, notchDepth);
			pixels.add(pixel);			
		}

		// build up the array of charge traps
		for (int iType=0; iType<nType; iType++) {
			int tempSeed;
			if (sp.equals("parallel")) {
				tempSeed = (testRow+1)*10000 + trapLocationRandomSeed*(iType+1);
			} else {
				tempSeed = trapLocationRandomSeed*(iType+1);
			}
			srp.setIseedIndex(tempSeed);
			sruHeight.setIseedIndex(tempSeed+10);
			double trapLifetime;
			double trapsPerPixel;
			if (sp.equals("parallel")) {
				trapLifetime = parallelTrapLifetimes[iType];
				trapsPerPixel = parallelTrapsPerPixels[iType];
			} else {
				trapLifetime = serialTrapLifetimes[iType];
				trapsPerPixel = serialTrapsPerPixels[iType];
			}
			SimRandomExponential sre = new SimRandomExponential(trapDecayRandomSeed, 1.0/trapLifetime, "", false);
			for (int iPixel=0; iPixel<nPixels; iPixel++) {
				PixelWithTraps pixel = (PixelWithTraps) pixels.get(iPixel);
				if (quantizeTraps) {
					int nTrap = srp.next(trapsPerPixel);
					for (int iTrap=0; iTrap<nTrap; iTrap++) {
						for (int iLevel=0; iLevel<nLevels; iLevel++) {
							double trapHeightFraction = sruHeight.next();
							ElectronTrap trap = new ElectronTrapCB1(sre, trapHeightFraction, 1.0/nLevels);
							pixel.addTrap(trap);
						}
					}
				} else {
					ElectronTrap trap = new ElectronTrapCB1Mean(trapsPerPixel, trapLifetime, fullWell, defaultPower, defaultNotchDepth, nLevels);
					pixel.addTrap(trap);
				}			
			}
		}
		double readoutFromPix0 = 0.0;
		for (int iToDo=0; iToDo<nToDo; iToDo++) {
			if (iToDo % 10 == 0) System.out.println("ClockCharge doTest:  iToDo="+iToDo+"/"+nToDo);
			double charge;
			for (int i=0; i<nPixels; i++) {
				pixels.get(i).releaseTrappedCharge();
				if (i == pix0) {
					charge = charge0;
				} else {
					charge = 0;
				}
				charge += sky;
				pixels.get(i).setCharge(charge);
			}
			PixelRegister pr = new PixelRegister(pixels);
			int iPixelMax = nPixels;
			for (int i=0; i<pix0; i++) {
				int decaySeed = trapDecayRandomSeed+iToDo;
				int tempSeed = -1;
				if (quantizeCharge) {
					if (sp.equals("parallel")) {
						tempSeed = decaySeed + i + testCol*10000;
					} else {
						tempSeed = decaySeed + i + testRow;
					}
					pr.setDecaySeed(tempSeed);
				}
				pr.clock(iPixelMax, 1.0, quantizeCharge);
				iPixelMax--;
			}
			
			for (int i=pix0; i<nPixels; i++) {
				int decaySeed = trapDecayRandomSeed+iToDo;
				int tempSeed = -1;
				if (quantizeCharge) {
					if (sp.equals("parallel")) {
						tempSeed = decaySeed + i + testCol*10000;
					} else {
						tempSeed = decaySeed + i + testRow;
					}
					pr.setDecaySeed(tempSeed);
				}
				// this was quantizeTraps 
				double clockedCharge = pr.clock(iPixelMax, 1.0, quantizeCharge);
				int iBin = i-pix0;
				double value =  clockedCharge/nToDo;				
				if (i == pix0) {
					readoutFromPix0 += value;
				}
				hg.fill(iBin, value);
				iPixelMax--;
			}
		}
		hg.addConstant(-sky);
		readoutFromPix0 -= sky;
		double retval = 0.0;
		{
			FitFunction ff = null;
			if (testTruth.length() > 1) {
				String key = testTruth+"-"+sp;
				double[] params = ClockCharge.getTruth(key);
				params[0] *= pix0*charge0;
				params[2] *= pix0*charge0;
				ff = new FitFunctionDoubleExponential();
				ff.setName(key);
				ff.setParameters(params);				
				retval = hg.getChi2(ff, 1);
			}
			if (writeFiles) {
				hg.addConstant(-skyValue);
				JFreeChart chart = Histogram.getChart(hg, ff, true);
				JFreeChartUtil.toPng(chart, testTag+"clockChargeDoTest-"+sp+".png");
				RowListStarTable rlst = hg.getRlst(ff);
				DescribedValue describedValue1 = new DescribedValue(new DefaultValueInfo("QINPIX0", Double.class, "charge read out from pixel 0"), new Double(readoutFromPix0));
				rlst.setParameter(describedValue1);
				ImageRepository.write(testTag+"clockChargeDoTest-"+sp+".fit", rlst, true);
			}
		}
		return retval;
	}

	/**
	 * reads the input image, applies sky, gain, and bin factor, clocks the charge and writes to outFileName
	 * @param inFileName where to read the input image
	 * @param outFileName where to write
	 * @param quantizeChargeIn whether to quantize charge
	 */
	public void doClock2(String inFileName, String outFileName, boolean quantizeChargeIn) {
		doClock2(inFileName, outFileName, null, quantizeChargeIn);
	}

	/**
	 * @param inFileName
	 * @param outFileName
	 * @param range defines the row,column range to be clocked
	 * @param quantizeChargeIn whether to quantize charge
	 */
	public void doClock2(String inFileName, String outFileName, Rectangle range, boolean quantizeChargeIn) {
		StopwatchCollection.start(" i/o");
		PixeledImage pi = ImageRepository.readPixeledImage(inFileName);
		if (sky != 0.0) {
			pi.addScalar(sky);
		}
		if (gain != 1.0) {
			pi.multiplyScalar(gain);
		}
		StopwatchCollection.stop(" i/o");

		if (range == null) {
			range = new Rectangle(pi.getColumnMin(), pi.getRowMin(), pi.getColumnCount(), pi.getRowCount());
		}

		//int nRow = range.height;
		//int nCol = range.width;
		//int row0PlusRow1 = 2*range.y+range.height;
		//int col0PlusCol1 = 2*range.x+range.width;
		//int nTransferSerial = nRow*nCol*col0PlusCol1/2;
		//int nTransferParallel = nRow*nCol*row0PlusRow1/2;
		//int nTransfer = nTransferSerial+nTransferParallel;
		//System.out.println("  nTransferSerial="+nTransferSerial);
		//System.out.println("nTransferParallel="+nTransferParallel);
		//System.out.println("        nTransfer="+nTransfer);
		StopwatchCollection.start(" all doClock2");
		doClock2(pi, range);
		StopwatchCollection.stop(" all doClock2");
		StopwatchCollection.start(" i/o");
		ImageRepository.write(outFileName, pi, format, true);
		StopwatchCollection.stop(" i/o");
	}

	/**
	 * @param pi image to modify with the specified traps
	 * @param quantizeTrapsIn whether to quantize charge
	 */
	public void doClock2(PixeledImage pi, boolean quantizeTrapsIn) {
		doClock2(pi, null);
	}	
	/**
	 * @param pi
	 * @param range row,col range to be clocked
	 */
	@SuppressWarnings("unchecked")
	public void doClock2(PixeledImage pi, Rectangle range) {
		if (range == null) {
			range = new Rectangle(pi.getColumnMin(), pi.getRowMin(), pi.getColumnCount(), pi.getRowCount());			
		}
		if (!userSpecifiedParallelCti) {

			int nRow = pi.getRowCount();
			int nCol = pi.getColumnCount();
			ArrayList<ElectronTrap>[][] parallelTraps = (ArrayList<ElectronTrap>[][]) new ArrayList<?>[nRow][nCol];
			SimRandomPoisson srpParallel       = new SimRandomPoisson();
			SimRandomUniform sruParallelHeight = new SimRandomUniform();
			SimRandomExponential sre = null;
			if (parallelTrapsPerPixels != null) {
				for (int col=range.x; col<range.x+range.width; col++) {	
					for (int iType=0; iType<parallelTrapsPerPixels.length; iType++) {
						double parallelTrapsPerPixel = parallelTrapsPerPixels[iType];
						double parallelTrapLifetime = parallelTrapLifetimes[iType];
						sre = new SimRandomExponential(trapDecayRandomSeed+iType, 1.0/parallelTrapLifetime,"",false);
						int tempSeed = (col+1)*10000 + trapLocationRandomSeed*(iType+1);
						srpParallel.setIseedIndex(tempSeed);
						sruParallelHeight.setIseedIndex(tempSeed+10);
						for (int row=pi.getRowMin(); row<range.y+range.height; row++) {
							if (quantizeTraps) {
								int nTraps = srpParallel.next(parallelTrapsPerPixel);
								if (nTraps > 0) {
									ArrayList<ElectronTrap> traps = getParallelTraps(pi, parallelTraps, col, row);
									for (int iTrap=0; iTrap<nTraps; iTrap++) {
										for (int iLevel=0; iLevel<nLevels; iLevel++) {
											double trapHeightFraction = sruParallelHeight.next();
											ElectronTrap trap = new ElectronTrapCB1(sre, trapHeightFraction, 1.0/nLevels);
											traps.add(trap);
										}
									}
								}
							} else {
								if (parallelTrapsPerPixel > 0.0) {
									ArrayList<ElectronTrap> traps = getParallelTraps(pi, parallelTraps, col, row);
									ElectronTrapCB1Mean trap;
									if (col == range.x) {
										trap = new ElectronTrapCB1Mean(parallelTrapsPerPixel, parallelTrapLifetime, parallelFullWell, parallelPower, parallelNotchDepth, nLevels);
										traps.add(trap);
									} else {
										ArrayList<ElectronTrap> trapsFirstColumn = getParallelTraps(pi, parallelTraps, range.x, row);
										trap = (ElectronTrapCB1Mean) trapsFirstColumn.get(iType);
										trap.setLifetimeTrapsPerPixel(parallelTrapLifetime, parallelTrapsPerPixel);
										traps.add(trap);
									}

								}
							}
						}
					}
					int rowMin = Math.max(pi.getRowMin(), range.y);
					int rowMax = Math.min(pi.getRowMax(), range.y+range.height);
					Rectangle rangeOneColumn = new Rectangle(col, rowMin, 1, rowMax-rowMin);
					StopwatchCollection.start("clock charge");
					pi.clockCharge(parallelTraps, 1.0, parallelFullWell, parallelPower, parallelNotchDepth,
							null, 1.0, serialFullWell, serialPower, serialNotchDepth,
							verbosity, rangeOneColumn, quantizeCharge, trapDecayRandomSeed);				
					StopwatchCollection.stop("clock charge");
				}
			}
			ArrayList<ElectronTrap>[] serialTraps = (ArrayList<ElectronTrap>[]) new ArrayList<?>[nCol];
			SimRandomPoisson srpSerial       = new SimRandomPoisson();
			SimRandomUniform sruSerialHeight = new SimRandomUniform();
			if (serialTrapsPerPixels != null) {
				StopwatchCollection.start("define traps");
				for (int iType=0; iType<serialTrapsPerPixels.length; iType++) {
					int tempSeed = trapLocationRandomSeed*(iType+1);				
					srpSerial.setIseedIndex(tempSeed);
					sruSerialHeight.setIseedIndex(tempSeed+10);
					double serialTrapsPerPixel = serialTrapsPerPixels[iType];
					double serialTrapLifetime = serialTrapLifetimes[iType];
					sre = new SimRandomExponential(trapDecayRandomSeed, 1.0/serialTrapLifetime,"",false);
					for (int col=pi.getColumnMin(); col<range.x+range.width; col++) {
						if (quantizeTraps) {
							int nTraps = srpSerial.next(serialTrapsPerPixel);
							if (nTraps > 0) {
								ArrayList<ElectronTrap> traps = getSerialTraps(pi, serialTraps, col);
								for (int iTrap=0; iTrap<nTraps; iTrap++) {
									for (int iLevel=0; iLevel<nLevels; iLevel++) {
										ElectronTrap trap = new ElectronTrapCB1(sre, sruSerialHeight.next(), 1.0/nLevels);
										traps.add(trap);
									}
								}
							}
						} else {
							if (serialTrapsPerPixel > 0.0) {
								ArrayList<ElectronTrap> traps = getSerialTraps(pi, serialTraps, col);
								ElectronTrapCB1Mean trap = new ElectronTrapCB1Mean(serialTrapsPerPixel, serialTrapLifetime, serialFullWell, serialPower, serialNotchDepth, nLevels);
								traps.add(trap);
							}
						}
					}
				}
				StopwatchCollection.stop("define traps");
				StopwatchCollection.start("clock charge");
				pi.clockCharge(null, 1.0, parallelFullWell, parallelPower, parallelNotchDepth, 
						serialTraps, 1.0, serialFullWell, serialPower, serialNotchDepth, verbosity, 
						range, quantizeCharge, trapDecayRandomSeed);				
				StopwatchCollection.stop("clock charge");
			}
		}
	}

	private ArrayList<ElectronTrap> getSerialTraps(PixeledImage pi,
			ArrayList<ElectronTrap>[] serialTraps, int col) {
		ArrayList<ElectronTrap> traps = serialTraps[col-pi.getColumnMin()];
		if (traps == null) {
			traps = new ArrayList<ElectronTrap>();
			serialTraps[col-pi.getColumnMin()] = traps;
		}
		return traps;
	}

	private ArrayList<ElectronTrap> getParallelTraps(PixeledImage pi, ArrayList<ElectronTrap>[][] parallelTraps, int col, int row) {
		int r = row-pi.getRowMin();
		int c = col-pi.getColumnMin();
		ArrayList<ElectronTrap> traps = parallelTraps[r][c];
		if (traps == null) {
			traps = new ArrayList<ElectronTrap>();
			parallelTraps[r][c] = traps;
		}
		return traps;
	}		

	private static double[] parse(String in) {
		String[] tokens = in.split(",");
		double[] retval = new double[tokens.length];
		for (int i=0; i<tokens.length; i++) {
			retval[i] = Double.parseDouble(tokens[i]);
		}
		return retval;
	}
	private void parseArgs(String[] args) throws JSAPException {
		JSAP jsap = new JSAP();

		FlaggedOption testOpt = new FlaggedOption("test")
		.setStringParser(JSAP.BOOLEAN_PARSER)
		.setDefault("false")
		.setRequired(true)
		.setLongFlag("doTest");
		jsap.registerParameter(testOpt);

		FlaggedOption testRowOpt = new FlaggedOption("testRow")
		.setStringParser(JSAP.INTEGER_PARSER)
		.setDefault("100")
		.setRequired(false)
		.setLongFlag("testRow");
		jsap.registerParameter(testRowOpt);

		FlaggedOption testColOpt = new FlaggedOption("testCol")
		.setStringParser(JSAP.INTEGER_PARSER)
		.setDefault("100")
		.setRequired(false)
		.setLongFlag("testCol");
		jsap.registerParameter(testColOpt);

		FlaggedOption testNToDoOpt = new FlaggedOption("testNToDo")
		.setStringParser(JSAP.INTEGER_PARSER)
		.setDefault("200")
		.setRequired(false)
		.setLongFlag("testNToDo");
		jsap.registerParameter(testNToDoOpt);

		FlaggedOption testTagOpt = new FlaggedOption("testTag")
		.setStringParser(JSAP.STRING_PARSER)
		.setDefault("default")
		.setRequired(false)
		.setLongFlag("testTag");
		jsap.registerParameter(testTagOpt);

		FlaggedOption testTrailLengthOpt = new FlaggedOption("testTrailLength")
		.setStringParser(JSAP.INTEGER_PARSER)
		.setDefault("45")
		.setRequired(true)
		.setLongFlag("testTrailLength");
		jsap.registerParameter(testTrailLengthOpt);

		FlaggedOption testTruthOpt = new FlaggedOption("testTruth")
		.setStringParser(JSAP.STRING_PARSER)
		.setDefault("Post-Anneal-11sep")
		.setRequired(true)
		.setLongFlag("testTruth");
		jsap.registerParameter(testTruthOpt);


		FlaggedOption parallelCtiOpt = new FlaggedOption("parallelCti")
		.setStringParser(JSAP.DOUBLE_PARSER)
		.setDefault("1e-6")
		.setRequired(false)
		.setLongFlag("parallelCti");
		jsap.registerParameter(parallelCtiOpt);

		FlaggedOption serialCtiOpt = new FlaggedOption("serialCti")
		.setStringParser(JSAP.DOUBLE_PARSER)
		.setDefault("1e-6")
		.setRequired(false)
		.setLongFlag("serialCti");
		jsap.registerParameter(serialCtiOpt);

		FlaggedOption serialCtiChargeNoughtOpt = new FlaggedOption("serialCtiChargeNought")
		.setStringParser(JSAP.DOUBLE_PARSER)
		.setDefault("1620.0")
		.setRequired(true)
		.setLongFlag("serialCtiChargeNought");
		jsap.registerParameter(serialCtiChargeNoughtOpt);

		FlaggedOption parallelCtiChargeNoughtOpt = new FlaggedOption("parallelCtiChargeNought")
		.setStringParser(JSAP.DOUBLE_PARSER)
		.setDefault("1620.0")
		.setRequired(true)
		.setLongFlag("parallelCtiChargeNought");
		jsap.registerParameter(parallelCtiChargeNoughtOpt);

		String parallelTrapsPerPixelDefault = "0.13,7.0";
		FlaggedOption parallelTrapsPerPixelOpt = new FlaggedOption("parallelTrapsPerPixel")
		.setStringParser(JSAP.STRING_PARSER)
		.setDefault(parallelTrapsPerPixelDefault)
		.setRequired(true)
		.setLongFlag("parallelTrapsPerPixel");
		jsap.registerParameter(parallelTrapsPerPixelOpt);

		String parallelTrapLifetimeDefault = "10.0,0.5";
		FlaggedOption parallelTrapLifetimeOpt = new FlaggedOption("parallelTrapLifetime")
		.setStringParser(JSAP.STRING_PARSER)
		.setDefault(parallelTrapLifetimeDefault)
		.setRequired(true)
		.setLongFlag("parallelTrapLifetime");
		jsap.registerParameter(parallelTrapLifetimeOpt);

		FlaggedOption parallelFullWellOpt = new FlaggedOption("parallelFullWell")
		.setStringParser(JSAP.DOUBLE_PARSER)
		.setDefault("150000.0")
		.setRequired(true)
		.setLongFlag("parallelFullWell");
		jsap.registerParameter(parallelFullWellOpt);

		FlaggedOption parallelPowerOpt = new FlaggedOption("parallelPower")
		.setStringParser(JSAP.DOUBLE_PARSER)
		.setDefault("0.5")
		.setRequired(true)
		.setLongFlag("parallelPower");
		jsap.registerParameter(parallelPowerOpt);

		FlaggedOption parallelNotchDepthOpt = new FlaggedOption("parallelNotchDepth")
		.setStringParser(JSAP.DOUBLE_PARSER)
		.setDefault("0.0")
		.setRequired(true)
		.setLongFlag("parallelNotchDepth");
		jsap.registerParameter(parallelNotchDepthOpt);

		String serialTrapsPerPixelDefault = "0.7,0.3";
		FlaggedOption serialTrapsPerPixelOpt = new FlaggedOption("serialTrapsPerPixel")
		.setStringParser(JSAP.STRING_PARSER)
		.setDefault(serialTrapsPerPixelDefault)
		.setRequired(true)
		.setLongFlag("serialTrapsPerPixel");
		jsap.registerParameter(serialTrapsPerPixelOpt);

		String serialTrapLifetimeDefault = "15.0,1.0";
		FlaggedOption serialTrapLifetimeOpt = new FlaggedOption("serialTrapLifetime")
		.setStringParser(JSAP.STRING_PARSER)
		.setDefault(serialTrapLifetimeDefault)
		.setRequired(true)
		.setLongFlag("serialTrapLifetime");
		jsap.registerParameter(serialTrapLifetimeOpt);

		FlaggedOption serialFullWellOpt = new FlaggedOption("serialFullWell")
		.setStringParser(JSAP.DOUBLE_PARSER)
		.setDefault("150000.0")
		.setRequired(true)
		.setLongFlag("serialFullWell");
		jsap.registerParameter(serialFullWellOpt);

		FlaggedOption serialPowerOpt = new FlaggedOption("serialPower")
		.setStringParser(JSAP.DOUBLE_PARSER)
		.setDefault("0.5")
		.setRequired(true)
		.setLongFlag("serialPower");
		jsap.registerParameter(serialPowerOpt);

		FlaggedOption serialNotchDepthOpt = new FlaggedOption("serialNotchDepth")
		.setStringParser(JSAP.DOUBLE_PARSER)
		.setDefault("0.0")
		.setRequired(true)
		.setLongFlag("serialNotchDepth");
		jsap.registerParameter(serialNotchDepthOpt);

		UnflaggedOption fileNamesOpt = new UnflaggedOption("fileNames")
		.setStringParser(JSAP.STRING_PARSER)
		.setDefault("none")
		.setRequired(true)
		.setGreedy(true);
		jsap.registerParameter(fileNamesOpt);

		FlaggedOption trapLocationRandomSeedOpt = new FlaggedOption("trapLocationRandomSeed")
		.setStringParser(JSAP.INTEGER_PARSER)
		.setDefault("12")
		.setRequired(true)
		.setLongFlag("trapLocationRandomSeed");
		jsap.registerParameter(trapLocationRandomSeedOpt);

		FlaggedOption trapDecayRandomSeedOpt = new FlaggedOption("trapDecayRandomSeed")
		.setStringParser(JSAP.INTEGER_PARSER)
		.setDefault("13")
		.setRequired(true)
		.setLongFlag("trapDecayRandomSeed");
		jsap.registerParameter(trapDecayRandomSeedOpt);

		FlaggedOption outputFormatOpt = new FlaggedOption("outputFormat")
		.setStringParser(JSAP.STRING_PARSER)
		.setDefault("float")
		.setRequired(true)
		.setLongFlag("outputFormat");
		jsap.registerParameter(outputFormatOpt);

		FlaggedOption showStopwatchOpt = new FlaggedOption("showStopwatch")
		.setStringParser(JSAP.BOOLEAN_PARSER)
		.setDefault("false")
		.setRequired(true)
		.setLongFlag("showStopwatch");
		jsap.registerParameter(showStopwatchOpt);

		FlaggedOption skyPerPixelOpt = new FlaggedOption("skyPerPixel")
		.setStringParser(JSAP.DOUBLE_PARSER)
		.setDefault("0.0")
		.setRequired(true)
		.setLongFlag("skyPerPixel");
		jsap.registerParameter(skyPerPixelOpt);

		FlaggedOption gainOpt = new FlaggedOption("gain")
		.setStringParser(JSAP.DOUBLE_PARSER)
		.setDefault("1.0")
		.setRequired(true)
		.setLongFlag("gain");
		jsap.registerParameter(gainOpt);

		FlaggedOption rangeOpt = new FlaggedOption("range")
		.setStringParser(JSAP.STRING_PARSER)
		.setDefault("default")
		.setRequired(true)
		.setLongFlag("range");
		jsap.registerParameter(rangeOpt);

		FlaggedOption verbosityOpt = new FlaggedOption("verbosity")
		.setStringParser(JSAP.INTEGER_PARSER)
		.setDefault("0")
		.setRequired(true)
		.setLongFlag("verbosity");
		jsap.registerParameter(verbosityOpt);

		FlaggedOption quantizeChargeOpt = new FlaggedOption("quantizeCharge")
		.setStringParser(JSAP.BOOLEAN_PARSER)
		.setDefault("true")
		.setRequired(true)
		.setLongFlag("quantizeCharge");
		jsap.registerParameter(quantizeChargeOpt);

		FlaggedOption quantizeTrapsOpt = new FlaggedOption("quantizeTraps")
		.setStringParser(JSAP.BOOLEAN_PARSER)
		.setDefault("true")
		.setRequired(true)
		.setLongFlag("quantizeTraps");
		jsap.registerParameter(quantizeTrapsOpt);

		FlaggedOption nLevelsOpt = new FlaggedOption("nLevels")
		.setStringParser(JSAP.INTEGER_PARSER)
		.setDefault("1")
		.setRequired(true)
		.setLongFlag("nLevels");
		jsap.registerParameter(nLevelsOpt);

		config = jsap.parse(args);

		if (!config.success() ) {
			System.err.println();
			Iterator<?> iterator = config.getErrorMessageIterator();
			while (iterator.hasNext()) {
				Object next = iterator.next();
				System.err.println(next);
			}
			System.err.println();
			System.err.print("Usage: java "
					+ ClockCharge.class.getName());
			System.err.println(" "
					+ jsap.getUsage());
			System.err.println();
			throw new JSAPException();
		}

		userSpecifiedParallelCti = config.userSpecified("parallelCti");
		trapLocationRandomSeed = config.getInt("trapLocationRandomSeed");
		trapDecayRandomSeed = config.getInt("trapDecayRandomSeed");
		parallelTrapsPerPixels = parse(config.getString("parallelTrapsPerPixel"));
		parallelTrapLifetimes = parse(config.getString("parallelTrapLifetime"));
		serialTrapsPerPixels = parse(config.getString("serialTrapsPerPixel"));
		serialTrapLifetimes = parse(config.getString("serialTrapLifetime"));
		parallelFullWell = config.getDouble("parallelFullWell");
		parallelPower = config.getDouble("parallelPower");
		parallelNotchDepth = config.getDouble("parallelNotchDepth");
		nLevels = config.getInt("nLevels");
		serialFullWell = config.getDouble("serialFullWell");
		serialPower = config.getDouble("serialPower");
		serialNotchDepth = config.getDouble("serialNotchDepth");
		testTag = config.getString("testTag")+"-";
		sky = config.getDouble("skyPerPixel");
		gain = config.getDouble("gain");
		testRow = config.getInt("testRow");
		testCol = config.getInt("testCol");
		testTrailLength = config.getInt("testTrailLength");
		trapLocationRandomSeed = config.getInt("trapLocationRandomSeed");
		trapDecayRandomSeed = config.getInt("trapDecayRandomSeed");
		quantizeTraps = config.getBoolean("quantizeTraps"); 
		quantizeCharge = config.getBoolean("quantizeCharge"); 
		nToDo = config.getInt("testNToDo");
		verbosity = config.getInt("verbosity");
		String outputFormat = config.getString("outputFormat");
		if (outputFormat.equalsIgnoreCase("float")) {
			format = FitsDataFormat.FLOAT;
		} else if (outputFormat.equalsIgnoreCase("double")) {
			format = FitsDataFormat.DOUBLE;
		} else if (outputFormat.equalsIgnoreCase("ushort")) {
			format = FitsDataFormat.USHORT;
		} else {
			throw new IllegalArgumentException("format must be one of float, double, ushort but format="+outputFormat);
		}
		parallelCtiChargeNought = config.getDouble("parallelCtiChargeNought");
		serialCtiChargeNought = config.getDouble("serialCtiChargeNought");
		testTruth = config.getString("testTruth");
	}
	/**
	 * @param nLevels number of levels used in mean traps
	 */
	public void setNLevels(int nLevels) {
		this.nLevels = nLevels;
	}

	/**
	 * @param sky the sky level in electrons/pixel
	 */
	public void setSky(double sky) {
		this.sky = sky;
	}

	/**
	 * @param gain in electrons/adu
	 */
	public void setGain(double gain) {
		this.gain = gain;
	}

	/**
	 * @param verbosity 0 for quiet, 1 for a little, 2 for a lot of output
	 */
	public void setVerbosity(int verbosity) {
		this.verbosity = verbosity;
	}

	/**
	 * @param quantizeTraps whether to quantize traps in the doTest methods
	 */
	public void setQuantizeTraps(boolean quantizeTraps) {
		this.quantizeTraps = quantizeTraps;
		if (!quantizeTraps) {
			this.quantizeCharge=false;
		}
	}

	/**
	 * zero out the parallel traps per pixel, used in ClockChargeRunner
	 */
	public void setParallelTrapsPerPixelZero() {
		for (int i=0; i<parallelTrapsPerPixels.length; i++) {
			parallelTrapsPerPixels[i] = 0;
		}
	}
	/**
	 * zero out the serial traps per pixel, used in ClockChargeRunner
	 */
	public void setSerialTrapsPerPixelZero() {
		for (int i=0; i<serialTrapsPerPixels.length; i++) {
			serialTrapsPerPixels[i] = 0;
		}			
	}

	/**
	 * @param charge
	 * @param notchDepth
	 * @param fullWell
	 * @param power
	 * @return 0 if charge<notchDepth or fullWell<notchDepth; otherwise the charge height = ((charge-notchDepth)/(fullWell-notchDepth))^power
	 */
	public static double chargeHeightFraction(double charge, double notchDepth, double fullWell, double power ) {
		if (fullWell < notchDepth) return 0.0;
		if (charge < notchDepth) return 0.0;
		double retval = Math.pow((charge-notchDepth)/(fullWell-notchDepth), power);
		return retval;
	}
}
