package gov.fnal.eag.sim.pixel;

import gov.fnal.eag.sim.repository.image.ImageRepository;

import com.martiansoftware.jsap.FlaggedOption;
import com.martiansoftware.jsap.JSAP;
import com.martiansoftware.jsap.JSAPResult;
import com.martiansoftware.jsap.UnflaggedOption;

/**
 * combines pixeled images from FITS files
 * @author stoughto
 *
 */
public class Drizzle {

	/**
	 * 
	 * @param args optional flags --drizzle and --outputFileName, and list of files to drizzle together
	 * @throws Exception if there is trouble registering parameters (coding error!)
	 */
	public static void main(String[] args) throws Exception {
		JSAP jsap = new JSAP();
        FlaggedOption drizzleOpt = new FlaggedOption("drizzle")
        .setStringParser(JSAP.DOUBLE_PARSER)
        .setDefault("0.5")
        .setRequired(true)
        .setShortFlag('d')
        .setLongFlag("drizzle");
        jsap.registerParameter(drizzleOpt);
        
        FlaggedOption outputFileNameOpt = new FlaggedOption("outputFileName")
        .setStringParser(JSAP.STRING_PARSER)
        .setDefault("drizzleOut.fit")
        .setRequired(true)
        .setShortFlag('o')
        .setLongFlag("outputFileName");
        jsap.registerParameter(outputFileNameOpt);

        String imageRepositoryDefault = "imageRepositoryDefault";
        FlaggedOption imageRepositoryOpt = new FlaggedOption("imageRepository")
        .setStringParser(JSAP.STRING_PARSER)
        .setDefault(imageRepositoryDefault)
        .setRequired(true)
        .setShortFlag('i')
        .setLongFlag("imageRepository");
        jsap.registerParameter(imageRepositoryOpt);
        
        UnflaggedOption inputFileNamesOpt = new UnflaggedOption("inputFileNames")
        .setStringParser(JSAP.STRING_PARSER)
        .setDefault("none")
        .setRequired(true)
        .setGreedy(true);
        jsap.registerParameter(inputFileNamesOpt);

        
        JSAPResult config = jsap.parse(args);    

        // check whether the command line was valid, and if it wasn't,
        // display usage information and exit.
        if (!config.success() ) {
            System.err.println();
            System.err.print("Usage: java "
                                + Drizzle.class.getName());
            System.err.println(" "
                                + jsap.getUsage());
            System.err.println();
            System.exit(1);
        }        
        double drizzle = config.getDouble("drizzle");
        String outputFileName = config.getString("outputFileName");
        String[] inputFileNames = config.getStringArray("inputFileNames");

        String imageRepository = config.getString("imageRepository");
        ImageRepository ir;
        if (imageRepository.equals(imageRepositoryDefault)) {
        	ir = new ImageRepository();
        } else {
        	ir = new ImageRepository(imageRepository);
        }
        
        PixeledImage destination = ir.getPixeledImage(inputFileNames[0]);
        for (int i=1; i<inputFileNames.length; i++) {
        	PixeledImage source = ir.getPixeledImage(inputFileNames[i]);
        	PixeledImage.drizzle(destination, source, drizzle);
        }
        ir.put(outputFileName, destination, true);
	}

}
