/**
 * ElectronTrap.java stoughto Jan 25, 2007 11:02:06 AM
 */
package gov.fnal.eag.sim.pixel;

import gov.fnal.eag.sim.random.SimRandom;

import java.util.ArrayList;

/**
 * @author stoughto
 *
 */
public interface ElectronTrap {
	
	/**
	 * @return the number of electrons in this trap
	 */
	public double getContents();

	/**
	 * see if the trap decays in this duration and empty the trap if it does
	 * @param time in seconds
	 * @param quantizeCharge if true, use random exponential number to decide whether or not to decay; otherwise, return the mean
	 * @param chargeHeight effective height of charge in the pixel
	 * @return amount of charge released in this duration
	 */
	public double decay(double time, boolean quantizeCharge, double chargeHeight);
	
	/**
	 * see if this trap will absorb an electron
	 * @param chargeIn the amount of charge in the pixel
	 * @return the amount of charge absorbed by the trap
	 */
	public double absorb(double chargeIn);

	/**
	 * @return the amount of charge released from this trap
	 */
	public double releaseTrappedCharge();

	/**
	 * @param decaySeed
	 */
	public void setDecaySeed(int decaySeed);

	/**
	 * @return SimRandom associated with this trap
	 */
	public ArrayList<SimRandom> getSimRandoms();

	
	/**
	 * @return the location of this trap in a pixel, from 0.0 to 1.0 inclusive
	 */
	public double getTrapHeightFraction();
	
	/**
	 * @return the amount of charge this trap can hold
	 */
	public double getTrapDepth();
	

}
