package gov.fnal.eag.sim.pixel;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.geom.Point2D;

/**
 * things to put on plots
 * @author stoughto
 *
 */
public abstract class ImageGlyph {
	private static final Color DEFAULT_FG_COLOR = Color.white;
	private static final Color DEFAULT_BG_COLOR = Color.black;
	private Point2D.Double location;
	private Color fgColor = ImageGlyph.DEFAULT_FG_COLOR;
	private Color bgColor = ImageGlyph.DEFAULT_BG_COLOR;
	private boolean pixelCoordinates = true;
	/**
	 * @return the background color
	 * 
	 */
	public Color getBgColor() {
		return bgColor;
	}
	/**
	 * @param bgColor the background color
	 * @return this glyph
	 */
	public ImageGlyph setBgColor(Color bgColor) {
		this.bgColor = bgColor;
		return this;
	}
	/**
	 * @return the foreground color
	 */
	public Color getFgColor() {
		return fgColor;
	}
	/**
	 * @param fgColor the foreground color
	 * @return this glyph
	 */
	public ImageGlyph setFgColor(Color fgColor) {
		this.fgColor = fgColor;
		return this;
	}
	/**
	 * @return the location of this glyph
	 */
	public Point2D.Double getLocation() {
		return location;
	}
	/**
	 * @param location the location of this glyph
	 * @return this glyph
	 */
	public ImageGlyph setLocation(Point2D.Double location) {
		this.location = location;
		return this;
	}
	
	/**
	 * @return true if the location is in pixel coordinates
	 */
	public boolean isPixelCoordinates() {
		return pixelCoordinates;
	}
	/**
	 * @param pixelCoordinates
	 * @return this glyph
	 */
	public ImageGlyph setPixelCoordinates(boolean pixelCoordinates) {
		this.pixelCoordinates = pixelCoordinates;
		return this;
	}
	/**
	 * render this glyph in the graphics
	 * @param g2d
	 */
	public abstract void render(Graphics2D g2d);
}
