package gov.fnal.eag.sim.pixel;


/**
 * captures behavior of LookupTables for converting between electrons and compressed ADU values
 * @author stoughto
 *
 */
public abstract class LookupTable {
	/**
	 * @return the number of bits in the compressed values
	 */
	public abstract int getNBits();
	/**
	 * @param electrons number of electrons
	 * @return compressed value
	 */
	public abstract int compress(double electrons);
	
	/**
	 * @param compressed
	 * @return number of electrons
	 */
	public abstract double restore(int compressed);

	/**
	 * @param compressed
	 * @return the number of original values that map to this compressed value
	 */
	public abstract int nValues(int compressed);
	
	/**
	 * compress and restore the values in the PixeledImage
	 * @param pi 
	 */
	public void compressDecompress(PixeledImage pi) {
		for (int row=pi.getRowMin(); row<pi.getRowMax(); row++) {
			for (int col=pi.getColumnMin(); col<pi.getColumnMax(); col++) {
				double value = pi.getByRowCol(row,col);
				int compressed = compress(value);
				double restored = restore(compressed);
				pi.setByRowCol(row, col, restored);
			}
		}
	}
}
