
package gov.fnal.eag.sim.pixel;

//import javax.media.jai.KernelJAI;
import gov.fnal.eag.wcs.Coordinate;

import java.awt.geom.Point2D;

//import javax.vecmath.Point2d;

/**
 * a point spread function
 */
public interface PSF {

	
	/**
	 * generate the next point, relative to the center, using this Psf.
	 * @return Point2d
	 */
	public Point2D.Double nextPoint();

	/**
	 * get the normalized flux per pixel
	 * @param location the point where you want the flux
	 * @return the flux
	 */
	public double intensity(Point2D.Double location);
	
	/**
	 * number of rows over which the Psf is defined
	 * @return the number of rows
	 */
	public int nrow();

	/**
	 * number of columns over which the Psf is defined
	 * @return the number of columns
	 */
	public int ncol();
	/**
	 * get JAI kernel for PSF convolution.
	 * @return JAI kernel 
	 */
	
	//public KernelJAI getKernel();
	public PixeledImage getKernel();

	/**
	 * get the eigenvalues that describe this PSF
	 * @return the eigenvalues
	 */
	public double[][] getEigenvalues();
	/**
	 * convolve the image and replace the pixel values
	 * @param pixeledImage the image to convolve
	 */
	public void convolve(PixeledImage pixeledImage) ;

	/**
	 * convolve the image and replace the pixel values
	 * @param pixeledImage the image to convolve
	 * @param backgroundLight model to get values outside of the image
	 */
	public void convolve(PixeledImage pixeledImage, BackgroundLight backgroundLight);

	/**
	 * get the shapelet scale for this PSF
	 * @return the scale
	 */
	public double getScale();	

	
	/**
	 * convolve with the other psf and replace the values in this psf
	 * @param psf the other psf
	 */
	public void convolve(PSF psf);

	/**
	 * change the scale of the PSF
	 * @param factor by multiplying it by this conversion factor
	 */
	public void multiplyScale(double factor);	
	
	/**
	 * @return the location of this PSF in the focal plane
	 */
	public Coordinate getXyfpMeters();
}
