
package gov.fnal.eag.sim.pixel;

import gov.fnal.eag.wcs.Coordinate;
import gov.fnal.eag.wcs.SphericalCoordinate;



/**
 * produce an appropriate PSF at a pixel position
 */
public interface PSFFactory {
	/**
	 * get the PSF at a position on the CCD
	 * @param coordinate  in row,col position
	 * @return the PSF in pixel units
	 */
	public PSF getPsf(Coordinate coordinate);

	/**
	 * @param sc
	 * @return the psf at this position
	 */
	public PSF getPsf(SphericalCoordinate sc);
	/**
	 * @param detectorObject
	 * @return the psf in pixel units
	 */
	public PSF getPsf(DetectorObject detectorObject);	

	/**
	 * @param pixeledImage image to convolve
	 * @throws Exception 
	 */
	public void convolve(PixeledImage pixeledImage) throws Exception;
	
	/**
	 * @param bright image to convolve
	 * @param backgroundLight model to calculate light outside of the image
	 */
	public void convolve(PixeledImage bright, BackgroundLight backgroundLight);
	
	/**
	 * @param sc where the bore site of the telescope points
	 */
	public void moveTelescope(SphericalCoordinate sc);
}
