package gov.fnal.eag.sim.pixel;

import gov.fnal.eag.sim.random.SimRandomUniform;
import gov.fnal.eag.wcs.Coordinate;
import gov.fnal.eag.wcs.SphericalCoordinate;

/**
 * always return the same sech psf
 */
public class PSFFactoryConstantSech implements PSFFactory {
	/**
	 * the psf for this factory
	 */
	private static PSF psf;
	/**
	 * fully-specified constructor
	 * @param sigma sigma of the psf
	 * @param radiusMax the maximum radius to consider
	 * @param nPoints the number of points to use in Profiles
	 * @param simRandomUniform SimRandomUniform generator
	 */
	public PSFFactoryConstantSech(double sigma, double radiusMax, int nPoints, SimRandomUniform simRandomUniform) {
		psf = new PSFSech(sigma, radiusMax, nPoints, simRandomUniform);
	}
	
	/**
	 * convenience constructor with sigma of 5 pixels and default Profiles
	 */
	public PSFFactoryConstantSech() {
		this(5.0);
	}

	/**
	 * convenience constructor
	 * @param sigma with this sigma in pixels
	 */
	public PSFFactoryConstantSech(double sigma) {
		this(sigma, 10.0*sigma, 3000, new SimRandomUniform());
	}

	/**
	 * @see gov.fnal.eag.sim.pixel.PSFFactory#getPsf(gov.fnal.eag.sim.pixel.DetectorObject)
	 */
	public PSF getPsf(DetectorObject detectorObject) {
		// You can get row, col from detectorObject if you need them.
		return psf;
	}

	/**
	 * @see gov.fnal.eag.sim.pixel.PSFFactory#getPsf(gov.fnal.eag.wcs.Coordinate)
	 */
	public PSF getPsf(Coordinate coordinate) {
		return psf;
	}

	/**
	 * @see gov.fnal.eag.sim.pixel.PSFFactory#convolve(gov.fnal.eag.sim.pixel.PixeledImage)
	 */
	public void convolve(PixeledImage pixeledImage) throws Exception {
			// lucky for us the PSF is constant, so this is easy!
			psf.convolve(pixeledImage);
			return;
	}

	/**
	 * @see gov.fnal.eag.sim.pixel.PSFFactory#convolve(gov.fnal.eag.sim.pixel.PixeledImage, gov.fnal.eag.sim.pixel.BackgroundLight)
	 */
	public void convolve(PixeledImage pixeledImage, BackgroundLight backgroundLight) {
		psf.convolve(pixeledImage, backgroundLight);
		return;
	}

	/**
	 * @see gov.fnal.eag.sim.pixel.PSFFactory#moveTelescope(gov.fnal.eag.wcs.SphericalCoordinate)
	 */
	public void moveTelescope(SphericalCoordinate sc) {
		// TODO Auto-generated method stub
		
	}

	/**
	 * @see gov.fnal.eag.sim.pixel.PSFFactory#getPsf(gov.fnal.eag.wcs.SphericalCoordinate)
	 */
	public PSF getPsf(SphericalCoordinate sc) {
		return psf;
	}
}
