package gov.fnal.eag.sim.pixel;

import gov.fnal.eag.sim.repository.image.ImageRepository;
import gov.fnal.eag.wcs.Coordinate;
import gov.fnal.eag.wcs.TransformTangentPlaneA7;

import java.awt.geom.Point2D.Double;
import java.io.File;
import java.io.InputStream;

/**
 * defines a PSF as a PixeledImage
 * @author stoughto
 *
 */
public class PSFPixeledImage implements PSF {
	PixeledImage pi;
	double pixSizeArcseconds;
	Coordinate xyfpMeters;
	/**
	 * @param fileName
	 */
	public PSFPixeledImage(String fileName) {
		File file = new File(fileName);
		if (file.canRead()) {
			pi = ImageRepository.readPixeledImage(fileName);
		} else {
			String baseFileName = "data"+File.separator+"psf"+File.separator+fileName;
			InputStream inputStream = PSFPixeledImage.class.getResourceAsStream(baseFileName);
			pi = ImageRepository.readPixeledImage(inputStream);
			if (pi == null) {
				throw new IllegalArgumentException("can not read fileName="+fileName);
			}
		}
		init();
	}
	
	/**
	 * constructor with the specified image
	 * @param pi
	 */
	public PSFPixeledImage(PixeledImage pi) {
		this.pi = pi;
		init();
	}
	
	private void init() {
		pixSizeArcseconds = pi.header.getDoubleValue("PIXSIZE");
		
		xyfpMeters = new Coordinate(
				pi.header.getDoubleValue("XFPM", 0.0),
				pi.header.getDoubleValue("YFPM", 0.0));

		
		TransformTangentPlaneA7 transform = new TransformTangentPlaneA7(pi.getColumnCount()/2.0, pi.getRowCount()/2.0, 0.0, 0.0, pixSizeArcseconds/3600.0, 0.0, 0.0);
		transform.toHeader(pi.header);
	}
	/**
	 * @see gov.fnal.eag.sim.pixel.PSF#convolve(gov.fnal.eag.sim.pixel.PixeledImage)
	 */
	public void convolve(PixeledImage pixeledImage) {
		throw new UnsupportedOperationException();
	}

	/**
	 * @see gov.fnal.eag.sim.pixel.PSF#convolve(gov.fnal.eag.sim.pixel.PixeledImage, gov.fnal.eag.sim.pixel.BackgroundLight)
	 */
	public void convolve(PixeledImage pixeledImage, BackgroundLight backgroundLight) {
		throw new UnsupportedOperationException();
	}

	/**
	 * @see gov.fnal.eag.sim.pixel.PSF#convolve(gov.fnal.eag.sim.pixel.PSF)
	 */
	public void convolve(PSF psf) {
		throw new UnsupportedOperationException();
	}

	/**
	 * @see gov.fnal.eag.sim.pixel.PSF#getEigenvalues()
	 */
	public double[][] getEigenvalues() {
		throw new UnsupportedOperationException();
	}

	/**
	 * @see gov.fnal.eag.sim.pixel.PSF#getKernel()
	 */
	public PixeledImage getKernel() {
		PixeledImage retval = new PixeledImage(pi);
		TransformTangentPlaneA7 transform = new TransformTangentPlaneA7(retval.getColumnCount()/2.0, retval.getRowCount()/2.0, 0.0, 0.0, pixSizeArcseconds/3600.0, 0.0, 0.0);
		transform.toHeader(retval.header);
		return retval;
	}

	/**
	 * @see gov.fnal.eag.sim.pixel.PSF#getScale()
	 */
	public double getScale() {
		throw new UnsupportedOperationException();
	}

	/**
	 * @see gov.fnal.eag.sim.pixel.PSF#intensity(java.awt.geom.Point2D.Double)
	 */
	public double intensity(Double location) {
		throw new UnsupportedOperationException();
	}

	/**
	 * @see gov.fnal.eag.sim.pixel.PSF#multiplyScale(double)
	 */
	public void multiplyScale(double factor) {
		throw new UnsupportedOperationException();
	}

	/**
	 * @see gov.fnal.eag.sim.pixel.PSF#ncol()
	 */
	public int ncol() {
		throw new UnsupportedOperationException();
	}

	/**
	 * @see gov.fnal.eag.sim.pixel.PSF#nextPoint()
	 */
	public Double nextPoint() {
		throw new UnsupportedOperationException();
	}

	/**
	 * @see gov.fnal.eag.sim.pixel.PSF#nrow()
	 */
	public int nrow() {
		throw new UnsupportedOperationException();
	}
	/**
	 * drizzle this PSF to a PixeledImage with a different pixel size
	 * @param newPixSizeArcseconds
	 * @return the resampled image
	 */
	public PixeledImage resample(double newPixSizeArcseconds) {
		int nRowNew = (int) Math.ceil(pi.getRowCount()*pixSizeArcseconds/newPixSizeArcseconds);
		int nColNew = (int) Math.ceil(pi.getColumnCount()*pixSizeArcseconds/newPixSizeArcseconds);
		PixeledImage retval = new PixeledImage(nRowNew, nColNew);
		TransformTangentPlaneA7 transform = new TransformTangentPlaneA7(retval.getColumnCount()/2.0, retval.getRowCount()/2.0, 0.0, 0.0, newPixSizeArcseconds/3600.0, 0.0, 0.0);
		transform.toHeader(retval.header);
		PixeledImage.drizzle(retval, pi, 0.5);
		return retval;
	}

	/**
	 * @see gov.fnal.eag.sim.pixel.PSF#getXyfpMeters()
	 */
	public Coordinate getXyfpMeters() {
		return xyfpMeters;
	}
	

}
