package gov.fnal.eag.sim.pixel;

import gov.fnal.eag.shapelets.functions.BasisFunction;
import gov.fnal.eag.shapelets.functions.Convolution;
import gov.fnal.eag.wcs.Coordinate;

import java.awt.geom.Point2D.Double;

/**
 * defines a PSF by shapelet coefficients
 * @author stoughto
 *
 */
public class PSFShapelet implements PSF {

	Coordinate coordinate = new Coordinate(0,0);
	private double[][] eigenvalues;
	private double scale;
	/**
	 * construct a PSF defined by a shapelet
	 * @param coef the shapelet coefficients
	 * @param scale scale, in arcseconds
	 */
	public PSFShapelet(double[][] coef, double scale) {
		eigenvalues = coef;
		this.scale = scale;
	}
	/**
	 * construct a PSF defined by a shapelet
	 * @param coef the shapelet coefficients
	 * @param scale scale, in arcseconds
	 */
	public PSFShapelet(double[] coef, double scale) {
		eigenvalues = BasisFunction.vector2Tab(coef);
		this.scale = scale;
	}
	
	/**
	 * @see gov.fnal.eag.sim.pixel.PSF#convolve(gov.fnal.eag.sim.pixel.PixeledImage)
	 */
	public void convolve(PixeledImage pixeledImage) {
		throw new UnsupportedOperationException("need to implement this");
	}

	/**
	 * @see gov.fnal.eag.sim.pixel.PSF#convolve(gov.fnal.eag.sim.pixel.PixeledImage, gov.fnal.eag.sim.pixel.BackgroundLight)
	 */
	public void convolve(PixeledImage pixeledImage, BackgroundLight backgroundLight) {
		throw new UnsupportedOperationException("need to implement this");
	}

	/**
	 * @see gov.fnal.eag.sim.pixel.PSF#getEigenvalues()
	 */
	public double[][] getEigenvalues() {
		return eigenvalues;
	}

	/**
	 * @see gov.fnal.eag.sim.pixel.PSF#getKernel()
	 */
	public PixeledImage getKernel() {
		throw new UnsupportedOperationException("need to implement this");
	}

	/**
	 * @see gov.fnal.eag.sim.pixel.PSF#getScale()
	 */
	public double getScale() {
		return scale;
	}

	/**
	 * @see gov.fnal.eag.sim.pixel.PSF#intensity(java.awt.geom.Point2D.Double)
	 */
	public double intensity(Double location) {
		throw new UnsupportedOperationException("need to implement this");
	}

	/**
	 * @see gov.fnal.eag.sim.pixel.PSF#ncol()
	 */
	public int ncol() {
		throw new UnsupportedOperationException("need to implement this");
	}

	/**
	 * @see gov.fnal.eag.sim.pixel.PSF#nextPoint()
	 */
	public Double nextPoint() {
		throw new UnsupportedOperationException("need to implement this");
	}

	/**
	 * @see gov.fnal.eag.sim.pixel.PSF#nrow()
	 */
	public int nrow() {
		throw new UnsupportedOperationException("need to implement this");
	}

	/**
	 * @see gov.fnal.eag.sim.pixel.PSF#convolve(gov.fnal.eag.sim.pixel.PSF)
	 */
	public void convolve(final PSF that) {
		double convolvedScale = Math.sqrt(scale*scale + that.getScale()*that.getScale());
		double [][] thatEigenvalues = that.getEigenvalues();
		int maxNOrder = Math.max(eigenvalues.length, thatEigenvalues.length);
		Convolution convolution = new Convolution(scale, that.getScale(), maxNOrder-1);
		double[][] convolvedEigenvalues = convolution.convolve(eigenvalues, thatEigenvalues);
		eigenvalues = convolvedEigenvalues;
		scale = convolvedScale;
	}

	/**
	 * @see gov.fnal.eag.sim.pixel.PSF#multiplyScale(double)
	 */
	public void multiplyScale(double factor) {
		scale *= factor;		
	}
	
	/**
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		String retval = super.toString()+" scale="+scale;
		return retval;
	}
	/**
	 * @see gov.fnal.eag.sim.pixel.PSF#getXyfpMeters()
	 */
	public Coordinate getXyfpMeters() {
		return coordinate;
	}

}
