/**
 * Pixel.java stoughto Jan 25, 2007 11:35:55 AM
 */
package gov.fnal.eag.sim.pixel;

import gov.fnal.eag.sim.random.SimRandom;

import java.util.ArrayList;

/**
 * @author stoughto
 *
 */
public interface Pixel {
	/**
	 * @return the amount of charge in this pixel
	 */
	public double getCharge();
	/**
	 * @param newCharge new charge coming in
	 * @param time time to accomplish charge transfer (seconds)
	 * @param quantizeCharge true if charge is quantized
	 * @return the amount of charge transferred out
	 */
	public double transferCharge(double newCharge, double time, boolean quantizeCharge);
	
	/**
	 * put charge into the pixel
	 * @param newCharge
	 */
	public void setCharge(double newCharge);
	/**
	 * @return the number of electrons trapped in the pixel
	 */
	public double getNTrapped();
	
	/**
	 * @param charge
	 * @return a copy of this pixel with the specified amount of charge
	 */
	public Pixel copy(double charge);
	
	/**
	 * @return the amount of trapped charge released from this Pixel
	 */
	public double releaseTrappedCharge();
	
	/**
	 * sets the offset of the decay seed
	 * @param decaySeed
	 */
	public void setDecaySeed(int decaySeed);
	
	/**
	 * @return the SimRandom generators associated with this pixel
	 */
	public ArrayList<SimRandom> getSimRandoms();
	/**
	 * @return the number of traps in this pixel
	 */
	public double getNTraps();
}
