/**
 * PixelCti.java stoughto Jan 25, 2007 2:16:41 PM
 */
package gov.fnal.eag.sim.pixel;

import gov.fnal.eag.sim.random.SimRandom;

import java.util.ArrayList;

/**
 * A Pixel that holds charge and transfers it out with a specified charge transfer inefficiency
 * @author stoughto
 *
 */
public class PixelCtiConstant implements Pixel {
	private double cte;
	private double charge;
	
	/**
	 * put charge in a pixel
	 * @param cti the charge transfer inefficiency
	 * @param charge the amount of charge you start with
	 */
	public PixelCtiConstant(double cti, double charge) {
		if (cti < 0.0 || cti > 1.0) {
			throw new Error("cti must be between 0.0 and 1.0 but cti="+cti);
		}
		cte = 1.0-cti;
		if (charge < 0) {
			throw new Error("charge must be >= 0 but charge="+charge);
		}
		setCharge(charge);
	}
	/**
	 * @see gov.fnal.eag.sim.pixel.Pixel#getCharge()
	 */
	public double getCharge() {
		return charge;
	}

	/**
	 * @see gov.fnal.eag.sim.pixel.Pixel#transferCharge(double, double, boolean)
	 */
	public double transferCharge(double newCharge, double time, boolean quantizeCharge) {
		double retval = cte*charge;
		charge = newCharge+(1-cte)*charge;
		return retval;
	}
	/**
	 * @see gov.fnal.eag.sim.pixel.Pixel#getNTrapped()
	 */
	public double getNTrapped() {
		return 0;
	}

	/**
	 * @see gov.fnal.eag.sim.pixel.Pixel#copy(double)
	 */
	public Pixel copy(double chargeIn) {
		PixelCtiConstant retval = new PixelCtiConstant(1-cte, chargeIn);
		return retval;
	}
	/**
	 * @see gov.fnal.eag.sim.pixel.Pixel#setCharge(double)
	 */
	public void setCharge(double charge) {
		this.charge = charge;
	}
	
	/**
	 * @see gov.fnal.eag.sim.pixel.Pixel#releaseTrappedCharge()
	 */
	public double releaseTrappedCharge() {
		return 0.0;
	}
	
	/**
	 * @see gov.fnal.eag.sim.pixel.Pixel#setDecaySeed(int)
	 */
	public void setDecaySeed(int decaySeed) {
		// nothing to do here since there are not random numbers involved
	}
	
	/**
	 * @see gov.fnal.eag.sim.pixel.Pixel#getSimRandoms()
	 */
	public ArrayList<SimRandom> getSimRandoms() {
		return new ArrayList<SimRandom>();
	}
	
	/**
	 * @see gov.fnal.eag.sim.pixel.Pixel#getNTraps()
	 */
	public double getNTraps() {
		return 0;
	}
}
