/**
 * PixelCti.java stoughto Jan 25, 2007 2:16:41 PM
 */
package gov.fnal.eag.sim.pixel;

import gov.fnal.eag.sim.random.SimRandom;

import java.util.ArrayList;

/**
 * A Pixel that holds charge and transfers it out with a specified charge transfer inefficiency
 * @author stoughto
 *
 */
public class PixelCtiSqrtQ implements Pixel {
	private double cte;
	private double chargeNought;
	private double charge;
	
	/**
	 * put charge in a pixel
	 * @param cti the charge transfer inefficiency
	 * @param charge the amount of charge you start with
	 */
	public PixelCtiSqrtQ(double cti, double charge) {
		this(cti, charge, 1620);
	}
		/**
		 * put charge in a pixel
		 * @param cti the charge transfer inefficiency
		 * @param charge the amount of charge you start with
		 * @param chargeNought the charge that has the specifies cti
		 */
		public PixelCtiSqrtQ(double cti, double charge, double chargeNought) {
		if (cti < 0.0 || cti > 1.0) {
			throw new Error("cti must be between 0.0 and 1.0 but cti="+cti);
		}
		cte = 1.0-cti;
		if (charge < 0) {
			throw new Error("charge must be >= 0 but charge="+charge);
		}
		this.charge = charge;
		this.chargeNought = chargeNought;
	}
	/**
	 * @see gov.fnal.eag.sim.pixel.Pixel#getCharge()
	 */
	public double getCharge() {
		return charge;
	}

	/**
	 * @see gov.fnal.eag.sim.pixel.Pixel#transferCharge(double, double, boolean)
	 */
	public double transferCharge(double newCharge, double time, boolean quantizeCharge) {
		if (charge > 0) {
			double effectiveCti = Math.min(1.0, (1-cte)*Math.sqrt(chargeNought/charge));
			double effectiveCte = 1-effectiveCti;
			double retval = effectiveCte*charge;
			setCharge(newCharge+(1-effectiveCte)*charge);
			return retval;
		} 
		setCharge(newCharge);
		return 0;
	}
	/**
	 * @see gov.fnal.eag.sim.pixel.Pixel#getNTrapped()
	 */
	public double getNTrapped() {
		return 0.0;
	}
	
	/**
	 * @see gov.fnal.eag.sim.pixel.Pixel#copy(double)
	 */
	public Pixel copy(double chargeIn) {
		PixelCtiSqrtQ retval = new PixelCtiSqrtQ(1-cte, chargeIn);
		return retval;
	}
	/**
	 * @see gov.fnal.eag.sim.pixel.Pixel#setCharge(double)
	 */
	public void setCharge(double charge) {
		this.charge = charge;
	}
	/**
	 * @see gov.fnal.eag.sim.pixel.Pixel#releaseTrappedCharge()
	 */
	public double releaseTrappedCharge() {
		return 0.0;		
	}
	
	/**
	 * @see gov.fnal.eag.sim.pixel.Pixel#setDecaySeed(int)
	 */
	public void setDecaySeed(int decaySeed) {
		//nothing to do here since no random numbers are involved.
	}
	
	/**
	 * @see gov.fnal.eag.sim.pixel.Pixel#getSimRandoms()
	 */
	public ArrayList<SimRandom> getSimRandoms() {
		return new ArrayList<SimRandom>();
	}
	
	/**
	 * @see gov.fnal.eag.sim.pixel.Pixel#getNTraps()
	 */
	public double getNTraps() {
		return 0;
	}

}
