package gov.fnal.eag.sim.pixel;

import java.io.PrintStream;

import gov.fnal.eag.sim.random.SimRandomNormal;

/**
 * add gaussian noise to the input value
 */
public class PixelValueTransformationConstantGaussianNoise implements PixelValueTransformation {
	/**
	 * useful name for this transformation
	 */
	private String name;
	
	/**
	 * sigma of the normal distribution
	 */
	private double sigma;

	/**
	 * normal random number generator
	 */
	SimRandomNormal simRandomNormal;

	/**
	 * constructor with the specified sigma and random number generator
	 * @param sigma sigma of the gaussian distribution
	 * @param name an informative name for this transform
	 * @param simRandomNormal normal random number generator
	 */
	public PixelValueTransformationConstantGaussianNoise(double  sigma, String name, SimRandomNormal simRandomNormal) {
		this.sigma = sigma;
		this.name = name;
		this.simRandomNormal = simRandomNormal;
	}
	/**
	 * default constructor with sigma of zero and random number generator with default seed
	 * @throws Exception if there is trouble
	 *
	 */
	public PixelValueTransformationConstantGaussianNoise() throws Exception {
		this(0.0, "zero width gaussian", new SimRandomNormal());
	}
	/**
	 * transform the value
	 * @param x original value
	 * @return transformed value
	 */
	public double transform(double x) {
		return x + sigma*simRandomNormal.next();
	}

	/**
	 * @see gov.fnal.eag.sim.pixel.PixelValueTransformation#transform(double, double, double)
	 */
	public double transform(double x, double row, double col) {
		return transform(x);
	}
	/**
	 * @see gov.fnal.eag.sim.pixel.PixelValueTransformation#getName()
	 */
	public String getName() {
		return name;
	}


	/**
	 * set the seed for the SimRandomNormal 
	 * @param seedIndex
	 */
	public void setSeed(int seedIndex) {
		simRandomNormal.setIseedIndex(seedIndex);
	}

	/**
	 * @see gov.fnal.eag.sim.pixel.PixelValueTransformation#report(java.io.PrintStream)
	 */
	public void report(PrintStream ps) {
		ps.println("      name="+name+"  gauss noise sigma="+sigma);	
	}

}
