package gov.fnal.eag.sim.pixel;

import java.io.PrintStream;

/**
 * digitize the pixel value with the number of bits specified
 */
public class PixelValueTransformationDigitize implements PixelValueTransformation {
	/**
	 * usefule name for this transformation
	 */
	private String name;
	/**
	 * number of bits to use
	 */
	private int nBits;
	private double maxValue;
	/**
	 * constructor with number of bits
	 * @param nBits specified
	 * @param name informative name
	 */
	public PixelValueTransformationDigitize(int nBits, String name) {
		this.name = name;
		this.nBits = nBits;
		maxValue = Math.pow(2.0, nBits);	
	}
	/**
	 * default constructor with 16 bits
	 *
	 */
	public PixelValueTransformationDigitize() {
		this(16, "sixteen bit digitization");
	}
	/**
	 * transform the value by digitizing it to an integer
	 * @param x original value
	 * @return transformed value
	 */
	public double transform(double x) {
		if (x < 0) {return 0.0;}
		if (x >= maxValue) {return maxValue-1;}
		return Math.floor(x);
	}

	/**
	 * transform the value by digitizing it to an integer
	 * @param x original value
	 * @param row row position, not used
	 * @param col column position, not used
	 * @return transformed value
	 */
	public double transform(double x, double row, double col) {
		return transform(x);
	}

	/**
	 * @see gov.fnal.eag.sim.pixel.PixelValueTransformation#getName()
	 */
	public String getName() {
		return name;
	}

	/**
	 * @see gov.fnal.eag.sim.pixel.PixelValueTransformation#report(java.io.PrintStream)
	 */
	public void report(PrintStream ps) {
		ps.println("      name="+name+"  digitize with nBits="+nBits);
	}


}
