package gov.fnal.eag.sim.pixel;

import java.io.PrintStream;

/**
 * divides the pixeledImage value by the corresponding value in pi
 */
public class PixelValueTransformationDividePixeledImage implements PixelValueTransformation {
	/**
	 * a useful name for this transformation
	 */
	String name = "unknown";
	/**
	 * pixeledImage to multiply
	 */
	private PixeledImage pixeledImage = null;
	
	/**
	 * constructor with the pixeledImage defined
	 * @param pixeledImage the value to divide by in the transformation
     * @param name name of this transformation
	 */
	public PixelValueTransformationDividePixeledImage(PixeledImage pixeledImage, String name) {
		this.pixeledImage = pixeledImage;
		this.name = name;
	}
	/**
	 * creates a no-op transformation
	 */
	public PixelValueTransformationDividePixeledImage() {
		this(null, "identity");
	}


	/**
	 * @see gov.fnal.eag.sim.pixel.PixelValueTransformation#transform(double, double, double)
	 */
	public double transform(double x, double row, double col) {
	   if (pixeledImage == null) {
           return x;
       }
	   return x / pixeledImage.getByRowCol(row,col);
	}

	/**
	 * @see gov.fnal.eag.sim.pixel.PixelValueTransformation#getName()
	 */
	public String getName() {
		return name;
	}

	/**
	 * @see gov.fnal.eag.sim.pixel.PixelValueTransformation#report(java.io.PrintStream)
	 */
	public void report(PrintStream ps) {
		ps.println("      name="+name+"  divide pixeledImage="+pixeledImage);
	}

}
