package gov.fnal.eag.sim.pixel;

import java.io.PrintStream;

/**
 * piecewise linear gain
 */
public class PixelValueTransformationGainPiecewiseLinear implements PixelValueTransformation {
	/**
	 * useful name of this transformation
	 */
	private String name;
	/** 
	 * array of valueIn to define linear segments
	 */
	private double[]  valueIn;
	/** 
	 * array of valueIn to define linear segments
	 */
	private double[]  valueOut;
	/**
	 * Fully-specified constructor
	 * @param valueIn input values
	 * @param valueOut output values
	 * @param name informative name
	 * @throws Exception if valueIn and valueOut are not the same length or are null
	 */
	public PixelValueTransformationGainPiecewiseLinear(double[] valueIn, double[] valueOut, String name ) throws Exception {
		if (valueIn == null || valueOut == null ||  valueIn.length != valueOut.length || valueIn.length < 2) {
			throw new Exception ("lengths of x and y must be the same and > 1");
		}
		this.valueIn = valueIn;
		this.valueOut = valueOut;
		this.name = name;
	}
	/**
	 * applies linear gain, piecewise, to the value
	 * @param x input value
	 * @return transformed value
	 */
	public double transform(double x) {
		int index = Profiles.binarySearch(valueIn, x);
		if (index == 0) {
			return valueOut[0];
		} else if (index >= valueOut.length){
			return valueOut[valueOut.length-1];
		} else {
			return valueOut[index-1]+(x-valueIn[index-1])*(valueOut[index]-valueOut[index-1])/(valueIn[index]-valueIn[index-1]);
		}
	}

	/**
	 * apply linear gain, piecewise, to the value
	 * @param x input value
	 * @param row row position, not used
	 * @param col column position, not used
	 * @return transformed value
	 */
	public double transform(double x, double row, double col) {
		return transform(x);
	}
	/**
	 * @see gov.fnal.eag.sim.pixel.PixelValueTransformation#getName()
	 */
	public String getName() {
		return name;
	}
	/**
	 * @see gov.fnal.eag.sim.pixel.PixelValueTransformation#report(java.io.PrintStream)
	 */
	public void report(PrintStream ps) {
		ps.println("      name="+name+"  piecewise gain with n endpoints = "+valueIn.length);
		
	}
}
