package gov.fnal.eag.sim.pixel;

import java.io.PrintStream;

/**
 * multiply by a constant, usually inverse of gain(electrons/ADU) or a quantum efficiency
 */
public class PixelValueTransformationMultiplyConstant implements PixelValueTransformation {
	/**
	 * a useful name for this transformation
	 */
	String name = "unknown";
	/** 
	 * constant factor
	 */
	private double constant;
	/**
	 * constructor with the constant
	 * @param constant input
	 * @param name name of the transformation
	 */
	public PixelValueTransformationMultiplyConstant(double constant, String name) {
		this.constant = constant;
		this.name = name;		
	}
	/**
	 * default constructor multiplies by 1.0
	 *
	 */
	public PixelValueTransformationMultiplyConstant() {
		this(1.0, "identity");
	}
	/**
	 * multiplies the value by a constant
	 * @param x original value
	 * @return transformed value
	 */
	public double transform(double x) {
		return x*constant;
	}

	/**
	 * multiplies the value by a constant
	 * @param x original value
	 * @param row row position, not used
	 * @param col column position, not used
	 * @return transformed value
	 */
	public double transform(double x, double row, double col) {
		return transform(x);
	}

	/**
	 * @see gov.fnal.eag.sim.pixel.PixelValueTransformation#getName()
	 */
	public String getName() {
		return name;
	}

	/**
	 * @see gov.fnal.eag.sim.pixel.PixelValueTransformation#report(java.io.PrintStream)
	 */
	public void report(PrintStream ps) {
		ps.println("      name="+name+"  multiply constant="+constant);
		
	}
}
