package gov.fnal.eag.sim.pixel;

import java.io.PrintStream;

/**
 * keep similar PixelValueTransformation in different regions of a detector
 * @author stoughto
 *
 */
public class PixelValueTransformationSplit implements PixelValueTransformation {

	int[] splitColumns;
	PixelValueTransformation[] pvts;
	
	/**
	 * construct pvts.length regions, where splitColums.lengt+1 == pvts.length
	 * @param splitColumns column location where a new region begins
	 * @param pvts the PixelValueTransformation
	 */
	public PixelValueTransformationSplit(int[] splitColumns, PixelValueTransformation[] pvts) {
		if (splitColumns.length+1 == pvts.length) {
			this.splitColumns = splitColumns;
			this.pvts = pvts;
		} else {
			throw new Error("length of splitColumns="+splitColumns.length+" must be one less than length of pvts="+pvts.length);
		}
	}
	/**
	 * @see gov.fnal.eag.sim.pixel.PixelValueTransformation#getName()
	 */
	public String getName() {
		String name = "PixelValueTransformationSplit";
		return name;
	}

	/**
	 * @see gov.fnal.eag.sim.pixel.PixelValueTransformation#report(java.io.PrintStream)
	 */
	public void report(PrintStream ps) {
		for (PixelValueTransformation pvt : pvts) {
			pvt.report(ps);
		}
	}

	/**
	 * @see gov.fnal.eag.sim.pixel.PixelValueTransformation#transform(double, double, double)
	 */
	public double transform(double x, double row, double col) {
		int i=0;
		for (int splitColumn : splitColumns) {
			if (col < splitColumn) return pvts[i].transform(x, row, col);
			i++;
		}
		return pvts[i].transform(x, row, col);
	}

}
