package gov.fnal.eag.sim.pixel;


import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.geom.Line2D;
import java.awt.geom.Point2D;

/**
 * handles drawing a vertical and horizontal line on a plot to make a cross hair
 * @author stoughto
 *
 */
public class SimpleCrosshair extends ImageGlyph {
	/**
	 * default color for drawing crosshair
	 */
	public static final Color DEFAULT_COLOR = Color.RED;
	private double radius;
	/**
	 * @param row
	 * @param col
	 * @param radius
	 */
	public SimpleCrosshair(double row, double col, double radius) {
		this(row, col, radius, DEFAULT_COLOR);
	}
	/**
	 * @param row
	 * @param col
	 * @param radius
	 * @param color
	 */
	public SimpleCrosshair(double row, double col, double radius, Color color) {
 		setLocation(new Point2D.Double(col, row));
 		this.radius = radius;
		setFgColor(color);
	}
	
	/**
	 * @see gov.fnal.eag.sim.pixel.ImageGlyph#render(java.awt.Graphics2D)
	 */
	@Override
	public void render(Graphics2D g2d) {
		final double x = getLocation().x;
		final double y = getLocation().y;
		g2d.setColor(getFgColor());
		g2d.draw(new Line2D.Double(x, y-radius, x, y+radius));
		g2d.draw(new Line2D.Double(x-radius, y, x+radius, y));
	}	
}
