package gov.fnal.eag.sim.pixel;



import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.geom.AffineTransform;
import java.awt.geom.Area;
import java.awt.geom.Ellipse2D;
import java.awt.geom.Point2D;

/**
 * handles drawing an ellipse on a plot
 * @author stoughto
 *
 */
public class SimpleEllipse extends ImageGlyph {
	
	private static final Color DEFAULT_ELLIPSE_COLOR = Color.MAGENTA;
	double scale, ellipticity, positionAngle;
	/**
	 * @param row
	 * @param col
	 * @param scale
	 * @param ellipticity 0.0 is round
	 * @param positionAngle in degrees
	 */
	public SimpleEllipse(double row, double col, double scale, double ellipticity, double positionAngle) {
		setLocation(new Point2D.Double(col,row));
		this.scale = scale;
		this.ellipticity = ellipticity;
		this.positionAngle = positionAngle;
		setFgColor(DEFAULT_ELLIPSE_COLOR);
	}
	/**
	 * @see gov.fnal.eag.sim.pixel.ImageGlyph#render(java.awt.Graphics2D)
	 */
	@Override
	public void render(Graphics2D g2d) {
		g2d.setColor(getFgColor());
		double a = scale;
		double b = (1-ellipticity)*a;
		//Ellipse2D.Double ellipse = new Ellipse2D.Double(so.getY()-a*scale, so.getX()-b*scale, 2*a*scale, 2*b*scale);
		double x = getLocation().x;
		double y = getLocation().y;
		System.out.println("SimpleEllipse.render a b x y "+a+" "+b+" "+x+" "+y);
		Ellipse2D.Double ellipse = new Ellipse2D.Double(x-a, y-b, 2*a, 2*b);
		if (Math.abs(Math.sin(Math.toRadians(positionAngle))) < 0.01) {
			g2d.draw(ellipse);
		} else {
			AffineTransform affineTransform = new AffineTransform();
			affineTransform.rotate(positionAngle*Math.PI/180, getLocation().x, getLocation().y);
			Area area = new Area(ellipse);
			Area rotatedEllipse = area.createTransformedArea(affineTransform);
			g2d.draw(rotatedEllipse);
		}
	}
	
}
