package gov.fnal.eag.sim.pixel;



import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.geom.AffineTransform;

/**
 * represents the progress of a value from min to max
 * @author stoughto
 *
 */
public class SimpleProgressBar extends ImageGlyph {
	private static final Color DEFAULT_PROGRESS_BG_COLOR = Color.darkGray;
	private static final Color DEFAULT_PROGRESS_FG_COLOR = Color.yellow;
	private double min,max,value;
	private int yOffset;
	String format = "%.2f";
	private int height;
	
	/**
	 * @param min
	 * @param max
	 * @param value
	 * @param height
	 */
	public SimpleProgressBar(double min, double max, double value, int height) {
		this(min, max, value, height, 0);
	}

	/**
	 * @param min
	 * @param max
	 * @param value
	 * @param height
	 * @param yOffset
	 */
	public SimpleProgressBar(double min, double max, double value, int height, int yOffset) {
		this.min = min;
		this.max = max;
		this.value = value;
		setBgColor(DEFAULT_PROGRESS_BG_COLOR);
		setFgColor(DEFAULT_PROGRESS_FG_COLOR);
		setHeight(height);
		this.yOffset = yOffset;
	}
	/**
	 * @param height
	 * @return this glyph
	 */
	public SimpleProgressBar setHeight(int height) {
		this.height = height;
		return this;
	}
	
	/**
	 * @return the height
	 */
	public int getHeight() {
		return height;
	}

	/**
	 * @param value
	 */
	public void setValue(double value) {
		this.value = value;
	}
	/**
	 * @see gov.fnal.eag.sim.pixel.ImageGlyph#render(java.awt.Graphics2D)
	 */
	@Override
	public void render(Graphics2D g2d) {
		AffineTransform original = g2d.getTransform();
		g2d.setTransform(new AffineTransform());
		int width = g2d.getDeviceConfiguration().getBounds().width;	
		int progress = (int) (width * (value-min)/(max-min));
		g2d.setColor(getBgColor());
		g2d.fillRect(progress, yOffset, width-progress, getHeight()+yOffset);
		g2d.setColor(getFgColor());
		g2d.fillRect(0, yOffset, progress, getHeight()+yOffset);
		g2d.setTransform(original);
	}
	
}
