package gov.fnal.eag.sim.pixel;



import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.geom.AffineTransform;
import java.awt.geom.Line2D;

/**
 * represents a scale table and color table combination
 * @author stoughto
 *
 */
public class SimpleTableBar extends ImageGlyph {
	private static final Color DEFAULT_TABLE_BG_COLOR = Color.darkGray;
	private static final Color DEFAULT_TABLE_FG_COLOR = Color.yellow;
	private ScaleTable scaleTable;
	private ColorTable colorTable;
	private int yOffset;
	String format = "%.3f";
	int nTick = 5;
	private int height;
	
	/**
	 * @param format
	 */
	public void setFormat(String format) {
		this.format = format;
	}
	
	/**
	 * @param scaleTable
	 * @param colorTable
	 * @param height
	 */
	public SimpleTableBar(ScaleTable scaleTable, ColorTable colorTable, int height) {
		this(scaleTable, colorTable, height, 0);
	}

	/**
	 * @param scaleTable
	 * @param colorTable
	 * @param height
	 * @param yOffset 
	 */
	public SimpleTableBar(ScaleTable scaleTable, ColorTable colorTable, int height, int yOffset) {
		this.scaleTable = scaleTable;
		this.colorTable = colorTable;
		setBgColor(DEFAULT_TABLE_BG_COLOR);
		setFgColor(DEFAULT_TABLE_FG_COLOR);
		setHeight(height);
		this.yOffset = yOffset;
	}
	/**
	 * @see gov.fnal.eag.sim.pixel.ImageGlyph#render(java.awt.Graphics2D)
	 */
	@Override
	public void render(Graphics2D g2d) {
		AffineTransform original = g2d.getTransform();
		g2d.setTransform(new AffineTransform());
		int width = g2d.getDeviceConfiguration().getBounds().width;	
		g2d.setColor(getBgColor());
		g2d.fillRect(0, yOffset, width, getHeight()*2);
		for (int iColor=0; iColor<ColorTable.ncolor; iColor++) {
			int min = (int) (0.1*width + iColor*(0.8*width)/ColorTable.ncolor);
			int max = (int) (0.1*width + (iColor+1)*(0.8*width)/ColorTable.ncolor);
			g2d.setColor(colorTable.get(iColor));
			g2d.fillRect(min, yOffset, max-min, getHeight());
		}
		//System.out.println("width="+width);
		double xMin = scaleTable.getXMin();
		double xMax = scaleTable.getXMax();
		//System.out.println("xMin="+xMin+" xMax="+xMax);
		double dx = (xMax-xMin)/(nTick-1);
		for (int iTick=0; iTick<nTick; iTick++) {
			double x = xMin+iTick*dx;
			int i = scaleTable.lookup(x);
			//System.out.println("iTick="+iTick+" i="+i+" x="+x);
			g2d.setColor(getFgColor());
			int ix = (int) (0.1*width + i*width*0.8/scaleTable.getNLookup());
			g2d.draw(new Line2D.Double(ix, yOffset, ix, 0.1*getHeight()+yOffset));
			String label = String.format(format, x);
			g2d.drawString(label,  ix-10, (int) (yOffset+1.8*getHeight()));
		}
		//int nColor = colorTable.ncolor;
		
		/*
		g2d.setColor(getBgColor());
		g2d.fillRect(progress, yOffset, width-progress, getHeight()+yOffset);
		g2d.setColor(getFgColor());
		g2d.fillRect(0, yOffset, progress, getHeight()+yOffset);
		*/
		g2d.setTransform(original);
	}

	/**
	 * sets the height
	 * @param height
	 */
	public void setHeight(int height) {
		this.height = height;
	}
	
	/**
	 * @return the height
	 */
	public int getHeight() {
		return height;
	}
}
