package gov.fnal.eag.sim.pixel.test;

import gov.fnal.eag.shapelets.functions.BasisFunction;
import gov.fnal.eag.shapelets.functions.Decompose;
import gov.fnal.eag.shapelets.functions.ShapeRecomp;
import gov.fnal.eag.shapelets.functions.ShapeletsTransform;
import gov.fnal.eag.sim.pixel.ColorTable;
import gov.fnal.eag.sim.pixel.PixeledImage;
import gov.fnal.eag.sim.pixel.PixeledImagePlot;
import gov.fnal.eag.sim.pixel.ScaleTable;

import java.util.Hashtable;

import com.imsl.math.Complex;
/**
 * @author kuropat
 *
 */
public class DelutionDemo {

	/**
	 * This is an example of compression - delution and translation
	 * transformations of a shapelet object
	 * @param args
	 * @throws Exception 
	 */
	public static void main(String[] args) throws Exception {


		double [][] resPic = new double[200][200];
		for (int i=0; i<200; i++) {
			for (int j=0; j<200; j++) {
				resPic[i][j] = 0.;
			}
		}
		int size = 100;
		double[][] pix = new double[size][size];
		double i0 = size/2.0;
		double j0 = size/2.0;

		double height = 1000;
		double width = 20;
		for (int i=0; i<size; i++) {
			double di=i+0.5-i0;
			for (int j=0; j<size; j++) {
				double dj=j+0.5-j0;
				double r = Math.sqrt(di*di+dj*dj);
				double flux = height-r*height/width;
				if (flux > 0) pix[i][j] = flux;
			}
		}

		// decompose the object
		double alpha = width/2.58;
		Decompose d = new Decompose(pix, i0, j0, alpha);
		int nMax = 15;
		double[][] comp = d.getFnn2(nMax);
        double[] cart = BasisFunction.tab2Vector(comp);


		double gamma1 = 0.1;
		double gamma2 = 0.1;

		
 
        Complex[] pol = ShapeletsTransform.c2p(cart);
        ShapeRecomp sr = new ShapeRecomp(size,size,100.,3.,cart,15);
		double[][] origPic = sr.getImage();

        Complex[] shear = ShapeletsTransform.dilate(pol,gamma1);
        Complex[] shear1 = ShapeletsTransform.dilate(pol,-gamma2);
        Complex[] shear2 = ShapeletsTransform.trans(pol,2*gamma1,2*gamma2);
        double[] shearCart = ShapeletsTransform.p2c(shear);
        double[] shearCart1 = ShapeletsTransform.p2c(shear1);
        double[] shearCart2 = ShapeletsTransform.p2c(shear2);
        ShapeRecomp sr1 = new ShapeRecomp(size,size,100.,3.,shearCart,15);
        double[][] gamma1Pic = sr1.getImage();
        ShapeRecomp sr2 = new ShapeRecomp(size,size,100.,3.,shearCart1,15);
        double[][] gamma2Pic = sr2.getImage();
        ShapeRecomp sr3 = new ShapeRecomp(size,size,100.,3.,shearCart2,15);
        double[][] gamma3Pic = sr3.getImage();

 

        for (int i=0; i<size;i++) {
        	for (int j=0; j<size;j++) {
        		resPic[i][j] = origPic[i][j];
        	}
        }
        	int xshift = 100;
        	int yshift = 100;
        	for (int i=0; i<size;i++) {
            	for (int j=0; j<size;j++) {
            		resPic[i+xshift][j] = gamma1Pic[i][j];
            	}
            }
           	for (int i=0; i<size;i++) {
            	for (int j=0; j<size;j++) {
            		resPic[i][j+yshift] = gamma2Pic[i][j];
            	}
           	}
          	for (int i=0; i<size;i++) {
            	for (int j=0; j<size;j++) {
            		resPic[i+xshift][j+yshift] = gamma3Pic[i][j];
            	}
           	}
        	PixeledImage pi = new PixeledImage(resPic);
        	Hashtable<String,Double> extrema = pi.extrema();
    		double xMin = extrema.get("min");
    		double xMax = extrema.get("max");
    		
    		ScaleTable st = new ScaleTable(xMin, xMax);
    		ColorTable ct = new ColorTable("rainbow");
    		PixeledImagePlot pip = new PixeledImagePlot(pi, st, ct);
    		pip.toPng("delutionImage.png", 700);		
//        	ImageRepository.write("delutionImage.fits",pi);
		}
		
	}


