package gov.fnal.eag.sim.pixel.test;


import gov.fnal.eag.shapelets.functions.Decompose;
import gov.fnal.eag.sim.pixel.ColorTable;
import gov.fnal.eag.sim.pixel.DetectorObjectSurfaceBrightness;
import gov.fnal.eag.sim.pixel.PixeledImage;
import gov.fnal.eag.sim.pixel.PixeledImagePlot;
import gov.fnal.eag.sim.pixel.ScaleTable;
import gov.fnal.eag.sim.repository.image.ImageRepository;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Stroke;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Hashtable;

import junit.framework.TestCase;

import org.jfree.chart.ChartUtilities;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.annotations.XYAnnotation;
import org.jfree.chart.annotations.XYLineAnnotation;
import org.jfree.chart.annotations.XYTextAnnotation;

/**
 * demonstrate and test Shapelets
 * @author stoughto
 *
 */
public class ShapeletDemo extends TestCase {
	
	/**
	 * make the file gov.fnal.eag.sim.pixel.test.ShapeletDemo.png
	 * 
	 */
	public void testDemo1() {
		ArrayList<XYAnnotation> annotations = new ArrayList<XYAnnotation>();
		int objectDiameter = 50;
		double scaleFactor = 10.0;
		double scale = objectDiameter/scaleFactor;
		int nOrder = 5;
		int piDiameter = nOrder*objectDiameter;
		PixeledImage pi = new PixeledImage(piDiameter, piDiameter);
		Stroke stroke = new BasicStroke();		
		for (int ix=0; ix<nOrder; ix++) {
			double col = (ix+0.5)*objectDiameter;
			for (int iy=0; iy<nOrder; iy++) {
				if (ix+iy <= nOrder-1) {
					double row = (iy+0.5)*objectDiameter;
					double[][] eigenvalues = new double[15][15];
					eigenvalues[ix][iy] = 1.0;
					String label = String.format("(%d,%d)",ix,iy);
					annotations.add(new XYTextAnnotation(label, col, row-objectDiameter/2.2));
					annotations.add(new XYLineAnnotation(col-100*scale, row, col+100*scale, row, stroke, Color.black));
					annotations.add(new XYLineAnnotation(col, row-100*scale, col, row+100*scale, stroke, Color.black));
					DetectorObjectSurfaceBrightness dosb = new DetectorObjectSurfaceBrightness(row, col, 10.0, eigenvalues, scale);
					for (int r=iy*objectDiameter; r<(iy+1)*objectDiameter; r++) {
						for (int c=ix*objectDiameter; c<(ix+1)*objectDiameter; c++) {
							double flux = dosb.getFluxByPix(r, c);
							pi.setByRowCol(r,c, flux);
						}
					}
				}
			}
		}
		Hashtable<String,Double> extrema = pi.extrema();
		double min = extrema.get("min");
		double max = extrema.get("max");
		
		ImageRepository.write("shapeletDemo.fit", pi, true);
		ScaleTable st = new ScaleTable(min,max);
		//Color[] colors = {Color.red, Color.white, Color.green};
		Color[] colors = {Color.black, Color.gray, Color.white};
		ColorTable ct = new ColorTable(colors);
		PixeledImagePlot pip = new PixeledImagePlot(pi, st, ct);
	
		JFreeChart chart = pip.getChart();
		for (XYAnnotation annotation : annotations) {
			chart.getXYPlot().addAnnotation(annotation);
		}
		
		int pngWidth = 600;
		int pngHeight = (int) (pngWidth * (double) pi.getRowCount() / pi.getColumnCount());	
		String fileName = "gov.fnal.eag.sim.pixel.test.ShapeletDemo.png";
		try {
			ChartUtilities.saveChartAsPNG(new File(fileName), chart, pngWidth, pngHeight);
		} catch (IOException e) {
			throw new Error(e);
		}		
	}
	
	/**
	 * make the file gov.fnal.eag.sim.pixel.test.ShapeletDemo2.png
	 * creates shapelet object as decomposition of the Gaussian then
	 * perform reconstruction and comparison of the first order of the 
	 * obtained image and original object
	 * 
	 */
	public void testDemo2() {
		ArrayList<XYAnnotation> annotations = new ArrayList<XYAnnotation>();
		int piDiameter = 50; // Size of the pixeled image
		//int objectDiameter = 30; // size of the object
		double[][] pix = new double[50][50];
		double scale = 3.;  //sigma
//		double center = (double)piDiameter/2. ;
		double center = 25.5 ;
		
		// First create the image
		for (int i=0; i<piDiameter; i++) {
			double x = center - i - 0.5;
			for (int j=0; j< piDiameter; j++) {
				double y = center - j - 0.5;
				double dist = Math.sqrt(x*x + y*y);
				double flux = Math.exp(-(dist*dist)/(2.*(scale*scale)))/(scale*Math.sqrt(2.*Math.PI));
				if (dist<15. ) {
					pix[i][j] = flux;
				} else {
					pix[i][j] = 0.;
				}
			}
		}
		PixeledImage origIm = new PixeledImage(pix);
		Hashtable<String,Double> extrema = origIm.extrema();
		double min = extrema.get("min");
		double max = extrema.get("max");
		Hashtable<String, Number>  centroid = origIm.centroid(0.);
		//Set<String> keys = centroid.keySet();
		//Iterator it = keys.iterator();
//		while (it.hasNext()) {
//			System.out.println("key :"+it.next());
//		}
		double rowSigma = ((Double)centroid.get("rowSigma")).doubleValue();
		double rowCentroid = ((Double)centroid.get("rowCentroid")).doubleValue();
		double colCentroid = ((Double)centroid.get("colCentroid")).doubleValue();
		System.out.println("sigma="+rowSigma+" row centroid="+rowCentroid+" colCentroid="+
				colCentroid);
		ScaleTable st = new ScaleTable(min,max);
		//Color[] colors = {Color.red, Color.white, Color.green};
		Color[] colors = {Color.black, Color.gray, Color.white};
		ColorTable ct = new ColorTable(colors);
		PixeledImagePlot pip = new PixeledImagePlot(origIm, st, ct);
	
		JFreeChart chart = pip.getChart();
		for (XYAnnotation annotation : annotations) {
			chart.getXYPlot().addAnnotation(annotation);
		}
		
		int pngWidth = 600;
		int pngHeight = (int) (pngWidth * (double) origIm.getRowCount() / origIm.getColumnCount());	
		String fileName = "gov.fnal.eag.sim.pixel.test.OrigIM.png";
		try {
			ChartUtilities.saveChartAsPNG(new File(fileName), chart, pngWidth, pngHeight);
		} catch (IOException e) {
			throw new Error(e);
		}	
		
		// Now decompose the image
		int nMax = 20;
		Decompose decomp = new Decompose(pix, 25.5, 25.5, 2.*scale);
		double[] ev1 = decomp.getFnnV(nMax);
		//double[][] ev = decomp.getFnn2(nMax);
		// and reconstruct the image
		DetectorObjectSurfaceBrightness dosb = new DetectorObjectSurfaceBrightness(
				25.5, 25.5, 10.0, ev1, 2.*scale);
		double xdet = dosb.getCol();
		double ydet = dosb.getRow();
		System.out.println(" put object at x="+xdet+" y="+ydet);
		PixeledImage pi = new PixeledImage(piDiameter, piDiameter);
		for (int i=0; i<piDiameter; i++) {
			for (int j=0; j<piDiameter; j++) {
					double r = i + 0.5;
					double c = j + 0.5;
						pi.setByRowCol(r, c, dosb.getFluxByPix(i, j));
					}
				}				
		//ShapeRecomp recomp = new ShapeRecomp(50, 50,  2.*scale, 10.0, ev, nMax);
		//double[][] pix1 = recomp.getImage();
		//PixeledImage reconIm = new PixeledImage(pix1);
		
		//Hashtable<String,Double> extrema1 = pi.extrema();
//		double min1 = extrema1.get("min");
//		double max1 = extrema1.get("max");
		Hashtable<String, Number>  centroid1 = pi.centroid(0.);
		//Set<String> keys1 = centroid1.keySet();
		//Iterator it1 = keys1.iterator();
//		while (it.hasNext()) {
//			System.out.println("key :"+it.next());
//		}
		double rowSigma1 = ((Double)centroid1.get("rowSigma")).doubleValue();
		double rowCentroid1 = ((Double)centroid1.get("rowCentroid")).doubleValue();
		double colCentroid1 = ((Double)centroid1.get("colCentroid")).doubleValue();
		System.out.println("Recon sigma="+rowSigma1+" row centroid="+rowCentroid1+" colCentroid="+
				colCentroid1);
//		ScaleTable st1 = new ScaleTable(min,max);
		//Color[] colors = {Color.red, Color.white, Color.green};
//		Color[] colors = {Color.black, Color.gray, Color.white};
//		ColorTable ct = new ColorTable(colors);
		PixeledImagePlot pip1 = new PixeledImagePlot(pi, st, ct);
	
		JFreeChart chart1 = pip1.getChart();
		for (XYAnnotation annotation : annotations) {
			chart1.getXYPlot().addAnnotation(annotation);
		}
		
//		int pngWidth = 600;
//		int pngHeight = (int) (pngWidth * (double) origIm.getRowCount() / origIm.getColumnCount());	
		String fileName1 = "gov.fnal.eag.sim.pixel.test.ReconIM.png";
		try {
			ChartUtilities.saveChartAsPNG(new File(fileName1), chart1, pngWidth, pngHeight);
		} catch (IOException e) {
			throw new Error(e);
		}	
		

	}
}
