package gov.fnal.eag.sim.pixel.wavelet;

/**
 * @author mcginnis
 *
 */
public class ArcFit extends SimplexOptimize 
{

	double[]	m_dcenter_of_g = new double[2];
	double[]	m_darc_center = new double[2];
	private double[]	m_dcos_sin_theta = new double[2];
	private PixieCollection	pc = null;
	double		m_drot_angle = 0.0;
	double		m_dlength = 0.0;
	double		m_dmin_max_width = 0.0;
	double		m_drms_width = 0.0;
	double		m_drad_curv = 0.0;
	double		m_dtheta_start;
	double		m_dtheta_stop;

	ArcFit(PixieCollection	pixieCollection, int fitOrder)
	{
		this.pc = pixieCollection;
		vCurveFit(fitOrder);
	}

	void vLocalCoords(double[] dlocal_coord, double[] dglobal_coord)
	{
		dlocal_coord[0] 
			=  m_dcos_sin_theta[0] * (dglobal_coord[0] - m_dcenter_of_g[0])
			+  m_dcos_sin_theta[1] * (dglobal_coord[1] - m_dcenter_of_g[1]);
		dlocal_coord[1] 
			= -m_dcos_sin_theta[1] * (dglobal_coord[0] - m_dcenter_of_g[0])
			+  m_dcos_sin_theta[0] * (dglobal_coord[1] - m_dcenter_of_g[1]);
	}
	void vGlobalCoords(double[] dlocal_coord, double[] dglobal_coord)
	{
		dglobal_coord[0] 
			=  m_dcos_sin_theta[0] * dlocal_coord[0] 
			-  m_dcos_sin_theta[1] * dlocal_coord[1]
			+  m_dcenter_of_g[0];
		dglobal_coord[1] 
			=  m_dcos_sin_theta[1] * dlocal_coord[0]
			+  m_dcos_sin_theta[0] * dlocal_coord[1]
			+  m_dcenter_of_g[1];
	}
	void vCurveFit(int fitOrder)
	{
		double[][]	da = null;
		double[][]	db = null;
		double[]	dxp = null;
		double[]	du = new double[3];
		double[]	dv = new double[3];		
		double		dyp;
		double		dwp;
		double[]	dulimits = new double[2];
		double[]	dvlimits = new double[2];
		double[]	dlocal_coord = new double[2];
		double[]	dglobal_coord = new double[2];
		int			ii;
		int			ij;
		int			ip;
		int			ik;
		boolean		istatus;
		int			iorder_linear;
		double		dmax;
		double		dmin;
		double[]	dm = new double[2];
		double[]	dr = new double[2];
		double		dtheta;
		double		ddelta_theta_slice;
		int[]		itheta_count = new int[401];
		int			itheta_gap_start;
		int			itheta_gap_stop;
		int			inum_theta_slices;
//		double		PI = 3.141592654;
		double		PI2 = 6.283185308;
		double		m_dsum;

		iorder_linear = 1;
		double[] fitCoef  = new double[fitOrder + 1];

		m_dsum = 0.0;
		m_dcenter_of_g[0] = 0.0;
		m_dcenter_of_g[1] = 0.0;

		for ( ip = 0; ip < pc.inumPixies; ++ip)
		{
			dwp = pc.pixieArray[ip].val;
			m_dsum = m_dsum + dwp;
			m_dcenter_of_g[0] = m_dcenter_of_g[0] 
				+ pc.pixieArray[ip].col * dwp;
			m_dcenter_of_g[1] = m_dcenter_of_g[1] 
				+ pc.pixieArray[ip].row * dwp;
		}
		m_dcenter_of_g[0] = m_dcenter_of_g[0] / m_dsum;
		m_dcenter_of_g[1] = m_dcenter_of_g[1] / m_dsum;

		if ( pc.inumPixies < 2 * (fitOrder + 1))
		{
			m_dcos_sin_theta[0] = 1.0;
			m_dcos_sin_theta[1] = 0.0;
			m_drot_angle = 0.0;
			m_dlength = 0.0;
			m_dmin_max_width = 0.0;
			m_drms_width = 0.0;
			m_drad_curv = 0.0;
			return;
		}

// add one element extra onto matrix because NR starts index at 1		
		da = new double[fitOrder + 1 + 1][fitOrder + 1 + 1];
		db = new double[fitOrder  + 1 + 1][2 + 1];
		dxp = new double[2 * fitOrder + 1];

		for (ii = 0; ii <= fitOrder; ++ii)
		{
			db[ii + 1][1] = 0.0;
			for (ij = 0; ij <= fitOrder; ++ij)
			{
				da[ii + 1][ij + 1] = 0.0;
			}
		}
		for ( ip = 0; ip < pc.inumPixies; ++ip)
		{
			dxp[0] = 1.0;
			for (ik = 1; ik <= 2 * iorder_linear; ++ik)
			{
				dxp[ik] = dxp[ik - 1] 
					* (pc.pixieArray[ip].col - m_dcenter_of_g[0]);
			}
			dyp = (pc.pixieArray[ip].row - m_dcenter_of_g[1]);
			dwp = pc.pixieArray[ip].val;
			for (ii = 0; ii <= iorder_linear; ++ii)
			{
				db[ii + 1][1] = db[ii + 1][1] + dwp * dxp[ii] * dyp;
				for (ij = 0; ij <= iorder_linear; ++ij)
				{
					da[ii + 1][ij + 1] = da[ii + 1][ij + 1] 
						+ dwp * dxp[ii + ij];
				}
			}
		}
		istatus = NR.Guassj( da, iorder_linear + 1, db, 1);

		for (ii = 0; ii <= iorder_linear; ++ii)
		{
			fitCoef[ii] = db[ii + 1][1];
		}
		m_drot_angle = Math.atan(fitCoef[1]);
		m_dcos_sin_theta[0] = Math.cos(m_drot_angle);
		m_dcos_sin_theta[1] = Math.sin(m_drot_angle);

		for (ii = 0; ii <= fitOrder; ++ii)
		{
			db[ii + 1][1] = 0.0;
			for (ij = 0; ij <= fitOrder; ++ij)
			{
				da[ii + 1][ij + 1] = 0.0;
			}
		}
		for ( ip = 0; ip < pc.inumPixies; ++ip)
		{
			dglobal_coord[0] = pc.pixieArray[ip].col;
			dglobal_coord[1] = pc.pixieArray[ip].row;
			vLocalCoords(dlocal_coord, dglobal_coord);
			dxp[0] = 1.0;
			for (ik = 1; ik <= 2 * fitOrder; ++ik)
			{
				dxp[ik] = dxp[ik - 1] * dlocal_coord[0];
			}
			dyp = dlocal_coord[1];
			dwp = pc.pixieArray[ip].val;
			for (ii = 0; ii <= fitOrder; ++ii)
			{
				db[ii + 1][1] = db[ii + 1][1] + dwp * dxp[ii] * dyp;
				for (ij = 0; ij <= fitOrder; ++ij)
				{
					da[ii + 1][ij + 1] = da[ii + 1][ij + 1] 
						+ dwp * dxp[ii + ij];
				}
			}
		}
		istatus = NR.Guassj( da, fitOrder + 1, db, 1);
		fitCoef[0] = 0.0;
		fitCoef[1] = 0.0;
		for (ii = 0; ii <= fitOrder; ++ii)
		{
			fitCoef[ii] = db[ii + 1][1];
		}
		dulimits[0] = -1.0e33;
		dulimits[1] =  1.0e33;
		dvlimits[0] = -1.0e33;
		dvlimits[1] =  1.0e33;
		for ( ip = 0; ip < pc.inumPixies; ++ip)
		{
			dglobal_coord[0] = pc.pixieArray[ip].col;
			dglobal_coord[1] = pc.pixieArray[ip].row;
			vLocalCoords(dlocal_coord, dglobal_coord);
			if (dlocal_coord[0] > dulimits[0] ) 
				dulimits[0] = dlocal_coord[0];
			if (dlocal_coord[0] < dulimits[1] ) 
				dulimits[1] = dlocal_coord[0];

			dlocal_coord[1] = dlocal_coord[1]
				- dPoly(dlocal_coord[0], fitCoef, fitOrder);
			if (dlocal_coord[1] > dvlimits[0] ) 
				dvlimits[0] = dlocal_coord[1];
			if (dlocal_coord[1] < dvlimits[1] ) 
				dvlimits[1] = dlocal_coord[1];
		}


//	 finding rad of curve be fitting 2 perp lines to curve and finding
//	 intersection
		du[0] = dulimits[0] + 0.33 * (dulimits[1] - dulimits[0]);
		du[1] = dulimits[0] + 0.67 * (dulimits[1] - dulimits[0]);
		dv[0] = dPoly(du[0], fitCoef, fitOrder);
		dv[1] = dPoly(du[1], fitCoef, fitOrder);
		dm[0] = -1.0 / dPoly1stDeriv(du[0], fitCoef, fitOrder);
		dm[1] = -1.0 / dPoly1stDeriv(du[1], fitCoef, fitOrder);
		du[2] = ( (dv[0] - dm[0] * du[0]) - (dv[1] - dm[1] * du[1]) )
			/ (dm[1] - dm[0]);
		dv[2] = ( dm[1] * (dv[0] - dm[0] * du[0]) - dm[0] * (dv[1] - dm[1] * du[1]) )
			/ (dm[1] - dm[0]);
		dr[0] = (du[0] - du[2]) * (du[0] - du[2]) 
			  + (dv[0] - dv[2]) * (dv[0] - dv[2]);
		dr[0] = Math.sqrt(dr[0]);
		dr[1] = (du[1] - du[2]) * (du[1] - du[2]) 
			  + (dv[1] - dv[2]) * (dv[1] - dv[2]);
		dr[1] = Math.sqrt(dr[0]);
		m_drad_curv = 0.5 * (dr[0] + dr[1]);

		dlocal_coord[0] = du[2];
		dlocal_coord[1] = dv[2];

		vGlobalCoords(dlocal_coord, m_darc_center);
		setupOptimization();
		Optimize(0.001);
		for ( ii = 0; ii <= 400; ++ii) itheta_count[ii] = 0;

		m_dmin_max_width = 0.0;
		m_drms_width = 0.0;
		dmax = -1.0e+33;
		dmin =  1.0e+33;
		ddelta_theta_slice = 1.5 / m_drad_curv; // 1.5 pixels
		if (ddelta_theta_slice < 0.0175) ddelta_theta_slice = 0.0175; //1 degree
		inum_theta_slices = (int) Math.floor(PI2 / ddelta_theta_slice);
		if (inum_theta_slices < 1) inum_theta_slices = 1;
		ddelta_theta_slice = PI2 / inum_theta_slices;
		for ( ip = 0; ip < pc.inumPixies; ++ip)
		{
			dglobal_coord[0] = pc.pixieArray[ip].col - m_darc_center[0];
			dglobal_coord[1] = pc.pixieArray[ip].row - m_darc_center[1];
			dtheta = Math.atan2(dglobal_coord[1], dglobal_coord[0] + 1.0e-10);
			if (dtheta < 0.0) dtheta = dtheta + PI2;
			ii = (int) Math.floor(dtheta / ddelta_theta_slice);
			itheta_count[ii] = itheta_count[ii] + 1;
			dwp = dglobal_coord[0] * dglobal_coord[0] 
				+ dglobal_coord[1] * dglobal_coord[1];
			dwp = Math.sqrt(dwp);
			dwp = dwp - m_drad_curv;
			if (dwp > dmax ) dmax = dwp;
			if (dwp < dmin ) dmin = dwp;
			m_drms_width = m_drms_width + dwp * dwp * pc.pixieArray[ip].val;
		}
		m_dmin_max_width = dmax - dmin;
		m_drms_width = 2.0 * Math.sqrt(m_drms_width / m_dsum);
		m_dlength = m_drad_curv * (m_dtheta_stop - m_dtheta_start);
//	 Find the angles where there is no arc
		itheta_gap_start = 0;
		itheta_gap_stop = 0;
		istatus = false;
		while (!istatus && (itheta_gap_start < inum_theta_slices))
		{
			if (itheta_count[itheta_gap_start] == 0)
			{
				istatus = true;
			}
			else
			{
				itheta_gap_start = itheta_gap_start + 1;
			}
		}
		if (istatus == false)
		{
			m_dtheta_start =  0.0;
			m_dtheta_stop  =  PI2;
		}
		else
		{
			while (itheta_count[itheta_gap_start] == 0)
			{
				itheta_gap_start = itheta_gap_start - 1;
				if (itheta_gap_start < 0) itheta_gap_start = inum_theta_slices - 1;
			}
			itheta_gap_start = itheta_gap_start + 1;
			if (itheta_gap_start > (inum_theta_slices - 1)) itheta_gap_start = 0;
			itheta_gap_stop = itheta_gap_start;
			while (itheta_count[itheta_gap_stop] == 0)
			{
				itheta_gap_stop = itheta_gap_stop + 1;
				if (itheta_gap_stop > (inum_theta_slices - 1)) itheta_gap_stop = 0;
			}
			itheta_gap_stop = itheta_gap_stop - 1;
			if (itheta_gap_stop < 0) itheta_gap_stop = inum_theta_slices - 1;
		}
		m_dtheta_start = (itheta_gap_stop + 0.5) * ddelta_theta_slice;
		m_dtheta_stop = (itheta_gap_start - 0.5) * ddelta_theta_slice;
		if (m_dtheta_stop < m_dtheta_start) m_dtheta_stop = m_dtheta_stop + PI2;
		m_dlength = (m_dtheta_stop - m_dtheta_start) * m_drad_curv;

		da = null;
		db = null;
		dxp = null;
	}
	private double dPoly(double dx, double[] dfit_coef, int iorder)
	{
		double	dy;
		double	dxp;
		int		ii;

		dxp = 1.0;
		dy = dfit_coef[0];
		for ( ii = 1; ii <= iorder; ++ii)
		{
			dxp = dxp * dx;
			dy = dy + dxp * dfit_coef[ii];
		}
		return dy;
	}
	private double dPoly1stDeriv(double dx, double[] dfit_coef, int iorder)
	{
		double	dy;
		double	dxp;
		int		ii;

		if (iorder < 1) return 1.0e-33;
		dy = dfit_coef[1];
		if ( iorder < 2) return dy + 1.0e-33;
		dxp = 1.0;
		for ( ii = 2; ii <= iorder; ++ii)
		{
			dxp = dxp * dx;
			dy = dy + dxp * dfit_coef[ii] * ii;
		}
		return dy + 1.0e-33;
	}
	double dPoly2ndDeriv(double dx, double[] dfit_coef, int iorder)
	{
		double	dy;
		double	dxp;
		int		ii;

		if (iorder < 2) return 0.0;
		dy = dfit_coef[2] * (2.0);
		if ( iorder < 3) return dy;
		dxp = 1.0;
		for ( ii = 3; ii <= iorder; ++ii)
		{
			dxp = dxp * dx;
			dy = dy + dxp * dfit_coef[ii] * ii * (ii - 1);
		}
		return dy;
	}
	double	dRadiusFit(boolean iuse_pixel_value)
	{
		double	dsigma;
		double	dsum;
		double	dr;
		int		ipixel;

		m_drad_curv = 0.0;
		dsum = 0.0;
		dsigma = 0.0;
		for (ipixel = 0; ipixel < pc.inumPixies; ++ipixel)
		{
			dr =  (pc.pixieArray[ipixel].col - m_darc_center[0])
				* (pc.pixieArray[ipixel].col - m_darc_center[0])
				+ (pc.pixieArray[ipixel].row - m_darc_center[1])
				* (pc.pixieArray[ipixel].row - m_darc_center[1]);
			dr = Math.sqrt(dr);
			if (iuse_pixel_value)
			{
				dsum = dsum + pc.pixieArray[ipixel].val;
				m_drad_curv= m_drad_curv + dr * pc.pixieArray[ipixel].val;
			}
			else
			{
				dsum = dsum + 1.0;
				m_drad_curv = m_drad_curv + dr;
			}
		}
		m_drad_curv = m_drad_curv / dsum;
		for (ipixel = 0; ipixel < pc.inumPixies; ++ipixel)
		{
			dr =  (pc.pixieArray[ipixel].col - m_darc_center[0])
				* (pc.pixieArray[ipixel].col - m_darc_center[0])
				+ (pc.pixieArray[ipixel].row - m_darc_center[1])
				* (pc.pixieArray[ipixel].row - m_darc_center[1]);
			dr = Math.sqrt(dr);
			dr = (dr - m_drad_curv) * (dr - m_drad_curv);
			if (iuse_pixel_value)
			{
				dsigma = dsigma + dr * pc.pixieArray[ipixel].val;
			}
			else
			{
				dsigma = dsigma + dr;
			}
		}
		dsigma = Math.sqrt(dsigma / dsum);
		return dsigma;
	}
	@Override
	void setupOptimization()
	{
		m_inum_total_variables = 2;
		m_inum_opt_variables   = 2;
		m_variable = new COptimizeVariable[m_inum_total_variables];
		m_ilookup = new int[m_inum_total_variables];
		
		m_variable[0] = new COptimizeVariable();
		m_variable[1] = new COptimizeVariable();
		
		m_FwriteAmoebaText = null;
//		m_FwriteAmoebaText = fopen("AmoebaText.txt","w");
		m_variable[0].finit_value =   m_darc_center[0];
		m_variable[0].fmin        = -1.0e6;
		m_variable[0].fmax        =  1.0e6;
		m_variable[0].fstep       =   0.2;
		m_variable[0].fvalue      =   m_darc_center[0];
		m_variable[1].finit_value =   m_darc_center[1];
		m_variable[1].fmin        = -1.0e6;
		m_variable[1].fmax        =  1.0e6;
		m_variable[1].fstep       =   0.2;
		m_variable[1].fvalue     =    m_darc_center[1];
		m_ilookup[0] = 0;
		m_ilookup[1] = 1;

	/* Iterations */
		m_iopt_limit_type = 1;
	/* Time       */
	/*	m_iopt_limit_type = 2;
	/* Chi Square */
	/*	m_iopt_limit_type = 3; */

		m_fsim_limit = 100.0;
		return;
	}
	@Override
	double fChiSquare()
	{
		m_darc_center[0] = m_variable[0].fvalue;
		m_darc_center[1] = m_variable[1].fvalue;
		return dRadiusFit(false);
	}
}
