package gov.fnal.eag.sim.pixel.wavelet;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.FlowLayout;
import java.awt.GridLayout;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTextField;
import javax.swing.UIManager;

/**
 * @author mcginnis
 *
 */
public class ArcSearchGui extends JFrame 
{
	private static final long serialVersionUID = 1L;
	private String lastDirectoryPath = null;
 	JProgressBar progressBar = null;
  	static StatusBar statusBar = new StatusBar();
	JTextField scaleMaxTextField = null;
	JTextField[] scaleCutTextField = new JTextField[10];
	JTextField[] avgRadiusTextField = new JTextField[10];
	Wavelet waveletSettings = new Wavelet();
	boolean	batchCrop = true;
	boolean batchNormalize = true;
	boolean batchFilter = true;
	boolean batchWritePngImages = true;
	int		batchPngImageSize = 160;
	int		initBatchCropImageSize = 400;
	JTextField batchPngImageSizeTextField = null;
	JTextField initBatchCropImageSizeTextField = null;
	boolean	batchWriteFitsFiles = false;
	boolean	batchArcSearch = false;
	PixieImageRGBPlotter bwRGBplotter = new PixieImageRGBPlotter();
	PixieImageRGBPlotter clrRGBplotter = new PixieImageRGBPlotter();
 	JLabel[] imageIconLabel = new JLabel[6];
 	JPanel[] colorImagePanel = new JPanel[6];
 	JScrollPane[] colorImageScrollPane = new JScrollPane[6];

	private Component createComponents() 
	{
		JTabbedPane tabbedPane = new JTabbedPane();

		JPanel progressPanel = new JPanel();
	 	progressPanel.setBorder(BorderFactory.createCompoundBorder(BorderFactory.createTitledBorder("Progress"),BorderFactory.createEmptyBorder(5,5,5,5)));
	 	progressPanel.setLayout(new BoxLayout(progressPanel, BoxLayout.Y_AXIS));
	 	progressBar = new JProgressBar();
	 	progressBar.setIndeterminate(false);
	 	progressPanel.add(progressBar);
	 	
	 	tabbedPane.addTab("Images", setupImageDisplayPanel());
	 	tabbedPane.addTab("Progress", progressPanel);
	 	tabbedPane.addTab("Batch Settings", setupBatchSettingsPanel());
	 	tabbedPane.addTab("Wavelet Settings", setupWaveletSettingsPanel());
	 	
	 	JPanel mainPanel = new JPanel();
        mainPanel.setBorder(BorderFactory.createEmptyBorder(1,3,5,5));
        mainPanel.setLayout(new GridLayout(1,4,5,5));
        mainPanel.add(tabbedPane);
                
        return mainPanel;
	}
	JPanel setupBatchSettingsPanel()
	{
		JRadioButton batchCropButton = new JRadioButton("Crop");
		batchCropButton.setSelected(batchCrop);
		batchCropButton.addActionListener(new ArcSearchGuiActions(this, "Batch Crop"));

		JRadioButton batchNormalizeButton = new JRadioButton("Normalize");
		batchNormalizeButton.setSelected(batchNormalize);
		batchNormalizeButton.addActionListener(new ArcSearchGuiActions(this, "Batch Normalize"));

		JRadioButton batchFilterButton = new JRadioButton("Filter");
		batchFilterButton.setSelected(batchFilter);
		batchFilterButton.addActionListener(new ArcSearchGuiActions(this, "Batch Filter"));

		JRadioButton batchWritePngImagesButton = new JRadioButton("Write PNG Images");
		batchWritePngImagesButton.setSelected(batchWritePngImages);
		batchWritePngImagesButton.addActionListener(new ArcSearchGuiActions(this, "Batch Write PNG Images"));

		batchPngImageSizeTextField = new JTextField(8);
		batchPngImageSizeTextField.setText(Integer.toString(batchPngImageSize));
		JLabel batchPngImageSizeLabel = new JLabel("PNG Image Size");
		JPanel batchPngImageSizePanel = new JPanel();
		batchPngImageSizePanel.setLayout(new FlowLayout());
		batchPngImageSizePanel.add(batchPngImageSizeLabel);
		batchPngImageSizePanel.add(batchPngImageSizeTextField);
		
		initBatchCropImageSizeTextField = new JTextField(8);
		initBatchCropImageSizeTextField.setText(Integer.toString(initBatchCropImageSize));
		JLabel initBatchCropImageSizeLabel = new JLabel("Cropped Image Size");
		JPanel initBatchCropImageSizePanel = new JPanel();
		initBatchCropImageSizePanel.setLayout(new FlowLayout());
		initBatchCropImageSizePanel.add(initBatchCropImageSizeLabel);
		initBatchCropImageSizePanel.add(initBatchCropImageSizeTextField);
		
		JRadioButton batchWriteFitsFilesButton = new JRadioButton("Write Fits Files");
		batchWriteFitsFilesButton.setSelected(batchWriteFitsFiles);
		batchWriteFitsFilesButton.addActionListener(new ArcSearchGuiActions(this, "Batch Write Fits Files"));

		JRadioButton batchArcSearchButton = new JRadioButton("Arc Search");
		batchArcSearchButton.setSelected(batchArcSearch);
		batchArcSearchButton.addActionListener(new ArcSearchGuiActions(this, "Batch Arc Search"));

		JPanel batchSettingsPanel1 = new JPanel();
		batchSettingsPanel1.setLayout(new BoxLayout(batchSettingsPanel1, BoxLayout.Y_AXIS));
		batchSettingsPanel1.add(batchCropButton);
		batchSettingsPanel1.add(batchNormalizeButton);
		batchSettingsPanel1.add(batchFilterButton);
		batchSettingsPanel1.add(batchWriteFitsFilesButton);
		batchSettingsPanel1.add(batchArcSearchButton);
		batchSettingsPanel1.add(batchWritePngImagesButton);
		batchSettingsPanel1.add(batchPngImageSizePanel);
		batchSettingsPanel1.add(initBatchCropImageSizePanel);

		JPanel batchSettingsPanel2 = new JPanel();
		batchSettingsPanel2.setLayout(new BoxLayout(batchSettingsPanel2, BoxLayout.Y_AXIS));

		ArcSearchGuiActions[] bwGraphicsTypeAction = new ArcSearchGuiActions[6];
		bwGraphicsTypeAction[0] = new ArcSearchGuiActions(this, "Grey Linear");
		bwGraphicsTypeAction[1] = new ArcSearchGuiActions(this, "Grey Log");
		bwGraphicsTypeAction[2] = new ArcSearchGuiActions(this, "Grey Hist Eq");
		bwGraphicsTypeAction[3] = new ArcSearchGuiActions(this, "Grey Invert Image");
		bwGraphicsTypeAction[4] = new ArcSearchGuiActions(this, "Grey Auto Scale");
		bwGraphicsTypeAction[5] = new ArcSearchGuiActions(this, "Grey Color Spectrum");
		bwRGBplotter.setScaleType("log");
		bwRGBplotter.setInvertImage(false);
		bwRGBplotter.setAutoScale(false);
		bwRGBplotter.setColorSpectrum(true);
		bwRGBplotter.setPixelValueLimits(0.0, 10.0);
		JPanel bwGraphicsTypePanel = bwRGBplotter.pngGraphicsTypePanel("Grey Scale Images", bwGraphicsTypeAction);
	 	batchSettingsPanel2.add(bwGraphicsTypePanel);
		
		ArcSearchGuiActions[] clrGraphicsTypeAction = new ArcSearchGuiActions[6];
		clrGraphicsTypeAction[0] = new ArcSearchGuiActions(this, "Color Linear");
		clrGraphicsTypeAction[1] = new ArcSearchGuiActions(this, "Color Log");
		clrGraphicsTypeAction[2] = new ArcSearchGuiActions(this, "Color Hist Eq");
		clrGraphicsTypeAction[3] = new ArcSearchGuiActions(this, "Color Invert Image");
		clrGraphicsTypeAction[4] = new ArcSearchGuiActions(this, "Color Auto Scale");
		clrGraphicsTypeAction[5] = new ArcSearchGuiActions(this, "Color Color Spectrum");
		clrRGBplotter.setScaleType("log");
		clrRGBplotter.setInvertImage(false);
		clrRGBplotter.setAutoScale(false);
		clrRGBplotter.setColorSpectrum(false);
		clrRGBplotter.setPixelValueLimits(0.0, 10.0);
		JPanel clrGraphicsTypePanel = clrRGBplotter.pngGraphicsTypePanel("Color Images", clrGraphicsTypeAction);
	 	batchSettingsPanel2.add(clrGraphicsTypePanel);

	 	JPanel batchSettingsPanel = new JPanel();
		batchSettingsPanel.setLayout(new BoxLayout(batchSettingsPanel, BoxLayout.X_AXIS));
		batchSettingsPanel.add(batchSettingsPanel1);
		batchSettingsPanel.add(batchSettingsPanel2);
		
		return batchSettingsPanel;
	}
	private void readBatchSettingsTextFieldsFromGui()
	{
		batchPngImageSize = Integer.valueOf(batchPngImageSizeTextField.getText()).intValue();
		initBatchCropImageSize = Integer.valueOf(initBatchCropImageSizeTextField.getText()).intValue();
		bwRGBplotter.setPixelValueLimits(
				Double.valueOf(bwRGBplotter.minPixelValueTextField.getText()).doubleValue(), 
				Double.valueOf(bwRGBplotter.maxPixelValueTextField.getText()).doubleValue());
		clrRGBplotter.setPixelValueLimits(
				Double.valueOf(clrRGBplotter.minPixelValueTextField.getText()).doubleValue(), 
				Double.valueOf(clrRGBplotter.maxPixelValueTextField.getText()).doubleValue());
	}
	private JPanel setupWaveletSettingsPanel()
	{
		int		ii;
	 	JPanel settingsPanel = new JPanel();
	 	settingsPanel.setBorder(BorderFactory.createCompoundBorder(BorderFactory.createTitledBorder("Settings"),BorderFactory.createEmptyBorder(5,5,5,5)));
	 	settingsPanel.setLayout(new BoxLayout(settingsPanel, BoxLayout.Y_AXIS));
	 	JLabel scaleMaxLabel = new JLabel("Max. Scale");
	 	scaleMaxTextField = new JTextField(3);
	 	JPanel scaleMaxSubPanel = new JPanel();
	 	scaleMaxSubPanel.add(scaleMaxLabel);
	 	scaleMaxSubPanel.add(scaleMaxTextField);
	 	settingsPanel.add(scaleMaxSubPanel);
	 	scaleMaxTextField.setText(Integer.toString(waveletSettings.getScaleMax()));

	 	JPanel hardEdgeSubPanel = new JPanel();
		JRadioButton[] hardEdgeRadioButton = new JRadioButton[2];
	 	hardEdgeSubPanel.setBorder(BorderFactory.createCompoundBorder(BorderFactory.createTitledBorder("Hard Edge"),BorderFactory.createEmptyBorder(5,5,5,5)));
	 	hardEdgeSubPanel.setLayout(new FlowLayout());
	 	hardEdgeRadioButton[0] = new JRadioButton("On");
	 	hardEdgeRadioButton[0].setSelected(true);
	 	hardEdgeRadioButton[1] = new JRadioButton("Off");
	 	hardEdgeRadioButton[0].addActionListener(new ArcSearchGuiActions(this, "Hard Edge on"));
	 	hardEdgeRadioButton[1].addActionListener(new ArcSearchGuiActions(this, "Hard Edge off"));
	 	ButtonGroup hardEdgeButtonGroup = new ButtonGroup();
	 	hardEdgeButtonGroup.add(hardEdgeRadioButton[0]);
	 	hardEdgeButtonGroup.add(hardEdgeRadioButton[1]);
	 	hardEdgeSubPanel.add(hardEdgeRadioButton[0]);
	 	hardEdgeSubPanel.add(hardEdgeRadioButton[1]);
	 	settingsPanel.add(hardEdgeSubPanel);
	 	if (waveletSettings.getHardEdge())
	 	{
	 		hardEdgeRadioButton[0].setSelected(true);
	 		hardEdgeRadioButton[1].setSelected(false);
	 	}
	 	else
	 	{
	 		hardEdgeRadioButton[0].setSelected(false);
	 		hardEdgeRadioButton[1].setSelected(true);
	 	}		
		
		JPanel scaleCutPanel = new JPanel();
        scaleCutPanel.setBorder(BorderFactory.createCompoundBorder(BorderFactory.createTitledBorder("Scale Cuts"),BorderFactory.createEmptyBorder(5,5,5,5)));
		scaleCutPanel.setLayout(new BoxLayout(scaleCutPanel, BoxLayout.Y_AXIS));
	    JLabel[] scaleCutLabel = new JLabel[10];
	    JPanel[] scaleCutSubPanel = new JPanel[10];
		for (ii = 0; ii < 10; ++ii)
		{
			scaleCutTextField[ii] = new JTextField(8);
			scaleCutTextField[ii].setText(Double.toString(waveletSettings.getScaleSigmaCut(ii)));
			scaleCutLabel[ii] = new JLabel("Scale " + ii);
			scaleCutSubPanel[ii] = new JPanel();
			scaleCutSubPanel[ii].add(scaleCutLabel[ii]);
			scaleCutSubPanel[ii].add(scaleCutTextField[ii]);
			scaleCutPanel.add(scaleCutSubPanel[ii]);
		}

        JPanel avgRadiusPanel = new JPanel();
        avgRadiusPanel.setBorder(BorderFactory.createCompoundBorder(BorderFactory.createTitledBorder("Averaging Radius"),BorderFactory.createEmptyBorder(5,5,5,5)));
        avgRadiusPanel.setLayout(new BoxLayout(avgRadiusPanel, BoxLayout.Y_AXIS));
	    JLabel[] avgRadiusLabel = new JLabel[10];
	    JPanel[] avgRadiusSubPanel = new JPanel[10];
		for (ii = 0; ii < 10; ++ii)
		{
			avgRadiusTextField[ii] = new JTextField(8);
			avgRadiusTextField[ii].setText(Integer.toString(waveletSettings.getRadiusAvg(ii)));
			avgRadiusLabel[ii] = new JLabel("Scale " + ii);
			avgRadiusSubPanel[ii] = new JPanel();
			avgRadiusSubPanel[ii].add(avgRadiusLabel[ii]);
			avgRadiusSubPanel[ii].add(avgRadiusTextField[ii]);
			avgRadiusPanel.add(avgRadiusSubPanel[ii]);
		}
        JPanel waveletSettingsPanel = new JPanel();
        waveletSettingsPanel.add(settingsPanel);
        waveletSettingsPanel.add(scaleCutPanel);
        waveletSettingsPanel.add(avgRadiusPanel);
		return waveletSettingsPanel;
	}
	private JPanel setupImageDisplayPanel()
	{
	 	JPanel imagePanel = new JPanel();
	 	imagePanel.setBorder(BorderFactory.createCompoundBorder(BorderFactory.createTitledBorder(""),BorderFactory.createEmptyBorder(5,5,5,5)));
	 	imagePanel.setLayout(new GridLayout(2,3,5,5));
	 	ImageIcon imageIcon = new ImageIcon();
	 	for (int ii = 0; ii < 6; ++ii)
	 	{
	 		colorImagePanel[ii] = new JPanel();
	 		colorImagePanel[ii].setBorder(BorderFactory.createCompoundBorder(BorderFactory.createTitledBorder(""),BorderFactory.createEmptyBorder(5,5,5,5)));
		 	imageIconLabel[ii] = new JLabel(imageIcon);
		 	colorImagePanel[ii].add(imageIconLabel[ii]);
		 	colorImageScrollPane[ii] = new JScrollPane(colorImagePanel[ii]);
		 	imagePanel.add(colorImageScrollPane[ii]);
	 	}
	 	return imagePanel;
	}
	private void readWaveletSettingsTextFieldsFromGui()
	{
		int		ivalue;
		double	dvalue;
		int		ii;
		ivalue = Integer.valueOf(scaleMaxTextField.getText()).intValue();
		waveletSettings.setScaleMax(ivalue);	
		for (ii = 0; ii < 10; ++ii)
		{
			ivalue = Integer.valueOf(avgRadiusTextField[ii].getText()).intValue();
			dvalue = Double.valueOf(scaleCutTextField[ii].getText()).doubleValue();
			waveletSettings.setRadiusAvg(ii, ivalue);
			waveletSettings.setScaleSigmaCut(ii, dvalue);
		}
	}
	void onFileCreateBatchFile(boolean colorBatchFile)
    {
		File	batchFile;
		int		numOfFiles;
		File[]	fitsFile = null;
		FileWriter batchFileWriter = null;
		String	fileName = null;
		int		colorCharIndex;
		ArcSearchThread	arcSearchThread = new ArcSearchThread();
		String	dialogInstruction = "";
		int		ifile;
		
     	if (colorBatchFile)
     	{
     		dialogInstruction = "Choose Color Batch File";
     	}
     	else
     	{
     		dialogInstruction = "Choose Grey Batch File";
     	}
		if (okayCancelDialog(dialogInstruction, dialogInstruction) != true) return;
	   	FileChooserWrapper fc = new FileChooserWrapper(this);
		batchFile = fc.fOpenFile("txt",false, lastDirectoryPath);
		if (batchFile == null) return;
     	lastDirectoryPath = batchFile.getPath();
     	if (colorBatchFile)
     	{
     		dialogInstruction = "Choose Fits Files Sets";
     	}
     	else
     	{
     		dialogInstruction = "Choose Fits Files";
     	}
     	
		if (!okayCancelDialog(dialogInstruction, dialogInstruction))return;
     	numOfFiles = 1;
     	try
     	{
     		batchFileWriter = new FileWriter(batchFile);
	     	while(numOfFiles > 0)
	     	{
	     		fitsFile = null;
	     		fitsFile = fc.fOpenMultipleFiles("fit",lastDirectoryPath);
	     		if (fitsFile != null)
	     		{
	     			numOfFiles = fitsFile.length;
	     		}
	     		else
	     		{
	     			numOfFiles = 0;
	     		}
	     		if ((numOfFiles == 3 ) && colorBatchFile)
	     		{
	     			lastDirectoryPath = fitsFile[0].getPath();
	     			colorCharIndex = arcSearchThread.findColorCharIndexInFileName(fitsFile[0].getPath(), fitsFile[1].getPath());
					if (colorCharIndex == 0) throw new IOException();
					fileName = arcSearchThread.writeColorCharIntoFileName(fitsFile[0].getPath(), "$", colorCharIndex);
					batchFileWriter.write(fileName + "\n");
	     		}
	     		if ((numOfFiles > 0 ) && !colorBatchFile)
	     		{
	     			lastDirectoryPath = fitsFile[0].getPath();
	     			for ( ifile = 0; ifile < numOfFiles; ++ifile)
	     				batchFileWriter.write(fitsFile[ifile].getPath() + "\n");
	     		}
	     	}
 	     	batchFileWriter.write("\n" + "\n");
	     	batchFileWriter.close();
     	}
     	catch (Exception e) 
     	{
    		System.out.println("Problem Creating Batch file");
     	}
    }
    void onRunSingleWaveletFile()
    {
		File	inputFile;
		String	dialogInstruction = "Choose Fits File";
		
		if (okayCancelDialog(dialogInstruction, dialogInstruction) != true) return;
	   	FileChooserWrapper fc = new FileChooserWrapper(this);
	   	inputFile = fc.fOpenFile("fit",false, lastDirectoryPath);
		if (inputFile == null) return;
     	lastDirectoryPath = inputFile.getPath();
     	
 		String[] colorImagePanelTitle = {"Input","Output","   ","   ","   ","   "};
      	for (int ii = 0; ii < 6; ++ii)
     	{
     		imageIconLabel[ii].setIcon(new ImageIcon());
     		colorImagePanel[ii].setBorder(BorderFactory.createCompoundBorder(BorderFactory.createTitledBorder(colorImagePanelTitle[ii]),BorderFactory.createEmptyBorder(5,5,5,5)));
     	}
     	readWaveletSettingsTextFieldsFromGui();
     	readBatchSettingsTextFieldsFromGui();
     	
     	ArcSearchThread arcSearchThread = new ArcSearchThread();
     	arcSearchThread.setWaveletSettings(waveletSettings);
     	arcSearchThread.setProgressBar(progressBar);
     	arcSearchThread.setStatusBar(statusBar);
     	arcSearchThread.setBwRGBplotter(bwRGBplotter);
     	arcSearchThread.setImageIconLabel(imageIconLabel);
     	arcSearchThread.setColorImageScrollPane(colorImageScrollPane);
     	arcSearchThread.setRunSingleFileWaveletFilter(true);
     	arcSearchThread.setRunProcessBatchImages(false);
     	arcSearchThread.setSingleInputFile(inputFile);
     	arcSearchThread.setArcSearchGui(this);

     	new Thread(arcSearchThread).start();

    }
    void onRunBatchFile(boolean colorBatchFile)
    {
		File	inputBatchFile;
		String	dialogInstruction = "";
		
     	if (colorBatchFile)
     	{
     		dialogInstruction = "Choose Color Batch File";
     	}
     	else
     	{
     		dialogInstruction = "Choose Grey Batch File";
     	}
		if (okayCancelDialog(dialogInstruction, dialogInstruction) != true) return;
	   	FileChooserWrapper fc = new FileChooserWrapper(this);
	   	inputBatchFile = fc.fOpenFile("txt",false, lastDirectoryPath);
		if (inputBatchFile == null) return;
     	lastDirectoryPath = inputBatchFile.getPath();
     	
 		String[] colorImagePanelTitle = {"g  ","r  ","i  ","gri"};
     	if (colorBatchFile)
     	{
     		colorImagePanelTitle[0] = "g";
     		colorImagePanelTitle[1] = "r";
     		colorImagePanelTitle[2] = "i";
     		colorImagePanelTitle[3] = "gri";
     	}
     	else
     	{
     		colorImagePanelTitle[0] = "   ";
     		colorImagePanelTitle[1] = "   ";
     		colorImagePanelTitle[2] = "   ";
     		colorImagePanelTitle[3] = "   ";
      	}
     	for (int ii = 0; ii < 4; ++ii)
     	{
     		imageIconLabel[ii].setIcon(new ImageIcon());
     		colorImagePanel[ii].setBorder(BorderFactory.createCompoundBorder(BorderFactory.createTitledBorder(colorImagePanelTitle[ii]),BorderFactory.createEmptyBorder(5,5,5,5)));
     	}
     	readWaveletSettingsTextFieldsFromGui();
     	readBatchSettingsTextFieldsFromGui();
     	
     	ArcSearchThread arcSearchThread = new ArcSearchThread();
     	arcSearchThread.setWaveletSettings(waveletSettings);
     	arcSearchThread.setProgressBar(progressBar);
     	arcSearchThread.setStatusBar(statusBar);
     	arcSearchThread.setInputBatchFile(inputBatchFile);
     	arcSearchThread.setBatchCrop(batchCrop);
     	arcSearchThread.setBatchNormalize(batchNormalize);
     	arcSearchThread.setBatchFilter(batchFilter);
     	arcSearchThread.setBatchWritePngImages(batchWritePngImages);
     	arcSearchThread.setBatchPngImageSize(batchPngImageSize);
     	arcSearchThread.setInitBatchCropImageSize(initBatchCropImageSize);
     	arcSearchThread.setBatchWriteFitsFiles(batchWriteFitsFiles);
     	arcSearchThread.setBatchArcSearch(batchArcSearch);
     	arcSearchThread.setBwRGBplotter(bwRGBplotter);
     	arcSearchThread.setClrRGBplotter(clrRGBplotter);
     	arcSearchThread.setImageIconLabel(imageIconLabel);
     	arcSearchThread.setColorImageScrollPane(colorImageScrollPane);
     	arcSearchThread.setColorBatchFile(colorBatchFile);
     	arcSearchThread.setRunSingleFileWaveletFilter(false);
     	arcSearchThread.setRunProcessBatchImages(true);
     	arcSearchThread.setArcSearchGui(this);

     	new Thread(arcSearchThread).start();

    }
   /**
 * @param dialogTitle
 * @param message
 * @return true if 
 */
	public boolean okayCancelDialog(String dialogTitle, String message)
    {
		int		choice;
		Object[] options = { "OK", "CANCEL" };
		
		choice = JOptionPane.showOptionDialog(null, dialogTitle, message, 
					JOptionPane.DEFAULT_OPTION, JOptionPane.WARNING_MESSAGE,
					null, options, options[0]);
		if (choice == 0)
		{
			return true;
		}
		else
		{
			return false;
		}
    }
	/**
	 * @param dialogTitle
	 * @param message
	 * @return
	 */
	public boolean yesNoDialog(String dialogTitle, String message)
	{
		int		choice;
		Object[] options = { "Yes", "No" };
		
		choice = JOptionPane.showOptionDialog(null, dialogTitle, message, 
					JOptionPane.DEFAULT_OPTION, JOptionPane.WARNING_MESSAGE,
					null, options, options[0]);
		if (choice == 0)
		{
			return true;
		}
		else
		{
			return false;
		}
	}
   private  JMenuBar addMenu()
    {
        JMenuBar menuBar = new JMenuBar();
        
        String menuText[] = {"File", "Run", "Help"};
        String itemText[][] =
            {{"New Color Batch", "New Grey Batch", "Exit"},
             {"Color Batch File", "Grey Batch File", "Single Wavelet Filter"},
             {"Help", "About"}};

        for (int i = 0; i < menuText.length; i++)
        {
            JMenu menu = new JMenu(menuText[i]);
            menuBar.add (menu);
            
            for (int j = 0; j < itemText[i].length; j++)
            {
                JMenuItem item = new JMenuItem(itemText[i][j]);
                menu.add (item);
                item.addActionListener(new ArcSearchGuiActions(this, menuText[i] + "." +itemText[i][j]));
 
            }
        }
        
        return menuBar;
    }
	/**
	 * @param args
	 */
	public static void main(String[] args) 
	{
        try 
        {
            UIManager.setLookAndFeel(
            UIManager.getCrossPlatformLookAndFeelClassName());
        } catch (Exception e) {}
        
        //Create the top-level container and add contents to it.
        JFrame frame = new JFrame("Noah III");
        ArcSearchGui app = new ArcSearchGui();
        frame.setJMenuBar(app.addMenu());

        Component contents = app.createComponents();
        frame.getContentPane().add(contents, BorderLayout.CENTER);
        frame.getContentPane().add(statusBar, java.awt.BorderLayout.SOUTH);  
        
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        frame.pack();
        frame.setVisible(true);
	}

}
