package gov.fnal.eag.sim.pixel.wavelet;

import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.LineNumberReader;

import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JProgressBar;
import javax.swing.JScrollBar;
import javax.swing.JScrollPane;

import nom.tam.fits.FitsException;


/**
 * @author mcginnis
 *
 */
public class ArcSearchThread  implements Runnable
{
	double	regionMin = 0.5;
	double	regionMax = 10.0;
	double	colorCompWindow = 2.0;
	int		batchObjCnt = 0;
	int		batchArcCnt = 0;
	int		batchCropImageSize;
	File[]	coloredImageFitsFile = new File[4];
	PixieImage[] coloredImage = new PixieImage[3];
	PixieImage[] deblendedArcImage = new PixieImage[3];
	private PixieImageRGBPlotter bwRGBplotter = null;
	private PixieImageRGBPlotter clrRGBplotter = null;
	private ArcSearchGui arcSearchGui = null;
	
	private JProgressBar progressBar = null;
	private File	inputBatchFile;
	private File	singleInputFile;
	private StatusBar statusBar = null;
	private Wavelet	waveletSettings = null;
	private boolean	batchCrop = true;
	private boolean batchNormalize = true;
	private boolean batchFilter = true;
	private boolean batchWritePngImages = true;
	private int		batchPngImageSize = 300;
	private int		initBatchCropImageSize = 400;
	private boolean	batchWriteFitsFiles = false;
	private boolean	batchArcSearch = false;
	private boolean colorBatchFile = true;
	private boolean runProcessBatchImages = true;
	private boolean runSingleFileWaveletFilter = false;
	private JLabel[] imageIconLabel = null;
	private JScrollPane[] colorImageScrollPane = null;
	String delim = System.getProperty("file.separator");
	
	ArcSearchThread()
	{
	}
	public void run() 
	{
    	progressBar.setIndeterminate(true);
     	try 
     	{
     		if (runProcessBatchImages)
     		{
     			processBatchImages();
     			progressBar.setIndeterminate(false);
     			statusBar.setMessage("Ready");
     		}
     		if (runSingleFileWaveletFilter)
     		{
    			singleFileWaveletFilter();
    			progressBar.setIndeterminate(false);
    			statusBar.setMessage("Ready");
    		}
		} 
     	catch (IOException e) 
     	{
    		progressBar.setIndeterminate(false);
    		statusBar.setMessage("Problem Processing Batch Images");
        	JOptionPane.showMessageDialog(arcSearchGui, "Input Output Error");
//			e.printStackTrace();
		}
     	catch (FitsException e) 
     	{
    		progressBar.setIndeterminate(false);
    		statusBar.setMessage("Problem Processing Batch Images");
        	JOptionPane.showMessageDialog(arcSearchGui, "Fits File Error");
//			e.printStackTrace();
		}
        catch (OutOfMemoryError e) 
        {
    		progressBar.setIndeterminate(false);
    		statusBar.setMessage("Not Enough Memory");
        	JOptionPane.showMessageDialog(arcSearchGui, "Not Enough Memory\nTry Increasing the Memory of the Java VM");
        }
	}
	/**
	 * @throws FitsException
	 * @throws IOException
	 */
	public void processBatchImages()  throws FitsException, IOException
	{
		int				colorCharIndex;
		FileWriter		outBatchWriter = null;
		FileWriter		outBatchHtmlWriter = null;
		String			outBatchHtmlFileName = null;
		File			outBatchHtmlFile = null;
		String			outBatchFileName = null;
		File			outBatchFile = null;
		FileReader		inBatchReader = null;
		LineNumberReader		inBatchLineReader = null;
		String			inputFileName = null;	
		
		outBatchFileName = inputBatchFile.getPath().substring(0, inputBatchFile.getPath().length() - 4)
			+ "_out.txt";
		outBatchFile = new File(outBatchFileName);
		outBatchHtmlFileName = outBatchFile.getPath().substring(0, outBatchFile.getPath().length() - 4)
			+ ".htm";
		outBatchHtmlFile = new File(outBatchHtmlFileName);
		outBatchHtmlWriter = new FileWriter(outBatchHtmlFile);
		
		if (batchArcSearch && colorBatchFile)
		{
			outBatchWriter = new FileWriter(outBatchFile);
			outBatchWriter.write("Count\t");
			outBatchWriter.write("Object\t");
			outBatchWriter.write("Num Pixs\t");
			outBatchWriter.write("Xcg\t");
			outBatchWriter.write("Ycg\t");
			outBatchWriter.write("Rot.\t");
			outBatchWriter.write("Length\t");
			outBatchWriter.write("Max Width\t");
			outBatchWriter.write("RMS Width\t");
			outBatchWriter.write("Curve\t");
			outBatchWriter.write("G color A\t");
			outBatchWriter.write("G color M\t");
			outBatchWriter.write("R color A\t");
			outBatchWriter.write("R color M\t");
			outBatchWriter.write("I color A\t");
			outBatchWriter.write("I color M\t");
			outBatchWriter.write("M11\t");
			outBatchWriter.write("M20\t");
			outBatchWriter.write("M02\t");
			outBatchWriter.write("M21\t");
			outBatchWriter.write("M12\t");
			outBatchWriter.write("M30\t");
			outBatchWriter.write("Parent file\n");
		}
		
		outBatchHtmlWriter.write("<html>\n<head>\n");
		outBatchHtmlWriter.write("<title>" + outBatchHtmlFile.getName() + "</title>\n");
		outBatchHtmlWriter.write("</head>\n<body>\n");

		inBatchReader = new FileReader(inputBatchFile);
		inBatchLineReader = new LineNumberReader(inBatchReader);
		inputFileName = "File";
		batchObjCnt = 0;
		while(inputFileName.length() > 2)
		{
			inputFileName = inBatchLineReader.readLine();
    		if (inputFileName.length() > 2)
    		{
    			batchObjCnt = batchObjCnt + 1;
				coloredImageFitsFile[0] = new File(inputFileName);
				coloredImageFitsFile[3] = new File(inputFileName);
    			if (colorBatchFile)
    			{
	    			colorCharIndex = findColorCharIndexInFileName(inputFileName,'$');
					if (colorCharIndex == 0) throw new IOException();
					coloredImageFitsFile[0] = new File(writeColorCharIntoFileName(inputFileName, "g", colorCharIndex));
					coloredImageFitsFile[1] = new File(writeColorCharIntoFileName(inputFileName, "r", colorCharIndex));
					coloredImageFitsFile[2] = new File(writeColorCharIntoFileName(inputFileName, "i", colorCharIndex));
					coloredImageFitsFile[3] = new File(inputFileName);
    			}
				cropNormalizeFilterImages();
				if (batchArcSearch && colorBatchFile) 
				{
					statusBar.setMessage("Batch Cnt. " + Integer.toString(batchObjCnt) + " Finding Arcs");
					findArcs(outBatchFile, outBatchHtmlFile, outBatchWriter, outBatchHtmlWriter);
				}
				else
				{
					if (batchWritePngImages) makePngImages(outBatchHtmlFile, outBatchHtmlWriter);
					if (batchWriteFitsFiles) writeFilteredFitsFiles(outBatchHtmlFile);
				}
    		}
		}

		if (batchArcSearch && colorBatchFile) outBatchWriter.write("-1\n");
		outBatchHtmlWriter.write("</body>\n</html>\n");
		
		if (batchArcSearch && colorBatchFile) outBatchWriter.close();
		outBatchHtmlWriter.close();
		inBatchReader.close();	
		statusBar.setMessage("Finished Batch running");
			
	}
	/**
	 * @param fileName
	 * @param colorChar 
	 * @return
	 */
	public int findColorCharIndexInFileName(String fileName, char colorChar)
	{
		int		ii;
		int		colorCharIndex;
		colorCharIndex = 0;
		for (ii = 0; ii < fileName.length(); ++ii)
		{
			if(fileName.charAt(ii) == colorChar) colorCharIndex = ii;
		}
		return colorCharIndex;
	}
	/**
	 * @param fileName1
	 * @param fileName2
	 * @return
	 */
	public int findColorCharIndexInFileName(String fileName1, String fileName2)
	{
		int		ii;
		int		colorCharIndex;
		int		difCount;
		char	ctest1;
		char	ctest2;
		
		colorCharIndex = 0;
		difCount = 0;
		
		for (ii = 0; ii < fileName1.length(); ++ii)
		{
			ctest1 = fileName1.charAt(ii);
			ctest2 = fileName2.charAt(ii);
			
			if ( ctest1 != ctest2) 
			{
				difCount = difCount + 1;
				colorCharIndex = ii;
			}
		}
		if(difCount > 1) colorCharIndex = 0;
		return colorCharIndex;
	}
	/**
	 * @param inputFileName
	 * @param colorChar
	 * @param colorCharIndex
	 * @return
	 */
	public String writeColorCharIntoFileName(String inputFileName, String colorChar, int colorCharIndex)
	{
		String 	outputFileName = null;
		int		fileNameLength;
		String[]	fileNamePart = new String[3];
		fileNameLength = inputFileName.length();
		fileNamePart[0] = inputFileName.substring(0, colorCharIndex);
		fileNamePart[1] = colorChar;
		fileNamePart[2] = inputFileName.substring(colorCharIndex + 1, fileNameLength);
		outputFileName = fileNamePart[0] + fileNamePart[1] + fileNamePart[2];
		return outputFileName;
	}
	private	void cropNormalizeFilterImages() throws FitsException, IOException
	{
		int		ifile;
		int		maxFile = 1;
		PixieImage croppedImage = null;
		Wavelet wavelet = new Wavelet(waveletSettings);
		int nmaxRows;
		int nmaxCols;
		if (colorBatchFile) maxFile = 3;

		for (ifile = 0; ifile < maxFile; ++ifile)
		{
			statusBar.setMessage("Batch Cnt. " + Integer.toString(batchObjCnt) + " Reading File " + Integer.toString(ifile + 1));
			nmaxRows = PixieImage.getRowCount(coloredImageFitsFile[ifile]);
			nmaxCols = PixieImage.getColCount(coloredImageFitsFile[ifile]);
			batchCropImageSize = initBatchCropImageSize;
	    	if ( batchCropImageSize > nmaxRows) 
	    		batchCropImageSize = nmaxRows;
	    	if ( batchCropImageSize > nmaxCols) 
	    		batchCropImageSize = nmaxCols;
	    	if (batchCrop)
	    	{
		    	croppedImage = new PixieImage(coloredImageFitsFile[ifile],
		    			(nmaxRows / 2) - (batchCropImageSize / 2), 
		    			(nmaxCols / 2) - (batchCropImageSize / 2),
		    			batchCropImageSize, batchCropImageSize);
	    	}
	    	else
	    	{
	    		croppedImage = new PixieImage(coloredImageFitsFile[ifile]);
	    	}
	    	if (batchNormalize) 
    		{
	    		statusBar.setMessage("Batch Cnt. " + Integer.toString(batchObjCnt) + " Normalizing File " + Integer.toString(ifile + 1));
	    		wavelet.pixelNormalize(croppedImage);
    		}
	    	if (batchFilter)
	    	{
	    		statusBar.setMessage("Batch Cnt. " + Integer.toString(batchObjCnt) + " Filtering File " + Integer.toString(ifile + 1));
	    		coloredImage[ifile] = wavelet.waveFilter(croppedImage);
	    	}
	    	else
	    	{
	    		coloredImage[ifile] = new PixieImage(croppedImage);
	    	}
		}
//	Will do auto scaling if autoscale is set true otherwise it does nothing.
		bwRGBplotter.setPixelValueLimits(croppedImage);
		clrRGBplotter.setPixelValueLimits(croppedImage);
//	Will do histogram scaling if the scaletype is set to histogram otherwise it does nothing.
		bwRGBplotter.setScaleTable(croppedImage);
		clrRGBplotter.setScaleTable(croppedImage);
		wavelet = null;
		croppedImage = null;
		System.gc();
	}
	private PixieImage compositeColorImage(double threshold)
	{
		double[][] dpixComp = null;
		double[][][] dpixColor = new double[3][][];
		
		PixieImage piComp = new PixieImage(coloredImage[0]);
		Pixie pmax = new Pixie();
		pmax.row = piComp.getRowCount();
		pmax.col = piComp.getColCount();
		dpixComp = piComp.getPix();
		double	davg;
		
		
		for (int image = 0; image < 3; ++image)
		{
			dpixColor[image] = coloredImage[image].getPix();
		}
		for ( int ii = 0; ii < pmax.row; ++ii)
		{
			for ( int ij = 0; ij < pmax.col; ++ij)
			{
				davg = (dpixColor[0][ii][ij] + dpixColor[1][ii][ij] + dpixColor[2][ii][ij]) / 3.0;
				 
				if (davg > regionMin) 
				{
					dpixComp[ii][ij] 
					   = 1.0   * (dpixColor[0][ii][ij] / dpixColor[1][ii][ij])
					           * (dpixColor[0][ii][ij] / dpixColor[2][ii][ij]);
				}
				else
				{
					dpixComp[ii][ij] = 0.0;
				}
				if (dpixComp[ii][ij] > 0.4)
				{
					dpixComp[ii][ij] = dpixComp[ii][ij] * davg;
				}
				else
				{
					dpixComp[ii][ij] = 0.0;
				}
			}
		}
		
		return piComp;
	}
	private	void makePngImages(File outBatchHtmlFile, FileWriter outBatchHtmlWriter) throws IOException
	{
		int		ifile;
		int		maxFile = 0;
		String	plotFileName;
		String	shortPlotFileName;
		String	htmlStuff;
		String	q = "\"";
		PixieImageRGBPlotter plotter = null;
	 	ImageIcon imageIcon = null;
		
		htmlStuff 
			= "<p>File: "
			+ coloredImageFitsFile[3].getName()
			+ "</p>\n";
		outBatchHtmlWriter.write(htmlStuff);
		if (colorBatchFile) maxFile = 3;

		for (ifile = 0; ifile <= maxFile; ++ifile)
		{
			statusBar.setMessage("Batch Cnt. " + Integer.toString(batchObjCnt) + " Writing PNG File " + Integer.toString(ifile + 1));
			shortPlotFileName = coloredImageFitsFile[ifile].getName().substring(0, coloredImageFitsFile[ifile].getName().length() - 4);
			plotFileName 
				= outBatchHtmlFile.getParent() 
				+ delim 
				+ shortPlotFileName
				+ ".png";
			if (ifile < 3)
			{
				plotter = new PixieImageRGBPlotter();
				plotter.setImages(coloredImage[ifile], coloredImage[ifile], coloredImage[ifile]);
				plotter.copyScaling(bwRGBplotter);
			}
			if (ifile == 3)
			{
				plotter = new PixieImageRGBPlotter();
				plotter.setImages(coloredImage[2], coloredImage[1], coloredImage[0]);
				plotter.copyScaling(clrRGBplotter);
			}
			plotter.setScaleTable(null);
			plotter.toPNGFile(plotFileName);
		 	imageIcon = new ImageIcon(plotter.makeBufferedImage());
		 	imageIconLabel[ifile].setIcon(imageIcon);
			centerScrollBars(colorImageScrollPane[ifile]);
			
			htmlStuff
				= "<a href=" 
				+ q + shortPlotFileName + ".png" + q
				+ "><img border=" + q + "1" + q + " src="
				+ q + shortPlotFileName + ".png" + q
				+ " width=" + q + Integer.toString(batchPngImageSize) + q + " height=" + q + Integer.toString(batchPngImageSize) + q + "></a>\n";
			outBatchHtmlWriter.write(htmlStuff);
		}
		if (colorBatchFile)
		{
			PixieImage piComp = compositeColorImage(regionMin);
/*			shortPlotFileName = coloredImageFitsFile[3].getName().substring(0, coloredImageFitsFile[3].getName().length() - 4)
			+ "_g2ori";
			plotFileName 
				= outBatchHtmlFile.getParent() 
				+ delim 
				+ shortPlotFileName
				+ ".fit";
			try
			{
				piComp.writeToFitsFile(plotFileName);
			}
			catch (FitsException e)
			{
				
			}
			catch (IOException e)
			{
				
			}
*/			
			shortPlotFileName = coloredImageFitsFile[3].getName().substring(0, coloredImageFitsFile[3].getName().length() - 4)
				+ "_comp";
			plotFileName 
				= outBatchHtmlFile.getParent() 
				+ delim 
				+ shortPlotFileName
				+ ".png";
			plotter = new PixieImageRGBPlotter();
			plotter.setImages(piComp, piComp, piComp);
			plotter.copyScaling(bwRGBplotter);
			plotter.setScaleTable(null);
			plotter.toPNGFile(plotFileName);
		 	imageIcon = new ImageIcon(plotter.makeBufferedImage());
		 	imageIconLabel[4].setIcon(imageIcon);
			centerScrollBars(colorImageScrollPane[4]);
			htmlStuff
				= "<a href=" 
				+ q + shortPlotFileName + ".png" + q
				+ "><img border=" + q + "1" + q + " src="
				+ q + shortPlotFileName + ".png" + q
				+ " width=" + q + Integer.toString(batchPngImageSize) + q + " height=" + q + Integer.toString(batchPngImageSize) + q + "></a>\n";
			outBatchHtmlWriter.write(htmlStuff);
		}
		if (batchArcSearch && colorBatchFile)
		{
			shortPlotFileName = coloredImageFitsFile[3].getName().substring(0, coloredImageFitsFile[3].getName().length() - 4)
				+ "_arc"
				+ Integer.toString(batchArcCnt);
			plotFileName 
				= outBatchHtmlFile.getParent() 
				+ delim 
				+ shortPlotFileName
				+ ".png";
			plotter = new PixieImageRGBPlotter();
			plotter.setImages(deblendedArcImage[0], deblendedArcImage[1], deblendedArcImage[2]);
			plotter.copyScaling(bwRGBplotter);
			plotter.setScaleTable(null);
			plotter.toPNGFile(plotFileName);
		 	imageIcon = new ImageIcon(plotter.makeBufferedImage());
		 	imageIconLabel[5].setIcon(imageIcon);
			centerScrollBars(colorImageScrollPane[5]);
			htmlStuff
				= "<a href=" 
				+ q + shortPlotFileName + ".png" + q
				+ "><img border=" + q + "1" + q + " src="
				+ q + shortPlotFileName + ".png" + q
				+ " width=" + q + Integer.toString(batchPngImageSize) + q + " height=" + q + Integer.toString(batchPngImageSize) + q + "></a>\n";
			outBatchHtmlWriter.write(htmlStuff);
		}
		htmlStuff = "</p>\n<hr>\n";
		outBatchHtmlWriter.write(htmlStuff);
		plotter = null;
		imageIcon = null;
	}
	void findArcs(File outBatchFile, File outBatchHtmlFile, FileWriter outBatchWriter, FileWriter outBatchHtmlWriter) throws IOException, FitsException
	{
		double[][]	dpix1 = null;
		int			ii;
		int			ij;
		int			ik;
		Pixie		pmax = new Pixie();
		int[][]		iregion_array = null;
		int			inum_regions;
		boolean		idebug;
		Pixie		p  = new Pixie();
		int			iobject;
		int			icolor;

		dpix1  = coloredImage[0].getPix();
		pmax.row = coloredImage[0].getRowCount();
		pmax.col = coloredImage[0].getColCount();
		pmax.val = 0.0;
		iregion_array = new int[pmax.row][pmax.col];

		for (ii = 0; ii < pmax.row; ii++)
		{  
			for (ij = 0; ij < pmax.col; ij++) 
			{
				iregion_array[ii][ij] = 0;
			}
		}
		inum_regions = 0;
		for (ii = 0; ii < pmax.row; ii++)
		{  
			for (ij = 0; ij < pmax.col; ij++) 
			{
				if (iregion_array[ii][ij] == 0)
				{
					if (dpix1[ii][ij] >= regionMin)
					{
						inum_regions = inum_regions + 1;
						iregion_array[ii][ij] = inum_regions;
						p.row = ii;
						p.col = ij;
						BlendedArc.vGrowRegion(p, pmax, dpix1, iregion_array,
								  inum_regions, regionMin, -1.0);
					}
					else
					{
						iregion_array[ii][ij] = -1;
					}
				}
			}
		}
		int[] numPixels = new int[inum_regions];
		BlendedArc[]	blendedArc = new BlendedArc[inum_regions];
		for (ii = 0; ii < inum_regions; ii++)
		{
			numPixels[ii] = 0;
		}

		for (ii = 0; ii < pmax.row; ii++)
		{  
			for (ij = 0; ij < pmax.col; ij++) 
			{
				if (iregion_array[ii][ij] > -1)
				{
					iobject = iregion_array[ii][ij] - 1;
					numPixels[iobject] = numPixels[iobject] + 1;
				}
			}
		}
		for (ii = 0; ii < inum_regions; ii++)
		{
			blendedArc[ii] = new BlendedArc(numPixels[ii], coloredImage);
			numPixels[ii] = 0;
		}
		for (ii = 0; ii < pmax.row; ii++)
		{  
			for (ij = 0; ij < pmax.col; ij++) 
			{
				if (iregion_array[ii][ij] > -1)
				{
					iobject = iregion_array[ii][ij] - 1;
					ik = numPixels[iobject];
					blendedArc[iobject].m_pixel[ik].row = ii;
					blendedArc[iobject].m_pixel[ik].col = ij;
					blendedArc[iobject].m_pixel[ik].val = dpix1[ii][ij];
					for (icolor = 0; icolor < 3; ++icolor)
					{
						blendedArc[iobject].m_color_pixel[icolor][ik].row = ii;
						blendedArc[iobject].m_color_pixel[icolor][ik].col = ij;
						blendedArc[iobject].m_color_pixel[icolor][ik].val = coloredImage[icolor].getPix()[ii][ij];
					}
					numPixels[iobject] = numPixels[iobject] + 1;
				}
			}
		}
		numPixels = null;
		for (ii = 0; ii < inum_regions; ii++)
		{
			idebug = false;
/*			if (ii == 19) //14 for gal 19 for arc
			{
				idebug = true;
				if (idebug)
				{
					System.out.println("Hi");
				}
			}
*/			blendedArc[ii].vFindObjects(idebug);
		}

		for (ii = 0; ii < inum_regions; ii++)
		{
			if (blendedArc[ii].m_inum_objects > 0)
			{
				for ( ij = 0; ij < blendedArc[ii].m_inum_objects; ++ij)
				{
					if (blendedArc[ii].deblendedArc[ij].iPassedCuts() )
					{
						batchArcCnt = batchArcCnt + 1;
						deblendedArcImage[0] = blendedArc[ii].deblendedArc[ij].makePixieImage(pmax,coloredImage[0],true,false,false);
						deblendedArcImage[1] = blendedArc[ii].deblendedArc[ij].makePixieImage(pmax,coloredImage[0],false,true,false);
						deblendedArcImage[2] = blendedArc[ii].deblendedArc[ij].makePixieImage(pmax,coloredImage[0],false,false,true);
						outBatchWriter.write(String.format("  %d\t",batchArcCnt));
						outBatchWriter.write(String.format("  %d\t",ii + 1));
						outBatchWriter.write(String.format("  %d\t",blendedArc[ii].deblendedArc[ij].m_inum_pixels));
						outBatchWriter.write(String.format("%.1f\t",blendedArc[ii].deblendedArc[ij].m_dcenter_of_g[0]));
						outBatchWriter.write(String.format("%.1f\t",blendedArc[ii].deblendedArc[ij].m_dcenter_of_g[1]));
						outBatchWriter.write(String.format("%.1f\t",180.0 * blendedArc[ii].deblendedArc[ij].m_drot_angle / 3.14159276));
						outBatchWriter.write(String.format("%.1f\t",blendedArc[ii].deblendedArc[ij].m_dlength));
						outBatchWriter.write(String.format("%.1f\t",blendedArc[ii].deblendedArc[ij].m_dmin_max_width));
						outBatchWriter.write(String.format("%.1f\t",blendedArc[ii].deblendedArc[ij].m_drms_width));
						outBatchWriter.write(String.format("%.1f\t",blendedArc[ii].deblendedArc[ij].m_drad_curv));
						outBatchWriter.write(String.format("%.1f\t",blendedArc[ii].deblendedArc[ij].m_davg_color[0]));
						outBatchWriter.write(String.format("%.1f\t",blendedArc[ii].deblendedArc[ij].m_dmedium_color[0]));
						outBatchWriter.write(String.format("%.1f\t",blendedArc[ii].deblendedArc[ij].m_davg_color[1]));
						outBatchWriter.write(String.format("%.1f\t",blendedArc[ii].deblendedArc[ij].m_dmedium_color[1]));
						outBatchWriter.write(String.format("%.1f\t",blendedArc[ii].deblendedArc[ij].m_davg_color[2]));
						outBatchWriter.write(String.format("%.1f\t",blendedArc[ii].deblendedArc[ij].m_dmedium_color[2]));
						outBatchWriter.write(String.format("%.2e\t",blendedArc[ii].deblendedArc[ij].m_Cmoment.m_drotated[1][1]));
						outBatchWriter.write(String.format("%.2e\t",blendedArc[ii].deblendedArc[ij].m_Cmoment.m_drotated[2][0]));
						outBatchWriter.write(String.format("%.2e\t",blendedArc[ii].deblendedArc[ij].m_Cmoment.m_drotated[0][2]));
						outBatchWriter.write(String.format("%.2e\t",blendedArc[ii].deblendedArc[ij].m_Cmoment.m_drotated[2][1]));
						outBatchWriter.write(String.format("%.2e\t",blendedArc[ii].deblendedArc[ij].m_Cmoment.m_drotated[1][2]));
						outBatchWriter.write(String.format("%.2e\t",blendedArc[ii].deblendedArc[ij].m_Cmoment.m_drotated[3][0]));
						outBatchWriter.write(String.format("!%s\n",coloredImageFitsFile[3].getName()));
						makePngImages(outBatchHtmlFile, outBatchHtmlWriter);
					}
					blendedArc[ii].deblendedArc[ij] = null;
				}
			}
			blendedArc[ii].deblendedArc = null;
			blendedArc[ii] = null;
		}
		blendedArc = null;
		
 		outBatchWriter.write(String.format("-1\n"));
		PixieImage pi = new PixieImage(pmax.row, pmax.col);
		dpix1 = pi.getPix();
		for (ii = 0; ii < pmax.row; ii++)
		{  
			for (ij = 0; ij < pmax.col; ij++) 
			{
				dpix1[ii][ij] = (double) iregion_array[ii][ij];
			}
		}
		iregion_array = null;
		String objectFitsfileName = outBatchFile.getParent() + delim + "object.fit";
		pi.writeToFitsFile(objectFitsfileName);
		pi = null;
	}
	private void centerScrollBars(JScrollPane scrollPane)
	{
		int		iscrollBarMaxValue;
		int		iscrollBarMinValue;
		JScrollBar scrollBar = null;
		
	 	scrollBar = scrollPane.getVerticalScrollBar();
	 	scrollBar.setValue(100000);
	 	iscrollBarMaxValue = scrollBar.getValue();
	 	scrollBar.setValue(0);
	 	iscrollBarMinValue = scrollBar.getValue();
	 	scrollBar.setValue((iscrollBarMinValue + iscrollBarMaxValue)/2);

	 	scrollBar = scrollPane.getHorizontalScrollBar();
	 	scrollBar.setValue(100000);
	 	iscrollBarMaxValue = scrollBar.getValue();
	 	scrollBar.setValue(0);
	 	iscrollBarMinValue = scrollBar.getValue();
	 	scrollBar.setValue((iscrollBarMinValue + iscrollBarMaxValue)/2);
	}
	private void writeFilteredFitsFiles(File outBatchHtmlFile)  throws FitsException, IOException
	{
		String	plotFileName;
		int		ifile;

		for (ifile = 0; ifile < 3; ++ifile)
		{
			statusBar.setMessage("Batch Cnt. " + Integer.toString(batchObjCnt) + " Writing Fits File " + Integer.toString(ifile + 1));
			plotFileName 
				= outBatchHtmlFile.getParent()
				+ delim
				+ coloredImageFitsFile[ifile].getName().substring(0, coloredImageFitsFile[ifile].getName().length() - 4)
				+ "-wv.fit";
			coloredImage[ifile].writeToFitsFile(plotFileName);
		}
		
	}
	private void singleFileWaveletFilter()  throws FitsException, IOException
	{
		statusBar.setMessage("Wavelet Filtering");
		String plotFileName;
		Wavelet wavelet = new Wavelet(waveletSettings);
		PixieImage inputImage = new PixieImage(singleInputFile);
		PixieImage outputImage = wavelet.waveFilter(inputImage);
		plotFileName  = singleInputFile.getPath().substring(0, singleInputFile.getPath().length() - 4)
			+ "-wv.fit";
		outputImage.writeToFitsFile(plotFileName);

//		Will do auto scaling if autoscale is set true otherwise it does nothing.
		bwRGBplotter.setPixelValueLimits(inputImage);
//	Will do histogram scaling if the scaletype is set to histogram otherwise it does nothing.
		bwRGBplotter.setScaleTable(inputImage);

		PixieImageRGBPlotter plotter = new PixieImageRGBPlotter();
		plotter.setImages(inputImage, inputImage, inputImage);
		plotter.copyScaling(bwRGBplotter);
		plotter.setScaleTable(null);
	 	ImageIcon imageIcon = new ImageIcon(plotter.makeBufferedImage());
	 	imageIconLabel[0].setIcon(imageIcon);
		centerScrollBars(colorImageScrollPane[0]);

		plotter.setImages(outputImage, outputImage, outputImage);
		plotter.copyScaling(bwRGBplotter);
		plotter.setScaleTable(null);
	 	imageIcon = new ImageIcon(plotter.makeBufferedImage());
	 	imageIconLabel[1].setIcon(imageIcon);
		centerScrollBars(colorImageScrollPane[1]);

		plotter = null;
		imageIcon = null;
		wavelet = null;
		outputImage = null;
		inputImage = null;
		statusBar.setMessage("Wavelet Filtering Finished");
	}
	void setWaveletSettings(Wavelet waveletSettings)
	{
		this.waveletSettings = waveletSettings;
	}
	void setProgressBar(JProgressBar progressBar)
	{
		this.progressBar = progressBar;
	}
	void setStatusBar(StatusBar statusBar)
	{
		this.statusBar = statusBar;
	}
	void setInputBatchFile(File inputBatchFile)
	{
		this.inputBatchFile = inputBatchFile;
	}
	void setBatchCrop(boolean batchCrop)
	{
		this.batchCrop = batchCrop;
	}
	void setBatchNormalize(boolean batchNormalize)
	{
		this.batchNormalize = batchNormalize;
	}
	void setBatchFilter(boolean batchFilter)
	{
		this.batchFilter = batchFilter;
	}
	void setBatchWritePngImages(boolean batchWritePngImages)
	{
		this.batchWritePngImages = batchWritePngImages;
	}
	void setBatchPngImageSize(int batchPngImageSize)
	{
		this.batchPngImageSize = batchPngImageSize;
	}
	void setInitBatchCropImageSize(int initBatchCropImageSize)
	{
		this.initBatchCropImageSize = initBatchCropImageSize;
	}
	void setBatchWriteFitsFiles(boolean batchWriteFitsFiles)
	{
		this.batchWriteFitsFiles = batchWriteFitsFiles;
	}
	void setBatchArcSearch(boolean batchArcSearch)
	{
		this.batchArcSearch = batchArcSearch;
	}
	void setBwRGBplotter(PixieImageRGBPlotter bwRGBplotter)
	{
		this.bwRGBplotter = bwRGBplotter;
	}
	void setClrRGBplotter(PixieImageRGBPlotter clrRGBplotter)
	{
		this.clrRGBplotter = clrRGBplotter;
	}
	void setImageIconLabel(JLabel[] imageIconLabel)
	{
		this.imageIconLabel = imageIconLabel;
	}
	void setColorImageScrollPane(JScrollPane[] colorImageScrollPane)
	{
		this.colorImageScrollPane = colorImageScrollPane;
	}
	void setColorBatchFile(boolean colorBatchFile)
	{
		this.colorBatchFile = colorBatchFile;
	}
	void setRunProcessBatchImages(boolean runProcessBatchImages)
	{
		this.runProcessBatchImages = runProcessBatchImages;
	}
	void setRunSingleFileWaveletFilter(boolean runSingleFileWaveletFilter)
	{
		this.runSingleFileWaveletFilter = runSingleFileWaveletFilter;
	}
	void setSingleInputFile(File singleInputFile)
	{
		this.singleInputFile = singleInputFile;
	}
	void setArcSearchGui(ArcSearchGui arcSearchGui)
	{
		this.arcSearchGui = arcSearchGui;
	}
}
