package gov.fnal.eag.sim.pixel.wavelet;

/**
 * @author mcginnis
 *
 */
public class BlendedArc 
{
	int				m_inum_pixels;
	int				m_iregion_min_num_pix;
	double			m_dred_to_blue_pixel_cut;
	Pixie[]			m_pixel = null;
/* 0 = g, 1 = r, 2 = i  */
	Pixie[][]		m_color_pixel = new Pixie[3][];
	PixieImage[]	coloredImage = null;
	int				m_inum_objects;
	double			m_dmin_layer_step;
	double			m_dlayer_overlap_frac;
	int				m_imax_no_layers;
	DeblendedArc[]	deblendedArc = null;

	BlendedArc(int inum_pixels, PixieImage[] coloredImage)
	{
		int		icolor;
		
		this.m_inum_pixels = inum_pixels;
		this.coloredImage = coloredImage;

		m_iregion_min_num_pix = 30;
		m_dmin_layer_step = 3.0;
		m_dlayer_overlap_frac = 0.30;
		m_imax_no_layers = 8;
		m_dred_to_blue_pixel_cut = 1.3;

		m_pixel = new Pixie[m_inum_pixels];
		for (int ii = 0; ii < m_inum_pixels; ++ii) m_pixel[ii] = new Pixie();
		for (icolor = 0; icolor < 3; ++icolor)
		{
			m_color_pixel[icolor] = new Pixie[m_inum_pixels];
			for (int ii = 0; ii < m_inum_pixels; ++ii) m_color_pixel[icolor][ii] = new Pixie();
		}

		m_inum_objects = 0;

	}
	static void vGrowRegion(Pixie p, Pixie pmax,
							  double[][]dpix1, int[][] iregion_array,
							  int inum_regions, double dthreshold, 
							  double dmax_threshold)
	{
		int		iradius;
		int		inum_pixels_added;
		int		inum_pixels_added_ring;
		Pixie	pi = new Pixie();
		Pixie	po = new Pixie();
		Pixie	pp = new Pixie();
		Pixie	pm = new Pixie();
		int		ii;
		int		ij;
		int		ik;
		int		il;

		iradius = 0;
		inum_pixels_added = 1;
		while (inum_pixels_added > 0)
		{
			iradius = iradius + 1;
			inum_pixels_added = 0;
	/* First check inside radius */
	/*		Check inside of 4 corners */
			for (ii = -1; ii <= 1; ii = ii + 2)
			{
				po.row = p.row + iradius * ii;
				pi.row = po.row - ii;
				for (ij = -1; ij <= 1; ij = ij + 2)
				{
					po.col = p.col + iradius * ij;
					pi.col = po.col  - ij;
					if (iCheckAdjPix(po, pi,pmax,
								iregion_array, inum_regions, dpix1, 
								dthreshold, dmax_threshold) )
					{
						iregion_array[po.row][po.col] = inum_regions;
						inum_pixels_added = inum_pixels_added + 1;
					}
				}
			}
	/*		Check top and bottom */
			for (ii = -1; ii <= 1; ii = ii + 2)
			{
				po.row = p.row + iradius * ii;
				pi.row = po.row - ii;
				for (ij = -iradius + 1; ij <= iradius - 1; ij = ij + 1)
				{
					po.col = p.col + ij;
					for (ik = -1; ik <= 1; ik = ik + 1)
					{
						il = ij + ik;
						if (il < -iradius + 1) il = -iradius + 1;
						if (il >  iradius - 1) il =  iradius - 1;
						pi.col = p.col + il;
						if (iCheckAdjPix(po, pi,pmax,
									iregion_array, inum_regions, dpix1, 
									dthreshold, dmax_threshold) )
						{
							iregion_array[po.row][po.col] = inum_regions;
							inum_pixels_added = inum_pixels_added + 1;
						}
					}
				}
			}
	/*		Check left and right */
			for (ii = -1; ii <= 1; ii = ii + 2)
			{
				po.col = p.col + iradius * ii;
				pi.col = po.col - ii;
				for (ij = -iradius + 1; ij <= iradius - 1; ij = ij + 1)
				{
					po.row = p.row + ij;
					for (ik = -1; ik <= 1; ik = ik + 1)
					{
						il = ij + ik;
						if (il < -iradius + 1) il = -iradius + 1;
						if (il >  iradius - 1) il =  iradius - 1;
						pi.row = p.row + il;
						if (iCheckAdjPix(po, pi,pmax,
									iregion_array, inum_regions, dpix1, 
									dthreshold, dmax_threshold) )
						{
							iregion_array[po.row][po.col] = inum_regions;
							inum_pixels_added = inum_pixels_added + 1;
						}
					}
				}
			}
	/* Next check outside radius */
			inum_pixels_added_ring = 1;
			while (inum_pixels_added_ring > 0)
			{
				inum_pixels_added_ring = 0;
		/*		Check  4 corners */
				for (ii = -1; ii <= 1; ii = ii + 2)
				{
					po.row = p.row + iradius * ii;
					pp.row = po.row - ii;
					pm.row = po.row;
					for (ij = -1; ij <= 1; ij = ij + 2)
					{
						po.col = p.col + iradius * ij;
						pp.col = po.col;
						pm.col = po.col - ij;
						if (iCheckAdjPix(po, pp,pmax,
									iregion_array, inum_regions, dpix1, 
									dthreshold, dmax_threshold) )
						{
								iregion_array[po.row][po.col] = inum_regions;
								inum_pixels_added = inum_pixels_added + 1;
								inum_pixels_added_ring = inum_pixels_added_ring + 1;
						}
						if (iCheckAdjPix(po, pm,pmax,
									iregion_array, inum_regions, dpix1, 
									dthreshold, dmax_threshold) )
						{
								iregion_array[po.row][po.col] = inum_regions;
								inum_pixels_added = inum_pixels_added + 1;
								inum_pixels_added_ring = inum_pixels_added_ring + 1;
						}
					}
				}
	/*		Check top and bottom */
				for (ii = -1; ii <= 1; ii = ii + 2)
				{
					po.row = p.row + iradius * ii;
					pp.row = po.row;
					pm.row = po.row;
					for (ij = -iradius + 1; ij <= iradius - 1; ij = ij + 1)
					{
						po.col = p.col + ij;
						pp.col = po.col + 1;
						pm.col = po.col - 1;
						if (iCheckAdjPix(po, pp,pmax,
									iregion_array, inum_regions, dpix1, 
									dthreshold, dmax_threshold) )
						{
								iregion_array[po.row][po.col] = inum_regions;
								inum_pixels_added = inum_pixels_added + 1;
								inum_pixels_added_ring = inum_pixels_added_ring + 1;
						}
						if (iCheckAdjPix(po, pm,pmax,
									iregion_array, inum_regions, dpix1, 
									dthreshold, dmax_threshold) )
						{
								iregion_array[po.row][po.col] = inum_regions;
								inum_pixels_added = inum_pixels_added + 1;
								inum_pixels_added_ring = inum_pixels_added_ring + 1;
						}
					}
				}
	/*		Check left and right */
				for (ii = -1; ii <= 1; ii = ii + 2)
				{
					po.col = p.col + iradius * ii;
					pp.col = po.col;
					pm.col = po.col;
					for (ij = -iradius + 1; ij <= iradius - 1; ij = ij + 1)
					{
						po.row = p.row + ij;
						pp.row = po.row + 1;
						pm.row = po.row - 1;
						if (iCheckAdjPix(po, pp,pmax,
									iregion_array, inum_regions, dpix1, 
									dthreshold, dmax_threshold) )
						{
								iregion_array[po.row][po.col] = inum_regions;
								inum_pixels_added = inum_pixels_added + 1;
								inum_pixels_added_ring = inum_pixels_added_ring + 1;
						}
						if (iCheckAdjPix(po, pm, pmax,
									iregion_array, inum_regions, dpix1, 
									dthreshold, dmax_threshold) )
						{
								iregion_array[po.row][po.col] = inum_regions;
								inum_pixels_added = inum_pixels_added + 1;
								inum_pixels_added_ring = inum_pixels_added_ring + 1;
						}
					}
				}
			}
		}
	}
	static boolean iValidCoord(Pixie pi, Pixie pmax)
	{
		boolean		iok;

		iok = false;
		if ( (0 <= pi.row) && ( pi.row < pmax.row) )
		{
			if ( (0 <= pi.col) && ( pi.col < pmax.col) )
			{
				iok = true;
			}
		}
		return iok;
	}
	static boolean iCheckAdjPix(Pixie px, Pixie pi,
			Pixie pmax, int[][] iregion_array, int inum_regions,
							  double[][] dpix1, double dthreshold, 
							  double dmax_threshold)
	{
		boolean	iok;

		iok = false;
		if (	iValidCoord(pi , pmax) 
			&&	iValidCoord(px, pmax) )
		{
			if (	(iregion_array[pi.row ][pi.col ] == inum_regions)
				&&	(iregion_array[px.row][px.col] == 0)
				&&	(dpix1[px.row][px.col] >= dthreshold) ) 
			{
				if (dmax_threshold < 0.0)
				{
					iok = true;
				}
				else
				{
					if (dpix1[px.row][px.col] <= dmax_threshold)
					{
						iok = true;
					}
				}
			}
		}
		return iok;
	}
	double[][] dMakeImage(Pixie pstart, Pixie pmax, Pixie[] input_pix)
	{
		Pixie	pstop = null;
		int		ipix;
		double[][] dpix = null;
		int		ii;
		int		ij;

		pstart.row = input_pix[0].row;
		pstart.col = input_pix[0].col;
		pstop = new Pixie(input_pix[0]);

		for ( ipix = 1; ipix < m_inum_pixels; ++ipix)
		{
			if (pstart.row > input_pix[ipix].row)
				pstart.row = input_pix[ipix].row;
			if (pstart.col > input_pix[ipix].col)
				pstart.col = input_pix[ipix].col;
			if ( pstop.row < input_pix[ipix].row)
				pstop.row  = input_pix[ipix].row;
			if ( pstop.col < input_pix[ipix].col)
				pstop.col  = input_pix[ipix].col;
		}
		pmax.row = pstop.row - pstart.row + 1;
		pmax.col = pstop.col - pstart.col + 1;
		dpix = new double [pmax.row][pmax.col];

		for ( ii = 0; ii < pmax.row; ++ii)
		{
			for ( ij = 0; ij < pmax.col; ++ij)
			{
				dpix[ii][ij] = 0.0;
			}
		}
		for ( ipix = 0; ipix < m_inum_pixels; ++ipix)
		{
			dpix[input_pix[ipix].row - pstart.row][input_pix[ipix].col - pstart.col] 
				= input_pix[ipix].val;
		}
		return dpix;
	}
	void vMakeLogImage(double[][] dpix, Pixie pmax)
	{
		int		ii;
		int		ij;
		double	dmin;
		double	dval;

		dmin = 1.0e+33;
		for ( ii = 0; ii < pmax.row; ++ii)
		{
			for ( ij = 0; ij < pmax.col; ++ij)
			{
				if (dpix[ii][ij] > 0.0)
				{
					dval = 10.0 * Math.log(dpix[ii][ij]);
					if (dmin > dval) dmin = dval;
				}
			}
		}
		for ( ii = 0; ii < pmax.row; ++ii)
		{
			for ( ij = 0; ij < pmax.col; ++ij)
			{
				if (dpix[ii][ij] > 0.0)
				{
					dpix[ii][ij] = 10.0 * Math.log(dpix[ii][ij]) / 2.30258;
					dpix[ii][ij] = dpix[ii][ij] - dmin + 1.0;
				}
			}
		}
	}
	void vRemoveRedPixels(double dred_to_blue_cut, double[][] dpix, 
									  Pixie pmax, Pixie pstart)
	{
		int		ipix;
		Pixie	p = new Pixie();
		double	dratio;

		for ( ipix = 0; ipix < m_inum_pixels; ++ipix)
		{
			p.row = m_pixel[ipix].row - pstart.row;
			p.col = m_pixel[ipix].col - pstart.col;
			dratio = m_color_pixel[2][ipix].val 
				/ (m_color_pixel[0][ipix].val + 1.0e-33);
			if (dratio > dred_to_blue_cut)
			{
				dpix[p.row][p.col] = 0.0;
			}
		}
	}
	void vFindObjects(boolean idebug)
	{
		int[][]		iobject_matrix= null;
		Pixie		pmax = new Pixie();
		double[][]	dpix_orig;
		double[][]	dpix_resd;
		Pixie		pstart = new Pixie();
		int			ii;
		int			ij;
		int			iobject;
		double		dmax;
		int			idist;
		int			idist_min;
		int			iobject_min_dist;
		int[][]		ipixel_map = null;
		Pixie[]		object_max_pixel = null;
		Pixie		p = new Pixie();
		//	idebug = TRUE;
		if (m_inum_pixels < m_iregion_min_num_pix) return;
		dpix_orig      = dMakeImage(pstart, pmax, m_pixel);
//	  dpix_resd was setup to hold heavily filtered wavelet residual image
//	  This was done so to reduce noise fluctuations
//		dpix_resd      = dMakeImage(pstart,  pmax,  m_color_pixel[3]);
		dpix_resd      = dMakeImage(pstart, pmax, m_pixel);
		iobject_matrix      = new int[pmax.row][pmax.col];
		vRemoveRedPixels(m_dred_to_blue_pixel_cut, dpix_resd, pmax, pstart);
		vResetObjectMatrix(iobject_matrix, dpix_resd, pmax);
		vMakeLogImage(dpix_resd, pmax);

		if (idebug) vPrintLayerSearch(dpix_resd, iobject_matrix,pmax, 0, 0);
		
//	 Grow each object in sequence to determine number of objects and
//		objects center
		m_inum_objects = iFindNoOfObjects(pmax, dpix_resd, iobject_matrix, idebug);
		if (idebug) vPrintLayerSearch(dpix_resd, iobject_matrix,pmax, 0, 1);
		object_max_pixel = new Pixie[m_inum_objects + 1];
		for (iobject = 0; iobject <= m_inum_objects; ++iobject) object_max_pixel[iobject] = new Pixie();
		for (iobject = 1; iobject <= m_inum_objects; ++iobject)
		{
			dmax = dFindLocalMax(dpix_resd, iobject_matrix, pmax, 
					object_max_pixel[iobject], iobject);
		}
		ipixel_map = new int[m_inum_pixels][m_inum_objects + 1];
		for ( ii = 0; ii < m_inum_pixels; ++ii)
		{
			for ( ij = 0; ij <= m_inum_objects; ++ij)
			{
				ipixel_map[ii][ij] = 0;
			}
			ipixel_map[ii][0] = -1;
		}
//	 Grow each object in the abscence of all objects
		for (iobject = 1; iobject <= m_inum_objects; ++iobject)
		{
			vResetObjectMatrix(iobject_matrix, dpix_resd, pmax);
			vGrowObject(iobject, object_max_pixel[iobject], pmax, dpix_resd,iobject_matrix, idebug);
	if (idebug) vPrintLayerSearch(dpix_resd, iobject_matrix,pmax, iobject, 0);
			for ( ii = 0; ii < m_inum_pixels; ++ii)
			{
				p.row = m_pixel[ii].row - pstart.row;
				p.col = m_pixel[ii].col - pstart.col;
				if (iobject_matrix[p.row][p.col] == iobject)
				{
					ipixel_map[ii][iobject] = 1;
					ipixel_map[ii][0] = 0;
				}
			}
		}
//	 Grow top 20% of each object
		vResetObjectMatrix(iobject_matrix, dpix_resd, pmax);
		for (iobject = 1; iobject <= m_inum_objects; ++iobject)
		{
			p.row = object_max_pixel[iobject].row;
			p.col = object_max_pixel[iobject].col;
			iobject_matrix[p.row][p.col] = iobject;
			dmax = dpix_resd[p.row][p.col];
			vGrowRegion(p, pmax, dpix_resd, iobject_matrix, iobject, 
								dmax * 0.8, dmax * 1.01);
	if (idebug) vPrintLayerSearch(dpix_resd, iobject_matrix,pmax, iobject, 1);
		}
		for ( ii = 0; ii < m_inum_pixels; ++ii)
		{
			p.row = m_pixel[ii].row - pstart.row;
			p.col = m_pixel[ii].col - pstart.col;
			if (iobject_matrix[p.row][p.col] > 0)
			{
				ipixel_map[ii][0] = iobject_matrix[p.row][p.col];
			}
			if (ipixel_map[ii][0] == 0)
			{
				idist_min = pmax.row * pmax.row + pmax.col * pmax.col;
				iobject_min_dist = 0;
				for (iobject = 1; iobject <= m_inum_objects; ++iobject)
				{
					if (ipixel_map[ii][iobject] == 1)
					{
						idist = iDistToObj(iobject, p, iobject_matrix, pmax);
						if (idist < idist_min)
						{
							idist_min = idist;
							iobject_min_dist = iobject;
						}
					}
				}
				ipixel_map[ii][0] = iobject_min_dist;
			}
		}
		for ( ii = 0; ii < m_inum_pixels; ++ii)
		{
			if (ipixel_map[ii][0] > 0)
			{
				p.row = m_pixel[ii].row - pstart.row;
				p.col = m_pixel[ii].col - pstart.col;
				iobject_matrix[p.row][p.col] = ipixel_map[ii][0];
			}
		}
	if (idebug) vPrintLayerSearch(dpix_resd, iobject_matrix,pmax, 0, 2);
		vCreateObjects(dpix_orig, iobject_matrix, pstart, pmax);

		dpix_orig = null;
		dpix_resd = null;
		iobject_matrix = null;
		object_max_pixel = null;
		ipixel_map = null;
	}
	void vResetObjectMatrix(int[][] iobject_matrix, double[][] dpix, Pixie pmax)
	{
		int		ii;
		int		ij;
//			Reset the object matrix
		for ( ii = 0; ii < pmax.row; ++ii)
		{
			for ( ij = 0; ij < pmax.col; ++ij)
			{
				if ( dpix[ii][ij] > 0.0 )
				{
					iobject_matrix[ii][ij]      = 0;
				}
				else
				{
					iobject_matrix[ii][ij]      = -1;
				}
			}
		}
	}
	int iDistToObj(int iobject, Pixie p, int[][] iobject_matrix, Pixie pmax)
	{
		int		ii;
		int		ij;
		int		idist;
		int		idist_min;

		idist_min = pmax.row * pmax.row + pmax.col * pmax.col;

		for ( ii = 0; ii < pmax.row; ++ii)
		{
			for ( ij = 0; ij < pmax.col; ++ij)
			{
				if (iobject_matrix[ii][ij] == iobject)
				{
					idist = (ii - p.row) * (ii - p.row)
						  + (ij - p.col) * (ij - p.col);
					if (idist < idist_min) idist_min = idist;
				}
			}
		}
		return idist_min;
	}
	int iFindNoOfObjects(Pixie pmax, double[][] dpix, int[][] iobject_matrix, boolean idebug)
	{
		int			ii;
		int			ij;
		int			inum_pixels;
		int			iactive_pixels;
		int			inum_objects;
		Pixie		plocalMax = new Pixie();

		iactive_pixels = 0;
		for ( ii = 0; ii < pmax.row; ++ii)
		{
			for ( ij = 0; ij < pmax.col; ++ij)
			{
				if (dpix[ii][ij] > 0.0) iactive_pixels = iactive_pixels + 1;
			}
		}

		inum_pixels = 0;
		inum_objects = 0;
		while ( inum_pixels < iactive_pixels  )
		{
			inum_objects = inum_objects + 1;		
			dFindLocalMax(dpix, iobject_matrix, pmax,  plocalMax, 0);
			vGrowObject(inum_objects, plocalMax, pmax, dpix, 
							iobject_matrix, idebug);
			inum_pixels = 0;
			for ( ii = 0; ii < pmax.row; ++ii)
			{
				for ( ij = 0; ij < pmax.col; ++ij)
				{
					if (iobject_matrix[ii][ij] > 0)
					{
						inum_pixels = inum_pixels + 1;
					}
				}
			}
		}

		return inum_objects;
	}
	/******************************************************************************/
	void vGrowObject(int iobject, Pixie plocalMax, Pixie pmax, double[][] dpix,
								 int[][] iobject_matrix, boolean idebug)
	{
		double[][]	dpix_copy = null;
		int[][]		iobject_matrix_copy = null;
		int			ii;
		int			ij;
		double		dthreshold_max;
		double		dthreshold;
		boolean		istop_layers;
		int			ilayer;
		double		dwindow_min;
		double		dwindow_max;

		dpix_copy = new double[pmax.row][pmax.col];
		iobject_matrix_copy = new int[pmax.row][pmax.col];
		for ( ii = 0; ii < pmax.row; ++ii)
		{
			for ( ij = 0; ij < pmax.col; ++ij)
			{
				iobject_matrix_copy[ii][ij] = iobject_matrix[ii][ij];
				dpix_copy[ii][ij] = dpix[ii][ij];
			}
		}

		dthreshold_max = dpix[plocalMax.row][plocalMax.col];
		dthreshold = ((double) (m_imax_no_layers - 1)) * m_dmin_layer_step;
		if (dthreshold > dthreshold_max) 
			dthreshold = dthreshold_max - m_dmin_layer_step;
		if (dthreshold < m_dmin_layer_step) dthreshold = 0.0;
		istop_layers = false;
		ilayer = 1;
		while (!istop_layers)
		{
			iobject_matrix_copy[plocalMax.row][plocalMax.col] = iobject;
//	 Make overlapping windows with thresholds
			dwindow_min = dthreshold - m_dlayer_overlap_frac * m_dmin_layer_step;
			if (dwindow_min < 0.0) dwindow_min = 0.0;
			dwindow_max = dthreshold_max + m_dlayer_overlap_frac * m_dmin_layer_step;
			vGrowRegion(plocalMax, pmax, dpix_copy, 
							iobject_matrix_copy, iobject, 
							dwindow_min, dwindow_max);
			for ( ii = 0; ii < pmax.row; ++ii)
			{
				for ( ij = 0; ij < pmax.col; ++ij)
				{
					if (iobject_matrix_copy[ii][ij] == iobject)
					{
						dpix_copy[ii][ij] = dthreshold;
						iobject_matrix[ii][ij] = iobject_matrix_copy[ii][ij];
						iobject_matrix_copy[ii][ij] = 0;
					}
				}
			}
			if ( dthreshold == 0.0 ) istop_layers = true;
			dthreshold_max = dthreshold;
			dthreshold = dthreshold - m_dmin_layer_step;
			if (dthreshold < m_dmin_layer_step) dthreshold = 0.0;
			ilayer = ilayer + 1;
		}
		dpix_copy = null;
		iobject_matrix_copy = null;


	}
	void vPrintLayerSearch(double[][] dpix, int[][] iobject_matrix,
									  Pixie pmax, int iobject, int ilayer)
	{
/*		FILE*	Foutput;
		int		ii;
		int		ij;
		char	cfileName[256];
		CFileName	Cname;
		char	cshortName[256];

		sprintf(cshortName,"obj_O%d_L%d",iobject,ilayer);
		Cname.vCombineFileName(cfileName,m_cbatch_directory,cshortName,"dat");

		Foutput = fopen(cfileName,"w");
		for ( ii = 0; ii < inaxes[1]; ++ii)
		{
			for ( ij = 0; ij < inaxes[0]; ++ij)
			{
				fprintf(Foutput,"%d\t",iobject_matrix[ii][ij]);
			}
			fprintf(Foutput,"\n");
		}
		fclose(Foutput);

		sprintf(cshortName,"pix_O%d_L%d",iobject,ilayer);
		Cname.vCombineFileName(cfileName,m_cbatch_directory,cshortName,"dat");
		Foutput = fopen(cfileName,"w");
		for ( ii = 0; ii < inaxes[1]; ++ii)
		{
			for ( ij = 0; ij < inaxes[0]; ++ij)
			{
				fprintf(Foutput,"%.1f\t",dpix[ii][ij]);
			}
			fprintf(Foutput,"\n");
		}
		fclose(Foutput);
*/
	}
	double dFindLocalMax(double[][] dpix, int[][] iobject_matrix, 
								   Pixie pmax, Pixie plocalMax, int iobject)
	{
		int			ii;
		int			ij;
		double		dmax;

		plocalMax.row = 0;
		plocalMax.col = 0;
		dmax = dpix[plocalMax.row][plocalMax.col];
		dmax = 0.0;
		for ( ii = 0; ii < pmax.row; ++ii)
		{
			for ( ij = 0; ij < pmax.col; ++ij)
			{
				if (iobject_matrix[ii][ij] == iobject )
				{
					if (dmax < dpix[ii][ij]) 
					{
						dmax = dpix[ii][ij];
						plocalMax.row = ii;
						plocalMax.col = ij;
					}
				}
			}
		}
		return dmax;
	}
	void vCreateObjects(double[][] dpix, int[][] iobject_matrix,
									Pixie pstart, Pixie pmax)
	{
		int			ii;
		int			ij;
		int			ik;
		int			icolor;
		Pixie		pstop = new Pixie();
		int[]		numPixels = new int[m_inum_objects];
		double[][]	colorPixArray = null;
		int			iobject;

		for (ik = 0; ik < m_inum_objects; ik++)
		{
			numPixels[ik] = 0;
		}
		for (ii = 0; ii < pmax.row; ii++)
		{  
			for (ij = 0; ij < pmax.col; ij++) 
			{
				if (iobject_matrix[ii][ij] > -1)
				{
					numPixels[iobject_matrix[ii][ij] - 1] 
					          = numPixels[iobject_matrix[ii][ij] - 1] + 1;
				}
			}
		}

		pstop.row = pstart.row + pmax.row - 1;
		pstop.col = pstart.col + pmax.col - 1;

		deblendedArc = new DeblendedArc[m_inum_objects];
		for (ik = 0; ik < m_inum_objects; ik++)
		{
			deblendedArc[ik] = new DeblendedArc(numPixels[ik]);
			numPixels[ik] = 0;
		}

		for (ii = 0; ii < pmax.row; ii++)
		{  
			for (ij = 0; ij < pmax.col; ij++) 
			{
				iobject = iobject_matrix[ii][ij] - 1;
				if (iobject > -1)
				{
					ik = numPixels[iobject];
					deblendedArc[iobject].m_pixel[ik].row = ii + pstart.row;
					deblendedArc[iobject].m_pixel[ik].col = ij + pstart.col;
					deblendedArc[iobject].m_pixel[ik].val = dpix[ii][ij];
					for (icolor = 0; icolor < 3; ++icolor)
					{
						colorPixArray = coloredImage[icolor].getPix();
						deblendedArc[iobject].m_color_pixel[icolor][ik].row = ii + pstart.row;
						deblendedArc[iobject].m_color_pixel[icolor][ik].col = ij + pstart.col;
						deblendedArc[iobject].m_color_pixel[icolor][ik].val 
							= colorPixArray[ii + pstart.row][ij + pstart.col];
					}
					numPixels[iobject] = numPixels[iobject] + 1;
				}
			}
		}
		for (ii = 0; ii < m_inum_objects; ii++)
		{
			deblendedArc[ii].vCurveFit();	
			deblendedArc[ii].vFindColorInObject();
		}
	}
}
