package gov.fnal.eag.sim.pixel.wavelet;

/**
 * @author mcginnis
 *
 */
public class DeblendedArc 
{
	int			m_inum_pixels = 0;
	Pixie[]		m_pixel = null;
	double[]	m_dfit_coef = null;
	int			m_ifit_order;
	double		m_dsum;
	double[]	m_dcenter_of_g = new double[2];
	double[]	m_darc_center = new double[2];
	double[]	m_dcos_sin_theta = new double[2];
	double		m_drot_angle;
	double		m_dlength;
	double		m_dmin_max_width;
	double		m_drms_width;
	double		m_drad_curv;
	double		m_dtheta_start;
	double		m_dtheta_stop;
/* 0 = g, 1 = r, 2 = i */
	Pixie[][]	m_color_pixel = new Pixie[3][];
	double[]	m_davg_color = new double[3];
	double[]	m_dmedium_color = new double[3];
	int			m_iobject_min_num_pix;
	double		m_dobject_min_len;
	double		m_dobject_min_l_w;
	double		m_dobject_max_curve;
	double		m_dobject_min_curve;
	double		m_dmax_median_red_blue_color_ratio;
	int			m_isearch_radius;
	Moment		m_Cmoment = new Moment();
	
	DeblendedArc(int inum_pixels)
	{
		this.m_inum_pixels = inum_pixels;
		m_ifit_order = 4;

		m_iobject_min_num_pix = 40;
		m_dobject_min_len = 10.0;
		m_dobject_min_l_w = 0.0;
		m_dobject_max_curve = 1000.0;
		m_dobject_min_curve = 0.0;
		m_dmax_median_red_blue_color_ratio = 1.5;
		m_isearch_radius = 180;

		m_pixel = new Pixie[m_inum_pixels];
		for (int ii = 0; ii < m_inum_pixels; ++ii) m_pixel[ii] = new Pixie();
		for (int icolor = 0; icolor < 3; ++icolor)
		{
			m_color_pixel[icolor] = new Pixie[m_inum_pixels];
			for (int ii = 0; ii < m_inum_pixels; ++ii) m_color_pixel[icolor][ii] = new Pixie();
		}
		m_dfit_coef  = new double[m_ifit_order + 1];
	}
	void vLocalCoords(double[] dlocal_coord, double[] dglobal_coord)
	{
		dlocal_coord[0] 
			=  m_dcos_sin_theta[0] * (dglobal_coord[0] - m_dcenter_of_g[0])
			+  m_dcos_sin_theta[1] * (dglobal_coord[1] - m_dcenter_of_g[1]);
		dlocal_coord[1] 
			= -m_dcos_sin_theta[1] * (dglobal_coord[0] - m_dcenter_of_g[0])
			+  m_dcos_sin_theta[0] * (dglobal_coord[1] - m_dcenter_of_g[1]);
	}
	void vGlobalCoords(double[] dlocal_coord, double[] dglobal_coord)
	{
		dglobal_coord[0] 
			=  m_dcos_sin_theta[0] * dlocal_coord[0] 
			-  m_dcos_sin_theta[1] * dlocal_coord[1]
			+  m_dcenter_of_g[0];
		dglobal_coord[1] 
			=  m_dcos_sin_theta[1] * dlocal_coord[0]
			+  m_dcos_sin_theta[0] * dlocal_coord[1]
			+  m_dcenter_of_g[1];
	}
	void vCurveFit()
	{
		double[][]	da = null;
		double[][]	db = null;
		double[]	dxp = null;
		double[]	du = new double[3];
		double[]	dv = new double[3];		
		double		dyp;
		double		dwp;
		double[]	dulimits = new double[2];
		double[]	dvlimits = new double[2];
		double[]	dlocal_coord = new double[2];
		double[]	dglobal_coord = new double[2];
		int			ii;
		int			ij;
		int			ip;
		int			ik;
		boolean		istatus;
		int			iorder_linear;
		double		dmax;
		double		dmin;
		double[]	dm = new double[2];
		double[]	dr = new double[2];
		CircleFit	Cfit_circle = new CircleFit();
		double		dtheta;
		double		ddelta_theta_slice;
		int[]		itheta_count = new int[401];
		int			itheta_gap_start;
		int			itheta_gap_stop;
		int			inum_theta_slices;
//		double		PI = 3.141592654;
		double		PI2 = 6.283185308;


		iorder_linear = 1;

		m_dsum = 0.0;
		m_dcenter_of_g[0] = 0.0;
		m_dcenter_of_g[1] = 0.0;

		for ( ip = 0; ip < m_inum_pixels; ++ip)
		{
			dwp = m_pixel[ip].val;
			m_dsum = m_dsum + dwp;
			m_dcenter_of_g[0] = m_dcenter_of_g[0] 
				+ ((double) m_pixel[ip].col ) * dwp;
			m_dcenter_of_g[1] = m_dcenter_of_g[1] 
				+ ((double) m_pixel[ip].row ) * dwp;
		}
		m_dcenter_of_g[0] = m_dcenter_of_g[0] / m_dsum;
		m_dcenter_of_g[1] = m_dcenter_of_g[1] / m_dsum;

		if ( m_inum_pixels < 2 * (m_ifit_order + 1))
		{
			m_dcos_sin_theta[0] = 1.0;
			m_dcos_sin_theta[1] = 0.0;
			m_drot_angle = 0.0;
			m_dlength = 0.0;
			m_dmin_max_width = 0.0;
			m_drms_width = 0.0;
			m_drad_curv = 0.0;
			return;
		}

// add one element extra onto matrix because NR starts index at 1		
		da = new double[m_ifit_order + 1 + 1][m_ifit_order + 1 + 1];
		db = new double[m_ifit_order  + 1 + 1][2 + 1];
		dxp = new double[2 * m_ifit_order + 1];

		for (ii = 0; ii <= m_ifit_order; ++ii)
		{
			db[ii + 1][1] = 0.0;
			for (ij = 0; ij <= m_ifit_order; ++ij)
			{
				da[ii + 1][ij + 1] = 0.0;
			}
		}
		for ( ip = 0; ip < m_inum_pixels; ++ip)
		{
			dxp[0] = 1.0;
			for (ik = 1; ik <= 2 * iorder_linear; ++ik)
			{
				dxp[ik] = dxp[ik - 1] 
					* (((double) m_pixel[ip].col) - m_dcenter_of_g[0]);
			}
			dyp = (((double) m_pixel[ip].row) - m_dcenter_of_g[1]);
			dwp = m_pixel[ip].val;
			for (ii = 0; ii <= iorder_linear; ++ii)
			{
				db[ii + 1][1] = db[ii + 1][1] + dwp * dxp[ii] * dyp;
				for (ij = 0; ij <= iorder_linear; ++ij)
				{
					da[ii + 1][ij + 1] = da[ii + 1][ij + 1] 
						+ dwp * dxp[ii + ij];
				}
			}
		}
		istatus = NR.Guassj( da, iorder_linear + 1, db, 1);

		for (ii = 0; ii <= iorder_linear; ++ii)
		{
			m_dfit_coef[ii] = db[ii + 1][1];
		}
		m_drot_angle = Math.atan(m_dfit_coef[1]);
		m_dcos_sin_theta[0] = Math.cos(m_drot_angle);
		m_dcos_sin_theta[1] = Math.sin(m_drot_angle);

		for (ii = 0; ii <= m_ifit_order; ++ii)
		{
			db[ii + 1][1] = 0.0;
			for (ij = 0; ij <= m_ifit_order; ++ij)
			{
				da[ii + 1][ij + 1] = 0.0;
			}
		}
		for ( ip = 0; ip < m_inum_pixels; ++ip)
		{
			dglobal_coord[0] = (double) m_pixel[ip].col;
			dglobal_coord[1] = (double) m_pixel[ip].row;
			vLocalCoords(dlocal_coord, dglobal_coord);
			dxp[0] = 1.0;
			for (ik = 1; ik <= 2 * m_ifit_order; ++ik)
			{
				dxp[ik] = dxp[ik - 1] * dlocal_coord[0];
			}
			dyp = dlocal_coord[1];
			dwp = m_pixel[ip].val;
			for (ii = 0; ii <= m_ifit_order; ++ii)
			{
				db[ii + 1][1] = db[ii + 1][1] + dwp * dxp[ii] * dyp;
				for (ij = 0; ij <= m_ifit_order; ++ij)
				{
					da[ii + 1][ij + 1] = da[ii + 1][ij + 1] 
						+ dwp * dxp[ii + ij];
				}
			}
		}
		istatus = NR.Guassj( da, m_ifit_order + 1, db, 1);
		m_dfit_coef[0] = 0.0;
		m_dfit_coef[1] = 0.0;
		for (ii = 0; ii <= m_ifit_order; ++ii)
		{
			m_dfit_coef[ii] = db[ii + 1][1];
		}
		dulimits[0] = -1.0e33;
		dulimits[1] =  1.0e33;
		dvlimits[0] = -1.0e33;
		dvlimits[1] =  1.0e33;
		for ( ip = 0; ip < m_inum_pixels; ++ip)
		{
			dglobal_coord[0] = (double) m_pixel[ip].col;
			dglobal_coord[1] = (double) m_pixel[ip].row;
			vLocalCoords(dlocal_coord, dglobal_coord);
			if (dlocal_coord[0] > dulimits[0] ) 
				dulimits[0] = dlocal_coord[0];
			if (dlocal_coord[0] < dulimits[1] ) 
				dulimits[1] = dlocal_coord[0];

			dlocal_coord[1] = dlocal_coord[1]
				- dPoly(dlocal_coord[0], m_dfit_coef, m_ifit_order);
			if (dlocal_coord[1] > dvlimits[0] ) 
				dvlimits[0] = dlocal_coord[1];
			if (dlocal_coord[1] < dvlimits[1] ) 
				dvlimits[1] = dlocal_coord[1];
		}


//	 finding rad of curve be fitting 2 perp lines to curve and finding
//	 intersection
		du[0] = dulimits[0] + 0.33 * (dulimits[1] - dulimits[0]);
		du[1] = dulimits[0] + 0.67 * (dulimits[1] - dulimits[0]);
		dv[0] = dPoly(du[0], m_dfit_coef, m_ifit_order);
		dv[1] = dPoly(du[1], m_dfit_coef, m_ifit_order);
		dm[0] = -1.0 / dPoly1stDeriv(du[0], m_dfit_coef, m_ifit_order);
		dm[1] = -1.0 / dPoly1stDeriv(du[1], m_dfit_coef, m_ifit_order);
		du[2] = ( (dv[0] - dm[0] * du[0]) - (dv[1] - dm[1] * du[1]) )
			/ (dm[1] - dm[0]);
		dv[2] = ( dm[1] * (dv[0] - dm[0] * du[0]) - dm[0] * (dv[1] - dm[1] * du[1]) )
			/ (dm[1] - dm[0]);
		dr[0] = (du[0] - du[2]) * (du[0] - du[2]) 
			  + (dv[0] - dv[2]) * (dv[0] - dv[2]);
		dr[0] = Math.sqrt(dr[0]);
		dr[1] = (du[1] - du[2]) * (du[1] - du[2]) 
			  + (dv[1] - dv[2]) * (dv[1] - dv[2]);
		dr[1] = Math.sqrt(dr[0]);
		m_drad_curv = 0.5 * (dr[0] + dr[1]);

		dlocal_coord[0] = du[2];
		dlocal_coord[1] = dv[2];

		vGlobalCoords(dlocal_coord, m_darc_center);
		Cfit_circle.m_pro = this;
		Cfit_circle.vFitCircle();
		for ( ii = 0; ii <= 400; ++ii) itheta_count[ii] = 0;

		m_dmin_max_width = 0.0;
		m_drms_width = 0.0;
		dmax = -1.0e+33;
		dmin =  1.0e+33;
		ddelta_theta_slice = 1.5 / m_drad_curv; // 1.5 pixels
		if (ddelta_theta_slice < 0.0175) ddelta_theta_slice = 0.0175; //1 degree
		inum_theta_slices = (int) Math.floor(PI2 / ddelta_theta_slice);
		if (inum_theta_slices < 1) inum_theta_slices = 1;
		ddelta_theta_slice = PI2 / ((double) inum_theta_slices);
		for ( ip = 0; ip < m_inum_pixels; ++ip)
		{
			dglobal_coord[0] = ((double) m_pixel[ip].col) - m_darc_center[0];
			dglobal_coord[1] = ((double) m_pixel[ip].row) - m_darc_center[1];
			dtheta = Math.atan2(dglobal_coord[1], dglobal_coord[0] + 1.0e-10);
			if (dtheta < 0.0) dtheta = dtheta + PI2;
			ii = (int) Math.floor(dtheta / ddelta_theta_slice);
			itheta_count[ii] = itheta_count[ii] + 1;
			dwp = dglobal_coord[0] * dglobal_coord[0] 
				+ dglobal_coord[1] * dglobal_coord[1];
			dwp = Math.sqrt(dwp);
			dwp = dwp - m_drad_curv;
			if (dwp > dmax ) dmax = dwp;
			if (dwp < dmin ) dmin = dwp;
			m_drms_width = m_drms_width + dwp * dwp * m_pixel[ip].val;
		}
		m_dmin_max_width = dmax - dmin;
		m_drms_width = 2.0 * Math.sqrt(m_drms_width / m_dsum);
		m_dlength = m_drad_curv * (m_dtheta_stop - m_dtheta_start);
//	 Find the angles where there is no arc
		itheta_gap_start = 0;
		itheta_gap_stop = 0;
		istatus = false;
		while (!istatus && (itheta_gap_start < inum_theta_slices))
		{
			if (itheta_count[itheta_gap_start] == 0)
			{
				istatus = true;
			}
			else
			{
				itheta_gap_start = itheta_gap_start + 1;
			}
		}
		if (istatus == false)
		{
			m_dtheta_start =  0.0;
			m_dtheta_stop  =  PI2;
		}
		else
		{
			while (itheta_count[itheta_gap_start] == 0)
			{
				itheta_gap_start = itheta_gap_start - 1;
				if (itheta_gap_start < 0) itheta_gap_start = inum_theta_slices - 1;
			}
			itheta_gap_start = itheta_gap_start + 1;
			if (itheta_gap_start > (inum_theta_slices - 1)) itheta_gap_start = 0;
			itheta_gap_stop = itheta_gap_start;
			while (itheta_count[itheta_gap_stop] == 0)
			{
				itheta_gap_stop = itheta_gap_stop + 1;
				if (itheta_gap_stop > (inum_theta_slices - 1)) itheta_gap_stop = 0;
			}
			itheta_gap_stop = itheta_gap_stop - 1;
			if (itheta_gap_stop < 0) itheta_gap_stop = inum_theta_slices - 1;
		}
		m_dtheta_start = (((double) itheta_gap_stop) + 0.5) * ddelta_theta_slice;
		m_dtheta_stop = (((double) itheta_gap_start) - 0.5) * ddelta_theta_slice;
		if (m_dtheta_stop < m_dtheta_start) m_dtheta_stop = m_dtheta_stop + PI2;
		m_dlength = (m_dtheta_stop - m_dtheta_start) * m_drad_curv;

		da = null;
		db = null;
		dxp = null;

		m_Cmoment.m_iweight_on = true;
		m_Cmoment.m_inum_pixels = m_inum_pixels;
		m_Cmoment.m_pixel = m_pixel;
		m_Cmoment.vCalcRotatedMoments();
	}
	private double dPoly(double dx, double[] dfit_coef, int iorder)
	{
		double	dy;
		double	dxp;
		int		ii;

		dxp = 1.0;
		dy = dfit_coef[0];
		for ( ii = 1; ii <= iorder; ++ii)
		{
			dxp = dxp * dx;
			dy = dy + dxp * dfit_coef[ii];
		}
		return dy;
	}
	private double dPoly1stDeriv(double dx, double[] dfit_coef, int iorder)
	{
		double	dy;
		double	dxp;
		int		ii;

		if (iorder < 1) return 1.0e-33;
		dy = dfit_coef[1];
		if ( iorder < 2) return dy + 1.0e-33;
		dxp = 1.0;
		for ( ii = 2; ii <= iorder; ++ii)
		{
			dxp = dxp * dx;
			dy = dy + dxp * dfit_coef[ii] * ((double) ii);
		}
		return dy + 1.0e-33;
	}
	double dPoly2ndDeriv(double dx, double[] dfit_coef, int iorder)
	{
		double	dy;
		double	dxp;
		int		ii;

		if (iorder < 2) return 0.0;
		dy = dfit_coef[2] * (2.0);
		if ( iorder < 3) return dy;
		dxp = 1.0;
		for ( ii = 3; ii <= iorder; ++ii)
		{
			dxp = dxp * dx;
			dy = dy + dxp * dfit_coef[ii] * ((double) ii) * ((double) (ii - 1));
		}
		return dy;
	}
	void vFindColorInObject()
	{
		double[]	dpixel_value = null;
		int		ii;
		int		icolor;

//	 Numerical Recipes has all the arrays indexed at 1 
		dpixel_value = new double[m_inum_pixels + 1];
		for (icolor  = 0; icolor < 3; ++icolor)
		{
			m_davg_color[icolor] = 0.0;
			for ( ii = 0; ii < m_inum_pixels; ++ii)
			{
				dpixel_value[ii + 1] = 	m_color_pixel[icolor][ii].val;
				m_davg_color[icolor] = m_davg_color[icolor] + dpixel_value[ii + 1];
			}
			m_davg_color[icolor] = m_davg_color[icolor] / ((double) m_inum_pixels);

		// Numerical Recipes has all the arrays indexed at 1 
			m_dmedium_color[icolor] = NR.select(m_inum_pixels / 2 + 1, 
												m_inum_pixels, dpixel_value);
		}
		dpixel_value = null;
	}
	boolean iPassedCuts()
	{

		double	dl_over_w;
		double	dred_blue_color_ratio;

		dl_over_w = m_dlength / m_dmin_max_width;
		dred_blue_color_ratio = m_dmedium_color[2] / m_dmedium_color[0];

		if ( m_inum_pixels < m_iobject_min_num_pix ) return false;
		if ( m_dlength     < m_dobject_min_len )     return false;
		if ( dl_over_w     < m_dobject_min_l_w )     return false;
		if ( m_drad_curv   > m_dobject_max_curve )	 return false;
		if ( m_drad_curv   < m_dobject_min_curve )	 return false;
		if ( dred_blue_color_ratio   > m_dmax_median_red_blue_color_ratio )	 
			return false;
		
		return true;
	}
	double	dRadiusFit(boolean iuse_pixel_value)
	{
		double	dsigma;
		double	dsum;
		double	dr;
		int		ipixel;

		m_drad_curv = 0.0;
		dsum = 0.0;
		dsigma = 0.0;
		for (ipixel = 0; ipixel < m_inum_pixels; ++ipixel)
		{
			dr =  (((double) m_pixel[ipixel].col) - m_darc_center[0])
				* (((double) m_pixel[ipixel].col) - m_darc_center[0])
				+ (((double) m_pixel[ipixel].row) - m_darc_center[1])
				* (((double) m_pixel[ipixel].row) - m_darc_center[1]);
			dr = Math.sqrt(dr);
			if (iuse_pixel_value)
			{
				dsum = dsum + m_pixel[ipixel].val;
				m_drad_curv= m_drad_curv + dr * m_pixel[ipixel].val;
			}
			else
			{
				dsum = dsum + 1.0;
				m_drad_curv = m_drad_curv + dr;
			}
		}
		m_drad_curv = m_drad_curv / dsum;
		for (ipixel = 0; ipixel < m_inum_pixels; ++ipixel)
		{
			dr =  (((double) m_pixel[ipixel].col) - m_darc_center[0])
				* (((double) m_pixel[ipixel].col) - m_darc_center[0])
				+ (((double) m_pixel[ipixel].row) - m_darc_center[1])
				* (((double) m_pixel[ipixel].row) - m_darc_center[1]);
			dr = Math.sqrt(dr);
			dr = (dr - m_drad_curv) * (dr - m_drad_curv);
			if (iuse_pixel_value)
			{
				dsigma = dsigma + dr * m_pixel[ipixel].val;
			}
			else
			{
				dsigma = dsigma + dr;
			}
		}
		dsigma = Math.sqrt(dsigma / dsum);
		return dsigma;
	}
	PixieImage makePixieImage(Pixie pmax, PixieImage backgroundImage, boolean plotArcPix, boolean plotArcFit, boolean plotBackground)
	{
		
		int 		inum_int_pts = 100;
		PixieImage	pi = new PixieImage(pmax.row, pmax.col);
		double[][]	dpix = pi.getPix();
		int			ii ;
		int			ij;
		Pixie		p = new Pixie();
		
		double dmax = 0.0;
		if (plotBackground)
		{
			for (ii = 0; ii < pmax.row; ++ii)
			{
				for (ij = 0; ij < pmax.col; ++ij)
				{
					dpix[ii][ij] = dpix[ii][ij] + backgroundImage.getPix()[ii][ij];
				}
			}
		}
		for ( ii = 0; ii < m_inum_pixels; ++ii)
		{
			if (plotArcPix) 
			{
				dpix[m_pixel[ii].row][m_pixel[ii].col] = m_pixel[ii].val;
			}
			else
			{
				dpix[m_pixel[ii].row][m_pixel[ii].col] = 0.0;
			}
			if ( dmax < m_pixel[ii].val ) dmax = m_pixel[ii].val ;
		}
		for (ii = 0; ii <= inum_int_pts; ++ii)
		{
			double dtheta = m_dtheta_start + (m_dtheta_stop - m_dtheta_start)
				* ((double) ii) / ((double) inum_int_pts);
			p.col = (int) (m_drad_curv * Math.cos(dtheta) + m_darc_center[0]);
			p.row = (int) (m_drad_curv * Math.sin(dtheta) + m_darc_center[1]);
			if (p.col < 0 ) p.col = 0;
			if (p.col >= pmax.col) p.col = pmax.col - 1;
			if (p.row < 0 ) p.row = 0;
			if (p.row >= pmax.row ) p.row = pmax.row - 1;
			if (plotArcFit)
			{
				dpix[p.row][p.col] = 2.0 * dmax;
			}
			else
			{
				dpix[p.row][p.col] = 0.0;
			}
		}
		return pi;
	}
}

