package gov.fnal.eag.sim.pixel.wavelet;

/**
 * @author mcginnis
 *
 */
public class Moment 
{
	int			m_inum_pixels;
	Pixie[]		m_pixel = null;
	boolean		m_iweight_on;
	double		m_darea;
	double		m_drow_mean;
	double		m_dcol_mean;
	double[][]	m_dunrotated = new double[4][4];
	double[][]	m_drotated = new double[4][4];
	
	void vCalcAreaAndMean()
	{
		int		ii;

		m_darea = 0.0;
		m_drow_mean = 0.0;
		m_dcol_mean = 0.0;

		for (ii = 0; ii < m_inum_pixels; ++ii)
		{
			if (m_iweight_on)
			{	
				m_darea = m_darea + m_pixel[ii].val;
				m_drow_mean = m_drow_mean + m_pixel[ii].val 
					* m_pixel[ii].row;
				m_dcol_mean = m_dcol_mean + m_pixel[ii].val 
					* m_pixel[ii].col;
			}
			else
			{
				m_darea = m_darea + 1.0;
				m_drow_mean = m_drow_mean + m_pixel[ii].row;
				m_dcol_mean = m_dcol_mean + m_pixel[ii].col;
			}
		}
		m_drow_mean = m_drow_mean / m_darea;
		m_dcol_mean = m_dcol_mean / m_darea;
	}
	double dCalcUnRotatedMoment(int icol_order, int irow_order)
	{
		int		ii;
		double	dmoment;
		double	dx;
		double	dy;
		double	dscale;

		dmoment = 0.0;

		for (ii = 0; ii < m_inum_pixels; ++ii)
		{
			dx = dPow(m_pixel[ii].col - m_dcol_mean, icol_order);
			dy = dPow(m_pixel[ii].row - m_drow_mean, irow_order);

			if (m_iweight_on)
			{	
				dmoment = dmoment + m_pixel[ii].val * dx * dy; 
			}
			else
			{
				dmoment = dmoment + dx * dy; 
			}
		}
		dscale = dPow(Math.sqrt(m_darea), icol_order + irow_order + 2);
		dmoment = dmoment / dscale;
		return dmoment;
	}
	void vCalcUnRotatedMoments()
	{
		int		ii;
		int		ij;
		for (ii = 0; ii < 4; ++ii)
		{
			for (ij = 0; ij < 4; ++ij)
			{
				m_dunrotated[ii][ij] = 0.0;
			}
		}
		m_dunrotated[0][0] = 1.0;
		m_dunrotated[1][0] = 0.0;
		m_dunrotated[0][1] = 0.0;
		m_dunrotated[1][1] = dCalcUnRotatedMoment(1, 1);
		m_dunrotated[2][0] = dCalcUnRotatedMoment(2, 0);
		m_dunrotated[0][2] = dCalcUnRotatedMoment(0, 2);
		m_dunrotated[2][1] = dCalcUnRotatedMoment(2, 1);
		m_dunrotated[1][2] = dCalcUnRotatedMoment(1, 2);
		m_dunrotated[3][0] = dCalcUnRotatedMoment(3, 0);
		m_dunrotated[0][3] = dCalcUnRotatedMoment(0, 3);

		return;
	}
	double dTheta()
	{
	   double a, b, c, r, t1, t2 ;
	   double[] vectmin = new double[2];
	   double[] vectmax = new double[2];
	   double dtheta;
	   double		PI = 3.141592654;
	   double		PI2 = 6.283185308;

	   a = m_dunrotated[2][0] ;   // m20
	   b = m_dunrotated[1][1] ;   // m11
	   c = m_dunrotated[0][2] ;   // m02

	   r = Math.sqrt ( (a - c) * (a - c)  +  4.0 * b * b ) ;

	   t1 = a - c + r ;  // If the axes are nearly vertical / horizontal
	   t2 = a - c - r ;  // one of these will be tiny, along with b

	   if (Math.abs(t1) >= Math.abs(t2) ) {  // Avoid using the tiny one
	      vectmax[0] = t1 ;          // Use the larger
	      vectmax[1] = 2.0 * b ;     // And get the other
	      vectmin[0] = 2.0 * b ;     // by orthogonality
	      vectmin[1] = -t1 ;
	      }
	   else {
	      vectmax[0] = 2.0 * b ;
	      vectmax[1] = -t2 ;
	      vectmin[0] = t2 ;
	      vectmin[1] = 2.0 * b ;
	      }

	   dtheta = Math.atan2(vectmax[1], vectmax[0] + 1.0e-33);
	   if (m_dunrotated[3][0] < 0.0 ) dtheta = dtheta + PI;
	   if (dtheta > PI) dtheta = dtheta - PI2;
	   return dtheta;
	}
	double dPow ( double base , int power )
	{
	   double result = 1.0 ;

	   while (power-- > 0)
	      result *= base ;
	   return result ;
	}
	int ibinom ( int n , int m )
	{
	   int prod = 1 ;
	   int den = 1 ;

	   while (m > 0) {
	      prod *= n-- ;
	      den *= m-- ;
	      }

	   return prod / den ;
	}
	double dRotMom1 ( int p , int q , double[][] m , double theta )
	{
		int r, s; ;
		double sgn = -1.0;
		double sum = 0.0 ;
		double ct = Math.cos ( theta ) ;
		double st = Math.sin ( theta ) ;
		int		itest;

		for (r=0 ; r<=p ; r++) 
		{
			for (s=0 ; s<=q ; s++) 
			{
// check to see if q-s is odd
				itest = (q - s) / 2;
				itest = 2 * itest;
				if (itest != (q - s))
				{
					sgn = -1.0;
				}
				else
				{
					sgn = 1.0;
				}
				sum += sgn * ibinom ( p , r ) * ibinom ( q , s ) *
	                dPow ( ct , p-r+s ) * dPow ( st , q-s+r ) * m[p+q-r-s][r+s] ;
			}
		}
		return sum ;
	}
	void vCalcRotatedMoments()
	{
		double	dtheta;
		int		ii;
		int		ij;


		vCalcAreaAndMean();
		vCalcUnRotatedMoments();
		dtheta = dTheta();
	 	for (ii = 0; ii < 4; ++ii)
		{
			for (ij = 0; ij < 4; ++ij)
			{
				m_drotated[ii][ij] = 0.0;
			}
		}
		m_drotated[0][0] = 1.0;
		m_drotated[1][0] = 0.0;
		m_drotated[0][1] = 0.0;

		m_drotated[2][0] = dRotMom1( 2 , 0 , m_dunrotated , dtheta ) ;
		m_drotated[1][1] = dRotMom1( 1 , 1 , m_dunrotated , dtheta ) ;
		m_drotated[0][2] = dRotMom1( 0 , 2 , m_dunrotated , dtheta ) ;
		m_drotated[3][0] = dRotMom1( 3 , 0 , m_dunrotated , dtheta ) ;
		m_drotated[2][1] = dRotMom1( 2 , 1 , m_dunrotated , dtheta ) ;
		m_drotated[1][2] = dRotMom1( 1 , 2 , m_dunrotated , dtheta ) ;
		m_drotated[0][3] = dRotMom1( 0 , 3 , m_dunrotated , dtheta ) ;
	}

}
