package gov.fnal.eag.sim.pixel.wavelet;

import java.util.ArrayList;

/**
 * @author mcginnis
 *
 */
public class PixieCollection 
{
	int				inumPixies;
/* 0 = g, 1 = r, 2 = i  */
	Pixie[]		pixieArray = null;
	PixieCollection(int inumPixies)
	{
		this.inumPixies = inumPixies;
		pixieArray = new Pixie[this.inumPixies];
		for (int ii = 0; ii < this.inumPixies; ++ii) pixieArray[ii] = new Pixie();
	}
	PixieCollection(Pixie pseed, PixieImage pimage, 
			double dthreshold, double dmax_threshold, boolean contiguous)
	{
		if (contiguous)
		{
			PixieCollection pc = growPixieCollection( pseed,  pimage, dthreshold,  dmax_threshold);
			inumPixies = pc.inumPixies;
			pixieArray = pc.pixieArray;
		}
		else
		{
			double[][] dpix = pimage.getPix();
			inumPixies = 0;
			for (int ii = 0; ii < pimage.getRowCount(); ++ii)
			{
				for (int ij = 0; ij < pimage.getColCount(); ++ij)
				{
					if ((dthreshold <= dpix[ii][ij]) && (dpix[ii][ij] <= dmax_threshold))
					{
						inumPixies = inumPixies + 1;
					}
				}
			}
			pixieArray = new Pixie[inumPixies];
			inumPixies = 0;
			for (int ii = 0; ii < pimage.getRowCount(); ++ii)
			{
				for (int ij = 0; ij < pimage.getColCount(); ++ij)
				{
					if ((dthreshold <= dpix[ii][ij]) && (dpix[ii][ij] <= dmax_threshold))
					{
						pixieArray[inumPixies] = new Pixie(ii,ij,dpix[ii][ij]);
						inumPixies = inumPixies + 1;
					}
				}
			}
			
		}
	}
	PixieCollection(PixieCollection pc)
	{
		this.inumPixies = pc.inumPixies;
		pixieArray = new Pixie[this.inumPixies];
		for (int ii = 0; ii < this.inumPixies; ++ii) 
			pixieArray[ii] = new Pixie(pc.pixieArray[ii]);
	}
	PixieCollection(PixieCollection pc, PixieImage pi)
	{
		int irowMax = pi.getRowCount();
		int icolMax = pi.getColCount();
		double[][]	dpix = pi.getPix();
		inumPixies = 0;
		for (int ii = 0; ii < pc.inumPixies; ++ii)
		{
			if ((pc.pixieArray[ii].row < irowMax) && (pc.pixieArray[ii].col < icolMax))
				inumPixies = inumPixies + 1;
		}
		pixieArray = new Pixie[inumPixies];
		inumPixies = 0;
		for (int ii = 0; ii < pc.inumPixies; ++ii)
		{
			if ((pc.pixieArray[ii].row < irowMax) && (pc.pixieArray[ii].col < icolMax))
			{
				pixieArray[inumPixies] = new Pixie(
						pc.pixieArray[ii].row,
						pc.pixieArray[ii].col,
						dpix[pc.pixieArray[ii].row][pc.pixieArray[ii].col]);
				inumPixies = inumPixies + 1;
			}
		}
	}
	/**
	 * @param pmax
	 * @return
	 */
	public PixieImage makePixieImage(Pixie pmax)
	{
		PixieImage pi = makePixieImage(this, pmax);
		return pi;
	}
	/**
	 * @param pseed
	 * @param pimage
	 * @param dthreshold
	 * @param dmax_threshold
	 * @return
	 */
	public static PixieCollection growPixieCollection(Pixie pseed, PixieImage pimage, 
					double dthreshold, double dmax_threshold)
	{
		int		iradius;
		int		inum_pixels_added;
		int		inum_pixels_added_ring;
		Pixie	pi = new Pixie();
		Pixie	po = new Pixie();
		Pixie	pp = new Pixie();
		Pixie	pm = new Pixie();
		int		ii;
		int		ij;
		int		ik;
		int		il;
		
		Pixie pmax = pimage.getMaxPixie();
		int[][] iregion_array = new int[pmax.row + 1][pmax.col + 1];
		
		for (ii = 0; ii <= pmax.row; ++ii)
			for (ij =0; ij <= pmax.col; ++ij) iregion_array[ii][ij] = 0;
	
		iradius = 0;
		inum_pixels_added = 1;
		while (inum_pixels_added > 0)
		{
			iradius = iradius + 1;
			inum_pixels_added = 0;
	/* First check inside radius */
	/*		Check inside of 4 corners */
			for (ii = -1; ii <= 1; ii = ii + 2)
			{
				po.row = pseed.row + iradius * ii;
				pi.row = po.row - ii;
				for (ij = -1; ij <= 1; ij = ij + 2)
				{
					po.col = pseed.col + iradius * ij;
					pi.col = po.col  - ij;
					if (iCheckAdjPix(po, pi,pimage,
								iregion_array, dthreshold, dmax_threshold) )
					{
						iregion_array[po.row][po.col] = 1;
						inum_pixels_added = inum_pixels_added + 1;
					}
				}
			}
	/*		Check top and bottom */
			for (ii = -1; ii <= 1; ii = ii + 2)
			{
				po.row = pseed.row + iradius * ii;
				pi.row = po.row - ii;
				for (ij = -iradius + 1; ij <= iradius - 1; ij = ij + 1)
				{
					po.col = pseed.col + ij;
					for (ik = -1; ik <= 1; ik = ik + 1)
					{
						il = ij + ik;
						if (il < -iradius + 1) il = -iradius + 1;
						if (il >  iradius - 1) il =  iradius - 1;
						pi.col = pseed.col + il;
						if (iCheckAdjPix(po, pi,pimage,
									iregion_array,dthreshold, dmax_threshold) )
						{
							iregion_array[po.row][po.col] = 1;
							inum_pixels_added = inum_pixels_added + 1;
						}
					}
				}
			}
	/*		Check left and right */
			for (ii = -1; ii <= 1; ii = ii + 2)
			{
				po.col = pseed.col + iradius * ii;
				pi.col = po.col - ii;
				for (ij = -iradius + 1; ij <= iradius - 1; ij = ij + 1)
				{
					po.row = pseed.row + ij;
					for (ik = -1; ik <= 1; ik = ik + 1)
					{
						il = ij + ik;
						if (il < -iradius + 1) il = -iradius + 1;
						if (il >  iradius - 1) il =  iradius - 1;
						pi.row = pseed.row + il;
						if (iCheckAdjPix(po, pi,pimage,
									iregion_array, dthreshold, dmax_threshold) )
						{
							iregion_array[po.row][po.col] = 1;
							inum_pixels_added = inum_pixels_added + 1;
						}
					}
				}
			}
	/* Next check outside radius */
			inum_pixels_added_ring = 1;
			while (inum_pixels_added_ring > 0)
			{
				inum_pixels_added_ring = 0;
		/*		Check  4 corners */
				for (ii = -1; ii <= 1; ii = ii + 2)
				{
					po.row = pseed.row + iradius * ii;
					pp.row = po.row - ii;
					pm.row = po.row;
					for (ij = -1; ij <= 1; ij = ij + 2)
					{
						po.col = pseed.col + iradius * ij;
						pp.col = po.col;
						pm.col = po.col - ij;
						if (iCheckAdjPix(po, pp,pimage,
									iregion_array, dthreshold, dmax_threshold) )
						{
								iregion_array[po.row][po.col] = 1;
								inum_pixels_added = inum_pixels_added + 1;
								inum_pixels_added_ring = inum_pixels_added_ring + 1;
						}
						if (iCheckAdjPix(po, pm,pimage,
									iregion_array, dthreshold, dmax_threshold) )
						{
								iregion_array[po.row][po.col] = 1;
								inum_pixels_added = inum_pixels_added + 1;
								inum_pixels_added_ring = inum_pixels_added_ring + 1;
						}
					}
				}
	/*		Check top and bottom */
				for (ii = -1; ii <= 1; ii = ii + 2)
				{
					po.row = pseed.row + iradius * ii;
					pp.row = po.row;
					pm.row = po.row;
					for (ij = -iradius + 1; ij <= iradius - 1; ij = ij + 1)
					{
						po.col = pseed.col + ij;
						pp.col = po.col + 1;
						pm.col = po.col - 1;
						if (iCheckAdjPix(po, pp,pimage,
									iregion_array, dthreshold, dmax_threshold) )
						{
								iregion_array[po.row][po.col] = 1;
								inum_pixels_added = inum_pixels_added + 1;
								inum_pixels_added_ring = inum_pixels_added_ring + 1;
						}
						if (iCheckAdjPix(po, pm,pimage,
									iregion_array, dthreshold, dmax_threshold) )
						{
								iregion_array[po.row][po.col] = 1;
								inum_pixels_added = inum_pixels_added + 1;
								inum_pixels_added_ring = inum_pixels_added_ring + 1;
						}
					}
				}
	/*		Check left and right */
				for (ii = -1; ii <= 1; ii = ii + 2)
				{
					po.col = pseed.col + iradius * ii;
					pp.col = po.col;
					pm.col = po.col;
					for (ij = -iradius + 1; ij <= iradius - 1; ij = ij + 1)
					{
						po.row = pseed.row + ij;
						pp.row = po.row + 1;
						pm.row = po.row - 1;
						if (iCheckAdjPix(po, pp,pimage,
									iregion_array, dthreshold, dmax_threshold) )
						{
								iregion_array[po.row][po.col] = 1;
								inum_pixels_added = inum_pixels_added + 1;
								inum_pixels_added_ring = inum_pixels_added_ring + 1;
						}
						if (iCheckAdjPix(po, pm, pimage,
									iregion_array, dthreshold, dmax_threshold) )
						{
								iregion_array[po.row][po.col] = 1;
								inum_pixels_added = inum_pixels_added + 1;
								inum_pixels_added_ring = inum_pixels_added_ring + 1;
						}
					}
				}
			}
		}
		
		int numPixels = 0;

		for (ii = 0; ii <= pmax.row; ii++)
		{  
			for (ij = 0; ij <= pmax.col; ij++) 
			{
				if (iregion_array[ii][ij] > 0)
				{
					numPixels = numPixels + 1;
				}
			}
		}
		PixieCollection pixieCollection = new PixieCollection(numPixels);
		ik = 0;
		for (ii = 0; ii <= pmax.row; ii++)
		{  
			for (ij = 0; ij <= pmax.col; ij++) 
			{
				if (iregion_array[ii][ij] > 0)
				{
					pixieCollection.pixieArray[ik].row = ii;
					pixieCollection.pixieArray[ik].col = ij;
					pixieCollection.pixieArray[ik].val = pimage.getPix()[ii][ij];
					ik = ik + 1;
				}
			}
		}
		iregion_array = null;
		return pixieCollection;
	}
	static boolean iCheckAdjPix(Pixie px, Pixie pi, PixieImage pimage, 
			int[][] iregion_array, double dthreshold, double dmax_threshold)
	{
		boolean	iok;
	
		Pixie pmax = pimage.getMaxPixie();
		double[][] dpix1 = pimage.getPix();
		
		iok = false;
		if (	iValidCoord(pi , pmax) 
			&&	iValidCoord(px, pmax) )
		{
			if (	(iregion_array[pi.row ][pi.col ] == 1)
				&&	(iregion_array[px.row][px.col] == 0)
				&&	(dpix1[px.row][px.col] >= dthreshold) ) 
			{
				if (dmax_threshold < 0.0)
				{
					iok = true;
				}
				else
				{
					if (dpix1[px.row][px.col] <= dmax_threshold)
					{
						iok = true;
					}
				}
			}
		}
		return iok;
	}
	static boolean iValidCoord(Pixie pi, Pixie pmax)
	{
		boolean		iok;
	
		iok = false;
		if ( (0 <= pi.row) && ( pi.row <= pmax.row) )
		{
			if ( (0 <= pi.col) && ( pi.col <= pmax.col) )
			{
				iok = true;
			}
		}
		return iok;
	}
	/**
	 * @param pc
	 * @param pi
	 */
	public static void removePixieCollectionFromPixieImage(PixieCollection pc, PixieImage pi)
	{
		int irowMax = pi.getRowCount();
		int icolMax = pi.getColCount();
		double[][]	dpix = pi.getPix();
		for (int ii = 0; ii < pc.inumPixies; ++ii)
		{
			if ((pc.pixieArray[ii].row < irowMax) && (pc.pixieArray[ii].col < icolMax))
			{
				dpix[pc.pixieArray[ii].row][pc.pixieArray[ii].col] = 0.0;
			}
		}
	}
	/**
	 * @param pc1
	 * @param pc2
	 * @return
	 */
	public static PixieCollection addCollections(PixieCollection pc1, PixieCollection pc2)
	{
		int	ii;
		int inumPixiesSum = pc1.inumPixies + pc2.inumPixies;
		PixieCollection pcSum = new PixieCollection(inumPixiesSum);
		for (ii = 0; ii < pc1.inumPixies; ++ii)
		{
			pcSum.pixieArray[ii] = new Pixie(pc1.pixieArray[ii]);
		}
		for (ii = 0; ii < pc2.inumPixies; ++ii)
		{
			pcSum.pixieArray[ii + pc1.inumPixies] = new Pixie(pc2.pixieArray[ii]);
		}
		return pcSum;
	}
	/**
	 * @param pc1
	 * @param pc2
	 * @return
	 */
	public static PixieCollection subtractCollections(PixieCollection pc1, PixieCollection pc2)
	{
		Pixie	pmax = new Pixie(-1, -1, 0.0);
		PixieImage pi = makePixieImage(pc1, pmax);
		removePixieCollectionFromPixieImage(pc2, pi);
		Pixie pseed = new Pixie(-1, -1, 0.0);
		boolean contiguous = false;
		PixieCollection reducedPixieCollection 
			= new PixieCollection(pseed, pi, 0.0001, 1.0e-33, contiguous);
		return reducedPixieCollection;
	}
	/**
	 * @param pc
	 * @param pmax
	 * @return
	 */
	public static PixieImage makePixieImage(PixieCollection pc, Pixie pmax)
	{
		int	ii;
		
		if ((pmax.row < 0) || (pmax.col < 0))
		{
			for (ii = 0; ii < pc.inumPixies; ++ii)
			{
				if (pmax.row < pc.pixieArray[ii].row) pmax.row = pc.pixieArray[ii].row;
				if (pmax.col < pc.pixieArray[ii].col) pmax.col = pc.pixieArray[ii].col;
			}
		}
		PixieImage pi = new PixieImage(pmax);
		double[][] dpix = pi.getPix();
		for (ii = 0; ii < pc.inumPixies; ++ii)
		{
			if ((pc.pixieArray[ii].row <= pmax.row) && (pc.pixieArray[ii].col <= pmax.col))
				dpix[pc.pixieArray[ii].row][pc.pixieArray[ii].col] = pc.pixieArray[ii].val;
		}
		return pi;
	}
	/**
	 * @param pi
	 * @param dthreshold
	 * @return
	 */
	public static ArrayList<PixieCollection> findPixieCollections(PixieImage pi, double dthreshold)
	{
		PixieImage piCopy = new PixieImage(pi);
		double[][] dpix1 = piCopy.getPix();
		Pixie pmax = piCopy.getMaxPixie();
		ArrayList<PixieCollection> pcList = new ArrayList<PixieCollection>();
		int		pcListIndex = 0;
		PixieCollection pc = null;
		Pixie pseed = new Pixie();
		
		for (int ii = 0; ii <= pmax.row; ii++)
		{  
			for (int ij = 0; ij <= pmax.col; ij++) 
			{
				if (dpix1[ii][ij] >= dthreshold)
				{
					pseed.row = ii;
					pseed.col = ij;
					pc = growPixieCollection(pseed, piCopy, dthreshold, 1.0e+33);
					pcList.add(pcListIndex, (PixieCollection) pc);
					removePixieCollectionFromPixieImage(pc, piCopy);
					pcListIndex = pcListIndex + 1;
				}
			}
		}
		piCopy = null;
		return pcList;
	}
}
