package gov.fnal.eag.sim.random;

import gov.fnal.eag.sim.catalog.CoordinateGenerator;
import gov.fnal.eag.wcs.SphericalCoordinate;

import java.util.Iterator;

/**
 * @author kuropat, stoughto
 *
 */
public class ConstantCoordinateGenerator implements CoordinateGenerator {
	private SphericalCoordinate coord;
	
	/**
	 * @param coord the coordinate to return
	 */
	public ConstantCoordinateGenerator(SphericalCoordinate coord) {
		this.coord = coord;
	}
	/**
	 * @see gov.fnal.eag.sim.catalog.CoordinateGenerator#steradians()
	 */
	public double steradians() {
		return 0;
	}
	/**
	 * @see gov.fnal.eag.sim.catalog.CoordinateGenerator#copy()
	 */
	public CoordinateGenerator copy() {
		ConstantCoordinateGenerator retval = new ConstantCoordinateGenerator(coord.copy());
		return retval;
	}
	/**
	 * @see gov.fnal.eag.sim.catalog.CoordinateGenerator#getISeed()
	 */
	public int getISeed() {
		return 0;
	}
	/**
	 * @see java.lang.Iterable#iterator()
	 */
	public Iterator<SphericalCoordinate> iterator() {
		return new ConstantCoordinateIterator();
	}

	private class ConstantCoordinateIterator implements Iterator<SphericalCoordinate> {

		/**
		 * always returns true
		 * @see java.util.Iterator#hasNext()
		 */
		public boolean hasNext() {
			return true;
		}
		/**
		 * always returns the same SphericalCoordinate
		 * @see java.util.Iterator#next()
		 */
		public SphericalCoordinate next() {
			return coord;
		}
		/**
		 * @see java.util.Iterator#remove()
		 * @throws UnsupportedOperationException
		 */
		public void remove() {
			throw new UnsupportedOperationException();
		}
	}

}
