package gov.fnal.eag.sim.random;

import gov.fnal.eag.util.densityEstimate.Histogram;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.URL;

import org.jfree.chart.JFreeChart;
import org.jfree.data.xy.XYSeries;


/**
 * implements normal random number generation by reading in a table of pre-generated numbers
 */
public class SimRandomExponentialFast  extends SimRandom {
	private int n;
	private float[] x;
	private boolean doneReading = false;
	private float lifetime;
	private DataInputStream inputStream;
	/**
	 * initialize reading the list of numbers
	 * @param iSeedIndexIn offset into the list
	 * @param lifetime 
	 * @throws IOException 
	 */
	public SimRandomExponentialFast(int iSeedIndexIn, double lifetime) throws IOException {
		this.iSeedIndex = Math.abs(iSeedIndexIn);
		this.lifetime = (float) lifetime;
		String fileName = "data"+File.separator+"simRandomExponentialFast.dat";
		URL inputURL = this.getClass().getResource(fileName);
			InputStream tempInputStream = inputURL.openStream();
			inputStream = new DataInputStream(tempInputStream);
			n = inputStream.readInt();
			iSeedIndex %= n;
			x = new float[n];
			if (iSeedIndex > 0) {
				for (int i=0; i<iSeedIndex; i++) {
					x[i] = inputStream.readFloat()*this.lifetime;
				}
			}
		
	}
	
	/**
	 * create the file simRandomUniformFast.dat to be installed in the data directory of this package
	 * @param args makeNewData to create the data file; otherwise, print the first 10 values
	 * @throws IOException 
	 */
	public static void main(String[] args) throws IOException {
		if (args.length == 1 && args[0].equals("makeNewData")) {
			
			DataOutputStream dos = new DataOutputStream(new FileOutputStream("simRandomExponentialFast.dat"));
			int n = 514229; // a nice prime number
			dos.writeInt(n);
			double lambda = 1.0;
			SimRandomExponential sre = new SimRandomExponential(1, lambda, "", false);
			for (int i=0; i<n; i++) {				
				float x = (float) sre.next();
				if (i % 10000 == 0) {
					System.out.println("i="+i+"/"+n+" x="+x);
				}
				dos.writeFloat(x);				
			}
			dos.close();
		
		} else {
			SimRandomExponentialFast srnf = new SimRandomExponentialFast(1,1.0);
			for (int i=0; i<10; i++) {
				System.out.println("i="+i+" random="+srnf.next());
			}
		}
	}
	/**
	 * @see gov.fnal.eag.sim.random.SimRandom#getChart()
	 */
	@Override
	public JFreeChart getChart() throws Exception {
		throw new UnsupportedOperationException();
	}


	/**
	 * @see gov.fnal.eag.sim.random.SimRandom#next()
	 */
	@Override
	public double next() {
		if (iSeedIndex >= x.length) {
			iSeedIndex=0;
			doneReading = true;
			try {
				inputStream.close();
			} catch (IOException e) {
				throw new Error(e);
			}
		}
		if (!doneReading) {
			try {
				x[iSeedIndex] = inputStream.readFloat()*lifetime;
			} catch (IOException e) {
				throw new Error(e);
			}
		}
		double retval = x[iSeedIndex];
		iSeedIndex++;
		return retval;
	}

	/**
	 * @see gov.fnal.eag.sim.random.SimRandom#toPNG(java.lang.String, int, int)
	 */
	@Override
	public void toPNG(String fileName, int sizeX, int sizeY) throws Exception {
		throw new UnsupportedOperationException();
	}

	/**
	 * @return the number of pre-computed random numbers
	 */
	public int getN() {
		return n;		
	}

	@Override
	protected void makeHistogram() {
		if (histogram == null) {
			histogram = new Histogram("SimRandomNormal", "value", "dN/dValue", 100, 0.0, 1.0);
		}
	}
	
	/**
	 * @return the true points, with zero-sized error bars
	 */
	@Override
	public XYSeries getTrue() {
		throw new UnsupportedOperationException();
	}

	/**
	 * @see gov.fnal.eag.sim.random.SimRandom#setIseedIndex(int)
	 */
	@Override
	public void setIseedIndex(int seedIndex) {
		iSeedIndex = seedIndex;
	}



}
