package gov.fnal.eag.sim.random;

import gov.fnal.eag.sim.catalog.AstronomicalFilter;
import gov.fnal.eag.wcs.SphericalCoordinate;


/**
 * interface for luminosity functions
 * @author stoughto
 *
 */
public interface SimRandomLuminosity {

	/**
	 * set the filter name
	 * @param filterName
	 */
	public void setFilterName(String filterName);

	/**
	 * @return a filter name to be used to normalize the flux
	 */
	public String getFilterName();
	
	/**
	 * @return the next value chosen from the luminosity function
	 */
	public double next();

	/**
	 * @param returnValue if false, only increment the random number sequence, but don't calculate an actual value
	 * @return the next value, or Double.NaN if returnValue is false
	 */
	public double next(boolean returnValue);
	/**
	 * integrate over magnitude
	 * @return the space density of objects, in pc^-3;
	 */
	public double getSpaceDensity();

	/**
	 * @return the random number seed index used for this generator
	 */
	public int getISeed();

	/**
	 * @return a deep copy of this generator
	 */
	public SimRandomLuminosity copy();

	/**
	 * @return the filter associated with this luminosity generator
	 */
	public AstronomicalFilter getFilter();

	/**
	 * @return the surface density of objects for this generator in objects per degree squared
	 */
	public double getSurfaceDensity();

	/**
	 * @param sc
	 * @return the next coordinate
	 */
	public double next(SphericalCoordinate sc);

}