package gov.fnal.eag.sim.random;

import gov.fnal.eag.sim.catalog.AstronomicalFilter;
import gov.fnal.eag.wcs.SphericalCoordinate;

/**
 * returns the same luminosity each time
 * @author stoughto
 *
 */
public class SimRandomLuminosityConstant implements SimRandomLuminosity {

	double luminosity;
	String filterName;
	AstronomicalFilter filter = null;
	double surfaceDensity;
	/**
	 * @param luminosity the luminosity to return
	 * @param filterName 
	 */
	public SimRandomLuminosityConstant(double luminosity, String filterName) {
		this.luminosity = luminosity;
		this.filterName = filterName;
	}
	/**
	 * @param luminosity
	 * @param filter
	 * @param surfaceDensity objects/degree squared 
	 */
	public SimRandomLuminosityConstant(double luminosity, AstronomicalFilter filter, double surfaceDensity) {
		this.luminosity = luminosity;
		this.filter = filter;
		this.filterName = filter.getName();
		this.surfaceDensity = surfaceDensity;
	}
	/**
	 * @see gov.fnal.eag.sim.random.SimRandomLuminosity#copy()
	 */
	public SimRandomLuminosity copy() {
		return new SimRandomLuminosityConstant(luminosity, filterName);
	}

	/**
	 * @see gov.fnal.eag.sim.random.SimRandomLuminosity#getSpaceDensity()
	 */
	public double getSpaceDensity() {
		throw new UnsupportedOperationException();
	}
	
	/**
	 * @see gov.fnal.eag.sim.random.SimRandomLuminosity#getSurfaceDensity()
	 */
	public double getSurfaceDensity() {
		return surfaceDensity;
	}

	/**
	 * @see gov.fnal.eag.sim.random.SimRandomLuminosity#getFilterName()
	 */
	public String getFilterName() {
		return filterName;
	}

	/**
	 * @see gov.fnal.eag.sim.random.SimRandomLuminosity#getISeed()
	 */
	public int getISeed() {
		return 0;
	}

	/**
	 * @see gov.fnal.eag.sim.random.SimRandomLuminosity#next()
	 */
	public double next() {
		return luminosity;
	}

	/**
	 * @see gov.fnal.eag.sim.random.SimRandomLuminosity#next(boolean)
	 */
	public double next(boolean returnValue) {
		if (returnValue) {
			return luminosity;
		}
		return Double.NaN;
	}

	/**
	 * @see gov.fnal.eag.sim.random.SimRandomLuminosity#setFilterName(java.lang.String)
	 */
	public void setFilterName(String filterName) {
		this.filterName = filterName;
	}

	/**
	 * @param that
	 * @return true if the two are equal
	 */
	public boolean equals(SimRandomLuminosityConstant that) {
		if (luminosity != that.luminosity) return false;
		if (!filterName.equals(that.filterName)) return false;
		return true;
	}
	
	/**
	 * @see gov.fnal.eag.sim.random.SimRandomLuminosity#getFilter()
	 */
	public AstronomicalFilter getFilter() {
		return filter;
	}

	/**
	 * @see gov.fnal.eag.sim.random.SimRandomLuminosity#next(gov.fnal.eag.wcs.SphericalCoordinate)
	 */
	public double next(SphericalCoordinate sc) {
		return next(true);
	}
}
