
package gov.fnal.eag.sim.random;

import gov.fnal.eag.sim.catalog.CoordinateGenerator;
import gov.fnal.eag.util.astronomy.Cosmo;
import gov.fnal.eag.util.htm.HTMfunc;
import gov.fnal.eag.wcs.SphericalCoordinate;

/**
 * select redshifts
 * @author stoughto
 *
 */
public class SimRandomRedshift extends SimRandomDistribution {
	protected Cosmo cosmo;
	private int seed;
	private boolean definedAsDistancePc;
	/**
	 * default constructor
	 */
	public SimRandomRedshift() {
		
	}
	/**
	 * 
	 * @param cosmo 
	 * @param zMin 
	 * @param zMax 
	 * @param nBin 
	 * 
	 */
	public SimRandomRedshift(Cosmo cosmo, double zMin, double zMax, int nBin)  {
		this(cosmo, 678, zMin, zMax, nBin);
	}
	
	/**
	 * 
	 * @param cosmo
	 * @param seed
	 * @param zMin
	 * @param zMax
	 * @param nBin
	 */
	public SimRandomRedshift(Cosmo cosmo, int seed, double zMin, double zMax, int nBin) {
		if (zMin > zMax) {
			throw new IllegalArgumentException("zMin="+zMin+" zMax="+zMax+" does not make sense");
		} else if (nBin <=0) {
			throw new IllegalArgumentException("nBin="+nBin+" does not make sense");
		}
		this.cosmo = cosmo;
	    //distribution = cosmo.getdVdzFunc();
		this.minValue = zMin;
		this.maxValue = zMax;
		this.deltaValue = (zMax-zMin)/nBin;
		this.nBin = nBin;
		this.seed = seed;
	    setDistribution(cosmo.getdVdzFunc());
		simRandomUniform = new SimRandomUniform();
		setIseedIndex(seed);
	}

	/**
	 * calculate the volume in the shell between the minimum and maximum redshifts
	 * @return the volume in units of pc^3
	 */
	public double volume() {
		double volume = cosmo.volume(minValue, maxValue)*Math.pow(cosmo.getCH0(),3);		
		return volume;
	}

	/**
	 * @return a deep copy
	 */
	public SimRandomRedshift copy() {
		SimRandomRedshift retval = new SimRandomRedshift(cosmo.copy(), simRandomUniform.iSeedIndex, minValue, maxValue, nBin);
		return retval;
	}
	
	/**
	 * 
	 * @param sc
	 * @return the random redshift based on this coordinate and the seed value
	 */
	public double next(SphericalCoordinate sc) {
		long thisHtm = HTMfunc.lookupId(sc.getLongitude(), sc.getLatitude(), CoordinateGenerator.depth);
		int iSeedIndexTemp = HTMfunc.indexFromHtm(thisHtm+seed);
		setIseedIndex(iSeedIndexTemp);
		return next();
	}
	
	/**
	 * @param definedAsDistancePc
	 */
	public void setDefinedAsDistancePc(boolean definedAsDistancePc) {
		this.definedAsDistancePc = definedAsDistancePc;
	}
	/**
	 * @return true if these aren't really redshifts, but rather, distance in pc 
	 */
	public boolean getDefinedAsDistancePc() {
		return definedAsDistancePc;
	}

}	
