package gov.fnal.eag.sim.repository.image;

/**
 * describe popular formats 
 * @author stoughto
 *
 */
public enum FitsDataFormat {
	/**
	 * double precision (64 bits)
	 */
	DOUBLE (-64, 0, 1, -Double.MAX_VALUE, Double.MAX_VALUE),
	/**
	 * floating point (32 bits)
	 */
	FLOAT (-32, 0, 1, -Float.MAX_VALUE, Float.MAX_VALUE),
	/**
	 * unsigned short (16 bits)
	 */
	USHORT (16, Math.pow(2, 15), 1, 0, Math.pow(2, 16)-1);
	
	private final int bitpix;
	private final double bzero;
	private final double bscale; 
	private final double min;
	private final double max;
	
	FitsDataFormat(int bitpix, double bzero, double bscale, double min, double max) {
		this.bitpix = bitpix;
		this.bzero = bzero;
		this.bscale = bscale;
		this.min = min;
		this.max = max;
	}
	/**
	 * @return bitpix for the header
	 */
	public int getBitpix() {return bitpix;}
	/**
	 * @return bzero for the header
	 */
	public double getBzero() {return bzero;}
	/**
	 * @return bscale for the header
	 */
	public double getBscale() {return bscale;}
	
	/**
	 * clip the value to the allowed min and  max values
	 * @param value input value
	 * @return the clipped value
	 */
	public double clip(double value) {
		if (value < min) return min;
		if (value > max) return max;
		return value;
	}
	
	/**
	 * @return the minimum legal value for this format
	 */
	public double getMin() {
		return min;
	}
	/**
	 * @return the maximum legal value for this format
	 */
	public double getMax() {
		return max;
	}
}


