package gov.fnal.eag.sim.repository.image;

import gov.fnal.eag.sim.repository.image.ImageRepository.LosslessCompressionType;

import java.io.IOException;
import java.io.InputStream;
import java.io.StreamCorruptedException;
import java.util.ArrayList;

import org.freehep.util.io.BitInputStream;

/**
 * simple implementation of Huffman encoding
 * @author stoughto
 *
 */
public class HuffmanInputStream extends InputStream {
	BitInputStream bitInputStream;
	private HuffmanTree huffmanTree;
	/**
	 * @param inputStream where to write the bits
	 * @throws IOException
	 */
	public HuffmanInputStream(InputStream inputStream)  throws IOException {
		bitInputStream = new BitInputStream(inputStream);
		int iMagic = readInteger();
		int huffmanMagic = LosslessCompressionType.getHuffmanMagicNumber();
		if (iMagic != huffmanMagic) {
			throw new StreamCorruptedException("expected Huffman magic number="+huffmanMagic+" but read "+iMagic);
		}
		readEncoding();
	}
	private void readEncoding() throws IOException {
		int size = readInteger();
		ArrayList<Boolean> encoding = new ArrayList<Boolean>();
		for (int i=0; i<size; i++) {
			boolean bit = bitInputStream.readBitFlag();
			encoding.add(bit);
		}
		huffmanTree = new HuffmanTree(encoding);
	}
	/**
	 * reads 32 bits from the bitInputStream, from MSB to LSB
	 * @return the resulting value
	 * @throws IOException
	 */
	public int readInteger() throws IOException {
		int retval = 0;
		for (int i=0; i<32; i++) {
			retval <<= 1;
			boolean bit = bitInputStream.readBitFlag();
			if (bit) {
				retval += 1;
			}
		}
		return retval;
	}
	/**
	 * reads the next bit pattern from the bitInputStream and gets the value of the pattern
	 * @see java.io.InputStream#read()
	 */
	@Override
	public int read() throws IOException {
		int retval = huffmanTree.getValue(bitInputStream);
		return retval;
	}
}
