package gov.fnal.eag.sim.repository.image;

/**
 * one node in a HuffmanTree
 * @author stoughto
 *
 */
public  class HuffmanNode implements Comparable<HuffmanNode> {
	int weight;
	int value;
	HuffmanNode left;
	HuffmanNode right;
	
	/**
	 * construct one leaf node, with the weight (frequency) and the original and decoded value
	 * @param weight
	 * @param value
	 */
	public HuffmanNode(int weight, int value) {
		this.weight = weight;
		this.value = value;
		left = null;
		right = null;
	}
	
	/**
	 * construct a node that points down to two other nodes
	 * @param left
	 * @param right
	 */
	public HuffmanNode(HuffmanNode left, HuffmanNode right) {
		weight = left.weight + right.weight;
		value = 0;
		this.left = left;
		this.right = right;
	}
	/**
	 * @param that the node to compare
	 * @return -1, 0, or 1 
	 * @see java.lang.Comparable#compareTo(java.lang.Object)
	 */
	public int compareTo(HuffmanNode that) {
		
		int thatWeight = that.weight;
		HuffmanNode thatLeft = that.left;
		if (weight == thatWeight) {
			if (value == 0 && right == null) {
				return 1;
			}
			if (value == 0 && thatLeft == null) {
				return -1;
			}
		}
		return (weight > thatWeight) ? 1 : -1;
	}
	/**
	 * report the weight and value of this node
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		String retval = String.format("weight="+weight+" value="+value);
		return retval;
	}
	
}
