package gov.fnal.eag.sim.repository.image;

import gov.fnal.eag.sim.repository.image.ImageRepository.LosslessCompressionType;

import java.io.IOException;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.HashMap;

import org.freehep.util.io.BitOutputStream;

/**
 * writes data using Huffman encoding
 * @author stoughto
 *
 */
public class HuffmanOutputStream extends OutputStream {
	BitOutputStream bitOutputStream;
	HuffmanTree huffmanTree;
	HashMap<Integer, String> huffmanMap;
	ArrayList<Integer> integers;
	/**
	 * @param outputStream where to write the bits
	 * @param losslessCompressionType
	 * @throws IOException
	 */
	public HuffmanOutputStream(OutputStream outputStream, LosslessCompressionType losslessCompressionType) throws IOException {
		bitOutputStream = new BitOutputStream(outputStream);
		huffmanTree = losslessCompressionType.getHuffmanTree();
		huffmanMap = huffmanTree.getMap();
		int iMagic = LosslessCompressionType.getHuffmanMagicNumber();
		writeInteger(iMagic);
		writeEncoding();
	}
	private void writeEncoding() throws IOException {
		ArrayList<Boolean> encoding = huffmanTree.getEncoding();
		writeInteger(encoding.size());
		for (int i=0; i<encoding.size(); i++) {
			bitOutputStream.writeBitFlag(encoding.get(i));
		}
	}
	/**
	 * writes 32 bits to the bitOutputStream, from MSB to LSB
	 * @param value the value to write
	 * @throws IOException
	 */
	public void writeInteger(int value) throws IOException {
		for (int iBit=32; iBit>0; iBit--) {
			boolean bit = (value & (1<<(iBit-1))) != 0;
			bitOutputStream.writeBitFlag(bit);
		}
		//bitOutputStream.writeUBits(value, 16);
	}
	/**
	 * writes the encoded bit pattern for the value to bitOutputStream
	 * @param value the value to encode
	 * @throws IOException
	 */
	public void encode(long value) throws IOException {
		String bits = huffmanMap.get((int)value);
		for (int iBit=0; iBit<bits.length(); iBit++) {
			char c = bits.charAt(iBit);
			boolean bit = (c == '1');
			bitOutputStream.writeBitFlag(bit);
		}
	}
	/**
	 * this not supported
	 * @see java.io.OutputStream#write(int)
	 */
	@Override
	public void write(int b) throws IOException {
		throw new UnsupportedOperationException();
	}
	
	/**
	 * closes the bitOutputStream
	 * @see java.io.OutputStream#close()
	 */
	@Override
	public void close() throws IOException {
			bitOutputStream.close();
	}

}
