package gov.fnal.eag.sim.run;

import gov.fnal.eag.sim.pixel.BackgroundLight;
import gov.fnal.eag.sim.pixel.BackgroundLightConstant;
import gov.fnal.eag.sim.pixel.DetectorObject;
import gov.fnal.eag.sim.pixel.DetectorObjectSurfaceBrightness;
import gov.fnal.eag.sim.pixel.PSFFactory;
import gov.fnal.eag.sim.pixel.PSFFactoryNull;
import gov.fnal.eag.sim.pixel.PixelDetector;
import gov.fnal.eag.sim.pixel.PixeledImage;
import gov.fnal.eag.sim.pixel.PixeledImageFactory;
import gov.fnal.eag.sim.random.SimRandomNormal;
import gov.fnal.eag.sim.random.SimRandomPoisson;
import gov.fnal.eag.sim.random.SimRandomUniform;
import gov.fnal.eag.sim.repository.image.ImageRepository;

import java.io.FileInputStream;
import java.util.LinkedList;
import java.util.Properties;

/**
 * read in a parameter file and create a FITS image
 * @author stoughto
 *
 */
public class OneObject {

	/**
	 * @param baseFileName the base name of the .par file that will be used to generate a .fit file
	 * @throws Exception 
	 */
	public OneObject(String baseFileName) throws Exception {
		Properties properties = new Properties();
        FileInputStream in;
        in = new FileInputStream(baseFileName+".par");
        properties.load(in);
        in.close();
        Properties up = new Properties(properties);
       
        int nrow = Integer.parseInt(up.getProperty("nrow", "200"));
        int ncol = Integer.parseInt(up.getProperty("ncol", "200"));
        String object = up.getProperty("object","100 100 1000 5 1.0 0.3 0.2");
        String[] split = object.split(" ");
        double row = Double.parseDouble(split[0]);
        double col = Double.parseDouble(split[1]);
        double nPhoton = Double.parseDouble(split[2]);
        double scale = Double.parseDouble(split[3]);
        int nev = split.length-4;
        double ev[] = new double[nev];
        for (int i=0; i<nev; i++) {
        	ev[i] = Double.parseDouble(split[4+i]);
        }
        double backgroundPhotonsPerPixel = Double.parseDouble(up.getProperty("backgroundPhotonsPerPixel", "0.0"));
        double cosmicRayRate = Double.parseDouble(up.getProperty("cosmicRayRate", "0.0"));
        int poissonSeedIndex = Integer.parseInt(up.getProperty("poissonSeedIndex", "1"));
        int uniformSeedIndex = Integer.parseInt(up.getProperty("poissonSeedIndex", "2"));
        
        DetectorObjectSurfaceBrightness dosb  = new DetectorObjectSurfaceBrightness(row, col, nPhoton, ev, scale);
        PixelDetector pixelDetector = new PixelDetector(nrow, ncol);
        BackgroundLight backgroundLight = new BackgroundLightConstant(backgroundPhotonsPerPixel);
        PSFFactory psfFactory = new PSFFactoryNull();
        LinkedList<DetectorObject> detectorObjects = new LinkedList<DetectorObject>();
        detectorObjects.add(dosb);
        SimRandomPoisson simRandomPoisson = new SimRandomPoisson(poissonSeedIndex);
        SimRandomUniform simRandomUniform = new SimRandomUniform(uniformSeedIndex);
        SimRandomNormal simRandomNormal = new SimRandomNormal(uniformSeedIndex+2);
        PixeledImageFactory pif = new PixeledImageFactory(
        		pixelDetector,
        		backgroundLight,
        		psfFactory,
        		detectorObjects,
        		cosmicRayRate,
        		simRandomPoisson,
        		simRandomUniform,
        		simRandomNormal
        		);
        pif.setCountsPerPixelThreshold(1e-6);
        PixeledImage pi = pif.produce();
        ImageRepository ir = new ImageRepository();
        ir.put(baseFileName+".fit", pi, true);
	}
	/**
	 * @param args
	 * @throws Exception 
	 */
	public static void main(String[] args) throws Exception {
		new OneObject(args[0]);
	}

}
