package gov.fnal.eag.sim.wavelet;

import gov.fnal.eag.sim.pixie.PixieImage;
import gov.fnal.eag.sim.pixie.PixieImageException;
import gov.fnal.eag.sim.pixie.PixieImageRGBPlotter;

import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.LineNumberReader;
import java.util.Date;
import java.util.Scanner;

import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JOptionPane;

/**
 * @author mcginnis
 *
 */
public class WaveletBatchThread  implements Runnable
{
	double	centerRa;
	double	centerDec;
	int		cropImageSize;
	File[]	inputImageFitsFile = new File[4];
	File[]	outputImageFitsFile = new File[4];
	int		batchObjCnt = 0;
	PixieImage[] coloredImage = new PixieImage[3];
	private PixieImageRGBPlotter bwRGBplotter = null;
	private PixieImageRGBPlotter clrRGBplotter = null;
	private WaveletGui arcSearchGui = null;
	
	private File	inputBatchFile;
	private File	singleInputFile;
	private StatusBar statusBar = null;
	private Wavelet	waveletSettings = null;
	private boolean batchNormalize = true;
	private boolean batchFilter = true;
	private boolean batchWritePngImages = true;
	private int		batchPngImageSize = 300;
	private boolean	batchWriteFitsFiles = false;
	private boolean colorBatchFile = true;
	private boolean runProcessBatchImages = true;
	private boolean shortFitsFormat = true;
	private boolean runSingleFileWaveletFilter = false;
	private JLabel[] imageIconLabel = null;
	String delim = System.getProperty("file.separator");
	
	WaveletBatchThread()
	{
	}
	/** (non-Javadoc)
	 * @see java.lang.Runnable#run()
	 */
	public void run() 
	{
     	try 
     	{
     		if (runProcessBatchImages)
     		{
     			processBatchImages();
     			statusBar.setMessage("Ready");
     		}
     		if (runSingleFileWaveletFilter)
     		{
    			singleFileWaveletFilter();
    			statusBar.setMessage("Ready");
    		}
		} 
     	catch (PixieImageException e) 
     	{
    		statusBar.setMessage("Problem Processing Batch Images");
        	JOptionPane.showMessageDialog(arcSearchGui, "Input Output Error");
//			e.printStackTrace();
		}
     	catch (IOException e) 
     	{
    		statusBar.setMessage("Problem Processing Batch Images");
        	JOptionPane.showMessageDialog(arcSearchGui, "Input Output Error");
//			e.printStackTrace();
		}
        catch (OutOfMemoryError e) 
        {
    		statusBar.setMessage("Not Enough Memory");
        	JOptionPane.showMessageDialog(arcSearchGui, "Not Enough Memory\nTry Increasing the Memory of the Java VM");
        }
	}
	/**
	 * @throws IOException
	 */
	public void processBatchImages()  throws IOException
	{
		int				colorCharIndex;
		FileWriter		outBatchHtmlWriter = null;
		String			outBatchHtmlFileName = null;
		File			outBatchHtmlFile = null;
		FileReader		inBatchReader = null;
		LineNumberReader		inBatchLineReader = null;
		String			inputFileName = null;	
		String			inputLine = null;	
		String			addOnText = null;	
		boolean			goodImage = true;
		
		outBatchHtmlFileName = inputBatchFile.getPath().substring(0, inputBatchFile.getPath().length() - 4)
			+ "_out.htm";
		outBatchHtmlFile = new File(outBatchHtmlFileName);
		outBatchHtmlWriter = new FileWriter(outBatchHtmlFile);
				
		outBatchHtmlWriter.write("<html>\n<head>\n");
		outBatchHtmlWriter.write("<title>" + outBatchHtmlFile.getName() + "</title>\n");
		outBatchHtmlWriter.write("</head>\n<body>\n");

		inBatchReader = new FileReader(inputBatchFile);
		inBatchLineReader = new LineNumberReader(inBatchReader);
		batchObjCnt = 0;
		inputLine = inBatchLineReader.readLine();
		Scanner sc = new Scanner(inputLine).useDelimiter("\\s*;\\s*");
		while (sc.hasNext())
		{
			inputFileName = sc.next();
			centerRa = sc.nextDouble();
			centerDec = sc.nextDouble();
			cropImageSize = sc.nextInt();
			addOnText = sc.next();
		    sc.close(); 			
			batchObjCnt = batchObjCnt + 1;
			inputImageFitsFile[0] = new File(inputFileName);
			inputImageFitsFile[3] = new File(inputFileName);
			int ifile = 0;
			outputImageFitsFile[ifile] = new File(outBatchHtmlFile.getParent()+ delim + inputImageFitsFile[ifile].getName().substring(0, inputImageFitsFile[ifile].getName().length() - 4) + addOnText + ".fit");
			ifile = 3;
			outputImageFitsFile[ifile] = new File(outBatchHtmlFile.getParent()+ delim + inputImageFitsFile[ifile].getName().substring(0, inputImageFitsFile[ifile].getName().length() - 4) + addOnText + ".fit");
			if (colorBatchFile)
			{
    			colorCharIndex = PixieImage.findColorCharIndexInFileName(inputFileName,'$');
				if (colorCharIndex == 0) throw new IOException();
				inputImageFitsFile[0] = new File(PixieImage.writeColorCharIntoFileName(inputFileName, "g", colorCharIndex));
				inputImageFitsFile[1] = new File(PixieImage.writeColorCharIntoFileName(inputFileName, "r", colorCharIndex));
				inputImageFitsFile[2] = new File(PixieImage.writeColorCharIntoFileName(inputFileName, "i", colorCharIndex));
				inputImageFitsFile[3] = new File(inputFileName);
				for (ifile = 0; ifile < 4; ++ ifile)
				{
	    			outputImageFitsFile[ifile] = new File(outBatchHtmlFile.getParent()+ delim + inputImageFitsFile[ifile].getName().substring(0, inputImageFitsFile[ifile].getName().length() - 4) + addOnText + ".fit");
				}
			}
			goodImage = true;
			try
			{
				cropNormalizeFilterImages();
			}
			catch (PixieImageException e)
			{
	    		statusBar.setMessage("Problem Processing Batch Image " + Integer.toString(batchObjCnt));
	    		System.out.println("Problem Processing Batch Image " + Integer.toString(batchObjCnt));
				goodImage = false;
			}
			if (goodImage)
			{
				try
				{
					if (batchWritePngImages) makePngImages(outBatchHtmlFile, outBatchHtmlWriter);
					if (batchWriteFitsFiles) writeFilteredFitsFiles(outBatchHtmlFile);
				}
				catch (PixieImageException e)
				{
		    		statusBar.setMessage("Problem Processing Batch Image " + Integer.toString(batchObjCnt));
		    		System.out.println("Problem Processing Batch Image " + Integer.toString(batchObjCnt));
					goodImage = false;
				}
			}
			inputLine = inBatchLineReader.readLine();
			sc = new Scanner(inputLine).useDelimiter("\\s*;\\s*");
 		}
		sc.close();
		
		outBatchHtmlWriter.write("</body>\n</html>\n");
		
		outBatchHtmlWriter.close();
		inBatchReader.close();	
		statusBar.setMessage("Finished Batch running");
			
	}
	private	void cropNormalizeFilterImages() throws PixieImageException
	{
		int		ifile;
		int		maxFile = 1;
		Wavelet wavelet = new Wavelet(waveletSettings);
		if (colorBatchFile) maxFile = 3;
		PixieImage[] croppedImage = new PixieImage[maxFile];
		Date date = new Date();

		double[] noiseMeanSigma = null;
		double[] imageNoiseMean = new double[maxFile];
		double[] imageNoiseSigma = new double[maxFile];
		
		for (ifile = 0; ifile < maxFile; ++ifile)
		{
			statusBar.setMessage("Batch Cnt. " + Integer.toString(batchObjCnt) + " Reading File " + Integer.toString(ifile + 1));
	    	if (cropImageSize > 0)
	    	{
	    		int[] rowCol = PixieImage.rADecToRowCol(inputImageFitsFile[ifile].getPath(), centerRa,  centerDec);
		    	croppedImage[ifile] = new PixieImage(inputImageFitsFile[ifile],
		    			rowCol[0], rowCol[1],cropImageSize,cropImageSize);
	    	}
	    	else
	    	{
	    		croppedImage[ifile] = new PixieImage(inputImageFitsFile[ifile]);
	    	}
			croppedImage[ifile].addCommentToHeaderEnd("Wavelet filtered " + date.toString());
    		noiseMeanSigma = croppedImage[ifile].findNoiseMeanSigma();
    		imageNoiseMean[ifile] = noiseMeanSigma[0];
    		imageNoiseSigma[ifile] = noiseMeanSigma[1];
			croppedImage[ifile].addHeadercard("SKYAVG",imageNoiseMean[ifile],"Pixel noise average before filtering");
			croppedImage[ifile].addHeadercard("SKYSIG",imageNoiseSigma[ifile],"Pixel noise sigma before filtering");
		}
	    if (batchNormalize) 
    	{
			for (ifile = 0; ifile < maxFile; ++ifile)
			{
	    		statusBar.setMessage("Batch Cnt. " + Integer.toString(batchObjCnt) + " Normalizing File " + Integer.toString(ifile + 1));
				croppedImage[ifile].addHeadercard("DOFFS",-imageNoiseMean[ifile],"Pixel value Offset after normalizing");
				croppedImage[ifile].addHeadercard("DSCAL",1.0 / imageNoiseSigma[ifile],"Pixel value Scale Change after normalizing");
				croppedImage[ifile].rescalePixels(1.0, -imageNoiseMean[ifile]);
				croppedImage[ifile].rescalePixels(1.0 / imageNoiseSigma[ifile], 0.0);
			}
    	}
		for (ifile = 0; ifile < maxFile; ++ifile)
		{
	    	if (batchFilter)
	    	{
	    		statusBar.setMessage("Batch Cnt. " + Integer.toString(batchObjCnt) + " Filtering File " + Integer.toString(ifile + 1));
	    		coloredImage[ifile] = wavelet.waveFilter(croppedImage[ifile]);
	    	}
	    	else
	    	{
	    		coloredImage[ifile] = new PixieImage(croppedImage[ifile]);
	    	}
		}
//	Will do auto scaling if autoscale is set true otherwise it does nothing.
		bwRGBplotter.setPixelValueLimits(coloredImage[0]);
		clrRGBplotter.setPixelValueLimits(coloredImage[0]);
//	Will do histogram scaling if the scaletype is set to histogram otherwise it does nothing.
		bwRGBplotter.setScaleTable(croppedImage[0]);
		clrRGBplotter.setScaleTable(croppedImage[0]);
		wavelet = null;
		croppedImage = null;
		System.gc();
	}
	private	void makePngImages(File outBatchHtmlFile, FileWriter outBatchHtmlWriter) throws IOException
	{
		int		ifile;
		int		maxFile = 0;
		String	plotFileName;
		String	shortPlotFileName;
		String	htmlStuff;
		String	q = "\"";
		PixieImageRGBPlotter plotter = null;
	 	ImageIcon imageIcon = null;
		
		htmlStuff 
			= "<p>File: "
			+ inputImageFitsFile[3].getName()
			+ "</p>\n";
		outBatchHtmlWriter.write(htmlStuff);
		if (colorBatchFile) maxFile = 3;

		for (ifile = 0; ifile <= maxFile; ++ifile)
		{
			statusBar.setMessage("Batch Cnt. " + Integer.toString(batchObjCnt) + " Writing PNG File " + Integer.toString(ifile + 1));
			plotFileName = outputImageFitsFile[ifile].getPath().substring(0, outputImageFitsFile[ifile].getPath().length() - 4) + ".png";
			shortPlotFileName = outputImageFitsFile[ifile].getName().substring(0, outputImageFitsFile[ifile].getName().length() - 4) + ".png";
			if (ifile < 3)
			{
				plotter = new PixieImageRGBPlotter();
				plotter.setImages(coloredImage[ifile], coloredImage[ifile], coloredImage[ifile]);
				plotter.copyScaling(bwRGBplotter);
			}
			if (ifile == 3)
			{
				plotter = new PixieImageRGBPlotter();
				plotter.setImages(coloredImage[2], coloredImage[1], coloredImage[0]);
				plotter.copyScaling(clrRGBplotter);
			}
			plotter.setScaleTable(null);
			plotter.toPNGFile(plotFileName);
		 	imageIcon = new ImageIcon(plotter.makeBufferedImage());
		 	imageIconLabel[ifile].setIcon(imageIcon);
			
			htmlStuff
				= "<a href=" 
				+ q + shortPlotFileName + q
				+ "><img border=" + q + "1" + q + " src="
				+ q + shortPlotFileName + q
				+ " width=" + q + Integer.toString(batchPngImageSize) + q + " height=" + q + Integer.toString(batchPngImageSize) + q + "></a>\n";
			outBatchHtmlWriter.write(htmlStuff);
		}
		htmlStuff = "</p>\n<hr>\n";
		outBatchHtmlWriter.write(htmlStuff);
		plotter = null;
		imageIcon = null;
	}
	private void writeFilteredFitsFiles(File outBatchHtmlFile)  throws PixieImageException
	{
		int		ifile;
		int		maxFile = 1;
		if (colorBatchFile) maxFile = 3;

		for (ifile = 0; ifile < maxFile; ++ifile)
		{
			statusBar.setMessage("Batch Cnt. " + Integer.toString(batchObjCnt) + " Writing Fits File " + Integer.toString(ifile + 1));
			coloredImage[ifile].writeToFitsFile(outputImageFitsFile[ifile].getPath(), shortFitsFormat);
		}
		
	}
	private void singleFileWaveletFilter()  throws PixieImageException
	{
		statusBar.setMessage("Wavelet Filtering");
		String plotFileName;
		Wavelet wavelet = new Wavelet(waveletSettings);
		PixieImage inputImage = new PixieImage(singleInputFile);
		PixieImage outputImage = wavelet.waveFilter(inputImage);
		plotFileName  = singleInputFile.getPath().substring(0, singleInputFile.getPath().length() - 4)
			+ "-wv.fit";
		outputImage.writeToFitsFile(plotFileName, shortFitsFormat);

//		Will do auto scaling if autoscale is set true otherwise it does nothing.
		bwRGBplotter.setPixelValueLimits(inputImage);
//	Will do histogram scaling if the scaletype is set to histogram otherwise it does nothing.
		bwRGBplotter.setScaleTable(inputImage);

		PixieImageRGBPlotter plotter = new PixieImageRGBPlotter();
		plotter.setImages(inputImage, inputImage, inputImage);
		plotter.copyScaling(bwRGBplotter);
		plotter.setScaleTable(null);
	 	ImageIcon imageIcon = new ImageIcon(plotter.makeBufferedImage());
	 	imageIconLabel[0].setIcon(imageIcon);

		plotter.setImages(outputImage, outputImage, outputImage);
		plotter.copyScaling(bwRGBplotter);
		plotter.setScaleTable(null);
	 	imageIcon = new ImageIcon(plotter.makeBufferedImage());
	 	imageIconLabel[1].setIcon(imageIcon);

		plotter = null;
		imageIcon = null;
		wavelet = null;
		outputImage = null;
		inputImage = null;
		statusBar.setMessage("Wavelet Filtering Finished");
	}
	void setWaveletSettings(Wavelet waveletSettings)
	{
		this.waveletSettings = waveletSettings;
	}
	void setStatusBar(StatusBar statusBar)
	{
		this.statusBar = statusBar;
	}
	void setInputBatchFile(File inputBatchFile)
	{
		this.inputBatchFile = inputBatchFile;
	}
	void setBatchNormalize(boolean batchNormalize)
	{
		this.batchNormalize = batchNormalize;
	}
	void setBatchFilter(boolean batchFilter)
	{
		this.batchFilter = batchFilter;
	}
	void setBatchWritePngImages(boolean batchWritePngImages)
	{
		this.batchWritePngImages = batchWritePngImages;
	}
	void setBatchPngImageSize(int batchPngImageSize)
	{
		this.batchPngImageSize = batchPngImageSize;
	}
	void setBatchWriteFitsFiles(boolean batchWriteFitsFiles)
	{
		this.batchWriteFitsFiles = batchWriteFitsFiles;
	}
	void setBwRGBplotter(PixieImageRGBPlotter bwRGBplotter)
	{
		this.bwRGBplotter = bwRGBplotter;
	}
	void setClrRGBplotter(PixieImageRGBPlotter clrRGBplotter)
	{
		this.clrRGBplotter = clrRGBplotter;
	}
	void setImageIconLabel(JLabel[] imageIconLabel)
	{
		this.imageIconLabel = imageIconLabel;
	}
	void setColorBatchFile(boolean colorBatchFile)
	{
		this.colorBatchFile = colorBatchFile;
	}
	void setRunProcessBatchImages(boolean runProcessBatchImages)
	{
		this.runProcessBatchImages = runProcessBatchImages;
	}
	void setRunSingleFileWaveletFilter(boolean runSingleFileWaveletFilter)
	{
		this.runSingleFileWaveletFilter = runSingleFileWaveletFilter;
	}
	void setSingleInputFile(File singleInputFile)
	{
		this.singleInputFile = singleInputFile;
	}
	void setArcSearchGui(WaveletGui arcSearchGui)
	{
		this.arcSearchGui = arcSearchGui;
	}
	void setShortFitsFormat(boolean shortFitsFormat)
	{
		this.shortFitsFormat = shortFitsFormat;
	}
}
