package gov.fnal.eag.util.astronomy;

/**
 * @author dcharms
 *
 */
public class ReverseFinder {
	
	private SimpleFunction function;
	
	private double tolerance;
	
	private double min = 0;
	
	private double max = 100;
	
	/**
	 * Create new finder with a function and default tolerance of .001
	 * @param function the function that will be used to find a value
	 */
	public ReverseFinder(SimpleFunction function) {
		this(function, .001);
	}
	
	/**
	 * Create new finder with a function and a tolerance to which it will find x value
	 * @param function
	 * @param tolerance
	 */
	public ReverseFinder(SimpleFunction function, double tolerance) {
		this.function = function;
		this.tolerance = tolerance;
	}
	
	/**
	 * Set the starting minimum x-value
	 * @param min the minimum x-value
	 */
	public void setMin(double min) {
		this.min = min;
	}
	
	/**
	 * Set the starting maximum x-value
	 * @param max the maximum x-value
	 */
	public void setMax(double max) {
		this.max = max;
	}
	
	/**
	 * Get the x value for the corresponding y value
	 * @param y the y value to find the x value for
	 * @return the corresponding x value
	 */
	public double getX(double y) {
		double x1 = min;
		double x2 = max;
		double difference = 100*tolerance;
		double finalX;
		do {
			double lowerEstimate = function.f(x1);
			double upperEstimate = function.f(x2);
			if (y>upperEstimate || y< lowerEstimate)
				return -1;
			double middleX = (x2-x1)/2+x1;
			double middleY = function.f(middleX);
			if (y > middleY) {
				x1 = middleX;
				if (y-middleY > upperEstimate-y) {
					difference = upperEstimate-y;
					finalX = x2;
				}
				else {
					difference = y-middleY;
					finalX = middleX;
				}
			}
			else {
				x2 = middleX;
				if (y-lowerEstimate > middleY-y) {
					difference = middleY-y;
					finalX = middleX;
				}
				else {
					difference = y-lowerEstimate;
					finalX = x1;
				}
			}
		}while (difference>tolerance);
		return finalX;
	}
}
