package gov.fnal.eag.util.densityEstimate;
import java.util.ArrayList;
import java.util.Collection;

import org.freehep.math.minuit.FCNBase;

/**
 * calculated chi2 for a set of observed points and a FitFunction
 * @author stoughto
 *
 */
public class Chi2 implements FCNBase {
	
	Collection<double[]> points;
	FitFunction fitFunction;
	int ndf;
	/**
	 * @param hg contains the observed points, assuming poisson statistics
	 * @param ff the function to fit
	 */
	public Chi2(Histogram hg, FitFunction ff) {
		points = new ArrayList<double[]>();
		for (int i=0; i<hg.nBin; i++) {
			double observed = hg.getContents(i)/hg.getBinWidth(i);
			if (!Double.isNaN(observed) && observed > 0) {
				double x = hg.getBinPos(i);
				double observedError = Math.sqrt(hg.getContents(i))/hg.getBinWidth(i);
				points.add(new double[] {x, observed, observedError});
			}
		}
		this.fitFunction = ff;
		setNdf();
	}
	private void setNdf() {
		ndf = points.size()-fitFunction.getParameterCount();
		if (ndf <= 0) {
			throw new IllegalArgumentException("points.size()="+points.size()+" nParameters="+fitFunction.getParameterCount());
		}

	}
	/**
	 * @param points
	 * @param fitFunction
	 */
	public Chi2(Collection<double[]> points, FitFunction fitFunction) {
		this.points = points;
		this.fitFunction = fitFunction;
		setNdf();
	}
	/**
	 * @see org.freehep.math.minuit.FCNBase#valueOf(double[])
	 */
	public double valueOf(double[] parameters) {
		/*
		String line = "Chi2.valueOf: ";
		for (double v : parameters) {
			line += v+" ";
		}
		System.out.println(line);
		*/
		double sum = 0;
		fitFunction.setParameters(parameters);
		double observedError;
		for (double[] point : points) {
			double x = point[0];
			double observed = point[1];
			if (point.length >2) {
				observedError = point[2];
			} else {
				observedError = 1.0;
			}
			double fitVal = fitFunction.valueAt(x);
			sum += Math.pow((fitVal-observed)/observedError, 2);
		}
		double retval = sum/ndf;
		return retval;
	}
}
