package gov.fnal.eag.util.densityEstimate;

import gov.fnal.eag.util.astronomy.SimpleFunction;

/**
 * The Epanechnikov kernel function
 * 
 * See <cite>Smoothing Methods in Statistics</cite> (1996) 
 * by J. C. Simonoff.
 * 
 * @author neilsen
 * @version $Name:  $ ($Id: EpanechnikovKernel.java,v 1.6 2009/02/22 16:53:46 stoughto Exp $)
 */
public class EpanechnikovKernel extends SimpleFunction {

	double u;
	
	/**
	 * Create a new EpanechnikovKernel
	 * 
	 * @param width the width of the kernel
	 */
	public EpanechnikovKernel(double width) {
		this.u = width / 2.0;
	}

	/**
	 * Return the value of the function at x
	 * 
	 * @param x the desired point in the kernel
	 * @return the value
	 */
	@Override
	public double f(double x) {
		x = x/u;
		double k = Math.abs(x) < 1 ? 0.75*(1.0-x*x)/(2*u): 0.0;
		return k;
	}

}
