package gov.fnal.eag.util.densityEstimate;

import org.freehep.math.minuit.FunctionMinimum;

/**
 * helper class for fitting functional forms using JMinuit
 * @author stoughto
 *
 */
public abstract class FitFunction {

	/**
	 * name of this function
	 */
	public String name;
	protected String[] parameterNames;
	protected double[] parameters;
	protected double[] errors;
	/**
	 * the result of minuit's fit, if available, or null
	 */
	public FunctionMinimum functionMinimum = null;
	/**
	 * @return the number of parameters
	 */
	public int getParameterCount() {
		return parameters.length;
	}
	
	/**
	 * @return the names of the parameters
	 */
	public String[] getParameterNames() {
		return parameterNames.clone();
	}
	
	/**
	 * @return the values of the parameters
	 */
	public double[] getParameters() {
		return parameters.clone();
	}
	/**
	 * @return the errors of the parameters
	 */
	public double[] getErrors() {
		return errors.clone();
	}
	
	/**
	 * @return the name
	 */
	public String getName() {
		return name;
	}
	
	/**
	 * set the name
	 * @param name
	 */
	public void setName(String name) {
		this.name = name;
	}
	/**
	 * @param parameter
	 * @return the value of the parameter
	 */
	public double getParameter(String parameter) {
		for (int i=0; i<parameterNames.length; i++) {
			if (parameterNames[i].equalsIgnoreCase(parameter)) {
				return parameters[i];
			}
		}
		throw new IllegalArgumentException("not a valid parameter name: "+parameter+"");
	}
	/**
	 * @param parameters values to set
	 */
	public void setParameters(double[] parameters) {
		if (parameters == null) {
			throw new IllegalArgumentException("can not set parameters to null");
		}
		if (this.parameters.length != parameters.length) {
			throw new IllegalArgumentException("lengths need to be the same but parameter.lenght="+parameters.length+" is not "+getParameterCount());
		}
		for (int i=0; i<parameters.length; i++) {
			this.parameters[i] =  parameters[i];
		}
	}
	
	/**
	 * @param parameter
	 * @return the value of the error on the parameter
	 */
	public double getError(String parameter) {
		for (int i=0; i<parameterNames.length; i++) {
			if (parameterNames[i].equalsIgnoreCase(parameter)) {
				return errors[i];
			}
		}
		throw new IllegalArgumentException("not a valid parameter name: "+parameter+"");
	}
	/**
	 * @param errors values to set
	 */
	public void setErrors(double[] errors) {
		if (this.errors.length != errors.length) {
			throw new IllegalArgumentException("lengths need to be the same but errors.length="+errors.length+" is not "+getParameterCount());
		}
		for (int i=0; i<errors.length; i++) {
			this.errors[i] =  errors[i];
		}
	}
	

	/**
	 * @param x
	 * @return the value of the function at x
	 */
	public abstract double valueAt(double x);

	/**
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		String retval = name+"\n";
		for (int i=0; i<parameters.length; i++) {
			retval += parameterNames[i]+" "+parameters[i]+" +/- "+errors[i]+"\n";
		}
		return retval;
	}
	
}
