package gov.fnal.eag.util.densityEstimate;

/**
 * an array of exponentials of the form a*exp(lambda*x)
 * @author stoughto
 *
 */
public class FitFunctionExponentials extends FitFunction {

	int nExponentials;
	/**
	 * @param nExponentials
	 */
	public FitFunctionExponentials(int nExponentials) {
		if (nExponentials < 0) {
			throw new IllegalArgumentException("nExponentials must be >=0 but nExponentials="+nExponentials);
		}		
		this.nExponentials = nExponentials;
		name = "nExponentials = "+nExponentials;
		parameterNames = new String[nExponentials*2];
		for (int i=0; i<nExponentials; i++) {
			parameterNames[2*i] = "lambda_"+i;
			parameterNames[2*i+1] = "a_"+i;
		}
		parameters = new double[2*nExponentials];
		errors = new double[2*nExponentials];		
	}
	/**
	 * @see gov.fnal.eag.util.densityEstimate.FitFunction#valueAt(double)
	 */
	@Override
	public double valueAt(double x) {
		double retval = 0.0;
		for (int i=0; i<nExponentials; i++) {
			double lambda = parameters[2*i];
			double a = parameters[2*i+1];
			retval += a*Math.exp(lambda*x);
		}
		return retval;
	}

}
