package gov.fnal.eag.util.densityEstimate;

import cern.jet.stat.Probability;

/**
 * 
 * @author stoughto
 *
 */
public class FitFunctionGaussian extends FitFunction {
	
	/**
	 *  default constructor that defines the names
	 */
	public FitFunctionGaussian() {
		name = "gaussian";
		parameterNames = new String[] {"mean", "sigma", "normalization"};
		parameters = new double[3];
		errors = new double[3];
	}
	
	/**
	 * constructor with parameters specified
	 * @param mean
	 * @param sigma
	 * @param normalization
	 */
	public FitFunctionGaussian(double mean, double sigma, double normalization) {
		this();
		parameters[0] = mean;
		parameters[1] = sigma;
		parameters[2] = normalization;
	}

	/**
	 * @see gov.fnal.eag.util.densityEstimate.FitFunction#valueAt(double)
	 */
	@Override
	public double valueAt(double x) {
		double m = parameters[0];
		double s = parameters[1];
		double c = parameters[2];
		double value = c*Math.exp(-0.5*(x-m)*(x-m)/(s*s))/(Math.sqrt(2.*Math.PI)*s);
		return value;
	}
	
	/**
	 * @see gov.fnal.eag.util.densityEstimate.FitFunction#toString()
	 */
	@Override
	public String toString() {
		String retval = name+"\n";
		for (int i=0; i<parameters.length; i++) {
			retval += parameterNames[i]+" "+parameters[i]+" +/- "+errors[i]+"\n";
		}
		double expectedMeanSigma = parameters[1]/Math.sqrt(parameters[2]);
		retval+=" expectedMeanSigma="+expectedMeanSigma;
		return retval;
	}

	/**
	 * @param x0
	 * @param x1
	 * @return the integral from x0 to x1
	 */ 
	public double integrate(double x0, double x1) {
		double m = parameters[0];
		double s = parameters[1];
		double c = parameters[2];
		double total = c*s*Math.sqrt(2*Math.PI);
		double k = 1.0/s/Math.sqrt(2.0);
		double integral = total*(Probability.errorFunction(k*(x1-m))-Probability.errorFunction(k*(x0-m)))/2.0;
		return integral;
	}


}
