package gov.fnal.eag.util.densityEstimate;

/**
 * an array of gaussians 
 * @author stoughto
 *
 */
public class FitFunctionGaussians extends FitFunction {

	int nGaussians;
	/**
	 * @param nGaussians
	 */
	public FitFunctionGaussians(int nGaussians) {
		if (nGaussians < 0) {
			throw new IllegalArgumentException("nGaussians must be >=0 but nGaussians="+nGaussians);
		}		
		this.nGaussians = nGaussians;
		name = "nGaussians = "+nGaussians;
		parameterNames = new String[nGaussians*3];
		for (int i=0; i<nGaussians; i++) {
			parameterNames[3*i] = "mean_"+i;
			parameterNames[3*i+1] = "sigma_"+i;
			parameterNames[3*i+2] = "normalization_"+i;
		}
		parameters = new double[3*nGaussians];
		errors = new double[3*nGaussians];		
	}
	/**
	 * @see gov.fnal.eag.util.densityEstimate.FitFunction#valueAt(double)
	 */
	@Override
	public double valueAt(double x) {
		double retval = 0.0;
		for (int i=0; i<nGaussians; i++) {
			double m = parameters[2*i];
			double s = parameters[2*i+1];
			double c = parameters[2*i+2];
			retval += c*Math.exp(-0.5*(x-m)*(x-m)/(s*s))/(Math.sqrt(2.*Math.PI)*s);
		}
		return retval;
	}

}
