package gov.fnal.eag.util.densityEstimate;

/**
 * 
 * @author stoughto
 *
 */
public class FitFunctionPoisson extends FitFunction {

	private PoissonDistribution poissonDistribution;
	private double mean;
	private double normalization;
	/**
	 * constructor with parameters specified
	 * @param mean
	 * @param normalization
	 */
	public FitFunctionPoisson(double mean, double normalization) {
		name = "poisson";
		parameterNames = new String[] {"mean", "normalization"};
		parameters = new double[2];
		errors = new double[2];
		setMean(mean);
		setNormalization(normalization);
	}

	private void setNormalization(double normalization) {
		this.normalization = normalization;
		parameters[1] = normalization;
	}

	/**
	 * sets the mean to this value
	 * @param mean
	 */
	public void setMean(double mean) {
		poissonDistribution = new PoissonDistribution(mean);
		this.mean = mean;
		parameters[0] = mean;
	}
	/**
	 * @see gov.fnal.eag.util.densityEstimate.FitFunction#valueAt(double)
	 */
	@Override
	public double valueAt(double x) {
		if (parameters[0] != mean) {
			setMean(parameters[0]);
		}
		if (parameters[1] != normalization) {
			setNormalization(parameters[1]);
		}
		double value = normalization*poissonDistribution.f(x);
		return value;
	}
	
	/**
	 * @see gov.fnal.eag.util.densityEstimate.FitFunction#toString()
	 */
	@Override
	public String toString() {
		String retval = name+"\n";
		for (int i=0; i<parameters.length; i++) {
			retval += parameterNames[i]+" "+parameters[i]+" +/- "+errors[i]+"\n";
		}
		double expectedMeanSigma = parameters[1]/Math.sqrt(parameters[2]);
		retval+=" expectedMeanSigma="+expectedMeanSigma;
		return retval;
	}


}
