package gov.fnal.eag.util.densityEstimate;

import org.freehep.math.minuit.FCNBase;

/**
 * function to calculate chi2 based on the a,b matrices used in lapack fits
 * @author stoughto
 *
 */
public class LSChi2 implements FCNBase {
	int nData, nCoef;
	double[][] a, b;
	
	/**
	 * @param a the "x" values, dimensioned as nCoef,nData
	 * @param b the "measured" values, dimensioned as nData,1
	 */
	public LSChi2(double[][] a, double[][] b) {
		nData = b.length;
		nCoef = a.length;
		if (nData != a[0].length) {
			throw new IllegalArgumentException("nData="+nData+" a[0].length="+a[0].length);
		}
		this.a = a;
		this.b = b;
	}
	/**
	 * @see org.freehep.math.minuit.FCNBase#valueOf(double[])
	 */
	public double valueOf(double[] coefs) {
		if (nCoef != coefs.length) {
			throw new IllegalArgumentException("nCoef="+nCoef+" coefs.length="+coefs.length);
		}
		double chi2 = 0;
		for (int iData=0; iData<nData; iData++) {
			double fitValue = 0.0;
			for (int iCoef=0; iCoef<nCoef; iCoef++) {
				fitValue += a[iCoef][iData] * coefs[iCoef];
			}
			chi2 += Math.pow(fitValue-b[iData][0], 2);
		}
		chi2 /= nData-coefs.length;
		return chi2;
	}

}
