package gov.fnal.eag.util.densityEstimate;

import java.util.ArrayList;

import gov.fnal.eag.util.astronomy.SimpleFunction;

/**
 * @author dcharms
 *
 */


public class PoissonDistribution extends SimpleFunction {
	
	private double mean;
	private ArrayList<Double> fList = new ArrayList<Double>();
	
	
	/**
	 * prepare to generate random numbers in a poisson distribution
	 * @param meanIn mean of the distribution
	 */
	public PoissonDistribution(double meanIn) {
		mean = meanIn;
		//System.out.println(" in PoissonDistribution:  mean="+mean);
		fList.add(new Double(1));
	}
	/**
	 * @param x
	 * @return 1.0/(e*x!)
	 */
	@Override
	public double f(double x) {
		int ix = (int) Math.max(0.0, x);
		double t1 = Math.pow(mean, ix);
		double t2 = factorial(ix);
		double t3 = Math.exp(mean);
		double value = t1/t2/t3;
		return value;
	}
	
	/**
	 * Calculate factorial for given value
	 * @param x value to calculate factorial for
	 * @return value of factorial
	 */
	public double factorial(int x) {
		if (x > fList.size()-1) {
			for (int i = fList.size(); i <= x; i++) {
				double previous = fList.get(i-1);
				double current = i*previous;
				fList.add(current);
			}
		}
		double retval = fList.get(x);
		return retval;
	}

}
