package gov.fnal.eag.util.densityEstimate;

import gov.fnal.eag.util.astronomy.SimpleFunction;

/**
 * The uniform kernel (related to a histogram)
 * 
 * This kernel is intended to be used as a kernel by the 
 * {@link gov.fnal.eag.util.densityEstimate.KernelDensityEstimate} class.
 * See <cite>Smoothing Methods in Statistics</cite> (1996) 
 * by J. C. Simonoff.
 * 
 * @author neilsen
 * @version $Name:  $ ($Id: UniformKernel.java,v 1.5 2009/02/23 04:04:36 stoughto Exp $)
 */
public class UniformKernel extends SimpleFunction {

	double width;
	
	/**
	 * Create a uniform kernel
	 * 
	 * @param width
	 */
	public UniformKernel(double width) {
		this.width = width;
	}

	/**
	 * calculate the value of a uniform kernel
	 * 
	 * @param x the point on the kernel to calculate
	 * @return the value
	 */
	@Override
	public double f(double x) {
		double value = Math.abs(x) < (width/2.0) ? (1.0/width) : 0;
		return value;
	}

}
