package gov.fnal.eag.util.densityEstimate.examples;

import java.io.File;
import java.io.IOException;
import java.util.Random;

import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartUtilities;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.data.xy.XYSeries;
import org.jfree.data.xy.XYSeriesCollection;

import gov.fnal.eag.util.astronomy.SimpleFunction;
import gov.fnal.eag.util.densityEstimate.KernelDensityEstimate;
import gov.fnal.eag.util.densityEstimate.UniformKernel;
import gov.fnal.eag.util.misc.StringUtil;

/**
 * A sample use of the KernelDensityEstimate
 * 
 * @author neilsen
 *
 */
public class DensityEstimateExample {
	
	/**
	 * Plot a density estimate for a uniform random distribution
	 * 
	 * @param args
	 */
	public static void main(String[] args) {

		// Create array that contains 100 random values between 0 and 1
		double[] value = new double[100];
		Random generator = new Random();
		for (int i=0; i < 100; i++) {
			value[i] = generator.nextDouble();
		}
		
		// Create an object that holds the estimates for the
		// density at 100 evenly spaced points between 0 and 1
		
		double binWidth = 0.1; // Use a bin width of 0.1
		SimpleFunction kernel =  new UniformKernel(binWidth); // Use a uniform kernel
		
		double min = 0.0; // start the estimates at 0
		double max = 1.0; // end the estimates at 1
		int number = 1000; // make 1000 estimates
		KernelDensityEstimate density = new KernelDensityEstimate(min, max, number, kernel);
		
		// Use each data point to make the estimate
		for (int i=0; i < 100; i++) {
			density.add(value[i]);
		}
		
		// Now we set up the data structures needed to make the plot

		// Create a data set to plot
		XYSeries xySeries = new XYSeries("Sample Data Set Title");
		double[] y = density.getEstimates(); // get the x values to plot
		double[] x = density.getX(); // get the y value to plot
		// add each point to the data set
		for (int i=0; i < number; i++) {
			xySeries.add(x[i],y[i]);
		}

		// Create a set of data sets to add to the plot. We just have one
		// data set, so it is short
		XYSeriesCollection xySeriesCollection = new XYSeriesCollection();
		xySeriesCollection.addSeries(xySeries);
		
		// Create a chart object
		String plotTitle = "Sample plot"; // the title of the plot
		String xLabel = "value"; // the label on the x axis
		String yLabel = "estimate"; // the label on the y axis
		PlotOrientation orientation = PlotOrientation.VERTICAL; 
		boolean showLegend = true; // show the legend on the plot?
		boolean toolTips = false; // generate tooltips?
		boolean urls = false; // generate url's?
		JFreeChart chart = ChartFactory.createXYLineChart(
				plotTitle, xLabel, yLabel, 
				xySeriesCollection, 
				orientation, showLegend, toolTips, urls);
		
		// Generate an image from the chart object
		String imageFileName = StringUtil.getFileName("png"); // the file name of the sample created image
		int imageWidth = 500; // The width of the image in pixels
		int imageHeight = 300; // The height of the imags in pixels
		try {
			ChartUtilities.saveChartAsPNG(new File(imageFileName), chart, imageWidth, imageHeight);
		} catch (IOException e) {
			throw new Error(e);
		}
		
		
	}

}
