package gov.fnal.eag.util.htm;

/**********************************************************************
 * These are the core routines for the HTM.
 * They are a direct lift from cc_aux but modified along the lined of the new design
 * In practically all cases below a double[] is a vector of 3 numbers representing
 * x,y,z in the unit sphere.
 *
 **********************************************************************
 **/
public class HTMfunc {

	/**
	 * a small number
	 */
	public static double Epsilon = 1.0E-15;
	/**
	 * number of bits to use to express one HTMid
	 */
	public static final int  IDSIZE=Long.SIZE;
	private static final long IDHIGHBIT = 1L << 63;
	/**
	 * maximum length of a name expressed as a string
	 */
	public static final int  HTMNAMEMAX = 32;
	private static final int iS2=0;
	private static final int iN1=1;
	private static final int iS1=2;
	private static final int iN2=3;
	private static final int iS3=4;
	private static final int iN0=5;
	private static final int iS0=6;
	private static final int iN3=7;



	static Object[] anchor=  new Object[6];
	static Object S_indexes[]=  new Object[4];
	static Object N_indexes[]= new Object[4];

	// dont like static initializers but this is the only way I see to do this.
	static {
		int[] n0 = {1, 0, 4}; //N0
		N_indexes[0] = n0;
		int[] n1 = {4, 0, 3}; //N1
		N_indexes[1] = n1;
		int[] n2 = {3, 0, 2}; //N2
		N_indexes[2] = n2;
		int[] n3 = {2, 0, 1};//N3
		N_indexes[3] = n3;

		int[] s0 = {1, 5, 2}; //S0
		S_indexes[0] = s0;
		int[] s1 = {2, 5, 3}; //S1
		S_indexes[1] = s1;
		int[] s2 = {3, 5, 4}; //S2
		S_indexes[2] = s2;
		int[] s3 = {4, 5, 1};//S3
		S_indexes[3] = s3;

		double[] a0  = {0,  0,  1}; // 0
		anchor[0] =  a0;// 0
		double[] a1 = {1,  0,  0}; // 1
		anchor[1] =  a1;// 0
		double[] a2 =  {0,  1,  0}; // 2
		anchor[2] =  a2;// 0
		double[] a3 =  {-1,  0,  0}; // 3
		anchor[3] =  a3;// 0
		double[] a4 = {0, -1,  0}; // 4
		anchor[4] =  a4;// 0
		double[] a5 = {0,  0, -1}; // 5
		anchor[5] =  a5;// 0
	}

	static Base bases[] = {
		new Base("S2", 10, 3, 5, 4),
		new Base("N1", 13, 4, 0, 3),
		new Base("S1", 9, 2, 5, 3),
		new Base("N2", 14, 3, 0, 2),
		new Base("S3", 11, 4,5,1),
		new Base("N0", 12, 1, 0, 4),
		new Base("S0", 8, 1, 5, 2),
		new Base("N3", 15, 2, 0, 1)
	};




	/**
	 * where to start in the HTM quadtree when looking for a vectors home
	 * xin yin zin are thin input vector
	 * v1 v2 v3 and name are loaded with the initial tringle points
	 * and the name of the triangle
	 */
	static protected void startpane(
			double[] v1, double[] v2, double[] v3,
			double xin, double yin, double zin, StringBuffer name) 
	{
		double[] tvec;
		int baseindex = 0;
		if ((xin > 0) && (yin >= 0)){
			baseindex = (zin >= 0) ? iN3 : iS0;

		} else if ((xin <= 0) && (yin > 0)){
			baseindex = (zin >= 0) ? iN2 : iS1;

		} else if ((xin < 0) && (yin <= 0)){
			baseindex = (zin >= 0) ? iN1 : iS2;

		} else /*if ((xin >= 0) && (yin < 0)) */ {  // the clause is always true by the time you get here
			baseindex = (zin >= 0) ? iN0 : iS3;
		}

		tvec = (double[]) anchor[bases[baseindex].v1];
		v1[0] = tvec[0];
		v1[1] = tvec[1];
		v1[2] = tvec[2];

		tvec = (double[])anchor[bases[baseindex].v2];
		v2[0] = tvec[0];
		v2[1] = tvec[1];
		v2[2] = tvec[2];

		tvec = (double[])anchor[bases[baseindex].v3];
		v3[0] = tvec[0];
		v3[1] = tvec[1];
		v3[2] = tvec[2];

		name.append(bases[baseindex].name);
		return;
	}


	/* calculate midpoint fo vectors v1 and v2 the answer is put in
	 the provided w vector.
	 */
	protected static final void m4_midpoint(double[] v1, double[]
	                                                            v2, double[] w){
		w[0] = v1[0] + v2[0];
		w[1] = v1[1] + v2[1];
		w[2] = v1[2] + v2[2];
		double tmp = Math.sqrt(w[0] * w[0] + w[1] * w[1] + w[2]*w[2]);
		w[0] /= tmp;
		w[1] /= tmp;
		w[2] /= tmp;
	}


	/**
	 *  for a given vector x,y,z return the HTM ID to the given depth
	 * @param x 
	 * @param y 
	 * @param z 
	 * @param depth 
	 * @return the name of the HTM triangle
	 */
	static public String lookup(double x, double y, double z, int depth) {
		//long rstat = 0;
		//int startID;
		StringBuffer name= new StringBuffer(80);

		double v1[] = new double[3];
		double v2[] = new double[3];
		double v0[] = new double[3];
		double w1[] = new double[3];
		double w2[] = new double[3];
		double w0[] = new double[3];
		double p[] =  new double[3];

		p[0] = x;
		p[1] = y;
		p[2] = z;

		// Get the ID of the level0 triangle, and its starting vertices


		startpane(v0, v1, v2, x, y, z, name);

		// Start searching for the children
		//*/
		while(depth-- > 0){
			m4_midpoint(v0, v1, w2);
			m4_midpoint(v1, v2, w0);
			m4_midpoint(v2, v0, w1);

			if (isinside(p, v0, w2, w1)) {
				name.append('0');
				copy_vec(v1, w2);
				copy_vec(v2, w1);
			}
			else if (isinside(p, v1, w0, w2)) {
				name.append('1');
				copy_vec(v0, v1);
				copy_vec(v1, w0);
				copy_vec(v2, w2);
			}
			else if (isinside(p, v2, w1, w0)) {
				name.append('2');
				copy_vec(v0, v2);
				copy_vec(v1, w1);
				copy_vec(v2, w0);
			}
			else /* if (isinside(p, w0, w1, w2)) */ { // the clause is always true by the time you get here
				name.append('3');
				copy_vec(v0, w0);
				copy_vec(v1, w1);
				copy_vec(v2, w2);
			}
		}
		return name.toString();
	}
	/**
	 * @param direction location on the sphere
	 * @param depth 
	 * @return the HTM id
	 */
	public static long lookupid(Vector3d direction, int depth) {
		return lookupId(direction.x_, direction.y_, direction.z_, depth);
	}

	/** looks up the name of a vector x,y,z and converts the name to an id 
	 * @param x 
	 * @param y 
	 * @param z 
	 * @param depth 
	 * @return the HTM id
	 */
	static public long lookupId(double x, double y, double z, int depth) {
		String name = lookup(x,y,z,depth);
		long rstat = nameToId(name.toString());
		return rstat;
	}
	/**
	 * @param ra in degrees
	 * @param dec in degrees
	 * @param depth
	 * @return the HTM id for the position
	 */
	static public String lookup(double ra, double dec, int depth) {
		double v[] = radecToVector(ra,dec);
		return lookup(v[0],v[1],v[2],depth);
	}

	/** convert ra dec to a vector 
	 * @param ra in degrees
	 * @param dec in degrees
	 * @return vector of x,y,z values
	 */
	static public double[] radecToVector(double ra, double dec) {
		double[] vec = new double[3];
		double cd = Math.cos(Math.toRadians(dec));

		double diff;
		diff = 90.0 - dec;

		if (diff < Epsilon && diff > -Epsilon){
			vec[0] = 1.0;
			vec[1] = 0.0;
			vec[2] = 1.0;
			return vec;
		}

		diff = -90.0 - dec;
		if (diff < Epsilon && diff > -Epsilon){
			vec[0] = 1.0;
			vec[1] = 0.0;
			vec[2] = -1.0;
			return vec;
		}
		vec[2] = Math.sin(Math.toRadians(dec));
		double quadrant;
		double qint;
		int iint;
		quadrant = ra / 90.0; // how close is it to an integer?
		// if quadrant is (almost) an integer, force x, y to particular
		// values of quad:
		// quad,   (x,y)
		// 0       (1,0)
		// 1,      (0,1)
		// 2,      (-1,0)
		// 3,      (0,-1)
		// q>3, make q = q mod 4, and reduce to above
		// q mod 4 should be 0.
		qint = Math.round(quadrant);
		if (Math.abs(qint - quadrant) < Epsilon) {
			iint = (int) qint;
			iint %= 4;
			if (iint < 0) iint += 4;

			switch(iint){
			case 0:
				vec[0] = 1.0;
				vec[1] = 0.0;
				break;
			case 1:
				vec[0] = 0.0;
				vec[1] = 1.0;
				break;
			case 2:
				vec[0] = -1.0;
				vec[1] = 0.0;
				break;
			case 3:
				vec[0] = 0.0;
				vec[1] = -1.0;
				break;
			}
			return vec;
		}
		vec[0] = Math.cos(Math.toRadians(ra))* cd;
		vec[1] = Math.sin(Math.toRadians(ra)) * cd;

		return vec;
	}

	/**
	 * @param ra in degrees
	 * @param dec in degrees
	 * @param depth
	 * @return the HTM id of the position
	 */
	static public long lookupId(double ra, double dec, int depth) {
		double x, y , z;
		double cd = Math.cos(Math.toRadians(dec));
		x = Math.cos(Math.toRadians(ra)) * cd;
		y = Math.sin(Math.toRadians(ra)) * cd;
		z = Math.sin(Math.toRadians(dec));
		return lookupId(x,y,z,depth);
	}

	/**
	 * for a given vector p is it contained in the triangle whose corners are
	 *  given by the vectors v1, v2,v3.
	 */
	private static boolean isinside(double[] p, double[] v1, double[] v2, double[] v3) // p need not be nromalilzed!!!
	{
		double crossp[] = new double[3];

		// These lines never return false, so why keep them?
		//crossp[0] = v1[1] * v2[2] - v2[1] * v1[2];
		//crossp[1] = v1[2] * v2[0] - v2[2] * v1[0];
		//crossp[2] = v1[0] * v2[1] - v2[0] * v1[1];
		//if (p[0] * crossp[0] + p[1] * crossp[1] + p[2] * crossp[2] < -Epsilon)
		//return false;

		crossp[0] = v2[1] * v3[2] - v3[1] * v2[2];
		crossp[1] = v2[2] * v3[0] - v3[2] * v2[0];
		crossp[2] = v2[0] * v3[1] - v3[0] * v2[1];
		if (p[0] * crossp[0] + p[1] * crossp[1] + p[2] * crossp[2] < -Epsilon)
			return false;

		// These lines never return false, so why keep them?
		//crossp[0] = v3[1] * v1[2] - v1[1] * v3[2];
		//crossp[1] = v3[2] * v1[0] - v1[2] * v3[0];
		//crossp[2] = v3[0] * v1[1] - v1[0] * v3[1];
		//if (p[0] * crossp[0] + p[1] * crossp[1] + p[2] * crossp[2] < -Epsilon)
		//return false;

		return true;
	}

	/**
	 * for a given name i.e. N301022 convert it to its 64bit htmId
	 *  effectively this walks trough the string in reverse order and
	 * sets the bits of a long number according to the charector in the
	 * String.

	   The  name has always the same structure, it begins with
	   an N or S, indicating north or south cap and then numbers 0-3 follow
	   indicating which child to descend into. So for a depth-5-index we have
	   strings like
				 N012023  S000222  N102302  etc

	   Each of the numbers correspond to 2 bits of code (00 01 10 11) in the
	   uint64. The first two bits are 10 for S and 11 for N. For example

				 N 0 1 2 0 2 3
				 11000110001011  =  12683 (dec)

	   The leading bits are always 0
	 * @param name the name to convert
	 * @return the htmId

	 **/
	static public long nameToId(String name) {

		if(name == null || name.length()==0) {
			throw new IllegalArgumentException("name="+name);
		}
		if(name.charAt(0) != 'N' && name.charAt(0) != 'S') {
			throw new IllegalArgumentException("must begin with N or S but name="+name);
		}
		int size = name.length();	   // determine string length

		if(size < 2 || size > HTMNAMEMAX) {
			throw new IllegalArgumentException("must have 2 <= size <= "+HTMNAMEMAX+" but size="+size+" for name="+name);
		}
		long retval = 0;
		for(int i = size-1; i > 0; i--) {// set bits starting from the end
			if(name.charAt(i) > '3' || name.charAt(i) < '0') {
				throw new IllegalArgumentException("characters must be 0, 1, 2, or 3 but i="+i+"= name.charAt(i)="+name.charAt(i)+" for name="+name);
			}
			retval += ( (long)(name.charAt(i)-'0')) << 2*(size - i -1);
		}

		int i = 2;					 // set first pair of bits, first bit always set
		if(name.charAt(0)=='N') i++;	  // for north set second bit too
		long last = ((long)i << (2*size - 2) );
		retval += last;
		return retval;
	}


	/**
	 * @param htmid
	 * @return the level of the HTM id
	 */
	static public int idLevel(long htmid) {
		int size=0, i;


		// determine index of first set bit
		for(i = 0; i < IDSIZE; i+=2) {
			if ( ((htmid << i) & IDHIGHBIT ) ==  IDHIGHBIT) break;
		}
		size = (IDSIZE-i) >>> 1;
		// Size is the length of the string representing the name of the
		// trixel, so the level is size - 2
		int level = size-2;
		if (level < 0) {
			throw new IllegalArgumentException("htmid="+htmid+" is not a valid htmid since level="+level);
		}
		return level;
	}

	/**
	 * Walk the bits of the id and convert it to a string like N012.
	 * sucessively look at each pair of bits and convert it to 0,1,2 or 3.
	 * Finally use the highest bit to set N or S in the string.
	 *
	 * The first bit pair gives N (11) or S (10).
	 * The subsequent bit-pairs give the numbers 0-3: (00, 01, 10, 11).

    Example: In the first level there are 8 nodes, and we get
    the names from numbers 8 through 15 giving S0,S1,S2,S3,N0,N1,N2,N3.

	   The order is always ascending starting from S0000.. to N3333...
	 * @param id HTMid to decode
	 * @return the name

	 */

	static public String idToName(long id) {
		int size=0, i;
		int c; // a spare character;


		// determine index of first set bit
		for(i = 0; i < IDSIZE; i+=2) {
			long x8 = ((id << i) & IDHIGHBIT);
			if ( x8!=0 ) break;
		}
		if(id == 0) {
			throw new IllegalArgumentException("id should be zero now but id="+id);
		}
		size=(IDSIZE-i) >>> 1;
			if (size < 2) {
				throw new IllegalArgumentException("illegal_id="+String.format("size=%d id=0x%x",size,id));
			}
			char[] name = new char[size];
			// fill characters starting with the last one
			for(i = 0; i < size-1; i++) {
				c =  '0' + (int) ((id >>> i*2) & 3);
				name[size-i-1] = (char ) c;
			}

			// put in first character
			if( ((id >> (size*2-2)) & 1) > 0 ) {
				name[0] = 'N';
			} else {
				name[0] = 'S';
			}

			return new String(name);
	}

	/**
	 * @param id HTM id
	 * @return the three corners of this HTM id triangle
	 */
	static public Vector3d[] idToTriangle(long id) {
		String name = idToName(id);
		double[][] triangle = nameToTriangle(name);
		Vector3d[] retval = {new Vector3d(triangle[0]), new Vector3d(triangle[1]), new Vector3d(triangle[2])};
		return retval;
	}
	
	/**
	 * @param id
	 * @return area of this HTM triangle, in steradians
	 */
	static public double steradians(long id) {
		Vector3d[] corners = idToTriangle(id);
		double retval = Vector3d.getAreaSr(corners);
		return retval;
	}
	/**
	 *  Like name2Id but instead of returning the htmId
	 *  return v0,v1,v2 vectors representing the corners of the triangle
	 * @param name the name to transform
	 *  @return Object[] which is holding 3 double[] for v0 v1 v2
	 **/

	static public double[][] nameToTriangle(String name)
	{
		double w0[] = new double[3];
		double w1[] = new double[3];
		double w2[] = new double[3];
		double v0[] = new double[3];
		double v1[] = new double[3];
		double v2[] = new double[3];

		// Get the top level hemi-demi-semi space

		int k;
		int[] anchor_offsets= new int[3];
		k = name.charAt(1) - '0';

		if (name.charAt(0) == 'S') {
			anchor_offsets[0] = ((int[])S_indexes[k])[0];
			anchor_offsets[1] = ((int[])S_indexes[k])[1];
			anchor_offsets[2] = ((int[])S_indexes[k])[2];
		} else {
			anchor_offsets[0] = ((int[])N_indexes[k])[0];
			anchor_offsets[1] = ((int[])N_indexes[k])[1];
			anchor_offsets[2] = ((int[])N_indexes[k])[2];
		}
		copy_vec(v0, (double[])anchor[anchor_offsets[0]]);
		copy_vec(v1, (double[])anchor[anchor_offsets[1]]);
		copy_vec(v2, (double[])anchor[anchor_offsets[2]]);

		int offset = 2;
		int len = name.length();
		while(offset < len){
			char s = name.charAt(offset);
			m4_midpoint(v0, v1, w2);
			m4_midpoint(v1, v2, w0);
			m4_midpoint(v2, v0, w1);
			switch(s) {
			case '0':
				copy_vec(v1, w2);
				copy_vec(v2, w1);
				break;
			case '1':
				copy_vec(v0, v1);
				copy_vec(v1, w0);
				copy_vec(v2, w2);
				break;
			case '2':
				copy_vec(v0, v2);
				copy_vec(v1, w1);
				copy_vec(v2, w0);
				break;
			case '3':
				copy_vec(v0, w0);
				copy_vec(v1, w1);
				copy_vec(v2, w2);
				break;
			}
			offset++;
		}
		double[][] retval = {v0, v1, v2};
		return retval;
	}

	/**
	 * simple utility funtion which copies one vector over another.
	 * this uses System.arraycopy which should be efficient.
	 * It assumes both arrays are the same lenght however it will copy all
	 * odf s to d so theoretically d could be longer than s.
	 * @param d destination of copy
	 * @param s source of copy
	 */
	protected static void copy_vec(double d[] , double s[]) {
		System.arraycopy(s,0,d,0,s.length);
	}


	/**
	 * for a given ID get back the approximate center of the triangle.
	 * This may be used as an inverse of lookup however bear in mind many points
	 * may fall in a triangle while only the center point will be returned from
	 * this function.
	 * The name is used to get the triange corners and these are then averaged
	 * to get a center point.
	 * The resultant vector is not normalized.
	 * @param name the HTM id name
	 * @return array of x,y,z location of the center of the HTM triangle
	 */

	public static double[] idToPoint(String name) {
		double[][] tri = nameToTriangle(name);
		double[] ret = new double[3];
		double[] v0 = tri[0];
		double[] v1 = tri[1];
		double[] v2 = tri[2];
		double center_x, center_y, center_z, sum;

		center_x = v0[0] + v1[0] + v2[0];
		center_y = v0[1] + v1[1] + v2[1];
		center_z = v0[2] + v1[2] + v2[2];
		sum = center_x * center_x + center_y * center_y + center_z * center_z;
		sum = Math.sqrt(sum);
		center_x /= sum;
		center_y /= sum;
		center_z /= sum;
		ret[0] = center_x;
		ret[1]= center_y;
		ret[2]= center_z; // I don't want it nomralized or radec to be set,

		return ret;

	}

	/**
	 * gets the name from the id and calls idToPoint with it.
	 * @param htmId id 
	 * @return x,y,z coordinates of the center of the trixel
	 */
	public static double[] idToPoint(long htmId) {
		String name = idToName(htmId);
		return idToPoint(name);
	}
	/**
	 * @param htmid
	 * @return a vector pointing to the center of this triangle
	 */
	public static Vector3d idToVector3d(long htmid) {
		return new Vector3d(idToPoint(htmid));
	}


	/**
	 * return the angular distance between two vectors
	 * ACOS (V1 . V2)
	 * @param v1 coordinates of first point
	 * @param v2 coordinates of second point
	 * @return distance in radians
	 */
	public static double distance (double[] v1, double[] v2) {
		double prod = 0;
		for (int i = 0; i < v1.length; i++ ) {
			prod += v1[i]*v2[i] ;
		}
		double dist = Math.acos(prod);
		return dist;
	}

	/**
	 * return the angular distance between two htm names
	 * gets the vectors of the mid points and uses those to compute distance
	 * @param htm1 name of the first point
	 * @param htm2 name of the second point
	 * @return distance in radians
	 */
	public static double distance (String htm1, String htm2) {
		double[] v1 = idToPoint(htm1);
		double[] v2 = idToPoint(htm2);
		return distance(v1,v2);
	}

	/**
	 * @param htmId
	 * @return an array of the four children of this trixel
	 */
	public static long[] getChildren(long htmId) {
		if ( (htmId & IDHIGHBIT) != 0) {
			throw new IllegalArgumentException("htmId="+htmId+" with level="+idLevel(htmId)+" does not have children");
		}
		long[] retval = new long[4];
		retval[0] = htmId << 2;
		retval[1] = retval[0] | 1l;
		retval[2] = retval[0] | 2l;
		retval[3] = retval[0] | 3l;
		return retval;
	}

	/**
	 * get the siblings of this trixel
	 * @param htmId the parent trixel
	 * @param finalLevel the level of the siblings
	 * @return contains one range containing the sibling trixel ids
	 */
	public static HTMrange getSiblings(long htmId, int finalLevel)  {
		int parentLevel = idLevel(htmId);
		if (parentLevel > finalLevel) {
			throw new IllegalArgumentException("parent level can not be > final level but parentLevel="+parentLevel+" finalLevel="+finalLevel);
		}
		if (finalLevel >= HTMfunc.HTMNAMEMAX-2 ) {
			throw new IllegalArgumentException("finalLevel can not be >= "+(HTMfunc.HTMNAMEMAX-2)+" but finalLevel="+finalLevel);
		}
		int bitsToShift=2*(finalLevel-parentLevel);
		long min = htmId << bitsToShift;
		long max = min;
		for (int i=0; i<bitsToShift; i++) {
			max |= 1l << i;
		}
		HTMrange retval = new HTMrange();
		retval.addRange(min, max);
		return retval;
	}


	/**
	 * @param thisHtm
	 * @return an index to seed SimRandom classes
	 */
	public static int indexFromHtm(Long thisHtm) {
		int seedIndex = (int) (thisHtm % Integer.MAX_VALUE);
		return seedIndex;
	}
}




class  Base {
	String name;
	int ID;
	int v1, v2, v3;
	/**
	 * @param n
	 * @param id
	 * @param v1
	 * @param v2
	 * @param v3
	 */
	public Base(String n, int id, int v1, int v2, int v3) {
		this.name = n;
		this.ID=id;
		this.v1=v1;
		this.v2=v2;
		this.v3=v3;
	}
	

}


