package gov.fnal.eag.util.htm;


import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Paint;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.Stroke;
import java.awt.geom.Point2D.Double;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;

import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartUtilities;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.annotations.XYAnnotation;
import org.jfree.chart.annotations.XYPolygonAnnotation;
import org.jfree.chart.annotations.XYTextAnnotation;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.data.xy.XYSeriesCollection;

/**
 * utilities to help plot htm stuff
 * @author stoughto
 *
 */
public class PlotUtil {
	/**
	 * @param htmid id to draw
	 * @param sc0 central coordinate
	 * @param nPointsPerEdge 
	 * @param stroke the stroke to use
	 * @param paint the paint to use
	 * @param minMax rectangle that keeps track of boundaries for plotting
	 * @return triangle as an XYAnnotation
	 */
	public static XYAnnotation getTriangleAnnotation(long htmid, Vector3d sc0, int nPointsPerEdge, Stroke stroke, Paint paint, Rectangle.Double minMax) {
		double ra0 = sc0.ra();
		double dec0 = sc0.dec();
		Vector3d[] c = HTMfunc.idToTriangle(htmid);
		double[] polygon = new double[2*3*nPointsPerEdge];
		int i0 = 0;
		double xMin = minMax.x;
		double xMax = minMax.x+minMax.width;
		double yMin = minMax.y;
		double yMax = minMax.y+minMax.height;
		{
			ArrayList<Point.Double> points = getGCPoints(c[0], c[1], ra0, dec0, nPointsPerEdge);
			for (Point.Double p : points) {
				xMin = Math.min(p.x, xMin);
				xMax = Math.max(p.x, xMax);
				yMin = Math.min(p.y, yMin);
				yMax = Math.max(p.y, yMax);
				
				polygon[i0++] = p.x;
				polygon[i0++] = p.y;
			}
		}
		{
			ArrayList<Point.Double> points = getGCPoints(c[1], c[2], ra0, dec0, nPointsPerEdge);
			for (Point.Double p : points) {

				xMin = Math.min(p.x, xMin);
				xMax = Math.max(p.x, xMax);
				yMin = Math.min(p.y, yMin);
				yMax = Math.max(p.y, yMax);
				
				polygon[i0++] = p.x;
				polygon[i0++] = p.y;
			}
		}
		{
			ArrayList<Point.Double> points = getGCPoints(c[2], c[0], ra0, dec0, nPointsPerEdge);
			for (Point.Double p : points) {

				xMin = Math.min(p.x, xMin);
				xMax = Math.max(p.x, xMax);
				yMin = Math.min(p.y, yMin);
				yMax = Math.max(p.y, yMax);

				polygon[i0++] = p.x;
				polygon[i0++] = p.y;
			}
		}
		
		minMax.x = xMin;
		minMax.y = yMin;
		minMax.width = xMax-xMin;
		minMax.height = yMax-yMin;
		
		XYPolygonAnnotation retval = new XYPolygonAnnotation(polygon, stroke, paint);
		return retval;
	}


	/**
	 * @param p1 first point
	 * @param p2 second point
	 * @param ra0 ra of tangential plane projection
	 * @param dec0 dec of tangential plane projection
	 * @param nPoints number of points for interpolation
	 * @return x,y points 
	 */
	public static ArrayList<Point.Double> getGCPoints(Vector3d p1, Vector3d p2, double ra0, double dec0, int nPoints) {
		double ra1 = p1.ra();
		double dec1 = p1.dec();
		double cosDec0 = Math.cos(Math.toRadians(dec0));

		Vector3d p2Temp = new Vector3d(p2);
		p2Temp.rotZ(-ra1);
		p2Temp.rotY(dec1-90);
		
		double ra2Rotated =  p2Temp.ra();

		ArrayList<Point.Double> retval = new ArrayList<Point.Double>();

		double separation = p1.separation(p2);
		for (int i=0; i<nPoints; i++) {
			double decTemp = 90-separation*i/(nPoints);
			Vector3d temp = new Vector3d(0, decTemp);
			temp.rotZ(ra2Rotated);
			temp.rotY(90-dec1);
			temp.rotZ(ra1);
			double tempRa = temp.ra();
			if (ra0 < 180) {
				if (tempRa > 180) {
					tempRa -= 360;
				}
			} 
			double x = cosDec0*(tempRa-ra0);
			double y = temp.dec()-dec0;
			retval.add(new Point.Double(x,y));
		}
		return retval;
		
	}
	/**
	 * @param region
	 * @param center
	 * @param nPointsPerConstraint
	 * @param stroke
	 * @param paint
	 * @param minMax
	 * @return an XYAnnotation that defines the region
	 */
	public static XYAnnotation getConvexRegionAnnotation(ConvexRegion region, Vector3d center, int nPointsPerConstraint, BasicStroke stroke, Paint paint, Rectangle.Double minMax) {
		double ra0 = center.ra();
		double dec0 = center.dec();
		double xMin = minMax.x;
		double xMax = minMax.x+minMax.width;
		double yMin = minMax.y;
		double yMax = minMax.y+minMax.height;
		
		double[] polygon =  new double[2*nPointsPerConstraint*region.getNumberOfConstraints()];
		int i0 = 0;
		if (region.getNumberOfConstraints() == 1) {
			Constraint c = region.getConstraint(0);
			ArrayList<Point.Double> points = getConstraintPoints(c, ra0, dec0, nPointsPerConstraint);
			for (Point.Double p : points) {
				
				xMin = Math.min(p.x, xMin);
				xMax = Math.max(p.x, xMax);
				yMin = Math.min(p.y, yMin);
				yMax = Math.max(p.y, yMax);
				
				polygon[i0++] = p.x;
				polygon[i0++] = p.y;
			}
			
		} else {
			for (int i=0; i<region.getNumberOfConstraints(); i++) {
				Vector3d c0 = region.getCorner(i);
				Vector3d c1;
				try {
					c1 = region.getCorner(i);
				} catch (IndexOutOfBoundsException e) {
					c1 = region.getCorner(0);
				}
				
				ArrayList<Point.Double> points = getGCPoints(c0, c1, ra0, dec0, nPointsPerConstraint);
				for (Point.Double p : points) {
					
					xMin = Math.min(p.x, xMin);
					xMax = Math.max(p.x, xMax);
					yMin = Math.min(p.y, yMin);
					yMax = Math.max(p.y, yMax);
					
					polygon[i0++] = p.x;
					polygon[i0++] = p.y;
				}

				
			}
		}
		minMax.x = xMin;
		minMax.y = yMin;
		minMax.width = xMax-xMin;
		minMax.height = yMax-yMin;
		XYPolygonAnnotation retval = new XYPolygonAnnotation(polygon, stroke, paint);
		return retval;
	}

	private static ArrayList<Double> getConstraintPoints(Constraint c, double ra0, double dec0, int pointsPerConstraint) {
		double cosDec0 = Math.cos(Math.toRadians(dec0));
		ArrayList<Double> retval = new ArrayList<Double>();
		Vector3d direction = c.getDirection();
		double raDirection = direction.ra();
		double decDirection = direction.dec();
		double radius = Math.toDegrees(c.getAngle());
		for (int i=0; i<pointsPerConstraint; i++) {
			double ra = 360.0*i/pointsPerConstraint;
			Vector3d temp = new Vector3d(ra, 90-radius);
			temp.rotY(90-decDirection);
			temp.rotZ(raDirection);
			double tempRa = temp.ra();
			if (ra0 < 180) {
				if (tempRa > 180) {
					tempRa -= 360;
				}
			} 
			double x = cosDec0*(tempRa-ra0);
			double y = temp.dec()-dec0;
			retval.add(new Point.Double(x,y));
			
		}
		return retval;
	}
	/**
	 * make a plot
	 * @param convexRegion region to plot
	 * @param htmid defines the triangle to plot
	 * @param fileName where to write the plot
	 * @throws IOException 
	 */
	public static void plotOverlap(ConvexRegion convexRegion, long htmid, String fileName) throws IOException {
		Vector3d center = HTMfunc.idToVector3d(htmid);
		double ra0 = center.ra();
		double dec0 = center.dec();
		
		
		XYSeriesCollection xysc = new XYSeriesCollection();
		
		JFreeChart chart = ChartFactory.createScatterPlot("plotOverlap htmid="+HTMfunc.idToName(htmid),"ra-"+ra0,"dec-"+dec0, xysc, PlotOrientation.VERTICAL, true, true, true);

		Rectangle.Double minMax =  new Rectangle.Double();
		XYAnnotation triangleAnnotation = PlotUtil.getTriangleAnnotation(htmid, center, 100, new BasicStroke(), Color.red, minMax);
		chart.getXYPlot().addAnnotation(triangleAnnotation);

		XYAnnotation convexRegionAnnotation =  PlotUtil.getConvexRegionAnnotation(convexRegion, center, 100, new BasicStroke(), Color.blue, minMax);
		chart.getXYPlot().addAnnotation(convexRegionAnnotation);
		
		chart.getXYPlot().getDomainAxis().setRange(minMax.x-0.05*minMax.width, minMax.x+1.05*minMax.width);
		chart.getXYPlot().getRangeAxis().setRange(minMax.y-0.05*minMax.height, minMax.y+1.05*minMax.height);
		
		ChartUtilities.saveChartAsPNG(new File(fileName), chart, 500, 500);
		}

	/**
	 * plot a convex region and triangles 
	 * @param cr the region
	 * @param range the list of HTM
	 * @param center of tangential projection
	 * @param title written at the top of the plot
	 * @param annotate if true plot text annotation for each HTM element
	 * @param fileName where to write it
	 */
	public static void plotConvexRegionRange(ConvexRegion cr, HTMrange range, Vector3d center, String title,
			boolean annotate, String fileName) {
		plotConvexRegionRange(cr, range, center, title, fileName, annotate, null);
	}
	
	/**
	 * plot a convex region and triangles 
	 * @param cr the region
	 * @param range the list of HTM
	 * @param center of tangential projection
	 * @param title written at the top of the plot
	 * @param fileName where to write it
	 * @param annotate if true plot text annotation for each HTM element
	 * @param plotMinMax plot limits, or null for auto scaling
	 */
	public static void plotConvexRegionRange(ConvexRegion cr, HTMrange range, Vector3d center, String title, String fileName, 
			boolean annotate, Rectangle.Double plotMinMax) {
		double ra0 = center.ra();
		double dec0 = center.dec();
		
		
		XYSeriesCollection xysc = new XYSeriesCollection();		
		JFreeChart chart = ChartFactory.createScatterPlot(title,"ra-"+ra0,"dec-"+dec0, xysc, PlotOrientation.VERTICAL, true, true, true);
		Rectangle.Double minMax =  new Rectangle.Double();
		for (Long htmid : range) {
			XYAnnotation triangleAnnotation = PlotUtil.getTriangleAnnotation(htmid, center, 100, new BasicStroke(), Color.red, minMax);
			chart.getXYPlot().addAnnotation(triangleAnnotation);
			if (annotate) {
				XYAnnotation triangleTextAnnotation = PlotUtil.getTriangleTextAnnotation(htmid, center, Color.red, true);
				chart.getXYPlot().addAnnotation(triangleTextAnnotation);			
			}
		}

		XYAnnotation convexRegionAnnotation =  PlotUtil.getConvexRegionAnnotation(cr, center, 100, new BasicStroke(), Color.blue, minMax);
		chart.getXYPlot().addAnnotation(convexRegionAnnotation);

		if (plotMinMax != null) {
			minMax = plotMinMax;
		}
		chart.getXYPlot().getDomainAxis().setRange(minMax.x-0.05*minMax.width, minMax.x+1.05*minMax.width);
		chart.getXYPlot().getRangeAxis().setRange(minMax.y-0.05*minMax.height, minMax.y+1.05*minMax.height);

		try {
			ChartUtilities.saveChartAsPNG(new File(fileName), chart, 500, 500);
		} catch (IOException e) {
			throw new Error(e);
		}
		
		
	}


	/**
	 * @param htmid the identification of this triangle
	 * @param center plot as a tangent plane projection centered at this position
	 * @param textColor color for the text
	 * @param asName true to print HTMfunc.idToName(htmid); false to print htmid.toString()
	 * @return an annotation to plot this triangle
	 */
	public static XYAnnotation getTriangleTextAnnotation(Long htmid, Vector3d center, Color textColor, boolean asName) {
		double ra0 = center.ra();
		double dec0 = center.dec();
		double cosDec0 = Math.cos(Math.toRadians(dec0));
		Vector3d direction = HTMfunc.idToVector3d(htmid);
		double tempRa = direction.ra();
		if (ra0 < 180) {
			if (tempRa > 180) {
				tempRa -= 360;
			}
		} 
		double x = cosDec0*(tempRa-ra0);
		double y = direction.dec()-dec0;
		String text;
		if (asName) {
			text = HTMfunc.idToName(htmid);
		} else {
			text = htmid.toString();
		}
		XYTextAnnotation retval = new XYTextAnnotation(text, x, y);
		retval.setPaint(textColor);
		return retval;
	}
	

}