package gov.fnal.eag.util.jdbc;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.util.Properties;

import uk.ac.starlink.table.StarTable;
import uk.ac.starlink.table.StarTableFactory;
import uk.ac.starlink.table.TableFormatException;

/**
 * manages the creation of a Connection to an Oracle server
 * @author stoughto
 * 
 * Properties read from the file .oracle in the user's home area:
 * host host.domain
 * port 1521
 * sid sidName
 * username userName
 * password passWord
 *
 */
public class OracleConnection {

	/**
	 * 
	 * @return the connection
	 * @throws SQLException if the call to DriverManager.getConnection throws SQLException
	 * @throws ClassNotFoundException if oracle.jdbc.driver.OracleDriver can not be loaded
	 */
	public static Connection getConnection() throws SQLException, ClassNotFoundException {
		String connectionString = getConnectionString();
		System.out.println(" connectionString="+connectionString);
		Connection conn = DriverManager.getConnection(connectionString);
		return conn;
		
	}

	private static String getConnectionString() throws Error, ClassNotFoundException {
		Properties properties = getProperties();
		String host = properties.getProperty("host");
		String port = properties.getProperty("port");
		String sid  = properties.getProperty("sid");
		String username = properties.getProperty("username");
		String password = properties.getProperty("password");
		String connectionString = "jdbc:oracle:thin:"+username+"/"+password+"@"+host+":"+port+":"+sid;
		return connectionString;
	}

	private static Properties getProperties() throws Error, ClassNotFoundException {
		File propertyFile = new File(System.getProperty("user.home"), ".oracle");
		Properties properties = new Properties();
		try {
			properties.load(new FileInputStream(propertyFile));
		} catch (FileNotFoundException e) {
			throw new Error("Can not find file="+propertyFile.getAbsolutePath());
		} catch (IOException e) {
			throw new Error("IOException while reading file="+propertyFile.getAbsolutePath());
		}
		
		String oracleClass = "oracle.jdbc.driver.OracleDriver";
		Class.forName(oracleClass);

		return properties;
	}
	
	/**
	 * @param query
	 * @return the resulting table
	 * @throws ClassNotFoundException
	 * @throws TableFormatException
	 * @throws IOException
	 */
	public static StarTable getStarTable(String query) throws ClassNotFoundException, TableFormatException, IOException {
		String connectionString = getConnectionString();
	   	String databaseUrl = connectionString+"#"+query;
		StarTableFactory starTableFactory = new StarTableFactory(false);
		StarTable retval = starTableFactory.makeStarTable(databaseUrl);
		return retval;
	}
}
