package gov.fnal.eag.util.misc;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;

/**
 * interact with the command line of the operating system.
 * Note:  only deals with operating systems that understand the "which" command.
 * You are responsible for making "which" work on your system!
 * @author stoughto
 *
 */
public class CommandLine {

	/**
	 * see if the command exists
	 * @param command
	 * @return true if the command exists
	 */
	public static boolean exists(String command) {
		String whichCommand = "which "+command;
		String line = null;
		Process p;
		try {
			p = Runtime.getRuntime().exec(whichCommand);
		} catch (IOException e) {
			throw new Error(e);
		}
		BufferedReader is = new BufferedReader(new InputStreamReader(p.getInputStream()));

		try {
			if ( (line = is.readLine()) != null) {
			  if (line.indexOf(command) == -1) {
				System.out.println("command="+command+" not found");
				return false;
				}				
			} else {
				System.out.println("which does not seem to work on this system");
				return false;
			}
		} catch (IOException e) {
			throw new Error(e);
		}
		return true;
	}
	
	/**
	 * execute the command
	 * @param command
	 * @return return value from the command
	 * @throws Error if the command is not defined
	 */
	public static int run(String command) {
		Process p;
		try {
			p = Runtime.getRuntime().exec(command);
		} catch (IOException e) {
			throw new Error("command not found: "+command, e);
		}
		
		BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(p.getInputStream()));
		String line;
		try {
			while ((line = bufferedReader.readLine()) != null) {
				System.out.println(line);
			}
		} catch (IOException e) {
			System.out.println("IO Exception caught");
		}
		try {
			p.waitFor();
		} catch (InterruptedException e1) {
			throw new Error(e1);
		}
		int retval = p.exitValue();
		return retval;
	}
	
	

}
