package gov.fnal.eag.util.misc;

import uk.ac.starlink.util.PhysicalConstants;

/**
 * a convenient place to keep some convenient constants
 * @author stoughto
 *
 */
public class Constants {

	/**
	 * Planck's constants, in erg*sec
	 */
	public static final double h = PhysicalConstants.PLANCK * 1e7;
	
	/**
	 * speed of light in angstrom/sec
	 */
	public static final double c = PhysicalConstants.SPEED_OF_LIGHT * 1e10;

	/**
	 * number of meters per parsec
	 * taken from http://jscience.org/experimental/javadoc/constant-values.html#org.jscience.astronomy.AstronomyConstants.PARSEC
	 */
	public static final double METERS_PER_PARSEC = 3.0856775807e16;

	/**
	 * 10 parsec in cm: 10 parsec * (meters/parsec) * (cm/meter)
	 */
	public static final double tenPc = 10.0 * METERS_PER_PARSEC * 100;

	/**
	 * surface area of a sphere with radius of 10 parsec, in cm^2
	 */
	public static final double tenPcCm2 = 4 * Math.PI * tenPc * tenPc;

	/**
	 * Boltzman's constant in J/degree K.
	 */
	public final static double kb = 1.3806504E-23; 
	
	

}
