package gov.fnal.eag.util.misc;

import gov.fnal.eag.util.densityEstimate.FitFunctionGaussian;

import java.util.Hashtable;

import cern.colt.list.DoubleArrayList;
import cern.jet.stat.Descriptive;

/**
 * convenience static methods to calculate mean and sigma of a DoubleArrayList
 * @author stoughto
 *
 */
public class MeanSigma {
	/**
	 * @param dal
	 * @return key/value pairs for mean, meanError, sigma, sigmaError, and n
	 */
	public static Hashtable<String,Double> getMeanSigma(DoubleArrayList dal) {
		double mean = Descriptive.mean(dal);
		double sigma = Math.sqrt(Descriptive.sampleVariance(dal, mean));
		double meanError = sigma/Math.sqrt(dal.size());
		double sigmaError = sigma/(2.0*Math.sqrt(dal.size()));
		Hashtable<String, Double> retval = new Hashtable<String, Double>();
		retval.put("mean", mean);
		retval.put("meanError", meanError);
		retval.put("sigma", sigma);
		retval.put("sigmaError", sigmaError);
		retval.put("n", (double) dal.size());
		return retval;
	}
	
	/**
	 * @param dal
	 * @return the gaussian function with mean, sigma, and n set to the values calculated from dal
	 */
	public static FitFunctionGaussian getFitFunctionGaussian(DoubleArrayList dal) {
		Hashtable<String,Double> ht = MeanSigma.getMeanSigma(dal);
		FitFunctionGaussian retval = new FitFunctionGaussian();
		retval.setParameters(new double[] {ht.get("mean"), ht.get("sigma"), ht.get("n")});
		return retval;
	}
}
