package gov.fnal.eag.util.misc;

import java.util.Enumeration;
import java.util.Properties;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import com.martiansoftware.jsap.FlaggedOption;
import com.martiansoftware.jsap.JSAP;
import com.martiansoftware.jsap.JSAPException;
import com.martiansoftware.jsap.StringParser;

/**
 * remove duplicate, leading, and trailing whitespace from a CharSequence
 * @author stoughto
 * copied from 	http://javaalmanac.com/egs/java.util.regex/RemDupSpace.html
 *
 */
public class StringUtil {
		
	/**
	 * set this to non-null to test the getFileName methods for different versions of java
	 */
	public static String defaultJavaVersionString = null;
	/**
	 * do the actual work
	 * @param inputStr original text
	 * @return text with duplicate whitespace removed
	 */
	public static CharSequence removeDuplicateWhitespace(CharSequence inputStr) {
		String trimmed = inputStr.toString().trim();
		String patternStr = "\\s+";
		String replaceStr = " ";
		Pattern pattern = Pattern.compile(patternStr);
		Matcher matcher = pattern.matcher(trimmed);
		return matcher.replaceAll(replaceStr);
	}
	
	/**
	 * @param in white-space separated values
	 * @return the tokens as an array of Strings
	 */
	public static String[] getTokens(String in) {
		return removeDuplicateWhitespace(in).toString().split(" ");
	}

	/**
	 * 
	 * @param fileType such as "png" or "fit"
	 * @return the calling method + suffix
	 */
	public static String getFileName(String fileType) {
		int stackTraceIndex = getStackTraceIndex();
		String callingMethod = Thread.currentThread().getStackTrace()[stackTraceIndex].getMethodName();
		String callingClass = Thread.currentThread().getStackTrace()[stackTraceIndex].getClassName();
		String retval = callingClass+"."+callingMethod;
		if (fileType != null && fileType.length() > 0) retval += "."+fileType;
		return retval;
	}
	/**
	 * 
	 * @param suffix text inserted just before the .
	 * @param fileType such as "png" or "fit"
	 * @return the calling method + suffix
	 */
	public static String getFileName(String suffix, String fileType) {
		int stackTraceIndex = getStackTraceIndex();
		String callingMethod = Thread.currentThread().getStackTrace()[stackTraceIndex].getMethodName();
		String callingClass = Thread.currentThread().getStackTrace()[stackTraceIndex].getClassName();
		String retval = callingClass+"."+callingMethod+suffix;
		if (fileType != null && fileType.length() > 0) retval += "."+fileType;
		return retval;
	}

	private static int getStackTraceIndex() {
		String javaVersionString;
		if (defaultJavaVersionString == null) {
			javaVersionString = System.getProperty("java.specification.version");
		} else {
			javaVersionString = defaultJavaVersionString;
		}
		double javaVersion;
		javaVersion = Double.parseDouble(javaVersionString);
		int stackTraceIndex;
		if (javaVersion < 1.6) {
			stackTraceIndex = 3; // works for version 1.5 and earlier
		} else {
			stackTraceIndex = 2; // works for version 1.6
		}		
		return stackTraceIndex;
	}

	/**
	 * remove duplicate whitespace and comma separate
	 * @param line
	 * @return tokens
	 */
	public static String[] tokenize(String line) {
		String[] retval = StringUtil.removeDuplicateWhitespace(line).toString().split(",");
		return retval;
	}
	/**
	 * remove duplicate whitespace and separate at the separator
	 * @param line
	 * @param separator the separator between tokens
	 * @return tokens
	 */
	public static String[] tokenize(String line, String separator) {
		String temp = StringUtil.removeDuplicateWhitespace(line).toString();
		String[] retval = temp.split(separator);
		return retval;
	}
	
	/**
	 * remove duplicate whitespace and comma separate
	 * @param line
	 * @return tokens parsed to double values
	 */
	public static double[] toDoubleArray(String line) {
		return toDoubleArray(line, ",");
	}
	/**
	 * remove duplicate whitespace and comma separate
	 * @param line
	 * @param separator comma or space, for example
	 * @return tokens parsed to double values
	 */
	public static double[] toDoubleArray(String line, String separator) {
		String noWhitespace = StringUtil.removeDuplicateWhitespace(line).toString();
		if (noWhitespace.length() == 0) {
			return new double[0];
		}
		String[] temp = noWhitespace.split(separator);
		double[] retval = new double[temp.length];
		for (int i=0; i<retval.length; i++) {
			retval[i] = Double.parseDouble(temp[i]);
		}
		return retval;
	}

	/**
	 * @param line
	 * @return array parsed from the space separated line
	 */
	public static int[] toIntArray(String line) {
		return toIntArray(line, " ");
	}
	/**
	 * @param line
	 * @param separator 
	 * @return tokens parsed to integer values
	 */
	public static int[] toIntArray(String line, String separator) {
		String noWhitespace = StringUtil.removeDuplicateWhitespace(line).toString();
		if (noWhitespace.length() == 0) {
			return new int[0];
		}
		String[] temp = noWhitespace.split(separator);
		int[] retval = new int[temp.length];
		for (int i=0; i<retval.length; i++) {
			retval[i] = Integer.parseInt(temp[i]);
		}
		return retval;
	}

	/**
	 * @param line space separate tokens
	 * @return the result of Boolean.parseBoolean(String) on each token, which returns true if the String equals "true" ignoring case
	 */
	public static boolean[] toBooleanArray(String line) {
		String[] temp = StringUtil.removeDuplicateWhitespace(line).toString().split(" ");
		boolean[] retval = new boolean[temp.length];
		for (int i=0; i<retval.length; i++) {
			retval[i] = Boolean.parseBoolean(temp[i]);
		}
		return retval;
	}

	
	/**
	 * @param properties
	 * @param key
	 * @return the value of the proerty as a String
	 */
	public static String getPropertyAsString(Properties properties, String key) {
		String temp = properties.getProperty(key);
		return temp;
	}
	/**
	 * @param properties
	 * @param key
	 * @return the value of the property as an int
	 * @throws NumberFormatException generated by calling the Integer.toString method
	 */
	public static int getPropertyAsInt(Properties properties, String key) throws NumberFormatException {
		String temp = properties.getProperty(key);
		try {
			int retval = Integer.parseInt(temp);
			return retval;
		} catch (NumberFormatException e) {
			throw new Error("can not parse temp="+temp+" for key="+key);
		}
	}
	/**
	 * @param properties
	 * @param key
	 * @return the value of the property as a double
	 * @throws NumberFormatException generated by calling the Double.toString method
	 */
	public static double getPropertyAsDouble(Properties properties, String key) throws NumberFormatException {
		String temp = properties.getProperty(key);
		try {
			double retval = Double.parseDouble(temp);
			return retval;
		} catch (NullPointerException e) {
			throw new Error("can not parse value temp="+temp+" for key="+key);
		}
	}
	
	/**
	 * @param properties
	 * @param key
	 * @return the value of the property as a boolean
	 */
	public static boolean getPropertyAsBoolean(Properties properties, String key) {
		String temp = properties.getProperty(key);
		boolean retval = Boolean.parseBoolean(temp);
		return retval;
	}

	/**
	 * @param properties
	 * @param key
	 * @return the value of key as an array of strings.
	 */
	public static String[] getPropertyAsStringArray(Properties properties, String key) {
		String temp = properties.getProperty(key);
		String[] retval = getTokens(temp);
		return retval;
	}
	/**
	 * @param properties
	 * @param key
	 * @return the values of the key as an array of integers
	 * @throws NumberFormatException generated by call to Integer.parseInt method
	 */
	public static int[] getPropertyAsIntArray(Properties properties, String key) throws NumberFormatException {
		String temp = properties.getProperty(key);
		int[] retval = toIntArray(temp);
		return retval;
	}
	/**
	 * @param properties
	 * @param key
	 * @return the array
	 * @throws NumberFormatException generated by call to Double.parseInt method
	 */
	public static double[] getPropertyAsDoubleArray(Properties properties, String key) throws NumberFormatException {
		String temp = properties.getProperty(key);
		double[] retval = toDoubleArray(temp);
		return retval;
	}
	/**
	 * @param properties
	 * @param key
	 * @return the array
	 */
	public static boolean[] getPropertyAsBooleanArray(Properties properties, String key) {
		String temp = properties.getProperty(key);
		boolean[] retval = toBooleanArray(temp);
		return retval;
	}

	/**
	 * @param values
	 * @return a comma-separated representation of the array of double values 
	 */
	public static String toString(double[] values) {
		String retval = "";
		for (double value : values) {
			if (retval.length() > 0) {
				retval += ",";
			}
			retval += Double.toString(value);
		}
		return retval;
	}

	/**
	 * @param original
	 * @return a copy of properties.  Changes to this copy do not change original
	 */
	public static Properties copy(Properties original) {
		Properties retval = new Properties();
		Enumeration<?> names =  original.propertyNames();
		while (names.hasMoreElements()) {
			String key = (String) names.nextElement();
			String value = original.getProperty(key);
			retval.setProperty(key, value);
		}
		return retval;
	}

	/**
	 * @param jsap
	 * @param name
	 * @param parser
	 * @throws JSAPException
	 */
	public static void register(JSAP jsap, String name, StringParser parser) throws JSAPException {
		jsap.registerParameter( new FlaggedOption(name).setStringParser(parser).setRequired(true).setLongFlag(name));
	}

	/**
	 * @param fullFileName
	 * @return the name of the file before the first dot
	 * /aaa/bbb/ccc/ddd/eee.fff returns eee
	 */
	public static String getRootFileName(String fullFileName) {
		String[] t1 = tokenize(fullFileName, "/");
		String fileName = t1[t1.length-1];
		String[] t2 = tokenize(fileName, "\\.");
		String retval = t2[0];
		return retval;
	}
}
