package gov.fnal.eag.util.misc;



/**
 * @author kuropat
 * The time bomb class will run in a separate thread and kill the
 * job when preset time will be reached. The class sets an exit code -1
 * which can be detected by the submitting script and interpreted as error.
 * This will permit to terminate jobs that for unknown reason runs forever
 * on some OSG sites. The bomb can be disarmed by using stopTB method.
 */
public class TimeBomb extends Thread {



	private boolean stop = false; // stop the thread if not 0

	private int waitTime = 180; // in seconds

	/**
	 * constructor
	 * @param waitTime time in seconds after which the bomb will explode
	 */
	public TimeBomb(int waitTime) {
		this.waitTime = waitTime;
	}




	/**
	 * @see java.lang.Thread#run()
	 * this is the main loop of the thread
	 * if timeout ends before the main thread is finished the jobs exit code will be -1
	 * the stop flag does not make effect
	 */
	@Override
	public void run() {

		while (!stop) {
			if (stop) Thread.yield();
			try {
				Thread.sleep(waitTime * 1000);

				if (!stop) {
					System.out.println("The job is killed by time bomb");
					System.exit(-1); 
				}
			} catch (InterruptedException e) {
				System.exit(0);
			}
		}
		Thread.yield(); // return control to main thread
	}

	/**
	 * The method to stop the timeBomb
	 */
	public void stopTB() {
		stop = true;
	}

	/**
	 * an example of how to use the time bomb
	 * @param args
	 */
	public static void main(String[] args) {
		/* Perform just one cycle of garbage collector */
		int proctime=2; // in seconds
		TimeBomb tb = new TimeBomb(180);
		tb.start();
		try {
			System.out.println("Start processing");
//			Thread.sleep(10*1000);
//			tb.stopTB();  // this should not change anything
//			System.out.println("Set stop the bomb");
			Thread.sleep(proctime*2000);
			System.out.println("Normal termination");
			tb.stopTB();
			//System.exit(0);
		} catch (InterruptedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		tb.stopTB();

	}
}
