package gov.fnal.eag.util.plot;

 /**
 * This library is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU Lesser General Public License as published by 
 * the Free Software Foundation; either version 2.1 of the License, or 
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public 
 * License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License 
 * along with this library; if not, write to the Free Software Foundation, 
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 */
import java.io.File;

import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartUtilities;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.data.xy.XYDataset;

/**
 *  Demonstrates how to use XYErrorBarSeriesCollection and XYErrorBarRenderer
 *
 *@author    Joshua Gould
 */
public class ErrorBarDemo3 {

	/**
	 * demonstrate how to make a plot with error bars to ErrorBarDemo2.png
	 * @throws Exception
	 */
	public ErrorBarDemo3() throws Exception {
		System.out.println(" BEGIN ErrorBarDemo2");
		XYDataset dataset = createDataset();
		JFreeChart chart = createChart(dataset);
		ChartUtilities.saveChartAsPNG(new File("ErrorBarDemo2.png"), chart, 500, 300);				
	}


	/**
	 * call ErrorBarDemo2
	 * @param args ignored
	 * @throws Exception if there is trouble writing the plot file
	 */
	public static void main(String[] args) throws Exception {
		new ErrorBarDemo3();
	}


	private JFreeChart createChart(XYDataset dataset) {
		JFreeChart chart = ChartFactory.createScatterPlot(
				"Error Bar Demo",
		// title
				"x",
		// x-axis label
				"y",
		// y-axis label
				dataset,
		// data
				PlotOrientation.VERTICAL,
		// create legend?
				false,
		// generate tooltips?
				false, false
		// generate URLs?
				);
		XYErrorBarRenderer r = new XYErrorBarRenderer();
		chart.getXYPlot().setRenderer(r);
		r.addChangeListener(chart.getXYPlot());// see bug 1177884

		return chart;
	}


	private XYDataset createDataset() {
		XYErrorBarSeriesCollection dataset = new XYErrorBarSeriesCollection();
		XYErrorBarSeries series = new XYErrorBarSeries("Series 1");
		for (int i = 0; i < 10; i++) {
			series.add(i, i, i - 2, i + 2);
		}
		dataset.addSeries(series);
		return dataset;
	}

}
