package gov.fnal.eag.util.plot;

import gov.fnal.eag.util.densityEstimate.KernelDensityEstimate;

import java.awt.BasicStroke;
import java.awt.Color;

import org.jfree.chart.ChartFactory;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.annotations.XYLineAnnotation;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.plot.XYPlot;
import org.jfree.data.xy.XYSeries;
import org.jfree.data.xy.XYSeriesCollection;

/**
 * @author dcharms
 *
 */
public class KernelDensityEstimatePlot {
	private KernelDensityEstimate kde;
	
	private String title = "";
	
	/**
	 * Constructor for new plot
	 * @param kde the KernelDensityEstimate to be plotted
	 */
	public KernelDensityEstimatePlot(KernelDensityEstimate kde) {
		this.kde = kde;
	}

	/**
	 * Plot the density estimate
	 * @param xMin minimum x value to be included in plot
	 * @param xMax maximum x value to be included in plot
	 * @param showMean plot the mean 
	 * @param showSigma plot the sigma
	 * @return the generated chart
	 */
	public JFreeChart getPlot(double xMin, double xMax, boolean showMean, boolean showSigma) {
		XYSeries xySeries = new XYSeries(title);
		double y[] = kde.getEstimates();
		double x[] = kde.getX();
		double max = 0;
		for(int i=0; i<x.length; i++) {
			if (x[i] >= xMin && x[i]<=xMax)
				xySeries.add(x[i], y[i]);
			if (max < y[i])
				max = y[i];
		}
		XYSeriesCollection xyCollection = new XYSeriesCollection();
		xyCollection.addSeries(xySeries);
		JFreeChart chart = ChartFactory.createXYLineChart(title, "value of object", "number of objects", xyCollection, PlotOrientation.VERTICAL, true, false, false);
		if (showMean) {
			XYPlot plot = chart.getXYPlot();
			plot.addAnnotation(new XYLineAnnotation(kde.getMean(), 0, kde.getMean(), max, new BasicStroke(), Color.BLUE));
		}
		
		if (showSigma) {
			XYPlot plot = chart.getXYPlot();
			plot.addAnnotation(new XYLineAnnotation(kde.getMean()-kde.getSigma()*2.35/2, max/2, kde.getMean()+kde.getSigma()*2.35/2, max/2, new BasicStroke(), Color.ORANGE));
		}
		return chart;
	}
	
	
	/**
	 * Plot the density estimate
	 * @param showMean plot the mean
	 * @param showSigma plot the sigma
	 * @return the generated chart
	 */
	public JFreeChart getPlot(boolean showMean, boolean showSigma) {
		return getPlot(kde.getMin(), kde.getMax(), showMean, showSigma);
	}
	
	/**
	 * Plot the density estimate
	 * @return the generated chart
	 */
	public JFreeChart getPlot() {
		return getPlot(kde.getMin(), kde.getMax(), true, true);
	}
	
	/**
	 * Set title of chart
	 * @param title the title to be used in chart
	 */
	public void setTitle(String title) {
		this.title = title;
	}
}
