package gov.fnal.eag.util.plot;

import java.util.ArrayList;

import org.jfree.data.xy.XYSeries;
import org.jfree.data.xy.XYSeriesCollection;

/**
 * data and errors kept as XYSeries
 * @author stoughto
 *
 */
public class XYPM {
	protected XYSeries data;
	protected XYSeries errors;
	/**
	 * all data no error bars
	 * @param xys the data
	 */
	public XYPM(XYSeries xys) {
		data = xys;
		errors = null;
	}
	/**
	 * empty data and errors
	 * @param name
	 */
	public XYPM(String name) {
		data = new XYSeries(name);
		errors = new XYSeries(name+" errors");
	}
	/**
	 * add a data point and sigma
	 * @param x
	 * @param y
	 * @param s
	 */
	public void add(double x, double y, double s) {
		add(x, y, s, -s);
	}
	/**
	 * add a data point and y plus and y minus values
	 * @param x
	 * @param y
	 * @param yp
	 * @param ym
	 */
	public void add(double x, double y, double yp, double ym) {
		data.add(x,y);
		if (errors != null) {
			errors.add(yp, ym);
		}
	}
	/**
	 * @return the number of data points
	 */
	public Object getItemCount() {
		return data.getItemCount();
	}
	/**
	 * @param xysc a collection of data series
	 * @return XYPMs, defined with no error
	 */
	public static ArrayList<XYPM> makeXYPMs(XYSeriesCollection xysc) {
		ArrayList<XYPM> retval = new ArrayList<XYPM>();
		for (int i=0; i<xysc.getSeriesCount(); i++) {
			XYSeries xys = xysc.getSeries(i);
			XYPM xypm = new XYPM(xys);
			retval.add(xypm);
		}
		return retval;
	}
}
