package gov.fnal.eag.util.stopwatch;

import java.io.FileNotFoundException;
import java.io.PrintStream;
import java.util.Iterator;
import java.util.LinkedList;

/**
 * static collection of stopwatches
 */
public class StopwatchCollection {
	/**
	 * the list of stopwatches being managed
	 */
	static LinkedList<Stopwatch> stopwatches = new LinkedList<Stopwatch>();
	/**
	 * add a new stopwatch with this name to the list
	 * @param name
	 */
	public static void add(String name) {		
		add(new Stopwatch(name));
	}
	/**
	 * @param fileName where to write the report
	 */
	public static void report(String fileName) {
		try {
			PrintStream ps = new PrintStream(fileName);
			report(ps);
			ps.close();
		} catch (FileNotFoundException e) {
			throw new Error(e);
		}
		
	}

	/**
	 * write summary information
	 * @param ps PrintStream to receive the information
	 */
	public static void report(PrintStream ps) {
	    Iterator<Stopwatch> it = stopwatches.iterator();
	    while (it.hasNext()) {
	    	Stopwatch sw = it.next();
	    	sw.tops(ps);
	    }
	}
	/** 
	 * make ps and jpg plots of the names stopwatch to the filename
	 * @param name of the stopwatch
	 * @param filename base name for the plot files
	 */
	public static void plot(String name, String filename) {
		Stopwatch sw = find(name);
		if (sw != null) {
			sw.toPng(filename);
		}
	}	
	/**
	 * start the named stopwatch, after creating it if necessary
	 * @param name the stopwatch
	 */
	public static void start(String name) {
		Stopwatch sw = find(name);
		
		if (sw == null) {
			sw = new Stopwatch(name);
			add(sw);
		}
		sw.start();
	}
	
	/**
	 * add this stopwatch to the list
	 * @param sw Stopwatch to add
	 */
	private static void add(Stopwatch sw) {
		stopwatches.add(sw);
		
	}
	/**
	 * stop the named stopwatch
	 * @param name stopwatch name
	 */
	public static void stop(String name) {
		Stopwatch sw = find(name);
		if (sw != null) {
			sw.stop();
		}
	}

	/**
	 * reset the named stopwatch, if it exists
	 * @param name the stopwatch
	 */
	public static void reset(String name) {
		Stopwatch sw = find(name);
		if (sw != null) {
			sw.reset();
		}
	}

	/**
	 * find the named stopwatch
	 * @param name stopwatch to find
	 * @return the found stopwatch
	 */
	public static Stopwatch find(String name){
	    Iterator<Stopwatch> it = stopwatches.iterator();
	    while (it.hasNext()) {
	    	Stopwatch temp = it.next();
	    	if (temp.getName().equals(name)) {
	    		return temp;
	    	}
	    }
		return null;
	}
    /**
     * get the names of all the stopwatches
     * @return the list of names
     */
    public static LinkedList<String> getNames() {
        LinkedList<String> names = new LinkedList<String>();
         Iterator<Stopwatch> it = stopwatches.iterator();
        while (it.hasNext()) {
            names.add(   it.next().getName()   );
        }
        return names;
    }	
    
    /**
     * initialize the collection
     */
    public static void init() {
    	stopwatches = new LinkedList<Stopwatch>();
    }
    
    /**
     * @param name of the stopwatch
     * @return the elapsed time for the stopwatch
     */
    public static long getElapsed(String name) {
    	Stopwatch sw = find(name);
    	return sw.getElapsedTime();
    }
	/**
	 * reset each stopwatch in the collection
	 */
	public static void resetAll() {
		for (Stopwatch stopwatch : stopwatches) {
			stopwatch.reset();
		}
		
	}
}
