package gov.fnal.eag.wcs;
/**
 * defines a coordinate
 * @author stoughto
 *
 */
public class Coordinate {
	
	private double x;
	private double y;	
	
	/**
	 * @param x coordinate typically column or x
	 * @param y coordinate typically row or y
	 */
	public Coordinate(double x, double y) {
		this.x = x;
		this.y = y;
	}
	
	/**
	 * @return a deep copy of this
	 */
	@Override
	public Coordinate clone() {
		return new Coordinate(x,y);
	}
	
	/**
	 * copy constructor
	 * @param objectCoordinate
	 */
	public Coordinate(Coordinate objectCoordinate) {
		this(objectCoordinate.x, objectCoordinate.y);
	}

	/**
	 * @return x value
	 */
	public double getX() {
		return x;
	}
	
	
	/**
	 * @return y value
	 */
	public double getY() {
		return y;
	}

	/** 
	 * 
	 * @param x value to set
	 */
	public void setX(double x) {
		this.x = x;		
	}
	
	/**
	 * @param y value to set
	 */
	public void setY(double y) {
		this.y = y;
	}

	/**
	 * calculates the linear separation between this and that coordinates
	 * @param that
	 * @return the separation
	 */
	public double linearSeparation(Coordinate that) {
		double dx = getX()-that.getX();
		double dy = getY()-that.getY();
		return Math.sqrt(dx*dx+dy*dy);
	}
	
	/**
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		return "x="+getX()+" y="+getY(); //$NON-NLS-1$
	}

	/**
	 * @param that
	 * @return true if this and that are the same coordinate
	 */
	public boolean equals(Coordinate that) {
		if (x != that.x) return false;
		if (y != that.y) return false;
		return true;
	}
	
}
