package gov.fnal.eag.wcs;

import java.util.ArrayList;

import nom.tam.fits.Header;
import nom.tam.fits.HeaderCard;
import nom.tam.fits.HeaderCardException;
import nom.tam.util.Cursor;

/**
 * utilities to help extract WCS information from headers
 * @author stoughto
 *
 */
public class HUtil {
	/**
	 * @param header
	 * @return true if this header has at least one keyword that starts with "CD"
	 */
	public static boolean definedWithCdValues(Header header) {
		Cursor cursor = header.iterator();
		while (cursor.hasNext()) {
			HeaderCard card = (HeaderCard) cursor.next();
			String key = card.getKey().toUpperCase();
			if (key.equals("CD1_1")) return true;
			if (key.equals("CD1_2")) return true;
			if (key.equals("CD2_1")) return true;
			if (key.equals("CD2_2")) return true;
		}
		return false;
	}

	/**
	 * @param header
	 * @return true if this header has at least one keyword that starts with "PC"
	 */
	public static boolean definedWithPcValues(Header header) {
		Cursor cursor = header.iterator();
		while (cursor.hasNext()) {
			HeaderCard card = (HeaderCard) cursor.next();
			String key = card.getKey().toUpperCase();
			if (key.startsWith("PC")) return true;
			//if (key.startsWith("CDELT")) return true;
		}
		return false;
	}

	/**
	 * remove keyword related to WCS coordinates
	 * @param header clean up this header
	 */
	public static void eraseWcsInfo(Header header) {
		ArrayList<String> keysToDelete = new ArrayList<String>();
		Cursor cursor = header.iterator();
		while (cursor.hasNext()) {
			HeaderCard card = (HeaderCard) cursor.next();
			String key = card.getKey().toUpperCase();
			if (key.startsWith("CDELT")) keysToDelete.add(card.getKey());
			if (key.startsWith("CD1_1")) keysToDelete.add(card.getKey());
			if (key.startsWith("CD1_2")) keysToDelete.add(card.getKey());
			if (key.startsWith("CD2_1")) keysToDelete.add(card.getKey());
			if (key.startsWith("CD2_2")) keysToDelete.add(card.getKey());
			if (key.startsWith("PC1_1")) keysToDelete.add(card.getKey());
			if (key.startsWith("PC1_2")) keysToDelete.add(card.getKey());
			if (key.startsWith("PC2_1")) keysToDelete.add(card.getKey());
			if (key.startsWith("PC2_2")) keysToDelete.add(card.getKey());
			if (key.startsWith("CDELT")) keysToDelete.add(card.getKey());
			if (key.startsWith("CRPIX")) keysToDelete.add(card.getKey());
			if (key.startsWith("CRVAL")) keysToDelete.add(card.getKey());
			if (key.startsWith("CTYPE")) keysToDelete.add(card.getKey());
		}
		for (String key : keysToDelete) {
			header.deleteKey(key);
		}
		
	}

	/**
	 * @param header operates on this Header
	 * @param key keyword to delete and then replace
	 * @param val replacement value
	 * @param comment replacement comment
	 */
	public static void replaceKeywordValue(Header header, String key, double val, String comment) {
		header.deleteKey(key);
		try {
			header.addValue(key, val, comment);
		} catch (HeaderCardException e) {
			throw new Error(e);
		}
	}

	/**
	 * @param header operates on this Header
	 * @param key keyword to delete and then replace
	 * @param val replacement value
	 * @param comment replacement comment
	 */
	public static void replaceKeywordValue(Header header, String key, String val, String comment) {
		header.deleteKey(key);
		try {
			header.addValue(key, val, comment);
		} catch (HeaderCardException e) {
			throw new Error(e);
		}
	}
	/**
	 * @param header operates on this Header
	 * @param key keyword to delete and then replace
	 * @param val replacement value
	 * @param comment replacement comment
	 */
	public static void replaceKeywordValue(Header header, String key, int val, String comment) {
		header.deleteKey(key);
		try {
			header.addValue(key, val, comment);
		} catch (HeaderCardException e) {
			throw new Error(e);
		}
	}

	/**
	 * @param header operates on this Header
	 * @param key keyword to delete and then replace
	 * @param val replacement value
	 * @param comment replacement comment
	 */
	public static void replaceKeywordValue(Header header, String key, boolean val, String comment) {
		header.deleteKey(key);
		try {
			header.addValue(key, val, comment);
		} catch (HeaderCardException e) {
			throw new Error(e);
		}
	}
	
	
}
