package gov.fnal.eag.wcs;

import nom.tam.fits.Header;

/**
 * convenience class for checking sanity of values
 * @author stoughto
 *
 */
public class ValueCheck {
	/**
	 * @param value the value to check
	 * @throws IllegalArgumentException if the value isInfinite or isNaN
	 */
	public static void isSane(double value) {
		isSane(value, "");
	}
	/**
	 * @param value the value to check
	 * @param message prepended to message string in IllegalArgumentException call
	 * @throws IllegalArgumentException if the value isInfinite or isNaN
	 */
	public static void isSane(double value, String message) {
		if (Double.isInfinite(value)) {
			throw new IllegalArgumentException(message+" infinite value="+value);
		}
		if (Double.isNaN(value)) {
			throw new IllegalArgumentException(message+" NaN value="+value);
		}
	}
	
	/**
	 * @param header 
	 * @param key
	 * @param defaultValue 
	 * @return the double value of key in header
	 * @throws IllegalArgumentException is the value is infinite or NaN
	 */
	public static double getDoubleValue(Header header, String key, double defaultValue) {
		double retval = header.getDoubleValue(key, defaultValue);
		isSane(retval, key);
		return retval;
	}
}
