package jsky.coords;

import nom.tam.fits.Header;

/**
 * simple implementation of WCSKeywordProvider that uses a Header.  
 * It is up to the programmer to make sure the Header has correct and consistent keywords and values.
 * @author stoughto
 *
 */
public class WCSKeywordProviderHeader implements WCSKeywordProvider {

	private Header header;
	
	/**
	 * constructs a provider that dishes out information from the Header
	 * @param header the Header with all the necessary information, we presume
	 */
	public WCSKeywordProviderHeader( Header header) {
		this.header = header;
	}
	
	/**
	 * @see jsky.coords.WCSKeywordProvider#findKey(java.lang.String)
	 */
	public boolean findKey(String key) {
		String temp = header.findKey(key);
		if (temp == null) {
			return false;
		} else {
			return true;
		}
	}

	/**
	 * @see jsky.coords.WCSKeywordProvider#getStringValue(java.lang.String)
	 */
	public String getStringValue(String key) {	
		return header.getStringValue(key);
	}

	/**
	 * @see jsky.coords.WCSKeywordProvider#getStringValue(java.lang.String, java.lang.String)
	 */
	public String getStringValue(String key, String defaultValue) {
		String value = header.getStringValue(key);
		if (value == null) {
			return defaultValue;
		} else {
			return value;
		}
	}

	/**
	 * @see jsky.coords.WCSKeywordProvider#getDoubleValue(java.lang.String)
	 */
	public double getDoubleValue(String key) {
		return header.getDoubleValue(key);
	}

	/**
	 * @see jsky.coords.WCSKeywordProvider#getDoubleValue(java.lang.String, double)
	 */
	public double getDoubleValue(String key, double defaultValue) {
		return header.getDoubleValue(key, defaultValue);
	}

	/**
	 * @see jsky.coords.WCSKeywordProvider#getFloatValue(java.lang.String)
	 */
	public float getFloatValue(String key) {
		return	header.getFloatValue(key);
	}

	/**
	 * @see jsky.coords.WCSKeywordProvider#getFloatValue(java.lang.String, float)
	 */
	public float getFloatValue(String key, float defaultValue) {
		return header.getFloatValue(key, defaultValue);
	}

	/**
	 * @see jsky.coords.WCSKeywordProvider#getIntValue(java.lang.String)
	 */
	public int getIntValue(String key) {
		return header.getIntValue(key);
	}

	/**
	 * @see jsky.coords.WCSKeywordProvider#getIntValue(java.lang.String, int)
	 */
	public int getIntValue(String key, int defaultValue) {
		return header.getIntValue(key, defaultValue);
	}

}
