package jsky.coords.test;

import java.awt.geom.Point2D;

import jsky.coords.WCSTransform;
import junit.framework.TestCase;

public class WCSTransformTest extends TestCase {
    /**
     * test that position, width, and center work for a simple transformation
     */
    public void testWCSTransformSimple() {
        double centerRightAscension = 100.0;
        double centerDeclination = 0.0;
        double xArcsecPerPixel = 0.5;
        double yArcsecPerPixel = xArcsecPerPixel;
        double xReferencePixel = 500.0;
        double yReferencePixel = 500.0;
        int nXPixels = 1000;
        int nYPixels = 1000;
        double rotateClockwise = 0.0;
        int equinox = 2000;
        double epoch = 0.0;
        String projection = "TAN";
        double truePos1x = centerRightAscension-0.5*nXPixels*xArcsecPerPixel/3600.0;
        double truePos2x = centerRightAscension+0.5*nXPixels*xArcsecPerPixel/3600.0;
        double trueWidth = truePos2x-truePos1x;

        WCSTransform tr = new WCSTransform(centerRightAscension, centerDeclination, xArcsecPerPixel, yArcsecPerPixel,
                xReferencePixel, yReferencePixel, nXPixels, nYPixels, rotateClockwise, equinox, epoch, projection);

        double xpix=0.0;
        double ypix = 500.0;
        Point2D.Double p = tr.pix2wcs(xpix, ypix);
        assertEquals(" x position", truePos1x, p.x, 1e-10);
        double width = tr.getWidthInDeg();
        assertEquals(" width", trueWidth, width, 1e-10);

        Point2D.Double center = tr.getWCSCenter();
        assertEquals(" x center", centerRightAscension, center.x, 1e-10);

     }
    
    /**
     * test the forward and reverse transformations that shift a detector by one pixel.
     *
     */
    public void testWcsShift() {
    	// common values
    	int equinox = 2000;
    	double epoch = 2000.0;
    	String projection = "TAN";
    	double rotation = 0.0;
    	double centerRa = 100.0;
    	double centerDec = 0.0;
    	double xArcsecPerPixel = 1.0;
    	double yArcsecPerPixel = xArcsecPerPixel;
    	int nXPixels = 100;
    	int nYPixels = 100;
    	double xRef1 = 2.5;
    	double yRef1 = 0.0;
    	WCSTransform tr1 = new WCSTransform(centerRa, centerDec, xArcsecPerPixel, yArcsecPerPixel, xRef1, yRef1, nXPixels, nYPixels, rotation, equinox, epoch, projection);
    	Point2D.Double raDec = tr1.pix2wcs(xRef1, yRef1);
    	Point2D.Double rc1 = tr1.wcs2pix(raDec.x, raDec.y);
    	assertEquals(xRef1, rc1.x, 1e-9);
    	assertEquals(yRef1, rc1.y, 1e-9);
    	
    	double xRef2 = 1.5;
    	double yRef2 = 0.0;
    	WCSTransform tr2 = new WCSTransform(centerRa, centerDec, xArcsecPerPixel, yArcsecPerPixel, xRef2, yRef2, nXPixels, nYPixels, rotation, equinox, epoch, projection);
    	Point2D.Double rc2 = tr2.wcs2pix(raDec.x, raDec.y);
    	
    	assertEquals(xRef2, rc2.x, 1e-9);
    	assertEquals(yRef2, rc2.y, 1e-9);
    	
    }
    public void testWCSTransformDes() {
        double centerRightAscension = 100.0;
        double centerDeclination = 0.0;
        double xArcsecPerPixel = 7.5E-5*3600;
        double yArcsecPerPixel = xArcsecPerPixel;
        int nXPixels = 2048;
        int nYPixels = 4096;
        
        double xReferencePixel = nXPixels/2.0;
        double yReferencePixel = nYPixels/2.0;
       
        double rotateClockwise = 0.0;
        int equinox = 2000;
        double epoch = 2000.0;
        String projection = "TAN";
        double truePos1x = centerRightAscension-0.5*nXPixels*xArcsecPerPixel/3600.0;
        double truePos2x = centerRightAscension+0.5*nXPixels*xArcsecPerPixel/3600.0;
        double trueWidth = truePos2x-truePos1x;
        System.out.println(" trueWidth (degrees) = "+trueWidth);

        double truePos1y = centerDeclination-0.5*nYPixels*yArcsecPerPixel/3600.0;
        double truePos2y = centerDeclination+0.5*nYPixels*yArcsecPerPixel/3600.0;
        double trueHeight = truePos2y-truePos1y;
        System.out.println(" trueHeight (degrees) = "+trueHeight);
        
        WCSTransform tr = new WCSTransform(centerRightAscension, centerDeclination, xArcsecPerPixel, yArcsecPerPixel,
                xReferencePixel, yReferencePixel, nXPixels, nYPixels, rotateClockwise, equinox, epoch, projection);

        double xpix1=0.0;
        double ypix1 = 0.0;
        Point2D.Double p1 = tr.pix2wcs(xpix1, ypix1);
        assertEquals(" x1 position", truePos1x, p1.x, 1e-10);
        assertEquals(" y1 position", truePos1y, p1.y, 1e-10);

        double xpix2 = nXPixels;
        double ypix2 = nYPixels;
        Point2D.Double p2 = tr.pix2wcs(xpix2, ypix2);
        assertEquals(" x2 position", truePos2x, p2.x, 1e-10);
        assertEquals(" y2 position", truePos2y, p2.y, 1e-10);
        
        double width = tr.getWidthInDeg();
        assertEquals(" width", trueWidth, width, 1e-10);
        
        double height = tr.getHeightInDeg();
        assertEquals(" height", trueHeight, height, 1e-10);
        
        Point2D.Double center = tr.getWCSCenter();
        assertEquals(" x center", centerRightAscension, center.x, 1e-10);
        assertEquals(" y center", centerDeclination, center.y, 1e-10);
        
     }

}
