package snap.snoop.framework.visualize;

import hep.aida.IAnalysisFactory;
import hep.aida.IBaseHistogram;
import hep.aida.ICloud1D;
import hep.aida.ICloud2D;
import hep.aida.IDataPointSet;
import hep.aida.IDataPointSetFactory;
import hep.aida.IHistogram1D;
import hep.aida.IHistogram2D;
import hep.aida.IHistogramFactory;
import hep.aida.IPlotter;
import hep.aida.IPlotterFactory;
import hep.aida.IPlotterStyle;
import hep.aida.ITree;

/**
 * The class Subplot provides for a mutliplot layout. The plot window canvas is
 * divided into a grid of rows and colums provided by the user. Each cell in the
 * grid becomes a separate plot as specified by the user. For example, the user
 * who wishes to have 9 plots on the canvas could contruct the Subplot with rows
 * and columns of 3 x 3, e.g.
 * <p>
 * Subplot s = new Subplot (3, 3);
 * <p>
 * The above object contains a plotteing region with 9 cells numbered starting
 * with 0. Any of the Visualize plotting methods are then used to plot in the
 * specified regions as following,
 * <p>
 * s.plot(0, x, y); //where argumnet 0 is the first cell and x and y are arrays
 * of double
 * <p>
 * s.hist(1, x); //here argument 1 is the next cell in the grid and will show a
 * histogram of x
 * <p>
 * and so on...
 * <p>
 * 
 * @author Hemant Shukla
 */
public class SubPlot
{
    private IAnalysisFactory analysisFactory;

    private ITree tree;

    private IPlotterFactory plotterFactory;

    private IPlotter plotter;

    int columns;

    int rows;

	/** String to use for window title **/
	String title = "Visualize";

    public SubPlot (int x, int y)
    {
        rows = x;
        columns = y;

        init();
    }

    /***************************************************************************
     * Plot data and line plots
     **************************************************************************/

    public IPlotter plot (int plotRegion, double[] data)
    {
        //        clear(plotRegion);

        // Create data point set
        IDataPointSetFactory dps = analysisFactory
                .createDataPointSetFactory(tree);
        IDataPointSet dps1D = dps.create("dps1D", "<<Title>>", 1);

        for (int i = 0; i < data.length; i++)
        {
            dps1D.addPoint();
            dps1D.point(i).coordinate(0).setValue(data[i]);
        }

        //plot it
        plotter.region(plotRegion).plot(dps1D, setPlotterStyle());

        return plotter;
    }

    public IPlotter plot (int plotRegion, double[] x, double[] y)
    {
        //Create data point set
        IDataPointSetFactory dps = analysisFactory
                .createDataPointSetFactory(tree);
        IDataPointSet dps2D = dps.create("dps2D", "<<Title>>", 2);

        //TO DO:
        //check if x and y have same dimensions else
        //throw and exception...

        //fill it
        for (int i = 0; i < y.length; i++)
        {
            dps2D.addPoint();
            dps2D.point(i).coordinate(0).setValue(x[i]);
            dps2D.point(i).coordinate(1).setValue(y[i]);
        }

        //plot it
        plotter.region(plotRegion).plot(dps2D, setPlotterStyle());

        return plotter;
    }

    /**
     * The method takes a DataTuple type and plots the single column based on
     * the name provided by the string columnName. NOTE: This plots only double,
     * int and float for now. Needs to exit elegantly.
     * 
     * @param tuple
     *            of type DataTuple
     * @param columnNameY
     *            String name of the vector
     *  
     */
    public IPlotter plot (int plotRegion, DataTuple tuple, String columnNameY)
    {
        //Create data point set
        IDataPointSetFactory dps = analysisFactory
                .createDataPointSetFactory(tree);
        IDataPointSet dps1D = dps.create("dps1D", "<<Title>>", 1);

        Class type = tuple.getTuple().columnType(
                tuple.getTuple().findColumn(columnNameY));

        if (type.getName().compareTo("double") == 0)
        {
            double[] y = new double[tuple.getTupleSize()];
            y = tuple.getColumnAsDouble(columnNameY);

            //fill it
            for (int i = 0; i < y.length; i++)
            {
                dps1D.addPoint();
                dps1D.point(i).coordinate(0).setValue(y[i]);
            }
        }

        if (type.getName().compareTo("int") == 0)
        {
            int[] y = new int[tuple.getTupleSize()];
            y = tuple.getColumnAsInteger(columnNameY);

            //fill it
            for (int i = 0; i < y.length; i++)
            {
                dps1D.addPoint();
                dps1D.point(i).coordinate(0).setValue(y[i]);
            }
        }

        if (type.getName().compareTo("float") == 0)
        {
            float[] y = new float[tuple.getTupleSize()];
            y = tuple.getColumnAsFloat(columnNameY);

            //fill it
            for (int i = 0; i < y.length; i++)
            {
                dps1D.addPoint();
                dps1D.point(i).coordinate(0).setValue(y[i]);
            }
        }

        //plot it
        plotter.region(plotRegion).plot(dps1D, setPlotterStyle());

        return plotter;
    }

    /**
     * @param plotRegion
     * @param tuple
     * @param columnNameX
     * @param columnNameY
     * @return
     */
    public IPlotter plot (int plotRegion, DataTuple tuple, String columnNameX,
            String columnNameY)
    {
        //Create data point set
        IDataPointSetFactory dps = analysisFactory
                .createDataPointSetFactory(tree);
        IDataPointSet dps2D = dps.create("dps2D", "<<Title>>", 2);

        Class typeX = tuple.getTuple().columnType(
                tuple.getTuple().findColumn(columnNameX));
        Class typeY = tuple.getTuple().columnType(
                tuple.getTuple().findColumn(columnNameY));

        int size = tuple.getTupleSize();

        if (typeX.getName().compareTo("double") == 0)
        {
            double[] x = new double[size];
            x = tuple.getColumnAsDouble(columnNameX);

            //fill it
            for (int i = 0; i < x.length; i++)
            {
                dps2D.addPoint();
                dps2D.point(i).coordinate(0).setValue(x[i]);
            }
        }

        if (typeX.getName().compareTo("int") == 0)
        {
            int[] x = new int[size];
            x = tuple.getColumnAsInteger(columnNameX);

            //fill it
            for (int i = 0; i < x.length; i++)
            {
                dps2D.addPoint();
                dps2D.point(i).coordinate(0).setValue(x[i]);
            }
        }

        if (typeY.getName().compareTo("double") == 0)
        {
            double[] y = new double[size];
            y = tuple.getColumnAsDouble(columnNameY);

            //fill it
            for (int i = 0; i < y.length; i++)
            {
                dps2D.addPoint();
                dps2D.point(i).coordinate(1).setValue(y[i]);
            }
        }

        if (typeY.getName().compareTo("int") == 0)
        {
            int[] y = new int[size];
            y = tuple.getColumnAsInteger(columnNameY);

            //fill it
            for (int i = 0; i < y.length; i++)
            {
                dps2D.addPoint();
                dps2D.point(i).coordinate(1).setValue(y[i]);
            }
        }

        //plot it
        plotter.region(plotRegion).plot(dps2D, setPlotterStyle());

        return plotter;
    }

    /**
     * 
     * @param x
     * @param y
     * @return
     */
    public IPlotter plot (int plotRegion, double[] x, double[] y, double[] err)
    {
        //Create data point set
        IDataPointSetFactory dps = analysisFactory
                .createDataPointSetFactory(tree);
        IDataPointSet dps2D = dps.create("dps2D", "<<Title>>", 2);

        //TO DO:
        //check if x and y have same dimensions else
        //throw and exception...

        //fill it
        for (int i = 0; i < y.length; i++)
        {
            dps2D.addPoint();
            dps2D.point(i).coordinate(0).setValue(x[i]);
            dps2D.point(i).coordinate(1).setValue(y[i]);
            dps2D.point(i).coordinate(1).setErrorMinus(err[i]);
            dps2D.point(i).coordinate(1).setErrorPlus(err[i]);
        }

        //plot it
        plotter.region(plotRegion).plot(dps2D, setPlotterStyle());

        return plotter;
    }

    public void plotOverlay (int plotRegion, double[] data, IPlotterStyle style)
    {
        //        clear(plotRegion);

        // Create data point set
        IDataPointSetFactory dps = analysisFactory
                .createDataPointSetFactory(tree);
        IDataPointSet dps1D = dps.create("dps1D", "<<Title>>", 1);

        for (int i = 0; i < data.length; i++)
        {
            dps1D.addPoint();
            dps1D.point(i).coordinate(0).setValue(data[i]);
        }

        //plot it
        plotter.region(plotRegion).plot(dps1D, style, "mode=overlay");
    }
    
    public void plotOverlay (int plotRegion, double[] x, double[] y)
    {
        //Create data point set
        IDataPointSetFactory dps = analysisFactory
                .createDataPointSetFactory(tree);
        IDataPointSet dps2D = dps.create("dps2D", "<<Title>>", 2);

        //TO DO:
        //check if x and y have same dimensions else
        //throw and exception...

        //fill it
        for (int i = 0; i < y.length; i++)
        {
            dps2D.addPoint();
            dps2D.point(i).coordinate(0).setValue(x[i]);
            dps2D.point(i).coordinate(1).setValue(y[i]);
        }

        //plot it
        plotter.region(plotRegion).plot(dps2D, "mode=overlay");
    }

    public void plotOverlay (int plotRegion, double[] x, double[] y,
            double[] err)
    {
        //Create data point set
        IDataPointSetFactory dps = analysisFactory
                .createDataPointSetFactory(tree);
        IDataPointSet dps2D = dps.create("dps2D", "<<Title>>", 2);

        //TO DO:
        //check if x and y have same dimensions else
        //throw and exception...

        //fill it
        for (int i = 0; i < y.length; i++)
        {
            dps2D.addPoint();
            dps2D.point(i).coordinate(0).setValue(x[i]);
            dps2D.point(i).coordinate(1).setValue(y[i]);
            dps2D.point(i).coordinate(1).setErrorMinus(err[i]);
            dps2D.point(i).coordinate(1).setErrorPlus(err[i]);
        }

        //plot it
        plotter.region(plotRegion).plot(dps2D, "mode=overlay");
    }

    public void plotOverlay (int plotRegion, double[] x, double[] y,
            double[] err, IPlotterStyle style)
    {
        //Create data point set
        IDataPointSetFactory dps = analysisFactory
                .createDataPointSetFactory(tree);
        IDataPointSet dps2D = dps.create("dps2D", "<<Title>>", 2);

        //TO DO:
        //check if x and y have same dimensions else
        //throw and exception...

        //fill it
        for (int i = 0; i < y.length; i++)
        {
            dps2D.addPoint();
            dps2D.point(i).coordinate(0).setValue(x[i]);
            dps2D.point(i).coordinate(1).setValue(y[i]);
            dps2D.point(i).coordinate(1).setErrorMinus(err[i]);
            dps2D.point(i).coordinate(1).setErrorPlus(err[i]);
        }

        //plot it
        plotter.region(plotRegion).plot(dps2D, style,
                "mode=overlay");
    }

    /***************************************************************************
     * Plot Histogram
     **************************************************************************/

    /**
     * @param y
     * @return
     */
    public IPlotter hist (int plotRegion, double[] y)
    {
        IHistogramFactory hf = analysisFactory.createHistogramFactory(tree);
        IHistogram1D h = hf.createHistogram1D(" ", "", 50, min(y)
                + (min(y) * 0.2), max(y) + (max(y) * 0.2));

        for (int i = 0; i < y.length; i++)
        {
            h.fill(y[i]);
        }
        plotter.region(plotRegion).plot((IBaseHistogram) h, setPlotterStyle());

        return plotter;
    }

    /**
     * @param x
     * @param y
     * @return
     */
    public IPlotter hist (int plotRegion, double[] x, double[] y)
    {
        IHistogramFactory hf = analysisFactory.createHistogramFactory(tree);
        IHistogram2D h = hf.createHistogram2D(" ", "", 50, min(x)
                + (min(x) * 0.2), max(x) + (max(x) * 0.2), 50, min(y)
                + (min(y) * 0.2), max(y) + (max(y) * 0.2));

        for (int i = 0; i < y.length; i++)
        {
            h.fill(x[i], y[i]);
        }
        plotter.region(plotRegion).plot((IBaseHistogram) h, setPlotterStyle());

        return plotter;
    }

    /**
     * @param y
     * @param bin
     * @return
     */
    public IPlotter hist (int plotRegion, double[] y, int bin)
    {
        IHistogramFactory hf = analysisFactory.createHistogramFactory(tree);
        IHistogram1D h = hf.createHistogram1D(" ", "", bin, min(y)
                + (min(y) * 0.2), max(y) + (max(y) * 0.2));

        for (int i = 0; i < y.length; i++)
        {
            h.fill(y[i]);
        }

        plotter.region(plotRegion).plot((IBaseHistogram) h, setPlotterStyle());

        return plotter;
    }

    /***************************************************************************
     * Plot Scatter
     **************************************************************************/

    /**
     * @param y
     * @return
     */
    public IPlotter scatter (int plotRegion, double[] y)
    {
        IHistogramFactory hf = analysisFactory.createHistogramFactory(tree);
        ICloud1D c = hf.createCloud1D(" ");

        for (int i = 0; i < y.length; i++)
        {
            c.fill(y[i]);
        }

        plotter.region(plotRegion).plot(c, setPlotterStyle());

        return plotter;
    }

    /**
     * @param x
     * @param y
     * @return
     */
    public IPlotter scatter (int plotRegion, double[] x, double[] y)
    {
        IHistogramFactory hf = analysisFactory.createHistogramFactory(tree);
        ICloud2D c = hf.createCloud2D(" ");

        for (int i = 0; i < y.length; i++)
        {
            c.fill(x[i], y[i]);
        }

        plotter.region(plotRegion).plot(c, setPlotterStyle());

        return plotter;
    }

    /***************************************************************************
     * Public Utility Methods
     **************************************************************************/

    public IPlotter getPlotter ()
    {
        return plotter;
    }

    public void setTitle (IPlotter p, int r, String title)
    {
        p.region(r).setTitle(title);
    }

	public void setWindowTitle(String newTitle)
	{
		title = newTitle;
		plotter.setTitle(title);
	}
    
    public String getWindowTitle()
    {
        return title;
    }

    public IPlotterStyle getStyle ()
    {
        IPlotterFactory newPlotterFactory = analysisFactory
                .createPlotterFactory();
        IPlotterStyle style = newPlotterFactory.createPlotterStyle();
        return style;
    }

    /***************************************************************************
     * Private methods
     **************************************************************************/

    /**
     * @throws IllegalArgumentException
     */
    private void init () throws IllegalArgumentException
    {
        analysisFactory = IAnalysisFactory.create();
        tree = analysisFactory.createTreeFactory().create();

        plotterFactory = analysisFactory.createPlotterFactory();
        plotter = plotterFactory.create();
        plotter.createRegions(rows, columns);
    }

    /**
     * @param plotRegion
     */
    private void clear (int plotRegion)
    {
        //        plotter.setCurrentRegionNumber(plotRegion);
        //        plotter.currentRegion().clear();

        //        plotter.region(plotRegion).clear();
    }

    /**
     * @return
     */
    private IPlotterStyle setPlotterStyle ()
    {
        //This styles overwrite some of the region styles
        IPlotterStyle style = plotterFactory.createPlotterStyle();
        style.dataStyle().markerStyle().setParameter("size", "3");
        style.dataStyle().markerStyle().setParameter("shape", "8");
        style.dataStyle().markerStyle().setParameter("color", "blue");

        /**
         * Set fonts
         */
        String font = "Gill Sans";

        //Title
        style.titleStyle().textStyle().setFont(font);
        style.titleStyle().textStyle().setFontSize(14);

        //X Axis
        style.xAxisStyle().labelStyle().setFont(font);
        style.xAxisStyle().labelStyle().setFontSize(12);

        style.xAxisStyle().tickLabelStyle().setFont(font);
        style.xAxisStyle().tickLabelStyle().setFontSize(12);

        //Y Axis
        style.yAxisStyle().labelStyle().setFont(font);
        style.yAxisStyle().labelStyle().setFontSize(12);

        style.yAxisStyle().tickLabelStyle().setFont(font);
        style.yAxisStyle().tickLabelStyle().setFontSize(12);

        return style;
    }

    private static double min (double[] x)
    {
        double minX = x[0];

        for (int i = 1; i < x.length; i++)
        {
            if (minX > x[i])
            {
                minX = x[i];
            }
        }

        return minX;
    }

    private static double max (double[] x)
    {
        double maxX = x[0];

        for (int i = 1; i < x.length; i++)
        {
            if (maxX < x[i])
            {
                maxX = x[i];
            }
        }

        return maxX;
    }

    private static int min (int[] x)
    {
        int minX = x[0];

        for (int i = 1; i < x.length; i++)
        {
            if (minX > x[i])
            {
                minX = x[i];
            }
        }

        return minX;
    }

    private static int max (int[] x)
    {
        int maxX = x[0];

        for (int i = 1; i < x.length; i++)
        {
            if (maxX < x[i])
            {
                maxX = x[i];
            }
        }

        return maxX;
    }

    private static float min (float[] x)
    {
        float minX = x[0];

        for (int i = 1; i < x.length; i++)
        {
            if (minX > x[i])
            {
                minX = x[i];
            }
        }

        return minX;
    }

    private static float max (float[] x)
    {
        float maxX = x[0];

        for (int i = 1; i < x.length; i++)
        {
            if (maxX < x[i])
            {
                maxX = x[i];
            }
        }

        return maxX;
    }
}